@@ -886,7 +886,8 @@ static void ai_diplomacy_suggest(struct player *pplayer,
 void dai_diplomacy_first_contact(struct player *pplayer,
 				 struct player *aplayer)
 {
-  if (pplayer->ai_controlled && !ai_handicap(pplayer, H_AWAY)) {
+  if (pplayer->ai_controlled && !ai_handicap(pplayer, H_AWAY)
+      && player_diplstate_get(pplayer, aplayer)->type == DS_WAR) {
     notify(aplayer, _("*%s (AI)* Greetings %s! May we suggest a ceasefire "
            "while we get to know each other better?"),
            player_name(pplayer),
@@ -1478,7 +1479,7 @@ void dai_diplomacy_actions(struct player *pplayer)
     if (player_diplstate_get(pplayer, target)->has_reason_to_cancel > 0) {
       /* We have good reason */
       war_reason = WAR_REASON_EXCUSE;
-    } if (pplayer->ai_common.love[player_index(target)] < 0) {
+    } else if (pplayer->ai_common.love[player_index(target)] < 0) {
       /* We have a reason of sorts from way back, maybe? */
       war_reason = WAR_REASON_HATRED;
     } else {
