@@ -17,23 +17,31 @@
 **************************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
-#include <assert.h>
 #include <string.h>
 
+/* utility */
+#include "fciconv.h"
 #include "fcintl.h"
-#include "game.h"
 #include "log.h"
 #include "mem.h"
 #include "support.h"
 
-#include "agents.h"
+/* common */
+#include "game.h"
+#include "specialist.h"
+
+/* client */
 #include "attribute.h"
 
+/* client/agents */
+#include "agents.h"
+
 #include "cma_fec.h"
 
+
 #define RESULT_COLUMNS		10
 #define BUFFER_SIZE		100
 #define MAX_LEN_PRESET_NAME	80
@@ -81,14 +89,14 @@ void cmafec_init(void)
 }
 
 /**************************************************************************
- ...
+  Free resources allocated for presets system.
 **************************************************************************/
 void cmafec_free(void)
 {
   while (cmafec_preset_num() > 0) {
     cmafec_preset_remove(0);
   }
-  preset_list_free(preset_list);
+  preset_list_destroy(preset_list);
 }
 
 /**************************************************************************
@@ -135,7 +143,7 @@ void cmafec_preset_add(const char *descr_name, struct cm_parameter *pparam)
 
   cm_copy_parameter(&ppreset->parameter, pparam);
   ppreset->descr = fc_malloc(MAX_LEN_PRESET_NAME);
-  (void) mystrlcpy(ppreset->descr, descr_name, MAX_LEN_PRESET_NAME);
+  (void) fc_strlcpy(ppreset->descr, descr_name, MAX_LEN_PRESET_NAME);
   preset_list_prepend(preset_list, ppreset);
 }
 
@@ -146,10 +154,10 @@ void cmafec_preset_remove(int index)
 {
   struct cma_preset *ppreset;
 
-  assert(index >= 0 && index < cmafec_preset_num());
+  fc_assert_ret(index >= 0 && index < cmafec_preset_num());
 
   ppreset = preset_list_get(preset_list, index);
-  preset_list_unlink(preset_list, ppreset);
+  preset_list_remove(preset_list, ppreset);
 
   free(ppreset->descr);
   free(ppreset);
@@ -162,7 +170,7 @@ char *cmafec_preset_get_descr(int index)
 {
   struct cma_preset *ppreset;
 
-  assert(index >= 0 && index < cmafec_preset_num());
+  fc_assert_ret_val(index >= 0 && index < cmafec_preset_num(), NULL);
 
   ppreset = preset_list_get(preset_list, index);
   return ppreset->descr;
@@ -175,7 +183,7 @@ const struct cm_parameter *cmafec_preset_get_parameter(int index)
 {
   struct cma_preset *ppreset;
 
-  assert(index >= 0 && index < cmafec_preset_num());
+  fc_assert_ret_val(index >= 0 && index < cmafec_preset_num(), NULL);
 
   ppreset = preset_list_get(preset_list, index);
   return &ppreset->parameter;
@@ -208,7 +216,7 @@ int cmafec_preset_num(void)
 }
 
 /**************************************************************************
-...
+  Return short description of city governor preset
 **************************************************************************/
 const char *cmafec_get_short_descr_of_city(const struct city *pcity)
 {
@@ -238,7 +246,7 @@ const char *cmafec_get_short_descr(const struct cm_parameter *const
 }
 
 /**************************************************************************
-...
+  Return string describing when city is assumed to grow.
 **************************************************************************/
 static const char *get_city_growth_string(struct city *pcity, int surplus)
 {
@@ -246,12 +254,12 @@ static const char *get_city_growth_string(struct city *pcity, int surplus)
   static char buffer[50];
 
   if (surplus == 0) {
-    my_snprintf(buffer, sizeof(buffer), _("never"));
+    fc_snprintf(buffer, sizeof(buffer), _("never"));
     return buffer;
   }
 
   stock = pcity->food_stock;
-  cost = city_granary_size(pcity->size);
+  cost = city_granary_size(city_size_get(pcity));
 
   stock += surplus;
 
@@ -266,13 +274,13 @@ static const char *get_city_growth_string(struct city *pcity, int surplus)
       turns = (stock / surplus);
     }
   }
-  my_snprintf(buffer, sizeof(buffer), PL_("%d turn", "%d turns", turns),
-	      turns);
+  fc_snprintf(buffer, sizeof(buffer), PL_("%d turn", "%d turns", turns),
+              turns);
   return buffer;
 }
 
 /**************************************************************************
-...
+  Return string describing when city is assumed to finish current production
 **************************************************************************/
 static const char *get_prod_complete_string(struct city *pcity, int surplus)
 {
@@ -280,24 +288,17 @@ static const char *get_prod_complete_string(struct city *pcity, int surplus)
   static char buffer[50];
 
   if (surplus <= 0) {
-    my_snprintf(buffer, sizeof(buffer), _("never"));
+    fc_snprintf(buffer, sizeof(buffer), _("never"));
     return buffer;
   }
 
-  stock = pcity->shield_stock;
-  if (pcity->production.is_unit) {
-    cost = unit_build_shield_cost(utype_by_number(pcity->production.value));
-  } else {
-    if (!pcity->production.is_unit
-	&& improvement_has_flag(pcity->production.value, IF_GOLD)) {
-      my_snprintf(buffer, sizeof(buffer), "%s",
-		  improvement_name_translation(pcity->production.value));
-      return buffer;
-    }
-    cost = impr_build_shield_cost(pcity->production.value);
+  if (city_production_has_flag(pcity, IF_GOLD)) {
+    fc_strlcpy(buffer, improvement_name_translation
+               (pcity->production.value.building), sizeof(buffer));
+    return buffer;
   }
-
-  stock += surplus;
+  stock = pcity->shield_stock + surplus;
+  cost = city_production_build_shield_cost(pcity);
 
   if (stock >= cost) {
     turns = 1;
@@ -310,58 +311,66 @@ static const char *get_prod_complete_string(struct city *pcity, int surplus)
       turns = (stock / surplus);
     }
   }
-  my_snprintf(buffer, sizeof(buffer), PL_("%d turn", "%d turns", turns),
-	      turns);
+  fc_snprintf(buffer, sizeof(buffer), PL_("%d turn", "%d turns", turns),
+              turns);
   return buffer;
 }
 
 /**************************************************************************
-...
+  Return string describing result
 **************************************************************************/
 const char *cmafec_get_result_descr(struct city *pcity,
-				    const struct cm_result *const
-				    result,
-				    const struct cm_parameter *const
-				    parameter)
+                                    const struct cm_result *result,
+                                    const struct cm_parameter *const
+                                    parameter)
 {
   int j;
   char buf[RESULT_COLUMNS][BUFFER_SIZE];
+  char citizens[BUFFER_SIZE];
   static char buffer[600];
 
+  /* TRANS: "W" is worker citizens, as opposed to specialists;
+   * %s will represent the specialist types, for instance "E/S/T" */
+  fc_snprintf(citizens, BUFFER_SIZE, _("People (W/%s)"),
+              specialists_abbreviation_string());
+
   if (!result->found_a_valid) {
     for (j = 0; j < RESULT_COLUMNS; j++)
-      my_snprintf(buf[j], BUFFER_SIZE, "---");
+      fc_snprintf(buf[j], BUFFER_SIZE, "---");
   } else {
     output_type_iterate(j) {
-      my_snprintf(buf[j], BUFFER_SIZE, "%+3d", result->surplus[j]);
+      fc_snprintf(buf[j], BUFFER_SIZE, "%+3d", result->surplus[j]);
     } output_type_iterate_end;
 
-    my_snprintf(buf[6], BUFFER_SIZE, "%d/%s%s",
-		pcity->size - cm_count_specialist(pcity, result),
-		specialists_string(result->specialists),
-		result->happy ? _(" happy") : "");
+    fc_snprintf(buf[6], BUFFER_SIZE, "%d/%s%s",
+                city_size_get(pcity) - cm_result_specialists(result),
+                specialists_string(result->specialists),
+                result->happy ? _(" happy") : "");
 
-    my_snprintf(buf[7], BUFFER_SIZE, "%s",
-		get_city_growth_string(pcity, result->surplus[O_FOOD]));
-    my_snprintf(buf[8], BUFFER_SIZE, "%s",
-		get_prod_complete_string(pcity, result->surplus[O_SHIELD]));
-    my_snprintf(buf[9], BUFFER_SIZE, "%s",
-		cmafec_get_short_descr(parameter));
+    fc_snprintf(buf[7], BUFFER_SIZE, "%s",
+                get_city_growth_string(pcity, result->surplus[O_FOOD]));
+    fc_snprintf(buf[8], BUFFER_SIZE, "%s",
+                get_prod_complete_string(pcity, result->surplus[O_SHIELD]));
+    fc_snprintf(buf[9], BUFFER_SIZE, "%s",
+                cmafec_get_short_descr(parameter));
   }
 
-  my_snprintf(buffer, sizeof(buffer),
-	      _("Name: %s\n"
-		"Food:       %10s Gold:    %10s\n"
-		"Production: %10s Luxury:  %10s\n"
-		"Trade:      %10s Science: %10s\n"
-		"\n"
-		"    People (W/E/S/T): %s\n"
-		"          City grows: %s\n"
-		"Production completed: %s"),
-	      buf[9], buf[O_FOOD], buf[O_GOLD], buf[O_SHIELD], buf[O_LUXURY],
-	      buf[O_TRADE], buf[O_SCIENCE], buf[6], buf[7], buf[8]);
+  fc_snprintf(buffer, sizeof(buffer),
+              _("Name: %s\n"
+                "Food:       %10s Gold:    %10s\n"
+                "Production: %10s Luxury:  %10s\n"
+                "Trade:      %10s Science: %10s\n"
+                "\n"
+                "%*s%s: %s\n"
+                "          City grows: %s\n"
+                "Production completed: %s"),
+              buf[9], buf[O_FOOD], buf[O_GOLD], buf[O_SHIELD], buf[O_LUXURY],
+              buf[O_TRADE], buf[O_SCIENCE],
+              MAX(0, 20 - (int)get_internal_string_length(citizens)), "",
+              citizens,
+              buf[6], buf[7], buf[8]);
 
-  freelog(LOG_DEBUG, "\n%s", buffer);
+  log_debug("\n%s", buffer);
   return buffer;
 }
 
