@@ -219,11 +219,22 @@ bool client_start_server(void)
   output_window_append(ftc_client, _("Starting local server..."));
 
   /* find a free port */
-  internal_server_port = find_next_free_port(DEFAULT_SOCK_PORT, family);
+  /* Mitigate the risk of ending up with the port already
+   * used by standalone server on Windows where this is known to be buggy
+   * by not starting from DEFAULT_SOCK_PORT but from one higher. */
+  internal_server_port = find_next_free_port(DEFAULT_SOCK_PORT + 1,
+                                             family, "localhost");
+
+  if (internal_server_port < 0) {
+    output_window_append(ftc_client, _("Couldn't start the server."));
+    output_window_append(ftc_client,
+                         _("You'll have to start one manually. Sorry..."));
+    return FALSE;
+  }
 
 # ifdef HAVE_WORKING_FORK
   server_pid = fork();
-  
+
   if (server_pid == 0) {
     int fd, argc = 0;
     const int max_nargs = 18;
@@ -245,6 +256,8 @@ bool client_start_server(void)
     argv[argc++] = "~/.freeciv/saves";
     argv[argc++] = "--scenarios";
     argv[argc++] = "~/.freeciv/scenarios";
+    argv[argc++] = "-A";
+    argv[argc++] = "none";
     if (logfile) {
       argv[argc++] = "--debug";
       argv[argc++] = "3";
@@ -345,7 +358,7 @@ bool client_start_server(void)
 
   fc_snprintf(options, sizeof(options),
               "-p %d --bind localhost -q 1 -e%s%s --saves \"%s\" "
-              "--scenarios \"%s\"",
+              "--scenarios \"%s\" -A none",
               internal_server_port, logcmdline, scriptcmdline, savescmdline,
               scenscmdline);
   fc_snprintf(cmdline1, sizeof(cmdline1), "./fcser %s", options);
