@@ -13,6 +13,10 @@ Freeciv - Copyright (C) 2003 - The Freeciv Project
 #ifndef FC__CONNECTDLG_COMMON_H
 #define FC__CONNECTDLG_COMMON_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 #include "shared.h"
 
 #if defined(HAVE_WORKING_FORK) || defined(WIN32_NATIVE)
@@ -26,8 +30,12 @@ bool is_server_running(void);
 bool can_client_access_hack(void);
 
 void send_client_wants_hack(const char *filename);
-void send_save_game(char *filename);
+void send_save_game(const char *filename);
 
 void set_ruleset(const char *ruleset);
 
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
 #endif  /* FC__CONNECTDLG_COMMON_H */ 
