@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <stdarg.h>
@@ -38,6 +38,7 @@
 #include "mapview_common.h"
 #include "tilespec.h"
 
+/* client/include */
 #include "editgui_g.h"
 #include "mapview_g.h"
 
@@ -446,7 +447,7 @@ static void editor_grab_tool(const struct tile *ptile)
       } else {
         score = 2;
       }
-      if (punit->transported_by > 0) {
+      if (unit_transported(punit)) {
         score = 1;
       }
 
@@ -642,7 +643,7 @@ static void editor_end_selection_rectangle(int canvas_x, int canvas_y)
   gui_rect_iterate(mapview.gui_x0 + editor->selrect_x,
                    mapview.gui_y0 + editor->selrect_y,
                    editor->selrect_width, editor->selrect_height,
-                   ptile, pedge, pcorner, gui_x, gui_y) {
+                   ptile, pedge, pcorner) {
     if (ptile == NULL) {
       continue;
     }
@@ -1297,7 +1298,7 @@ int editor_selection_count(void)
 }
 
 /****************************************************************************
-  Creates a virtual unit (like create_unit_virtual) based on the current
+  Creates a virtual unit (like unit_virtual_create) based on the current
   editor state. You should free() the unit when it is no longer needed.
   If creation is not possible, then NULL is returned.
 
@@ -1305,7 +1306,7 @@ int editor_selection_count(void)
   corresponding to the current 'applied player' parameter and has unit type
   given by the sub-value of the unit tool (ETT_UNIT).
 ****************************************************************************/
-struct unit *editor_create_unit_virtual(void)
+struct unit *editor_unit_virtual_create(void)
 {
   struct unit *vunit;
   struct player *pplayer;
@@ -1325,7 +1326,7 @@ struct unit *editor_create_unit_virtual(void)
     return NULL;
   }
 
-  vunit = create_unit_virtual(pplayer, NULL, putype, 0);
+  vunit = unit_virtual_create(pplayer, NULL, putype, 0);
 
   return vunit;
 }
@@ -1425,8 +1426,7 @@ void edit_buffer_copy(struct edit_buffer *ebuf, const struct tile *ptile)
     dy = 0;
   }
   vtile = tile_virtual_new(NULL);
-  vtile->x = dx;
-  vtile->y = dy;
+  vtile->index = native_pos_to_index(dx, dy);
 
   edit_buffer_type_iterate(ebuf, type) {
     switch (type) {
@@ -1459,7 +1459,7 @@ void edit_buffer_copy(struct edit_buffer *ebuf, const struct tile *ptile)
         if (!punit) {
           continue;
         }
-        vunit = create_unit_virtual(unit_owner(punit), NULL,
+        vunit = unit_virtual_create(unit_owner(punit), NULL,
                                     unit_type(punit), punit->veteran);
         vunit->homecity = punit->homecity;
         vunit->hp = punit->hp;
@@ -1476,7 +1476,7 @@ void edit_buffer_copy(struct edit_buffer *ebuf, const struct tile *ptile)
         fc_snprintf(name, sizeof(name), "Copy of %s",
                     city_name(pcity));
         vcity = create_city_virtual(city_owner(pcity), NULL, name);
-        vcity->size = pcity->size;
+        city_size_set(vcity, city_size_get(pcity));
         improvement_iterate(pimprove) {
           if (!is_improvement(pimprove)
               || !city_has_building(pcity, pimprove)) {
@@ -1582,7 +1582,7 @@ static void paste_tile(struct edit_buffer *ebuf,
         continue;
       }
       owner = player_number(city_owner(vcity));
-      value = vcity->size;
+      value = city_size_get(vcity);
       dsend_packet_edit_city_create(my_conn, owner, tile, value, 0);
       break;
     default:
@@ -1602,14 +1602,19 @@ void edit_buffer_paste(struct edit_buffer *ebuf, const struct tile *dest)
 {
   struct connection *my_conn = &client.conn;
   const struct tile *ptile;
+  int dest_x, dest_y;
 
   if (!ebuf || !dest) {
     return;
   }
 
+  index_to_map_pos(&dest_x, &dest_y, tile_index(dest));
   connection_do_buffer(my_conn);
   tile_list_iterate(ebuf->vtiles, vtile) {
-    ptile = map_pos_to_tile(dest->x + vtile->x, dest->y + vtile->y);
+    int virt_x, virt_y;
+
+    index_to_map_pos(&virt_x, &virt_y, tile_index(vtile));
+    ptile = map_pos_to_tile(dest_x + virt_x, dest_y + virt_y);
     if (!ptile) {
       continue;
     }
