@@ -13,9 +13,13 @@
 #ifndef FC__GOTO_H
 #define FC__GOTO_H
 
-#include "map.h"
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
 
+#include "map.h"
 #include "path_finding.h"
+#include "unitlist.h"
 
 void init_client_goto(void);
 void free_client_goto(void);
@@ -43,4 +47,8 @@ void send_connect_route(enum unit_activity activity);
 
 struct pf_path *path_to_nearest_allied_city(struct unit *punit);
 
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
 #endif /* FC__GOTO_H */
