@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <gdk/gdkkeysyms.h>
@@ -34,6 +34,7 @@
 #include "cma_fec.h"
 #include "messagewin_g.h"
 
+/* gtk-2.0 */
 #include "cityrep.h"
 #include "dialogs.h"
 #include "gui_stuff.h"
@@ -80,7 +81,7 @@ static void set_hscales(const struct cm_parameter *const parameter,
 			struct cma_dialog *pdialog);
 
 /**************************************************************************
-...
+  Initialize cma front end system
 **************************************************************************/
 void cma_fe_init()
 {
@@ -88,7 +89,7 @@ void cma_fe_init()
 }
 
 /**************************************************************************
-...
+  Free resources allocated for cma front end system
 **************************************************************************/
 void cma_fe_done()
 {
@@ -110,14 +111,12 @@ void close_cma_dialog(struct city *pcity)
 }
 
 /**************************************************************************
-...
+  Destroy cma dialog
 **************************************************************************/
 static void cma_dialog_destroy_callback(GtkWidget *w, gpointer data)
 {
   struct cma_dialog *pdialog = (struct cma_dialog *) data;
 
-  g_object_unref(pdialog->tips);
-
   dialog_list_remove(dialog_list, pdialog);
   free(pdialog);
 }
@@ -137,7 +136,7 @@ struct cma_dialog *get_cma_dialog(struct city *pcity)
 }
 
 /**************************************************************************
-...
+  User has pressed button in cma dialog
 **************************************************************************/
 static gboolean button_press_callback(GtkTreeView *view, GdkEventButton *ev,
 				      gpointer data)
@@ -164,7 +163,7 @@ static gboolean button_press_callback(GtkTreeView *view, GdkEventButton *ev,
 }
 
 /**************************************************************************
-...
+  User has requested help
 **************************************************************************/
 static void help_callback(GtkWidget *w, gpointer data)
 {
@@ -172,7 +171,7 @@ static void help_callback(GtkWidget *w, gpointer data)
 }
 
 /**************************************************************************
-...
+  Cell data function for cma dialog 
 **************************************************************************/
 static void cell_data_func(GtkTreeViewColumn *col, GtkCellRenderer *cell,
 			   GtkTreeModel *model, GtkTreeIter *it, gpointer data)
@@ -223,10 +222,6 @@ struct cma_dialog *create_cma_dialog(struct city *pcity)
   g_signal_connect(pdialog->shell, "destroy",
 		   G_CALLBACK(cma_dialog_destroy_callback), pdialog);
 
-  pdialog->tips = gtk_tooltips_new();
-  g_object_ref(pdialog->tips);
-  gtk_object_sink(GTK_OBJECT(pdialog->tips));
-
   page = gtk_hbox_new(FALSE, 12);
   gtk_box_pack_start(GTK_BOX(pdialog->shell), page, TRUE, TRUE, 0);
 
@@ -251,12 +246,11 @@ struct cma_dialog *create_cma_dialog(struct city *pcity)
   g_signal_connect(pdialog->preset_list, "button_press_event",
       		   G_CALLBACK(button_press_callback), pdialog);
 
-  gtk_tooltips_set_tip(pdialog->tips, view,
-		       _("For information on\n"
-		         "the citizen governor and governor presets,\n"
-			 "including sample presets,\n"
-		         "see README.cma."),
-		       "");
+  gtk_widget_set_tooltip_text(view,
+                              _("For information on\n"
+                                "the citizen governor and governor presets,\n"
+                                "including sample presets,\n"
+                                "see README.cma."));
 
   rend = gtk_cell_renderer_text_new();
   column = gtk_tree_view_column_new_with_attributes(NULL, rend,
@@ -491,15 +485,11 @@ static void update_cma_preset_list(struct cma_dialog *pdialog)
 
   /* Append the presets */
   if (cmafec_preset_num()) {
-    gtk_tooltips_disable(pdialog->tips);
-
     for (i = 0; i < cmafec_preset_num(); i++) {
       fc_strlcpy(buf, cmafec_preset_get_descr(i), sizeof(buf));
       gtk_list_store_append(pdialog->store, &it);
       gtk_list_store_set(pdialog->store, &it, 0, buf, -1);
     }
-  } else {
-    gtk_tooltips_enable(pdialog->tips);
   }
 }
 
@@ -736,4 +726,3 @@ static void hscale_changed(GtkAdjustment *get, gpointer data)
     refresh_cma_dialog(pdialog->pcity, DONT_REFRESH_HSCALES);
   }
 }
-
