@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <gtk/gtk.h>
@@ -32,7 +32,7 @@
 #include "text.h"
 #include "tilespec.h"
 
-/* gui-gtk-2.0 */
+/* client/gui-gtk-2.0 */
 #include "graphics.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
@@ -78,7 +78,7 @@ static gboolean show_happiness_button_release(GtkWidget *w,
                                               gpointer data);
 
 /****************************************************************
-...
+  Create happiness dialog
 *****************************************************************/
 void happiness_dialog_init()
 {
@@ -86,15 +86,15 @@ void happiness_dialog_init()
 }
 
 /****************************************************************
-...
+  Remove happiness dialog
 *****************************************************************/
 void happiness_dialog_done()
 {
-  dialog_list_free(dialog_list);
+  dialog_list_destroy(dialog_list);
 }
 
 /****************************************************************
-...
+  Return happiness dialog for a city
 *****************************************************************/
 static struct happiness_dialog *get_happiness_dialog(struct city *pcity)
 {
@@ -264,7 +264,7 @@ static struct happiness_dialog *create_happiness_dialog(struct city *pcity)
 }
 
 /**************************************************************************
-...
+  Refresh citizens pixcomm
 **************************************************************************/
 static void refresh_pixcomm(GtkPixcomm *dst, struct city *pcity,
 			    enum citizen_feeling index)
@@ -286,7 +286,7 @@ static void refresh_pixcomm(GtkPixcomm *dst, struct city *pcity,
 }
 
 /**************************************************************************
-...
+  Refresh whole happiness dialog
 **************************************************************************/
 void refresh_happiness_dialog(struct city *pcity)
 {
@@ -299,7 +299,7 @@ void refresh_happiness_dialog(struct city *pcity)
 }
 
 /**************************************************************************
-...
+  Close happiness dialog of given city
 **************************************************************************/
 void close_happiness_dialog(struct city *pcity)
 {
@@ -311,14 +311,14 @@ void close_happiness_dialog(struct city *pcity)
 
   gtk_widget_hide(pdialog->shell);
 
-  dialog_list_unlink(dialog_list, pdialog);
+  dialog_list_remove(dialog_list, pdialog);
 
   gtk_widget_destroy(pdialog->shell);
   free(pdialog);
 }
 
 /**************************************************************************
-...
+  Create happiness dialog and get its widget
 **************************************************************************/
 GtkWidget *get_top_happiness_display(struct city *pcity)
 {
