@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <stdlib.h>
@@ -28,6 +28,7 @@
 /* common */
 #include "game.h"
 #include "government.h"
+#include "road.h"
 #include "unit.h"
 
 /* client */
@@ -58,7 +59,9 @@
 #include "plrdlg.h"
 #include "ratesdlg.h"
 #include "repodlgs.h"
+#include "luaconsole.h"
 #include "spaceshipdlg.h"
+#include "unitselect.h"
 #include "wldlg.h"
 
 #include "menu.h"
@@ -141,7 +144,7 @@ static void save_options_callback(GtkAction *action, gpointer data)
 *****************************************************************/
 static void reload_tileset_callback(GtkAction *action, gpointer data)
 {
-  tilespec_reread(NULL);
+  tilespec_reread(NULL, TRUE);
 }
 
 /****************************************************************
@@ -157,7 +160,23 @@ static void save_game_callback(GtkAction *action, gpointer data)
 *****************************************************************/
 static void save_game_as_callback(GtkAction *action, gpointer data)
 {
-  popup_save_dialog(FALSE);
+  save_game_dialog_popup();
+}
+
+/****************************************************************************
+  Action "SAVE_MAPIMG" callback.
+****************************************************************************/
+static void save_mapimg_callback(GtkAction *action, gpointer data)
+{
+  mapimg_client_save(NULL);
+}
+
+/****************************************************************************
+  Action "SAVE_MAPIMG_AS" callback.
+****************************************************************************/
+static void save_mapimg_as_callback(GtkAction *action, gpointer data)
+{
+  save_mapimg_dialog_popup();
 }
 
 /****************************************************************
@@ -260,6 +279,14 @@ static void report_messages_callback(GtkAction *action, gpointer data)
   meswin_dialog_popup(TRUE);
 }
 
+/****************************************************************
+  Action "CLIENT_LUA_SCRIPT" callback.
+*****************************************************************/
+static void client_lua_script_callback(GtkAction *action, gpointer data)
+{
+  luaconsole_dialog_popup(TRUE);
+}
+
 /****************************************************************
   Action "REPORT_DEMOGRAPHIC" callback.
 *****************************************************************/
@@ -523,6 +550,17 @@ static void show_national_borders_callback(GtkToggleAction *action,
   }
 }
 
+/****************************************************************
+  Action "SHOW_NATIVE_TILES" callback.
+*****************************************************************/
+static void show_native_tiles_callback(GtkToggleAction *action,
+                                       gpointer data)
+{
+  if (draw_native ^ gtk_toggle_action_get_active(action)) {
+    key_map_native_toggle();
+  }
+}
+
 /****************************************************************
   Action "SHOW_CITY_FULL_BAR" callback.
 *****************************************************************/
@@ -749,7 +787,7 @@ static void show_better_fog_of_war_callback(GtkToggleAction *action,
 }
 
 /****************************************************************
-  Action "SHOW_NATIONAL_BORDERS" callback.
+  Action "FULL_SCREEN" callback.
 *****************************************************************/
 static void full_screen_callback(GtkToggleAction *action, gpointer data)
 {
@@ -797,7 +835,7 @@ static void scenario_properties_callback(GtkAction *action, gpointer data)
 *****************************************************************/
 static void save_scenario_callback(GtkAction *action, gpointer data)
 {
-  popup_save_dialog(TRUE);
+  save_scenario_dialog_popup();
 }
 
 /****************************************************************
@@ -837,7 +875,15 @@ static void select_same_type_cont_callback(GtkAction *action, gpointer data)
 *****************************************************************/
 static void select_same_type_callback(GtkAction *action, gpointer data)
 {
-  request_unit_select(get_units_in_focus(), SELTYPE_SAME, SELLOC_ALL);
+  request_unit_select(get_units_in_focus(), SELTYPE_SAME, SELLOC_WORLD);
+}
+
+/*****************************************************************************
+  Open unit selection dialog.
+*****************************************************************************/
+static void select_dialog_callback(GtkAction *action, gpointer data)
+{
+  unit_select_dialog_popup(NULL);
 }
 
 /****************************************************************
@@ -1112,7 +1158,7 @@ static void build_fortress_callback(GtkAction *action, gpointer data)
     /* FIXME: this can provide different actions for different units...
      * not good! */
     struct base_type *pbase = get_base_by_gui_type(BASE_GUI_FORTRESS,
-                                                   punit, punit->tile);
+                                                   punit, unit_tile(punit));
 
     if (pbase && can_unit_do_activity_base(punit, pbase->item_number)) {
       request_new_unit_activity_base(punit, pbase);
@@ -1276,6 +1322,10 @@ static GtkActionGroup *get_safe_group(void)
        NULL, NULL, G_CALLBACK(save_game_callback)},
       {"GAME_SAVE_AS", GTK_STOCK_SAVE_AS, _("Save Game _As..."),
        NULL, NULL, G_CALLBACK(save_game_as_callback)},
+      {"MAPIMG_SAVE", NULL, _("Save Map _Image"),
+       NULL, NULL, G_CALLBACK(save_mapimg_callback)},
+      {"MAPIMG_SAVE_AS", NULL, _("Save _Map Image As..."),
+       NULL, NULL, G_CALLBACK(save_mapimg_as_callback)},
       {"LEAVE", NULL, _("_Leave"),
        NULL, NULL, G_CALLBACK(leave_callback)},
       {"QUIT", GTK_STOCK_QUIT, _("_Quit"),
@@ -1287,6 +1337,9 @@ static GtkActionGroup *get_safe_group(void)
       {"WORKLISTS", NULL, _("Work_lists"),
        "<Control>l", NULL, G_CALLBACK(worklists_callback)},
 
+      {"CLIENT_LUA_SCRIPT", NULL, _("Client _Lua Script"),
+       NULL, NULL, G_CALLBACK(client_lua_script_callback)},
+
       /* Civilization menu. */
       {"MAP_VIEW", NULL, Q_("?noun:_View"),
        "F1", NULL, G_CALLBACK(map_view_callback)},
@@ -1379,6 +1432,9 @@ static GtkActionGroup *get_safe_group(void)
       {"SHOW_NATIONAL_BORDERS", NULL, _("National _Borders"),
        "<Control>b", NULL,
        G_CALLBACK(show_national_borders_callback), FALSE},
+      {"SHOW_NATIVE_TILES", NULL, _("Native Tiles"),
+       "<Shift><Control>n", NULL,
+       G_CALLBACK(show_native_tiles_callback), FALSE},
       {"SHOW_CITY_FULL_BAR", NULL, _("City Full Bar"),
        NULL, NULL, G_CALLBACK(show_city_full_bar_callback), FALSE},
       {"SHOW_CITY_NAMES", NULL, _("City _Names"),
@@ -1503,6 +1559,9 @@ static GtkActionGroup *get_unit_group(void)
       {"SELECT_SAME_TYPE", NULL, _("Same Type _Everywhere"),
        "<shift>x", NULL, G_CALLBACK(select_same_type_callback)},
 
+      {"SELECT_DLG", NULL, _("Unit selection dialog"),
+       NULL, NULL, G_CALLBACK(select_dialog_callback)},
+
       {"UNIT_WAIT", NULL, _("_Wait"),
        "w", NULL, G_CALLBACK(unit_wait_callback)},
       {"UNIT_DONE", NULL, _("_Done"),
@@ -1701,7 +1760,8 @@ GtkWidget *setup_menus(GtkWidget *window)
 
   /* Creates the UI manager. */
   ui_manager = gtk_ui_manager_new();
-  gtk_ui_manager_set_add_tearoffs(ui_manager, TRUE);
+  /* FIXME - following line commented out due to Gna bug #17162 */
+  /* gtk_ui_manager_set_add_tearoffs(ui_manager, TRUE); */
   g_signal_connect(ui_manager, "add_widget",
                    G_CALLBACK(add_widget_callback), menubar);
 
@@ -1896,7 +1956,6 @@ void real_menus_update(void)
   GtkActionGroup *edit_group;
   GtkActionGroup *unit_group;
   GtkActionGroup *playing_group;
-  GtkActionGroup *player_group;
   struct unit_list *punits = NULL;
   bool units_all_same_tile = TRUE, units_all_same_type = TRUE;
   GtkMenu *menu;
@@ -1911,7 +1970,6 @@ void real_menus_update(void)
   edit_group = get_edit_group();
   unit_group = get_unit_group();
   playing_group = get_playing_group();
-  player_group = get_player_group();
 
   if (get_num_units_in_focus() > 0) {
     const struct tile *ptile = NULL;
@@ -1920,14 +1978,14 @@ void real_menus_update(void)
     unit_list_iterate(punits, punit) {
       fc_assert((ptile==NULL) == (ptype==NULL));
       if (ptile || ptype) {
-        if (punit->tile != ptile) {
+        if (unit_tile(punit) != ptile) {
           units_all_same_tile = FALSE;
         }
         if (unit_type(punit) != ptype) {
           units_all_same_type = FALSE;
         }
       } else {
-        ptile = punit->tile;
+        ptile = unit_tile(punit);
         ptype = unit_type(punit);
       }
     } unit_list_iterate_end;
@@ -1945,6 +2003,19 @@ void real_menus_update(void)
                       can_conn_enable_editing(&client.conn));
   editgui_refresh();
 
+  {
+    char road_buf[500];
+
+    /* TRANS: Connect with some road type (Road/Railroad) */  
+    snprintf(road_buf, sizeof(road_buf), _("Connect With %s"),
+             road_name_translation(road_by_number(ROAD_ROAD)));
+    menus_rename(unit_group, "CONNECT_ROAD", road_buf);
+
+    snprintf(road_buf, sizeof(road_buf), _("Connect With %s"),
+             road_name_translation(road_by_number(ROAD_RAILROAD)));
+    menus_rename(unit_group, "CONNECT_RAIL", road_buf);
+  }
+
   if (!can_client_issue_orders()) {
     return;
   }
@@ -2064,7 +2135,7 @@ void real_menus_update(void)
     /* FIXME: this overloading doesn't work well with multiple focus
      * units. */
     unit_list_iterate(punits, punit) {
-      if (tile_city(punit->tile)) {
+      if (tile_city(unit_tile(punit))) {
         city_on_tile = TRUE;
         break;
       }
@@ -2082,21 +2153,28 @@ void real_menus_update(void)
     menus_rename(unit_group, "BUILD_ROAD", _("Establish Trade _Route"));
   } else if (units_have_flag(punits, F_SETTLERS, TRUE)) {
     bool has_road = FALSE;
+    enum eroad rtype;
+    char road_item[500];
 
     /* FIXME: this overloading doesn't work well with multiple focus
      * units. */
     unit_list_iterate(punits, punit) {
-      if (tile_has_special(punit->tile, S_ROAD)) {
+      if (tile_has_special(unit_tile(punit), S_ROAD)) {
         has_road = TRUE;
         break;
       }
     } unit_list_iterate_end;
 
     if (has_road) {
-      menus_rename(unit_group, "BUILD_ROAD", _("Build _Railroad"));
+      rtype = ROAD_RAILROAD;
     } else {
-      menus_rename(unit_group, "BUILD_ROAD", _("Build _Road"));
+      rtype = ROAD_ROAD;
     }
+
+    /* TRANS: Build road of specific type (Road/Railroad) */
+    snprintf(road_item, sizeof(road_item), _("Build %s"),
+             road_name_translation(road_by_number(rtype)));
+    menus_rename(unit_group, "BUILD_ROAD", road_item);
   } else {
     menus_rename(unit_group, "BUILD_ROAD", _("Build _Road"));
   }
@@ -2145,13 +2223,13 @@ void real_menus_update(void)
   if (units_all_same_tile) {
     struct unit *punit = unit_list_get(punits, 0);
 
-    pterrain = tile_terrain(punit->tile);
+    pterrain = tile_terrain(unit_tile(punit));
     if (pterrain->irrigation_result != T_NONE
         && pterrain->irrigation_result != pterrain) {
       fc_snprintf(irrtext, sizeof(irrtext), _("Change to %s"),
-                  get_tile_change_menu_text(punit->tile,
+                  get_tile_change_menu_text(unit_tile(punit),
                                             ACTIVITY_IRRIGATE));
-    } else if (tile_has_special(punit->tile, S_IRRIGATION)
+    } else if (tile_has_special(unit_tile(punit), S_IRRIGATION)
                && player_knows_techs_with_flag(unit_owner(punit),
                                                TF_FARMLAND)) {
       sz_strlcpy(irrtext, _("Bu_ild Farmland"));
@@ -2162,7 +2240,7 @@ void real_menus_update(void)
     if (pterrain->mining_result != T_NONE
         && pterrain->mining_result != pterrain) {
       fc_snprintf(mintext, sizeof(mintext), _("Change to %s"),
-                  get_tile_change_menu_text(punit->tile, ACTIVITY_MINE));
+                  get_tile_change_menu_text(unit_tile(punit), ACTIVITY_MINE));
     } else {
       sz_strlcpy(mintext, _("Build _Mine"));
     }
@@ -2170,7 +2248,7 @@ void real_menus_update(void)
     if (pterrain->transform_result != T_NONE
         && pterrain->transform_result != pterrain) {
       fc_snprintf(transtext, sizeof(transtext), _("Transf_orm to %s"),
-                  get_tile_change_menu_text(punit->tile,
+                  get_tile_change_menu_text(unit_tile(punit),
                                             ACTIVITY_TRANSFORM));
     } else {
       sz_strlcpy(transtext, _("Transf_orm Terrain"));
@@ -2322,6 +2400,7 @@ void real_menus_init(void)
   menus_set_active(safe_group, "SHOW_NATIONAL_BORDERS", draw_borders);
   menus_set_sensitive(safe_group, "SHOW_NATIONAL_BORDERS",
                       BORDERS_DISABLED != game.info.borders);
+  menus_set_active(safe_group, "SHOW_NATIVE_TILES", draw_native);
   menus_set_active(safe_group, "SHOW_CITY_FULL_BAR", draw_full_citybar);
   menus_set_active(safe_group, "SHOW_CITY_NAMES", draw_city_names);
   menus_set_active(safe_group, "SHOW_CITY_GROWTH", draw_city_growth);
