@@ -50,7 +50,7 @@
 #include "reqtree.h"
 #include "text.h"
 
-/* client/gui-gtk-3.0 */
+/* client/gui-gtk-2.0 */
 #include "canvas.h"
 #include "cityrep.h"
 #include "dialogs.h"
@@ -86,10 +86,9 @@ static bool science_report_combo_get_active(GtkComboBox *combo,
 static void science_report_combo_set_active(GtkComboBox *combo,
                                             Tech_type_id tech);
 static gboolean science_diagram_button_release_callback(GtkWidget *widget,
-    GdkEventButton *event, gpointer data);
-static gboolean science_diagram_update(GtkWidget *widget,
-                                       cairo_t *cr,
-                                       gpointer data);
+                                                        GdkEventButton *event,
+                                                        gpointer data);
+static void science_diagram_update(GtkWidget *widget, gpointer data);
 static GtkWidget *science_diagram_new(void);
 static void science_diagram_data(GtkWidget *widget, bool reachable);
 static void science_diagram_center(GtkWidget *diagram, Tech_type_id tech);
@@ -226,31 +225,18 @@ static gboolean science_diagram_button_release_callback(GtkWidget *widget,
 /****************************************************************************
   Draw the invalidated portion of the reqtree.
 ****************************************************************************/
-static gboolean science_diagram_update(GtkWidget *widget, cairo_t *cr, gpointer data)
+static void science_diagram_update(GtkWidget *widget, gpointer data)
 {
   /* FIXME: this currently redraws everything! */
-  struct canvas canvas = FC_STATIC_CANVAS_INIT;
+  struct canvas canvas = {
+    .type = CANVAS_PIXMAP,
+    .v.pixmap = GTK_LAYOUT(widget)->bin_window
+  };
   struct reqtree *reqtree = g_object_get_data(G_OBJECT(widget), "reqtree");
   int width, height;
-  GtkAdjustment *hadjustment;
-  GtkAdjustment *vadjustment;
-  gint hadjustment_value;
-  gint vadjustment_value;
-
-  hadjustment = gtk_scrollable_get_hadjustment(GTK_SCROLLABLE(widget));
-  vadjustment = gtk_scrollable_get_vadjustment(GTK_SCROLLABLE(widget));
-
-  hadjustment_value = (gint)gtk_adjustment_get_value(hadjustment);
-  vadjustment_value = (gint)gtk_adjustment_get_value(vadjustment);
-
-  cairo_translate(cr, -hadjustment_value, -vadjustment_value);
-
-  canvas.drawable = cr;
 
   get_reqtree_dimensions(reqtree, &width, &height);
   draw_reqtree(reqtree, &canvas, 0, 0, 0, 0, width, height);
-
-  return TRUE;
 }
 
 /****************************************************************************
@@ -265,7 +251,7 @@ static GtkWidget *science_diagram_new(void)
   gtk_widget_add_events(diagram,
                         GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
                         | GDK_BUTTON2_MOTION_MASK | GDK_BUTTON3_MOTION_MASK);
-  g_signal_connect(diagram, "draw",
+  g_signal_connect(diagram, "expose-event",
                    G_CALLBACK(science_diagram_update), NULL);
   g_signal_connect(diagram, "button-release-event",
                    G_CALLBACK(science_diagram_button_release_callback),
@@ -316,16 +302,12 @@ static void science_diagram_center(GtkWidget *diagram, Tech_type_id tech)
     gdouble value;
 
     adjust = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(sw));
-    value = (gtk_adjustment_get_lower(adjust)
-      + gtk_adjustment_get_upper(adjust)
-      - gtk_adjustment_get_page_size(adjust)) / width * x;
+    value = (adjust->lower + adjust->upper - adjust->page_size) / width * x;
     gtk_adjustment_set_value(adjust, value);
     gtk_adjustment_value_changed(adjust);
 
     adjust = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(sw));
-    value = (gtk_adjustment_get_lower(adjust)
-      + gtk_adjustment_get_upper(adjust)
-      - gtk_adjustment_get_page_size(adjust)) / height * y;
+    value = (adjust->lower + adjust->upper - adjust->page_size) / height * y;
     gtk_adjustment_set_value(adjust, value);
     gtk_adjustment_value_changed(adjust);
   }
@@ -442,7 +424,7 @@ static void science_report_update(struct science_report *preport)
 
   /* Collect all techs which are reachable in next 10 steps. */
   advance_index_iterate(A_FIRST, i) {
-    if (player_invention_reachable(client_player(), i, FALSE)
+    if (player_invention_reachable(client_player(), i, TRUE)
         && TECH_KNOWN != presearch->inventions[i].state
         && (i == presearch->tech_goal
             || 10 >= presearch->inventions[i].num_required_techs)) {
@@ -527,8 +509,7 @@ static void science_report_goal_callback(GtkComboBox *combo, gpointer data)
 static void science_report_init(struct science_report *preport)
 {
   GtkWidget *frame, *table, *help_button, *reachable_button, *sw, *w;
-  GtkSizeGroup *group;
-  GtkContainer *vbox;
+  GtkBox *vbox;
   GtkListStore *store;
   GtkCellRenderer *renderer;
 
@@ -541,27 +522,25 @@ static void science_report_init(struct science_report *preport)
   gui_dialog_add_button(preport->shell, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
   gui_dialog_set_default_response(preport->shell, GTK_RESPONSE_CLOSE);
 
-  vbox = GTK_CONTAINER(preport->shell->vbox);
-  group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
+  vbox = GTK_BOX(preport->shell->vbox);
 
   w = gtk_label_new(NULL);
-  gtk_container_add(vbox, w);
+  gtk_box_pack_start(vbox, w, FALSE, FALSE, 0);
   preport->main_label = GTK_LABEL(w);
 
   /* Current research target line. */
   frame = gtk_frame_new(_("Researching"));
-  gtk_container_add(vbox, frame);
+  gtk_box_pack_start(vbox, frame, FALSE, FALSE, 0);
 
-  table = gtk_grid_new();
-  gtk_grid_set_column_spacing(GTK_GRID(table), 4);
+  table = gtk_table_new(1, 6, TRUE);
+  gtk_table_set_col_spacings(GTK_TABLE(table), 4);
   gtk_container_add(GTK_CONTAINER(frame), table);
 
   help_button = gtk_check_button_new_with_label(_("Help"));
-  gtk_grid_attach(GTK_GRID(table), help_button, 5, 0, 1, 1);
+  gtk_table_attach(GTK_TABLE(table), help_button, 5, 6, 0, 1, 0, 0, 0, 0);
 
   store = science_report_store_new();
   w = gtk_combo_box_new_with_model(GTK_TREE_MODEL(store));
-  gtk_size_group_add_widget(group, w);
   g_object_unref(G_OBJECT(store));
   renderer = gtk_cell_renderer_text_new();
   gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(w), renderer, TRUE);
@@ -570,27 +549,24 @@ static void science_report_init(struct science_report *preport)
   gtk_widget_set_sensitive(w, can_client_issue_orders());
   g_signal_connect(w, "changed", G_CALLBACK(science_report_current_callback),
                    help_button);
-  gtk_grid_attach(GTK_GRID(table), w, 0, 0, 1, 1);
+  gtk_table_attach_defaults(GTK_TABLE(table), w, 0, 2, 0, 1);
   preport->reachable_techs = GTK_COMBO_BOX(w);
 
   w = gtk_progress_bar_new();
-  gtk_widget_set_hexpand(w, TRUE);
-  gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(w), TRUE);
-  gtk_grid_attach(GTK_GRID(table), w, 2, 0, 1, 1);
+  gtk_table_attach_defaults(GTK_TABLE(table), w, 2, 5, 0, 1);
   gtk_widget_set_size_request(w, -1, 25);
   preport->progress_bar = GTK_PROGRESS_BAR(w);
 
   /* Research goal line. */
   frame = gtk_frame_new( _("Goal"));
-  gtk_container_add(vbox, frame);
+  gtk_box_pack_start(vbox, frame, FALSE, FALSE, 0);
 
-  table = gtk_grid_new();
-  gtk_grid_set_column_spacing(GTK_GRID(table), 4);
+  table = gtk_table_new(1, 6, TRUE);
+  gtk_table_set_col_spacings(GTK_TABLE(table), 4);
   gtk_container_add(GTK_CONTAINER(frame),table);
 
   store = science_report_store_new();
   w = gtk_combo_box_new_with_model(GTK_TREE_MODEL(store));
-  gtk_size_group_add_widget(group, w);
   g_object_unref(G_OBJECT(store));
   renderer = gtk_cell_renderer_text_new();
   gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(w), renderer, TRUE);
@@ -603,19 +579,19 @@ static void science_report_init(struct science_report *preport)
   gtk_widget_set_sensitive(w, can_client_issue_orders());
   g_signal_connect(w, "changed", G_CALLBACK(science_report_goal_callback),
                    help_button);
-  gtk_grid_attach(GTK_GRID(table), w, 0, 0, 1, 1);
+  gtk_table_attach_defaults(GTK_TABLE(table), w, 0, 2, 0, 1);
   preport->reachable_goals = GTK_COMBO_BOX(w);
 
   w = gtk_label_new(NULL);
-  gtk_widget_set_hexpand(w, TRUE);
-  gtk_grid_attach(GTK_GRID(table), w, 2, 0, 1, 1);
+  gtk_table_attach_defaults(GTK_TABLE(table), w, 2, 5, 0, 1);
   gtk_widget_set_size_request(w, -1, 25);
   preport->goal_label = GTK_LABEL(w);
 
   /* Toggle unreachable button. */
   /* TRANS: As in 'Show all (even currently not reachable) techs'. */
   reachable_button = gtk_toggle_button_new_with_label(_("Show all"));
-  gtk_grid_attach(GTK_GRID(table), reachable_button, 5, 0, 1, 1);
+  gtk_table_attach(GTK_TABLE(table), reachable_button, 5, 6, 0, 1, 0, 0, 0,
+                   0);
   g_signal_connect(reachable_button, "toggled",
                    G_CALLBACK(science_report_unreachable_callback), preport);
   gtk_widget_set_sensitive(reachable_button, can_client_issue_orders()
@@ -626,18 +602,14 @@ static void science_report_init(struct science_report *preport)
   sw = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  gtk_container_add(vbox, sw);
+  gtk_box_pack_start(vbox, sw, TRUE, TRUE, 0);
 
   w = science_diagram_new();
-  gtk_widget_set_hexpand(w, TRUE);
-  gtk_widget_set_vexpand(w, TRUE);
   gtk_container_add(GTK_CONTAINER(sw), w);
   preport->drawing_area = GTK_LAYOUT(w);
 
   science_report_update(preport);
   gui_dialog_show_all(preport->shell);
-  gtk_widget_queue_draw(GTK_WIDGET(preport->drawing_area));
-  g_object_unref(group);
 
   /* This must be _after_ the dialog is drawn to really center it ... */
   science_report_redraw(preport);
@@ -722,7 +694,7 @@ struct economy_report {
   GtkLabel *label;
 };
 
-static struct economy_report economy_report = { NULL, NULL, NULL };
+static struct economy_report economy_report = { NULL, };
 
 enum economy_report_response {
   ERD_RES_SELL_REDUNDANT = 1,
@@ -803,7 +775,6 @@ static void economy_report_update(struct economy_report *preport)
   GtkTreeModel *model;
   GtkListStore *store;
   GtkTreeIter iter;
-  GdkPixbuf *pix;
   struct improvement_entry building_entries[B_LAST];
   struct unit_entry unit_entries[U_LAST];
   int entries_used, building_total, unit_total, tax, i;
@@ -833,10 +804,9 @@ static void economy_report_update(struct economy_report *preport)
     struct sprite *sprite = get_building_sprite(tileset, pimprove);
     cid cid = cid_encode_building(pimprove);
 
-    pix = sprite_get_pixbuf(sprite);
     gtk_list_store_append(store, &iter);
     gtk_list_store_set(store, &iter,
-                       ERD_COL_SPRITE, pix,
+                       ERD_COL_SPRITE, sprite_get_pixbuf(sprite),
                        ERD_COL_NAME, improvement_name_translation(pimprove),
                        ERD_COL_REDUNDANT, pentry->redundant,
                        ERD_COL_COUNT, pentry->count,
@@ -845,7 +815,6 @@ static void economy_report_update(struct economy_report *preport)
                        ERD_COL_IS_IMPROVEMENT, TRUE,
                        ERD_COL_CID, cid,
                        -1);
-    g_object_unref(G_OBJECT(pix));
     if (selected == cid) {
       /* Restore the selection. */
       gtk_tree_selection_select_iter(selection, &iter);
@@ -861,10 +830,9 @@ static void economy_report_update(struct economy_report *preport)
                                                 direction8_invalid(), TRUE);
     cid cid = cid_encode_unit(putype);
 
-    pix = sprite_get_pixbuf(sprite);
     gtk_list_store_append(store, &iter);
     gtk_list_store_set(store, &iter,
-                       ERD_COL_SPRITE, pix,
+                       ERD_COL_SPRITE, sprite_get_pixbuf(sprite),
                        ERD_COL_NAME, utype_name_translation(putype),
                        ERD_COL_REDUNDANT, 0,
                        ERD_COL_COUNT, pentry->count,
@@ -873,7 +841,6 @@ static void economy_report_update(struct economy_report *preport)
                        ERD_COL_IS_IMPROVEMENT, FALSE,
                        ERD_COL_CID, cid,
                        -1);
-    g_object_unref(G_OBJECT(pix));
     if (selected == cid) {
       /* Restore the selection. */
       gtk_tree_selection_select_iter(selection, &iter);
@@ -1055,10 +1022,10 @@ static void economy_report_selection_callback(GtkTreeSelection *selection,
 ****************************************************************************/
 static void economy_report_init(struct economy_report *preport)
 {
-  GtkWidget *view, *sw, *label, *button;
+  GtkWidget *view, *sw, *align, *label, *button;
   GtkListStore *store;
   GtkTreeSelection *selection;
-  GtkContainer *vbox;
+  GtkBox *vbox;
   const char *title;
   enum economy_report_columns i;
 
@@ -1066,19 +1033,20 @@ static void economy_report_init(struct economy_report *preport)
 
   gui_dialog_new(&preport->shell, GTK_NOTEBOOK(top_notebook), preport, TRUE);
   gui_dialog_set_title(preport->shell, _("Economy"));
-  vbox = GTK_CONTAINER(preport->shell->vbox);
+  vbox = GTK_BOX(preport->shell->vbox);
+
+  align = gtk_alignment_new(0.5, 0.0, 0.0, 1.0);
+  gtk_box_pack_start(vbox, align, TRUE, TRUE, 0);
 
   sw = gtk_scrolled_window_new(NULL, NULL);
-  gtk_widget_set_halign(sw, GTK_ALIGN_CENTER);
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
                                       GTK_SHADOW_ETCHED_IN);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-  gtk_container_add(GTK_CONTAINER(vbox), sw);
+  gtk_container_add(GTK_CONTAINER(align), sw);
 
   store = economy_report_store_new();
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
-  gtk_widget_set_vexpand(view, TRUE);
   g_object_unref(store);
   gtk_widget_set_name(view, "small_font");
   gtk_tree_view_columns_autosize(GTK_TREE_VIEW(view));
@@ -1129,24 +1097,24 @@ static void economy_report_init(struct economy_report *preport)
   }
 
   label = gtk_label_new(NULL);
-  gtk_container_add(vbox, label);
+  gtk_box_pack_start(vbox, label, FALSE, FALSE, 0);
   gtk_misc_set_padding(GTK_MISC(label), 5, 5);
   preport->label = GTK_LABEL(label);
 
-  gui_dialog_add_button(preport->shell, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
-
-  button = gui_dialog_add_button(preport->shell, _("_Disband"),
-                                 ERD_RES_DISBAND_UNITS);
+  button = gui_dialog_add_button(preport->shell, _("Sell _Redundant"),
+                                 ERD_RES_SELL_REDUNDANT);
   gtk_widget_set_sensitive(button, FALSE);
 
   button = gui_dialog_add_button(preport->shell, _("Sell _All"),
                                  ERD_RES_SELL_ALL);
   gtk_widget_set_sensitive(button, FALSE);
 
-  button = gui_dialog_add_button(preport->shell, _("Sell _Redundant"),
-                                 ERD_RES_SELL_REDUNDANT);
+  button = gui_dialog_add_button(preport->shell, _("_Disband"),
+                                 ERD_RES_DISBAND_UNITS);
   gtk_widget_set_sensitive(button, FALSE);
 
+  gui_dialog_add_button(preport->shell, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+
   gui_dialog_set_default_response(preport->shell, GTK_RESPONSE_CLOSE);
   gui_dialog_response_set_callback(preport->shell,
                                    economy_report_command_callback);
@@ -1216,19 +1184,19 @@ struct units_report {
   GtkTreeView *tree_view;
 };
 
-static struct units_report units_report = { NULL, NULL };
+static struct units_report units_report = { NULL, };
 
 enum units_report_response {
   URD_RES_NEAREST = 1,
   URD_RES_UPGRADE
 };
 
-/* Those values must match the functions units_report_store_new() and
- * units_report_column_name(). */
+/* Those values must match the order of unit_report_columns[]. */
 enum units_report_columns {
   URD_COL_UTYPE_NAME,
   URD_COL_UPGRADABLE,
-  URD_COL_IN_PROJECT,
+  URD_COL_N_UPGRADABLE,
+  URD_COL_IN_PROGRESS,
   URD_COL_ACTIVE,
   URD_COL_SHIELD,
   URD_COL_FOOD,
@@ -1236,59 +1204,57 @@ enum units_report_columns {
 
   /* Not visible. */
   URD_COL_TEXT_WEIGHT,
-  URD_COL_BOOL_VISIBLE,
+  URD_COL_UPG_VISIBLE,
+  URD_COL_NUPG_VISIBLE,
   URD_COL_UTYPE_ID,
 
   URD_COL_NUM
 };
 
+static const struct {
+  GType type;
+  const char *title;
+  const char *tooltip;
+  bool rightalign;
+  int visible_col;
+} unit_report_columns[] = {
+  { /* URD_COL_UTYPE_NAME */   G_TYPE_STRING,  N_("Unit Type"),
+    NULL,                         FALSE,  -1 },
+  { /* URD_COL_UPGRADABLE */   G_TYPE_BOOLEAN, N_("?Upgradable unit [short]:U"),
+    N_("Upgradable"),             TRUE,   URD_COL_UPG_VISIBLE },
+  { /* URD_COL_N_UPGRADABLE */ G_TYPE_INT,     "" /* merge with previous col */,
+    NULL,                         TRUE,   URD_COL_NUPG_VISIBLE },
+  /* TRANS: "In progress" abbrevation. */
+  { /* URD_COL_IN_PROGRESS */  G_TYPE_INT,     N_("In-Prog"),
+    N_("In progress"),            TRUE,   -1 },
+  { /* URD_COL_ACTIVE */       G_TYPE_INT,     N_("Active"),
+    NULL,                         TRUE,   -1 },
+  { /* URD_COL_SHIELD */       G_TYPE_INT,     N_("Shield"),
+    N_("Total shield upkeep"),    TRUE,   -1 },
+  { /* URD_COL_SHIELD */       G_TYPE_INT,     N_("Food"),
+    N_("Total food upkeep"),      TRUE,   -1 },
+  { /* URD_COL_SHIELD */       G_TYPE_INT,     N_("Gold"),
+    N_("Total gold upkeep"),      TRUE,   -1 },
+  { /* URD_COL_TEXT_WEIGHT */  G_TYPE_INT,     NULL /* ... */ },
+  { /* URD_COL_UPG_VISIBLE */  G_TYPE_BOOLEAN, NULL /* ... */ },
+  { /* URD_COL_NUPG_VISIBLE */ G_TYPE_BOOLEAN, NULL /* ... */ },
+  { /* URD_COL_UTYPE_ID */     G_TYPE_INT,     NULL /* ... */ }
+};
+
 /****************************************************************************
   Create a new units report list store.
 ****************************************************************************/
 static GtkListStore *units_report_store_new(void)
 {
-  return gtk_list_store_new(URD_COL_NUM,
-                            G_TYPE_STRING,      /* URD_COL_UTYPE_NAME */
-                            G_TYPE_BOOLEAN,     /* URD_COL_UPGRADABLE */
-                            G_TYPE_INT,         /* URD_COL_IN_PROJECT */
-                            G_TYPE_INT,         /* URD_COL_ACTIVE */
-                            G_TYPE_INT,         /* URD_COL_SHIELD */
-                            G_TYPE_INT,         /* URD_COL_FOOD */
-                            G_TYPE_INT,         /* URD_COL_GOLD */
-                            G_TYPE_INT,         /* URD_COL_TEXT_WEIGHT */
-                            G_TYPE_BOOLEAN,     /* URD_COL_BOOL_VISIBLE */
-                            G_TYPE_INT);        /* URD_COL_UTYPE_ID */
-}
+  int i;
+  GType cols[URD_COL_NUM];
+  fc_assert(ARRAY_SIZE(unit_report_columns) == URD_COL_NUM);
 
-/****************************************************************************
-  Returns the title of the column (translated).
-****************************************************************************/
-static const char *units_report_column_name(enum units_report_columns col)
-{
-  switch (col) {
-  case URD_COL_UTYPE_NAME:
-    return _("Unit Type");
-  case URD_COL_UPGRADABLE:
-    return Q_("?Upgradable unit [short]:U");
-  case URD_COL_IN_PROJECT:
-    /* TRANS: "In project" abbreviation. */
-    return _("In-Prog");
-  case URD_COL_ACTIVE:
-    return _("Active");
-  case URD_COL_SHIELD:
-    return _("Shield");
-  case URD_COL_FOOD:
-    return _("Food");
-  case URD_COL_GOLD:
-    return _("Gold");
-  case URD_COL_TEXT_WEIGHT:
-  case URD_COL_BOOL_VISIBLE:
-  case URD_COL_UTYPE_ID:
-  case URD_COL_NUM:
-    break;
+  for (i=0; i<URD_COL_NUM; i++) {
+    cols[i] = unit_report_columns[i].type;
   }
-
-  return NULL;
+  
+  return gtk_list_store_newv(URD_COL_NUM, cols);
 }
 
 /****************************************************************************
@@ -1305,6 +1271,7 @@ static void units_report_update(struct units_report *preport)
   struct urd_info unit_array[utype_count()];
   struct urd_info unit_totals;
   struct urd_info *info;
+  int total_upgradable_count = 0;
   GtkTreeSelection *selection;
   GtkTreeModel *model;
   GtkListStore *store;
@@ -1359,6 +1326,8 @@ static void units_report_update(struct units_report *preport)
   gtk_list_store_clear(store);
 
   unit_type_iterate(utype) {
+    bool upgradable;
+
     utype_id = utype_index(utype);
     info = unit_array + utype_id;
 
@@ -1366,20 +1335,22 @@ static void units_report_update(struct units_report *preport)
       continue;         /* We don't need a row for this type. */
     }
 
+    upgradable = client_has_player()
+                 && NULL != can_upgrade_unittype(client_player(), utype);
     
     gtk_list_store_append(store, &iter);
     gtk_list_store_set(store, &iter,
                        URD_COL_UTYPE_NAME, utype_name_translation(utype),
-                       URD_COL_UPGRADABLE, (client_has_player()
-                           && NULL != can_upgrade_unittype(client_player(),
-                                                           utype)),
-                       URD_COL_IN_PROJECT, info->building_count,
+                       URD_COL_UPGRADABLE, upgradable,
+                       URD_COL_N_UPGRADABLE, 0, /* never displayed */
+                       URD_COL_IN_PROGRESS, info->building_count,
                        URD_COL_ACTIVE, info->active_count,
                        URD_COL_SHIELD, info->upkeep[O_SHIELD],
                        URD_COL_FOOD, info->upkeep[O_FOOD],
                        URD_COL_GOLD, info->upkeep[O_GOLD],
                        URD_COL_TEXT_WEIGHT, PANGO_WEIGHT_NORMAL,
-                       URD_COL_BOOL_VISIBLE, TRUE,
+                       URD_COL_UPG_VISIBLE, TRUE,
+                       URD_COL_NUPG_VISIBLE, FALSE,
                        URD_COL_UTYPE_ID, utype_id,
                        -1);
     if (selected == utype_id) {
@@ -1393,20 +1364,25 @@ static void units_report_update(struct units_report *preport)
       unit_totals.upkeep[o] += info->upkeep[o];
     } output_type_iterate_end;
     unit_totals.building_count += info->building_count;
+    if (upgradable) {
+      total_upgradable_count += info->active_count;
+    }
   } unit_type_iterate_end;
 
   /* Add the total row. */
   gtk_list_store_append(store, &iter);
   gtk_list_store_set(store, &iter,
                      URD_COL_UTYPE_NAME, _("Totals:"),
-                     URD_COL_UPGRADABLE, FALSE,
-                     URD_COL_IN_PROJECT, unit_totals.building_count,
+                     URD_COL_UPGRADABLE, FALSE, /* never displayed */
+                     URD_COL_N_UPGRADABLE, total_upgradable_count,
+                     URD_COL_IN_PROGRESS, unit_totals.building_count,
                      URD_COL_ACTIVE, unit_totals.active_count,
                      URD_COL_SHIELD, unit_totals.upkeep[O_SHIELD],
                      URD_COL_FOOD, unit_totals.upkeep[O_FOOD],
                      URD_COL_GOLD, unit_totals.upkeep[O_GOLD],
                      URD_COL_TEXT_WEIGHT, PANGO_WEIGHT_BOLD,
-                     URD_COL_BOOL_VISIBLE, FALSE,
+                     URD_COL_UPG_VISIBLE, FALSE,
+                     URD_COL_NUPG_VISIBLE, TRUE,
                      URD_COL_UTYPE_ID, U_LAST,
                      -1);
   if (selected == U_LAST) {
@@ -1571,30 +1547,31 @@ static void units_report_command_callback(struct gui_dialog *pdialog,
 ****************************************************************************/
 static void units_report_init(struct units_report *preport)
 {
-  GtkWidget *view, *sw, *button;
+  GtkWidget *view, *sw, *align, *button;
   GtkListStore *store;
   GtkTreeSelection *selection;
-  GtkContainer *vbox;
-  const char *title;
+  GtkBox *vbox;
+  GtkTreeViewColumn *col = NULL;
   enum units_report_columns i;
 
   fc_assert_ret(NULL != preport);
 
   gui_dialog_new(&preport->shell, GTK_NOTEBOOK(top_notebook), preport, TRUE);
   gui_dialog_set_title(preport->shell, _("Units"));
-  vbox = GTK_CONTAINER(preport->shell->vbox);
+  vbox = GTK_BOX(preport->shell->vbox);
+
+  align = gtk_alignment_new(0.5, 0.0, 0.0, 1.0);
+  gtk_box_pack_start(vbox, align, TRUE, TRUE, 0);
 
   sw = gtk_scrolled_window_new(NULL,NULL);
-  gtk_widget_set_halign(sw, GTK_ALIGN_CENTER);
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
                                       GTK_SHADOW_ETCHED_IN);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-  gtk_container_add(GTK_CONTAINER(vbox), sw);
+  gtk_container_add(GTK_CONTAINER(align), sw);
 
   store = units_report_store_new();
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
-  gtk_widget_set_vexpand(view, TRUE);
   g_object_unref(store);
   gtk_widget_set_name(view, "small_font");
   gtk_tree_view_columns_autosize(GTK_TREE_VIEW(view));
@@ -1605,42 +1582,56 @@ static void units_report_init(struct units_report *preport)
   g_signal_connect(selection, "changed",
                    G_CALLBACK(units_report_selection_callback), preport);
 
-  for (i = 0; (title = units_report_column_name(i)); i++) {
+  for (i = 0; unit_report_columns[i].title != NULL; i++) {
     GtkCellRenderer *renderer;
-    GtkTreeViewColumn *col;
 
-    if (G_TYPE_BOOLEAN
-        == gtk_tree_model_get_column_type(GTK_TREE_MODEL(store), i)) {
+    if (strlen(unit_report_columns[i].title) > 0) {
+      GtkWidget *header = gtk_label_new(Q_(unit_report_columns[i].title));
+      if (unit_report_columns[i].tooltip) {
+        gtk_widget_set_tooltip_text(header,
+                                    Q_(unit_report_columns[i].tooltip));
+      }
+      gtk_widget_show(header);
+      col = gtk_tree_view_column_new();
+      gtk_tree_view_column_set_widget(col, header);
+      if (unit_report_columns[i].rightalign) {
+        gtk_tree_view_column_set_alignment(col, 1.0);
+      }
+      gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
+    } /* else add new renderer to previous TreeViewColumn */
+
+    fc_assert(col != NULL);
+    if (G_TYPE_BOOLEAN == unit_report_columns[i].type) {
       renderer = gtk_cell_renderer_toggle_new();
-      col = gtk_tree_view_column_new_with_attributes(title, renderer,
-                                                     "active", i, "visible",
-                                                     URD_COL_BOOL_VISIBLE,
-                                                     NULL);
+      gtk_tree_view_column_pack_start(col, renderer, FALSE);
+      gtk_tree_view_column_add_attribute(col, renderer, "active", i);
     } else {
       renderer = gtk_cell_renderer_text_new();
-      col = gtk_tree_view_column_new_with_attributes(title, renderer,
-                                                     "text", i, "weight",
-                                                     URD_COL_TEXT_WEIGHT,
-                                                     NULL);
+      gtk_tree_view_column_pack_start(col, renderer, TRUE);
+      gtk_tree_view_column_add_attribute(col, renderer, "text", i);
+      gtk_tree_view_column_add_attribute(col, renderer,
+                                         "weight", URD_COL_TEXT_WEIGHT);
     }
 
-    if (i > 0) {
+    if (unit_report_columns[i].visible_col >= 0) {
+      gtk_tree_view_column_add_attribute(col, renderer, "visible",
+                                         unit_report_columns[i].visible_col);
+    }
+
+    if (unit_report_columns[i].rightalign) {
       g_object_set(G_OBJECT(renderer), "xalign", 1.0, NULL);
-      gtk_tree_view_column_set_alignment(col, 1.0);
     }
-
-    gtk_tree_view_append_column(GTK_TREE_VIEW(view), col);
   }
 
-  gui_dialog_add_button(preport->shell, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+  button = gui_dialog_add_stockbutton(preport->shell, GTK_STOCK_FIND,
+                                      _("Find _Nearest"), URD_RES_NEAREST);
+  gtk_widget_set_sensitive(button, FALSE);
 
   button = gui_dialog_add_button(preport->shell, _("_Upgrade"),
                                  URD_RES_UPGRADE);
   gtk_widget_set_sensitive(button, FALSE);
 
-  button = gui_dialog_add_stockbutton(preport->shell, GTK_STOCK_FIND,
-                                      _("Find _Nearest"), URD_RES_NEAREST);
-  gtk_widget_set_sensitive(button, FALSE);
+  gui_dialog_add_button(preport->shell, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
 
   gui_dialog_set_default_response(preport->shell, GTK_RESPONSE_CLOSE);
   gui_dialog_response_set_callback(preport->shell,
@@ -1722,7 +1713,7 @@ enum endgame_report_columns {
   FRD_COL_NUM
 };
 
-static struct endgame_report endgame_report = { NULL, NULL };
+static struct endgame_report endgame_report = { NULL, };
 
 /****************************************************************************
   Returns the title of the column (translated).
@@ -1767,7 +1758,7 @@ static void endgame_report_update(struct endgame_report *preport,
   col_types[FRD_COL_PLAYER] = G_TYPE_STRING;
   col_types[FRD_COL_NATION] = GDK_TYPE_PIXBUF;
   col_types[FRD_COL_SCORE] = G_TYPE_INT;
-  for (i = FRD_COL_NUM; (guint)i < col_num; i++) {
+  for (i = FRD_COL_NUM; i < col_num; i++) {
     col_types[i] = G_TYPE_INT;
   }
   store = gtk_list_store_newv(col_num, col_types);
@@ -1775,7 +1766,7 @@ static void endgame_report_update(struct endgame_report *preport,
   g_object_unref(G_OBJECT(store));
 
   /* Create the new columns. */
-  for (i = 0; (guint)i < col_num; i++) {
+  for (i = 0; i < col_num; i++) {
     GtkCellRenderer *renderer;
     const char *title;
     const char *attribute;
@@ -1857,18 +1848,13 @@ static void endgame_report_init(struct endgame_report *preport)
                                       GTK_SHADOW_ETCHED_IN);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
-  gtk_container_add(GTK_CONTAINER(preport->shell->vbox), sw);
+  gtk_box_pack_start(GTK_BOX(preport->shell->vbox), sw, TRUE, TRUE, 0);
 
   view = gtk_tree_view_new();
   gtk_widget_set_name(view, "small_font");
   gtk_container_add(GTK_CONTAINER(sw), view);
   preport->tree_view = GTK_TREE_VIEW(view);
 
-  if (preport->shell->type == GUI_DIALOG_TAB) {
-    gtk_widget_set_hexpand(GTK_WIDGET(view), TRUE);
-    gtk_widget_set_vexpand(GTK_WIDGET(view), TRUE);
-  }
-
   gui_dialog_show_all(preport->shell);
 }
 
