@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <stdlib.h>
@@ -73,7 +73,7 @@ static void dst_row_callback(GtkTreeView *view, GtkTreePath *path,
                              GtkTreeViewColumn *col, gpointer data);
 
 /****************************************************************
-...
+  Illegal initialization value for max unit size variables
 *****************************************************************/
 void blank_max_unit_size(void)
 {
@@ -82,7 +82,7 @@ void blank_max_unit_size(void)
 }
 
 /****************************************************************
-...
+  Setup max unit sprite size.
 *****************************************************************/
 static void update_max_unit_size(void)
 {
@@ -91,7 +91,8 @@ static void update_max_unit_size(void)
 
   unit_type_iterate(i) {
     int x1, x2, y1, y2;
-    struct sprite *sprite = get_unittype_sprite(tileset, i);
+    struct sprite *sprite = get_unittype_sprite(tileset, i,
+                                                direction8_invalid(), TRUE);
 
     sprite_get_bounding_box(sprite, &x1, &y1, &x2, &y2);
     max_unit_width = MAX(max_unit_width, x2 - x1);
@@ -101,7 +102,7 @@ static void update_max_unit_size(void)
 
 
 /****************************************************************
-...
+  Worklists dialog being destroyed
 *****************************************************************/
 static void worklists_destroy_callback(GtkWidget *w, gpointer data)
 {
@@ -110,7 +111,7 @@ static void worklists_destroy_callback(GtkWidget *w, gpointer data)
 
 
 /****************************************************************
-...
+  Refresh global worklists list
 *****************************************************************/
 void update_worklist_report_dialog(void)
 {
@@ -128,7 +129,7 @@ void update_worklist_report_dialog(void)
 }
 
 /****************************************************************
-...
+  User has responded to worklist report
 *****************************************************************/
 static void worklists_response(GtkWidget *w, gint response)
 {
@@ -179,7 +180,7 @@ static void worklists_response(GtkWidget *w, gint response)
 }
 
 /****************************************************************
-...
+  Worklist cell edited
 *****************************************************************/
 static void cell_edited(GtkCellRendererText *cell,
                         const gchar *spath,
@@ -276,7 +277,7 @@ static GtkWidget *create_worklists_report(void)
 }
 
 /****************************************************************
-...
+  Open worklists report
 *****************************************************************/
 void popup_worklists_report(void)
 {
@@ -329,7 +330,7 @@ static GtkTargetEntry wl_dnd_targets[] = {
 
 
 /****************************************************************
-...
+  Add drag&drop target
 *****************************************************************/
 void add_worklist_dnd_target(GtkWidget *w)
 {
@@ -339,7 +340,7 @@ void add_worklist_dnd_target(GtkWidget *w)
 }
 
 /****************************************************************
-...
+  Get worklist by id
 *****************************************************************/
 static GtkWidget *get_worklist(int global_worklist_id)
 {
@@ -354,7 +355,7 @@ static GtkWidget *get_worklist(int global_worklist_id)
 }
 
 /****************************************************************
-...
+  Insert worklist to editor
 *****************************************************************/
 static void insert_worklist(int global_worklist_id, GtkWidget *editor)
 {
@@ -365,7 +366,7 @@ static void insert_worklist(int global_worklist_id, GtkWidget *editor)
 }
 
 /****************************************************************
-...
+  Remove worklist from hash
 *****************************************************************/
 static void delete_worklist(int global_worklist_id)
 {
@@ -375,7 +376,7 @@ static void delete_worklist(int global_worklist_id)
 }
 
 /****************************************************************
-...
+  User responded to worklist report
 *****************************************************************/
 static void worklist_response(GtkWidget *shell, gint response)
 {
@@ -417,7 +418,7 @@ static void popup_worklist(struct global_worklist *pgwl)
 }
 
 /****************************************************************
-...
+  Close worklist
 *****************************************************************/
 static void popdown_worklist(struct global_worklist *pgwl)
 {
@@ -432,7 +433,7 @@ static void popdown_worklist(struct global_worklist *pgwl)
 }
 
 /****************************************************************
-...
+  Destroy worklist
 *****************************************************************/
 static void worklist_destroy(GtkWidget *editor, gpointer data)
 {
@@ -448,7 +449,7 @@ static void worklist_destroy(GtkWidget *editor, gpointer data)
 }
 
 /****************************************************************
-...
+  Item activated from menu
 *****************************************************************/
 static void menu_item_callback(GtkMenuItem *item, struct worklist_data *ptr)
 {
@@ -481,7 +482,7 @@ static void menu_item_callback(GtkMenuItem *item, struct worklist_data *ptr)
 }
 
 /****************************************************************
-...
+  Open menu for adding items to worklist
 *****************************************************************/
 static void popup_add_menu(GtkMenuShell *menu, gpointer data)
 {
@@ -515,7 +516,7 @@ static void popup_add_menu(GtkMenuShell *menu, gpointer data)
 }
 
 /****************************************************************
-...
+  Help button clicked
 *****************************************************************/
 static void help_callback(GtkWidget *w, gpointer data)
 {
@@ -550,7 +551,7 @@ static void help_callback(GtkWidget *w, gpointer data)
 }
 
 /****************************************************************
-...
+  "Change Production" clicked
 *****************************************************************/
 static void change_callback(GtkWidget *w, gpointer data)
 {
@@ -572,7 +573,7 @@ static void change_callback(GtkWidget *w, gpointer data)
 }
 
 /****************************************************************
-...
+  Showing of future targets toggled
 *****************************************************************/
 static void future_callback(GtkToggleButton *toggle, gpointer data)
 {
@@ -585,7 +586,7 @@ static void future_callback(GtkToggleButton *toggle, gpointer data)
 }
 
 /****************************************************************
-...
+  Move item up in worklist
 *****************************************************************/
 static void queue_bubble_up(struct worklist_data *ptr)
 {
@@ -617,7 +618,7 @@ static void queue_bubble_up(struct worklist_data *ptr)
 }
 
 /****************************************************************
-...
+  Removal of the item requested
 *****************************************************************/
 static void queue_remove(struct worklist_data *ptr)
 {
@@ -632,7 +633,7 @@ static void queue_remove(struct worklist_data *ptr)
 }
 
 /****************************************************************
-...
+  Move item down in queue
 *****************************************************************/
 static void queue_bubble_down(struct worklist_data *ptr)
 {
@@ -663,7 +664,7 @@ static void queue_bubble_down(struct worklist_data *ptr)
 }
 
 /****************************************************************
-...
+  Insert item to queue
 *****************************************************************/
 static void queue_insert(struct worklist_data *ptr, bool prepend)
 {
@@ -709,7 +710,7 @@ static void queue_insert(struct worklist_data *ptr, bool prepend)
 }
 
 /****************************************************************
-...
+  Prepend item to worklist
 *****************************************************************/
 static void queue_prepend(struct worklist_data *ptr)
 {
@@ -717,7 +718,7 @@ static void queue_prepend(struct worklist_data *ptr)
 }
 
 /****************************************************************
-...
+  Append item to worklist
 *****************************************************************/
 static void queue_append(struct worklist_data *ptr)
 {
@@ -725,7 +726,7 @@ static void queue_append(struct worklist_data *ptr)
 }
 
 /****************************************************************
-...
+  Source row activated
 *****************************************************************/
 static void src_row_callback(GtkTreeView *view, GtkTreePath *path,
 			     GtkTreeViewColumn *col, gpointer data)
@@ -759,7 +760,7 @@ static void src_row_callback(GtkTreeView *view, GtkTreePath *path,
 }
 
 /****************************************************************
-...
+  Destination row activated
 *****************************************************************/
 static void dst_row_callback(GtkTreeView *view, GtkTreePath *path,
 			     GtkTreeViewColumn *col, gpointer data)
@@ -778,7 +779,7 @@ static void dst_row_callback(GtkTreeView *view, GtkTreePath *path,
 }
 
 /****************************************************************
-...
+  Key press for source
 *****************************************************************/
 static gboolean src_key_press_callback(GtkWidget *w, GdkEventKey *ev,
 				       gpointer data)
@@ -803,7 +804,7 @@ static gboolean src_key_press_callback(GtkWidget *w, GdkEventKey *ev,
 }
 
 /****************************************************************
-...
+  Key press for destination
 *****************************************************************/
 static gboolean dst_key_press_callback(GtkWidget *w, GdkEventKey *ev,
 				       gpointer data)
@@ -853,7 +854,7 @@ static gboolean dst_key_press_callback(GtkWidget *w, GdkEventKey *ev,
 }
 
 /****************************************************************
-...
+  Selection from source
 *****************************************************************/
 static void src_selection_callback(GtkTreeSelection *selection, gpointer data)
 {
@@ -884,7 +885,7 @@ static void src_selection_callback(GtkTreeSelection *selection, gpointer data)
 }
 
 /****************************************************************
-...
+  Selection from destination
 *****************************************************************/
 static void dst_selection_callback(GtkTreeSelection *selection, gpointer data)
 {
@@ -917,7 +918,7 @@ static void dst_selection_callback(GtkTreeSelection *selection, gpointer data)
 }
 
 /****************************************************************
-...
+  Drag&drop to destination
 *****************************************************************/
 static gboolean dst_dnd_callback(GtkWidget *w, GdkDragContext *context,
 				 struct worklist_data *ptr)
@@ -927,7 +928,7 @@ static gboolean dst_dnd_callback(GtkWidget *w, GdkDragContext *context,
 }
 
 /****************************************************************
-...
+  Render worklist cell
 *****************************************************************/
 static void cell_render_func(GtkTreeViewColumn *col, GtkCellRenderer *rend,
 			     GtkTreeModel *model, GtkTreeIter *it,
@@ -950,7 +951,7 @@ static void cell_render_func(GtkTreeViewColumn *col, GtkCellRenderer *rend,
 
       store.type = CANVAS_PIXBUF;
       store.v.pixbuf = pix;
-      create_overlay_unit(&store, target.value.utype);
+      create_overlay_unit(&store, target.value.utype, DIR8_SOUTH);
 
       g_object_set(rend, "pixbuf", pix, NULL);
       g_object_unref(pix);
@@ -988,7 +989,7 @@ static void cell_render_func(GtkTreeViewColumn *col, GtkCellRenderer *rend,
 }
 
 /****************************************************************
-...
+  Populate view with buildable item information
 *****************************************************************/
 static void populate_view(GtkTreeView *view, struct city **ppcity,
 			  GtkTreeViewColumn **pcol)
@@ -1210,7 +1211,7 @@ GtkWidget *create_worklist(void)
   gtk_box_set_spacing(GTK_BOX(bbox), 10);
   gtk_box_pack_start(GTK_BOX(editor), bbox, FALSE, FALSE, 0);
 
-  menubar = gtk_menu_bar_new();
+  menubar = gtk_aux_menu_bar_new();
   gtk_container_add(GTK_CONTAINER(bbox), menubar);
   gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(bbox), menubar, TRUE);
 
@@ -1275,7 +1276,7 @@ GtkWidget *create_worklist(void)
 
 
 /****************************************************************
-  ...
+  Reset worklist for city
 *****************************************************************/
 void reset_city_worklist(GtkWidget *editor, struct city *pcity)
 {
@@ -1300,7 +1301,7 @@ void reset_city_worklist(GtkWidget *editor, struct city *pcity)
 }
 
 /****************************************************************
-  ...
+  Reset one of the global worklists
 *****************************************************************/
 static void reset_global_worklist(GtkWidget *editor,
                                   struct global_worklist *pgwl)
@@ -1323,7 +1324,7 @@ static void reset_global_worklist(GtkWidget *editor,
 }
 
 /****************************************************************
-  ...
+  Refresh worklist info
 *****************************************************************/
 void refresh_worklist(GtkWidget *editor)
 {
@@ -1430,7 +1431,7 @@ void refresh_worklist(GtkWidget *editor)
 }
 
 /****************************************************************
-...
+  Commit worklist data to worklist
 *****************************************************************/
 static void commit_worklist(struct worklist_data *ptr)
 {
