@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <stdio.h>
@@ -34,7 +34,7 @@
 #include "packets.h"
 #include "unit.h"
 
-/* agents */
+/* client/agents */
 #include "cma_fec.h"
 
 /* client */
@@ -46,7 +46,7 @@
 #include "mapview_common.h"
 #include "options.h"
 
-/* gui-gtk-2.0 */
+/* client/gui-gtk-3.0 */
 #include "chatline.h"
 #include "citydlg.h"
 #include "gui_main.h"
@@ -317,7 +317,7 @@ void city_report_dialog_popdown(void)
 }
 
 /****************************************************************
-...
+  Make submenu listing possible build targets
 *****************************************************************/
 static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
 					     bool append_units,
@@ -342,7 +342,6 @@ static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
     ""
   };
 
-  gtk_menu_item_remove_submenu(parent_item);
   menu = gtk_menu_new();
   gtk_menu_item_set_submenu(parent_item, menu);
 
@@ -401,7 +400,8 @@ static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
     get_city_dialog_production_row(row, sizeof(buf[0]), target, NULL);
 
     menu_item = gtk_menu_item_new();
-    hbox = gtk_hbox_new(FALSE, 18);
+    hbox = gtk_grid_new();
+    gtk_grid_set_column_spacing(GTK_GRID(hbox), 18);
     gtk_container_add(GTK_CONTAINER(menu_item), hbox);
 
     for (i = 0; i < 3; i++) {
@@ -419,13 +419,20 @@ static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
       label = gtk_label_new(NULL);
       gtk_label_set_markup(GTK_LABEL(label), txt);
 
-      if (i < 2) {
-	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-      } else {
-	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
+      switch (i) {
+        case 0:
+          gtk_widget_set_halign(label, GTK_ALIGN_START);
+          gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+        break;
+        case 2:
+          gtk_widget_set_halign(label, GTK_ALIGN_END);
+          gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
+          break;
+        default:
+          break;
       }
 
-      gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, FALSE, 0);
+      gtk_container_add(GTK_CONTAINER(hbox), label);
       gtk_size_group_add_widget(group[i], label);
     }
 
@@ -554,12 +561,13 @@ static void sell_impr_iterate(GtkTreeModel *model, GtkTreePath *path,
 }
 
 /****************************************************************************
-  ...
+  Some build target, either improvement or unit, has been selected from
+  some menu.
 ****************************************************************************/
 static void select_impr_or_unit_callback(GtkWidget *w, gpointer data)
 {
   struct universal target = cid_decode(GPOINTER_TO_INT(data));
-  GObject *parent = G_OBJECT(w->parent);
+  GObject *parent = G_OBJECT(gtk_widget_get_parent(w));
   TestCityFunc test_func = g_object_get_data(parent, "freeciv_test_func");
   enum city_operation_type city_operation = 
     GPOINTER_TO_INT(g_object_get_data(parent, "freeciv_city_operation"));
@@ -632,7 +640,7 @@ static void select_impr_or_unit_callback(GtkWidget *w, gpointer data)
                                             sell_impr_iterate, &sd);
         if (sd.count > 0) {
           /* FIXME: plurality of sd.count is ignored! */
-          /* TRANS: "Sold 3 Harbour for 90 gold." (Pluralisation is in gold --
+          /* TRANS: "Sold 3 Harbor for 90 gold." (Pluralisation is in gold --
            * second %d -- not in buildings.) */
           fc_snprintf(buf, sizeof(buf), PL_("Sold %d %s for %d gold.",
                                             "Sold %d %s for %d gold.",
@@ -683,7 +691,7 @@ static void cma_iterate(GtkTreeModel *model, GtkTreePath *path,
 static void select_cma_callback(GtkWidget * w, gpointer data)
 {
   int idx = GPOINTER_TO_INT(data);
-  GObject *parent = G_OBJECT(w->parent);
+  GObject *parent = G_OBJECT(gtk_widget_get_parent(w));
   bool change_cma =
       GPOINTER_TO_INT(g_object_get_data(parent, "freeciv_change_cma"));
   struct cm_parameter parameter;
@@ -745,12 +753,12 @@ static void append_cma_to_menu_item(GtkMenuItem *parent_item, bool change_cma)
   GtkWidget *w;
 
   w = gtk_menu_item_get_submenu(parent_item);
-  if (w != NULL && GTK_WIDGET_VISIBLE(w)) {
+  if (w != NULL && gtk_widget_get_visible(w)) {
     return;
   }
 
-  gtk_menu_item_remove_submenu(parent_item);
   if (!can_client_issue_orders()) {
+    gtk_menu_item_set_submenu(parent_item, NULL);
     return;
   }
   menu = gtk_menu_new();
@@ -936,7 +944,7 @@ static void production_menu_shown(GtkWidget *widget, gpointer data)
   fc_assert_ret(callback != NULL);
 
   menu = gtk_menu_item_get_submenu(parent_item);
-  if (menu != NULL && GTK_WIDGET_VISIBLE(menu)) {
+  if (menu != NULL && gtk_widget_get_visible(menu)) {
     gtk_menu_shell_deactivate(GTK_MENU_SHELL(menu));
   }
 
@@ -969,7 +977,7 @@ static void production_menu_shown(GtkWidget *widget, gpointer data)
 }
 
 /****************************************************************
-...
+  Update city report views
 *****************************************************************/
 static void city_report_update_views(void)
 {
@@ -993,7 +1001,7 @@ static void city_report_update_views(void)
 }
 
 /****************************************************************
-...
+  User has toggled some column viewing option
 *****************************************************************/
 static void toggle_view(GtkCheckMenuItem *item, gpointer data)
 {
@@ -1004,7 +1012,7 @@ static void toggle_view(GtkCheckMenuItem *item, gpointer data)
 }
 
 /****************************************************************
-...
+  Create view menu for city report menubar.
 *****************************************************************/
 static void update_view_menu(GtkWidget *show_item)
 {
@@ -1023,18 +1031,20 @@ static void update_view_menu(GtkWidget *show_item)
 }
 
 /****************************************************************
-...
+  Create menubar for city report
 *****************************************************************/
 static GtkWidget *create_city_report_menubar(void)
 {
   GtkWidget *vbox, *sep, *menubar, *menu, *item;
 
-  vbox = gtk_vbox_new(FALSE, 0);
-  sep = gtk_hseparator_new();
-  gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 0);
+  vbox = gtk_grid_new();
+  gtk_orientable_set_orientation(GTK_ORIENTABLE(vbox),
+                                 GTK_ORIENTATION_VERTICAL);
+  sep = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
+  gtk_container_add(GTK_CONTAINER(vbox), sep);
 
   menubar = gtk_aux_menu_bar_new();
-  gtk_box_pack_start(GTK_BOX(vbox), menubar, TRUE, TRUE, 0);
+  gtk_container_add(GTK_CONTAINER(vbox), menubar);
   
   item = gtk_menu_item_new_with_mnemonic(_("_Production"));
   city_production_command = item;
@@ -1119,15 +1129,7 @@ static gint cityrep_sort_func(GtkTreeModel *model, GtkTreeIter *a,
 }
 
 /****************************************************************
-  Clean up when city report tree view is destroyed
-*****************************************************************/
-static void city_view_destroyed(GtkObject *object, gpointer column_tooltips)
-{
-  gtk_object_destroy(column_tooltips);
-}
-
-/****************************************************************
-...
+  Create city report dialog.
 *****************************************************************/
 static void create_city_report_dialog(bool make_modal)
 {
@@ -1136,7 +1138,6 @@ static void create_city_report_dialog(bool make_modal)
   struct city_report_spec *spec;
 
   GtkWidget *w, *sw, *menubar;
-  GtkTooltips *column_tooltips = gtk_tooltips_new();
   int i;
 
   gui_dialog_new(&city_dialog_shell, GTK_NOTEBOOK(top_notebook), NULL, TRUE);
@@ -1152,26 +1153,27 @@ static void create_city_report_dialog(bool make_modal)
   gui_dialog_add_widget(city_dialog_shell, menubar);
 
   /* buttons */
-  w = gui_dialog_add_stockbutton(city_dialog_shell, GTK_STOCK_ZOOM_FIT,
-      _("Cen_ter"), CITY_CENTER);
-  city_center_command = w;
+  gui_dialog_add_button(city_dialog_shell,
+			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
 
-  w = gui_dialog_add_stockbutton(city_dialog_shell, GTK_STOCK_ZOOM_IN,
-      _("_Inspect"), CITY_POPUP);
-  city_popup_command = w;
+  city_total_buy_cost_label = gtk_label_new(NULL);
+  gtk_widget_set_hexpand(city_total_buy_cost_label, TRUE);
+  gtk_label_set_ellipsize(GTK_LABEL(city_total_buy_cost_label),
+                          PANGO_ELLIPSIZE_START);
+  gtk_container_add(GTK_CONTAINER(city_dialog_shell->action_area),
+                    city_total_buy_cost_label);
 
   w = gui_dialog_add_stockbutton(city_dialog_shell, GTK_STOCK_EXECUTE,
       _("_Buy"), CITY_BUY);
   city_buy_command = w;
 
-  city_total_buy_cost_label = gtk_label_new(NULL);
-  gtk_label_set_ellipsize(GTK_LABEL(city_total_buy_cost_label),
-                          PANGO_ELLIPSIZE_START);
-  gtk_box_pack_end(GTK_BOX(city_dialog_shell->action_area),
-                   city_total_buy_cost_label, TRUE, TRUE, 0);
+  w = gui_dialog_add_stockbutton(city_dialog_shell, GTK_STOCK_ZOOM_IN,
+      _("_Inspect"), CITY_POPUP);
+  city_popup_command = w;
 
-  gui_dialog_add_button(city_dialog_shell,
-			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+  w = gui_dialog_add_stockbutton(city_dialog_shell, GTK_STOCK_ZOOM_FIT,
+      _("Cen_ter"), CITY_CENTER);
+  city_center_command = w;
 
   gui_dialog_set_default_response(city_dialog_shell,
 				  GTK_RESPONSE_CLOSE);
@@ -1187,6 +1189,8 @@ static void create_city_report_dialog(bool make_modal)
   city_model = city_report_dialog_store_new();
 
   city_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(city_model));
+  gtk_widget_set_hexpand(city_view, TRUE);
+  gtk_widget_set_vexpand(city_view, TRUE);
   g_object_unref(city_model);
   gtk_widget_set_name(city_view, "small_font");
   g_signal_connect(city_view, "row_activated",
@@ -1196,10 +1200,6 @@ static void create_city_report_dialog(bool make_modal)
   g_signal_connect(city_selection, "changed",
 	G_CALLBACK(city_selection_changed_callback), NULL);
 
-  /* Ensure column_tooltips gets cleaned up */
-  g_signal_connect(city_view, "destroy",
-                   G_CALLBACK(city_view_destroyed), column_tooltips);
-
   for (i = 0, spec = city_report_specs; i < NUM_CREPORT_COLS; i++, spec++) {
     GtkWidget *header;
     GtkCellRenderer *renderer;
@@ -1209,7 +1209,7 @@ static void create_city_report_dialog(bool make_modal)
     col = gtk_tree_view_column_new_with_attributes(NULL, renderer,
                                                    "text", i, NULL);
     header = gtk_label_new(titles[i]);
-    gtk_tooltips_set_tip(column_tooltips, header, spec->explanation, NULL);
+    gtk_widget_set_tooltip_text(header, spec->explanation);
     gtk_widget_show(header);
     gtk_tree_view_column_set_widget(col, header);
     gtk_tree_view_column_set_visible(col, spec->show);
@@ -1229,8 +1229,7 @@ static void create_city_report_dialog(bool make_modal)
                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_container_add(GTK_CONTAINER(sw), city_view);
 
-  gtk_box_pack_start(GTK_BOX(city_dialog_shell->vbox),
-                     sw, TRUE, TRUE, 0);
+  gtk_container_add(GTK_CONTAINER(city_dialog_shell->vbox), sw);
 
   city_model_fill(city_model, NULL, NULL);
   gui_dialog_show_all(city_dialog_shell);
@@ -1239,7 +1238,7 @@ static void create_city_report_dialog(bool make_modal)
 }
 
 /****************************************************************
-...
+  User has chosen to select all cities
 *****************************************************************/
 static void city_select_all_callback(GtkMenuItem *item, gpointer data)
 {
@@ -1247,7 +1246,7 @@ static void city_select_all_callback(GtkMenuItem *item, gpointer data)
 }
 
 /****************************************************************
-...
+  User has chosen to unselect all cities
 *****************************************************************/
 static void city_unselect_all_callback(GtkMenuItem *item, gpointer data)
 {
@@ -1255,7 +1254,7 @@ static void city_unselect_all_callback(GtkMenuItem *item, gpointer data)
 }
 
 /****************************************************************
-...
+  User has chosen to invert selection
 *****************************************************************/
 static void city_invert_selection_callback(GtkMenuItem *item, gpointer data)
 {
@@ -1272,7 +1271,7 @@ static void city_invert_selection_callback(GtkMenuItem *item, gpointer data)
 }
 
 /****************************************************************
-...
+  User has chosen to select coastal cities
 *****************************************************************/
 static void city_select_coastal_callback(GtkMenuItem *item, gpointer data)
 {
@@ -1315,7 +1314,7 @@ static void same_island_iterate(GtkTreeModel *model, GtkTreePath *path,
 }
 
 /****************************************************************
-...
+  User has chosen to select all cities on same island
 *****************************************************************/
 static void city_select_same_island_callback(GtkMenuItem *item, gpointer data)
 {
@@ -1323,7 +1322,7 @@ static void city_select_same_island_callback(GtkMenuItem *item, gpointer data)
 }
       
 /****************************************************************
-...
+  User has chosen to select cities with certain target in production
 *****************************************************************/
 static void city_select_building_callback(GtkMenuItem *item, gpointer data)
 {
@@ -1418,13 +1417,15 @@ static void city_command_callback(struct gui_dialog *dlg, int response,
 }
 
 /****************************************************************
-...
+  User has selected city row from city report.
 *****************************************************************/
 static void city_activated_callback(GtkTreeView *view, GtkTreePath *path,
 				    GtkTreeViewColumn *col, gpointer data)
 {
   GtkTreeModel *model;
   GtkTreeIter iter;
+  GdkWindow *win;
+  GdkDeviceManager *manager;
   GdkModifierType mask;
 
   model = gtk_tree_view_get_model(view);
@@ -1433,7 +1434,12 @@ static void city_activated_callback(GtkTreeView *view, GtkTreePath *path,
     return;
   }
 
-  gdk_window_get_pointer(NULL, NULL, NULL, &mask);
+  win = gdk_get_default_root_window();
+  manager = gdk_display_get_device_manager(gdk_window_get_display(win));
+
+  gdk_window_get_device_position(win,
+                                 gdk_device_manager_get_client_pointer(manager),
+                                 NULL, NULL, &mask);
 
   if (!(mask & GDK_CONTROL_MASK)) {
     popup_iterate(model, path, &iter, NULL);
@@ -1470,7 +1476,7 @@ void real_city_report_dialog_update(void)
   city_model_fill(city_model, city_selection, selected);
   g_hash_table_destroy(selected);
 
-  if (GTK_WIDGET_SENSITIVE(city_governor_command)) {
+  if (gtk_widget_get_sensitive(city_governor_command)) {
     append_cma_to_menu_item(GTK_MENU_ITEM(city_governor_command), TRUE);
   }
 
@@ -1497,7 +1503,7 @@ void real_city_report_update_city(struct city *pcity)
 }
 
 /****************************************************************
-...
+  Create submenu for changing production target
 *****************************************************************/
 static void create_change_menu(GtkWidget *item)
 {
@@ -1610,7 +1616,7 @@ static void create_sell_menu(GtkWidget *item)
 
 
 /****************************************************************
-...
+  Pops up menu where user can select build target.
 *****************************************************************/
 static void popup_change_menu(GtkMenuShell *menu, gpointer data)
 {
@@ -1770,7 +1776,7 @@ static void recreate_sell_menu(void)
 }
 
 /****************************************************************
-...
+  Creates select menu
 *****************************************************************/
 static void create_select_menu(GtkWidget *item)
 {
@@ -1889,7 +1895,7 @@ static void create_select_menu(GtkWidget *item)
 }
 
 /****************************************************************
-...
+  Returns whether city is building given target
 *****************************************************************/
 static bool city_building_impr_or_unit(const struct city *pcity,
 				       struct universal target)
@@ -1898,7 +1904,7 @@ static bool city_building_impr_or_unit(const struct city *pcity,
 }
 
 /****************************************************************
-...
+  Popup select menu
 *****************************************************************/
 static void popup_select_menu(GtkMenuShell *menu, gpointer data)
 {
