@@ -12,12 +12,13 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <gtk/gtk.h>
 
 /* utility */
+#include "astring.h"
 #include "support.h"
 
 /* common */
@@ -38,7 +39,7 @@
 #include "mapview.h"
 #include "packhand.h"
 
-/* client/gui-gtk-2.0 */
+/* client/gui-gtk-3.0 */
 #include "citydlg.h"
 #include "dialogs.h"
 #include "wldlg.h"
@@ -54,7 +55,7 @@ static GtkWidget  *spy_sabotage_shell;
 static int         sabotage_improvement;
 
 /****************************************************************
-...
+  User responded to bribe dialog
 *****************************************************************/
 static void bribe_response(GtkWidget *w, gint response)
 {
@@ -66,7 +67,7 @@ static void bribe_response(GtkWidget *w, gint response)
 }
 
 /****************************************************************
-...  Ask the server how much the bribe is
+  Ask the server how much the bribe is
 *****************************************************************/
 static void diplomat_bribe_callback(GtkWidget *w, gpointer data)
 {
@@ -79,7 +80,7 @@ static void diplomat_bribe_callback(GtkWidget *w, gpointer data)
 }
 
 /****************************************************************
-...
+  Popup unit bribe dialog
 *****************************************************************/
 void popup_bribe_dialog(struct unit *punit, int cost)
 {
@@ -120,7 +121,7 @@ void popup_bribe_dialog(struct unit *punit, int cost)
 }
 
 /****************************************************************
-...
+  User selected sabotaging from choice dialog
 *****************************************************************/
 static void diplomat_sabotage_callback(GtkWidget *w, gpointer data)
 {
@@ -133,7 +134,7 @@ static void diplomat_sabotage_callback(GtkWidget *w, gpointer data)
 }
 
 /****************************************************************
-...
+  User selected investigating from choice dialog
 *****************************************************************/
 static void diplomat_investigate_callback(GtkWidget *w, gpointer data)
 {
@@ -146,7 +147,7 @@ static void diplomat_investigate_callback(GtkWidget *w, gpointer data)
 }
 
 /****************************************************************
-...
+  User selected unit sabotaging from choice dialog
 *****************************************************************/
 static void spy_sabotage_unit_callback(GtkWidget *w, gpointer data)
 {
@@ -156,7 +157,7 @@ static void spy_sabotage_unit_callback(GtkWidget *w, gpointer data)
 }
 
 /****************************************************************
-...
+  User selected embassy establishing from choice dialog
 *****************************************************************/
 static void diplomat_embassy_callback(GtkWidget *w, gpointer data)
 {
@@ -169,7 +170,7 @@ static void diplomat_embassy_callback(GtkWidget *w, gpointer data)
 }
 
 /****************************************************************
-...
+  User selected poisoning from choice dialog
 *****************************************************************/
 static void spy_poison_callback(GtkWidget *w, gpointer data)
 {
@@ -181,7 +182,7 @@ static void spy_poison_callback(GtkWidget *w, gpointer data)
 }
 
 /****************************************************************
-...
+  User selected stealing from choice dialog
 *****************************************************************/
 static void diplomat_steal_callback(GtkWidget *w, gpointer data)
 {
@@ -194,7 +195,7 @@ static void diplomat_steal_callback(GtkWidget *w, gpointer data)
 }
 
 /****************************************************************
-...
+  User responded to steal advances dialog
 *****************************************************************/
 static void spy_advances_response(GtkWidget *w, gint response, gpointer data)
 {
@@ -210,7 +211,7 @@ static void spy_advances_response(GtkWidget *w, gint response, gpointer data)
 }
 
 /****************************************************************
-...
+  User selected entry in steal advances dialog
 *****************************************************************/
 static void spy_advances_callback(GtkTreeSelection *select, gpointer data)
 {
@@ -231,7 +232,7 @@ static void spy_advances_callback(GtkTreeSelection *select, gpointer data)
 }
 
 /****************************************************************
-...
+  Create spy's tech stealing dialog
 *****************************************************************/
 static void create_advances_list(struct player *pplayer,
 				 struct player *pvictim)
@@ -256,14 +257,19 @@ static void create_advances_list(struct player *pplayer,
 				  GTK_RESPONSE_ACCEPT);
 
   label = gtk_frame_new(_("Select Advance to Steal"));
-  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(spy_tech_shell)->vbox), label);
+  gtk_container_add(GTK_CONTAINER(gtk_dialog_get_content_area(GTK_DIALOG(spy_tech_shell))), label);
 
-  vbox = gtk_vbox_new(FALSE, 6);
+  vbox = gtk_grid_new();
+  gtk_orientable_set_orientation(GTK_ORIENTABLE(vbox),
+                                 GTK_ORIENTATION_VERTICAL);
+  gtk_grid_set_row_spacing(GTK_GRID(vbox), 6);
   gtk_container_add(GTK_CONTAINER(label), vbox);
       
   store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
 
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+  gtk_widget_set_hexpand(view, TRUE);
+  gtk_widget_set_vexpand(view, TRUE);
   g_object_unref(store);
   gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
 
@@ -315,7 +321,14 @@ static void create_advances_list(struct player *pplayer,
     gtk_list_store_append(store, &it);
 
     g_value_init(&value, G_TYPE_STRING);
-    g_value_set_static_string(&value, _("At Spy's Discretion"));
+    {
+      struct astring str = ASTRING_INIT;
+      /* TRANS: %s is a unit name, e.g., Spy */
+      astr_set(&str, _("At %s's Discretion"),
+               unit_name_translation(game_unit_by_number(diplomat_id)));
+      g_value_set_string(&value, astr_str(&str));
+      astr_free(&str);
+    }
     gtk_list_store_set_value(store, &it, 0, &value);
     g_value_unset(&value);
     gtk_list_store_set(store, &it, 1, A_UNSET, -1);
@@ -324,7 +337,7 @@ static void create_advances_list(struct player *pplayer,
   gtk_dialog_set_response_sensitive(GTK_DIALOG(spy_tech_shell),
     GTK_RESPONSE_ACCEPT, FALSE);
   
-  gtk_widget_show_all(GTK_DIALOG(spy_tech_shell)->vbox);
+  gtk_widget_show_all(gtk_dialog_get_content_area(GTK_DIALOG(spy_tech_shell)));
 
   g_signal_connect(gtk_tree_view_get_selection(GTK_TREE_VIEW(view)), "changed",
 		   G_CALLBACK(spy_advances_callback), NULL);
@@ -337,7 +350,7 @@ static void create_advances_list(struct player *pplayer,
 }
 
 /****************************************************************
-...
+  User has responded to spy's sabotage building dialog
 *****************************************************************/
 static void spy_improvements_response(GtkWidget *w, gint response, gpointer data)
 {
@@ -353,7 +366,7 @@ static void spy_improvements_response(GtkWidget *w, gint response, gpointer data
 }
 
 /****************************************************************
-...
+  User has selected new building from spy's sabotage dialog
 *****************************************************************/
 static void spy_improvements_callback(GtkTreeSelection *select, gpointer data)
 {
@@ -374,7 +387,7 @@ static void spy_improvements_callback(GtkTreeSelection *select, gpointer data)
 }
 
 /****************************************************************
-...
+  Creates spy's building sabotaging dialog
 *****************************************************************/
 static void create_improvements_list(struct player *pplayer,
 				     struct city *pcity)
@@ -400,14 +413,19 @@ static void create_improvements_list(struct player *pplayer,
 				  GTK_RESPONSE_ACCEPT);
 
   label = gtk_frame_new(_("Select Improvement to Sabotage"));
-  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(spy_sabotage_shell)->vbox), label);
+  gtk_container_add(GTK_CONTAINER(gtk_dialog_get_content_area(GTK_DIALOG(spy_sabotage_shell))), label);
 
-  vbox = gtk_vbox_new(FALSE, 6);
+  vbox = gtk_grid_new();
+  gtk_orientable_set_orientation(GTK_ORIENTABLE(vbox),
+                                 GTK_ORIENTATION_VERTICAL);
+  gtk_grid_set_row_spacing(GTK_GRID(vbox), 6);
   gtk_container_add(GTK_CONTAINER(label), vbox);
       
   store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
 
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+  gtk_widget_set_hexpand(view, TRUE);
+  gtk_widget_set_vexpand(view, TRUE);
   g_object_unref(store);
   gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
 
@@ -432,7 +450,7 @@ static void create_improvements_list(struct player *pplayer,
 
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
     GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
-  gtk_widget_set_size_request(sw, -1, 200);
+  gtk_scrolled_window_set_min_content_height(GTK_SCROLLED_WINDOW(sw), 200);
   
   gtk_container_add(GTK_CONTAINER(vbox), sw);
 
@@ -451,12 +469,19 @@ static void create_improvements_list(struct player *pplayer,
   } city_built_iterate_end;
 
   gtk_list_store_append(store, &it);
-  gtk_list_store_set(store, &it, 0, _("At Spy's Discretion"), 1, B_LAST, -1);
+  {
+    struct astring str = ASTRING_INIT;
+    /* TRANS: %s is a unit name, e.g., Spy */
+    astr_set(&str, _("At %s's Discretion"),
+             unit_name_translation(game_unit_by_number(diplomat_id)));
+    gtk_list_store_set(store, &it, 0, astr_str(&str), 1, B_LAST, -1);
+    astr_free(&str);
+  }
 
   gtk_dialog_set_response_sensitive(GTK_DIALOG(spy_sabotage_shell),
     GTK_RESPONSE_ACCEPT, FALSE);
   
-  gtk_widget_show_all(GTK_DIALOG(spy_sabotage_shell)->vbox);
+  gtk_widget_show_all(gtk_dialog_get_content_area(GTK_DIALOG(spy_sabotage_shell)));
 
   g_signal_connect(gtk_tree_view_get_selection(GTK_TREE_VIEW(view)), "changed",
 		   G_CALLBACK(spy_improvements_callback), NULL);
@@ -469,7 +494,7 @@ static void create_improvements_list(struct player *pplayer,
 }
 
 /****************************************************************
-...
+  Popup tech stealing dialog with list of possible techs
 *****************************************************************/
 static void spy_steal_popup(GtkWidget *w, gpointer data)
 {
@@ -530,7 +555,7 @@ static void diplomat_incite_callback(GtkWidget *w, gpointer data)
 }
 
 /****************************************************************
-...
+  User has responded to incite dialog
 *****************************************************************/
 static void incite_response(GtkWidget *w, gint response)
 {
@@ -605,7 +630,7 @@ static void diplomat_keep_moving_callback(GtkWidget *w, gpointer data)
 }
 
 /****************************************************************
-...
+  Diplomat dialog has been destoryed
 *****************************************************************/
 static void diplomat_destroy_callback(GtkWidget *w, gpointer data)
 {
@@ -614,7 +639,7 @@ static void diplomat_destroy_callback(GtkWidget *w, gpointer data)
 }
 
 /****************************************************************
-...
+  Diplomat dialog has been canceled
 *****************************************************************/
 static void diplomat_cancel_callback(GtkWidget *w, gpointer data)
 {
@@ -622,14 +647,14 @@ static void diplomat_cancel_callback(GtkWidget *w, gpointer data)
 }
 
 /****************************************************************
-...
+ Popup new diplomat dialog.
 *****************************************************************/
 void popup_diplomat_dialog(struct unit *punit, struct tile *dest_tile)
 {
   struct city *pcity;
   struct unit *ptunit;
   GtkWidget *shl;
-  char buf[128];
+  struct astring title = ASTRING_INIT, text = ASTRING_INIT;
 
   diplomat_id = punit->id;
 
@@ -637,14 +662,17 @@ void popup_diplomat_dialog(struct unit *punit, struct tile *dest_tile)
     /* Spy/Diplomat acting against a city */
 
     diplomat_target_id = pcity->id;
-    fc_snprintf(buf, sizeof(buf),
-                _("Your %s has arrived at %s.\nWhat is your command?"),
-                unit_name_translation(punit),
-                city_name(pcity));
+    astr_set(&title,
+             /* TRANS: %s is a unit name, e.g., Spy */
+             _("Choose Your %s's Strategy"), unit_name_translation(punit));
+    astr_set(&text, 
+             _("Your %s has arrived at %s.\nWhat is your command?"),
+             unit_name_translation(punit),
+             city_name(pcity));
 
     if (!unit_has_type_flag(punit, F_SPY)){
       shl = popup_choice_dialog(GTK_WINDOW(toplevel),
-	_("Choose Your Diplomat's Strategy"), buf,
+        astr_str(&title), astr_str(&text),
 	_("Establish _Embassy"), diplomat_embassy_callback, NULL,
 	_("_Investigate City"), diplomat_investigate_callback, NULL,
 	_("_Sabotage City"), diplomat_sabotage_callback, NULL,
@@ -668,7 +696,7 @@ void popup_diplomat_dialog(struct unit *punit, struct tile *dest_tile)
 	choice_dialog_button_set_sensitive(shl, 5, FALSE);
     } else {
        shl = popup_choice_dialog(GTK_WINDOW(toplevel),
-	_("Choose Your Spy's Strategy"), buf,
+        astr_str(&title), astr_str(&text),
 	_("Establish _Embassy"), diplomat_embassy_callback, NULL,
 	_("_Investigate City"), diplomat_investigate_callback, NULL,
 	_("_Poison City"), spy_poison_callback, NULL,
@@ -707,12 +735,13 @@ void popup_diplomat_dialog(struct unit *punit, struct tile *dest_tile)
       /* Spy/Diplomat acting against a unit */ 
        
       diplomat_target_id = ptunit->id;
+      astr_set(&text,
+               /* TRANS: %s is a unit name, e.g., Diplomat, Spy */
+               _("Your %s is waiting for your command."),
+               unit_name_translation(punit));
  
       shl = popup_choice_dialog(GTK_WINDOW(toplevel),
-	_("Subvert Enemy Unit"),
-	(!unit_has_type_flag(punit, F_SPY))?
-	_("The diplomat is waiting for your command"):
-	_("The spy is waiting for your command"),
+	_("Subvert Enemy Unit"), astr_str(&text),
 	_("_Bribe Enemy Unit"), diplomat_bribe_callback, NULL,
 	_("_Sabotage Enemy Unit"), spy_sabotage_unit_callback, NULL,
 	GTK_STOCK_CANCEL, diplomat_cancel_callback, NULL,
@@ -734,6 +763,8 @@ void popup_diplomat_dialog(struct unit *punit, struct tile *dest_tile)
 		       G_CALLBACK(diplomat_cancel_callback), NULL);
     }
   }
+  astr_free(&title);
+  astr_free(&text);
 }
 
 /****************************************************************
