@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <stdio.h>
@@ -39,7 +39,7 @@
 #include "client_main.h"
 #include "options.h"
 
-/* gui-gtk-2.0 */
+/* client/gui-gtk-3.0 */
 #include "chatline.h"
 #include "cityrep.h"
 #include "dialogs.h"
@@ -54,7 +54,7 @@ static GtkWidget *rates_dialog_shell;
 static GtkWidget *rates_gov_label;
 static GtkWidget *rates_tax_toggle, *rates_lux_toggle, *rates_sci_toggle;
 static GtkWidget *rates_tax_label, *rates_lux_label, *rates_sci_label;
-static GtkObject *rates_tax_adj, *rates_lux_adj, *rates_sci_adj;
+static GtkWidget *rates_tax_scale, *rates_lux_scale, *rates_sci_scale;
 
 static gulong     rates_tax_sig, rates_lux_sig, rates_sci_sig;
 /******************************************************************/
@@ -62,11 +62,11 @@ static gulong     rates_tax_sig, rates_lux_sig, rates_sci_sig;
 static int rates_tax_value, rates_lux_value, rates_sci_value;
 
 
-static void rates_changed_callback(GtkAdjustment *adj);
+static void rates_changed_callback(GtkWidget *range);
 
 
 /**************************************************************************
-...
+  Set tax values to display
 **************************************************************************/
 static void rates_set_values(int tax, int no_tax_scroll, 
 			     int lux, int no_lux_scroll,
@@ -76,9 +76,9 @@ static void rates_set_values(int tax, int no_tax_scroll,
   int tax_lock, lux_lock, sci_lock;
   int maxrate;
   
-  tax_lock	= GTK_TOGGLE_BUTTON(rates_tax_toggle)->active;
-  lux_lock	= GTK_TOGGLE_BUTTON(rates_lux_toggle)->active;
-  sci_lock	= GTK_TOGGLE_BUTTON(rates_sci_toggle)->active;
+  tax_lock	= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rates_tax_toggle));
+  lux_lock	= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rates_lux_toggle));
+  sci_lock	= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rates_sci_toggle));
 
   if (NULL != client.conn.playing) {
     maxrate = get_player_bonus(client.conn.playing, EFT_MAX_RATES);
@@ -132,39 +132,39 @@ static void rates_set_values(int tax, int no_tax_scroll,
 
   if (tax!=rates_tax_value) {
     fc_snprintf(buf, sizeof(buf), "%3d%%", tax);
-    if (strcmp(buf, GTK_LABEL(rates_tax_label)->label) != 0)
+    if (strcmp(buf, gtk_label_get_text(GTK_LABEL(rates_tax_label))) != 0)
 	gtk_label_set_text(GTK_LABEL(rates_tax_label), buf);
     if(!no_tax_scroll)
     {
-	g_signal_handler_block(rates_tax_adj, rates_tax_sig);
-	gtk_adjustment_set_value(GTK_ADJUSTMENT(rates_tax_adj), tax/10 );
-	g_signal_handler_unblock(rates_tax_adj, rates_tax_sig);
+	g_signal_handler_block(rates_tax_scale, rates_tax_sig);
+	gtk_range_set_value(GTK_RANGE(rates_tax_scale), tax/10 );
+	g_signal_handler_unblock(rates_tax_scale, rates_tax_sig);
     }
     rates_tax_value=tax;
   }
 
   if(lux!=rates_lux_value) {
     fc_snprintf(buf, sizeof(buf), "%3d%%", lux);
-    if (strcmp(buf, GTK_LABEL(rates_lux_label)->label) != 0)
+    if (strcmp(buf, gtk_label_get_text(GTK_LABEL(rates_lux_label))) != 0)
 	gtk_label_set_text(GTK_LABEL(rates_lux_label), buf);
     if(!no_lux_scroll)
     {
-	g_signal_handler_block(rates_lux_adj, rates_lux_sig);
-	gtk_adjustment_set_value(GTK_ADJUSTMENT(rates_lux_adj), lux/10 );
-	g_signal_handler_unblock(rates_lux_adj, rates_lux_sig);
+	g_signal_handler_block(rates_lux_scale, rates_lux_sig);
+	gtk_range_set_value(GTK_RANGE(rates_lux_scale), lux/10 );
+	g_signal_handler_unblock(rates_lux_scale, rates_lux_sig);
     }
     rates_lux_value=lux;
   }
 
   if(sci!=rates_sci_value) {
     fc_snprintf(buf, sizeof(buf), "%3d%%", sci);
-    if (strcmp(buf, GTK_LABEL(rates_sci_label)->label) != 0)
+    if (strcmp(buf, gtk_label_get_text(GTK_LABEL(rates_sci_label))) != 0)
 	gtk_label_set_text(GTK_LABEL(rates_sci_label),buf);
     if(!no_sci_scroll)
     {
-	g_signal_handler_block(rates_sci_adj, rates_sci_sig);
-	gtk_adjustment_set_value(GTK_ADJUSTMENT(rates_sci_adj), sci/10 );
-	g_signal_handler_unblock(rates_sci_adj, rates_sci_sig);
+	g_signal_handler_block(rates_sci_scale, rates_sci_sig);
+	gtk_range_set_value(GTK_RANGE(rates_sci_scale), sci/10 );
+	g_signal_handler_unblock(rates_sci_scale, rates_sci_sig);
     }
     rates_sci_value=sci;
   }
@@ -172,20 +172,20 @@ static void rates_set_values(int tax, int no_tax_scroll,
 
 
 /**************************************************************************
-...
+  User changes rates
 **************************************************************************/
-static void rates_changed_callback(GtkAdjustment *adj)
+static void rates_changed_callback(GtkWidget *range)
 {
-  int percent=adj->value;
+  int percent=gtk_range_get_value(GTK_RANGE(range));
 
-  if(adj==GTK_ADJUSTMENT(rates_tax_adj)) {
+  if(range==rates_tax_scale) {
     int tax_value;
 
     tax_value=10*percent;
     tax_value=MIN(tax_value, 100);
     rates_set_values(tax_value,1, rates_lux_value,0, rates_sci_value,0);
   }
-  else if(adj==GTK_ADJUSTMENT(rates_lux_adj)) {
+  else if(range==rates_lux_scale) {
     int lux_value;
 
     lux_value=10*percent;
@@ -203,7 +203,7 @@ static void rates_changed_callback(GtkAdjustment *adj)
 
 
 /**************************************************************************
-...
+  User has responded to rates dialog
 **************************************************************************/
 static void rates_command_callback(GtkWidget *w, gint response_id)
 {
@@ -216,14 +216,13 @@ static void rates_command_callback(GtkWidget *w, gint response_id)
 
 
 /****************************************************************
-... 
+  Create rates dialog
 *****************************************************************/
 static GtkWidget *create_rates_dialog(void)
 {
-  GtkWidget     *shell;
-  GtkWidget	*frame, *hbox;
-
-  GtkWidget	*scale;
+  GtkWidget     *shell, *content;
+  GtkWidget	*frame, *hgrid;
+  int i;
 
   if (!can_client_issue_orders()) {
     return NULL;
@@ -240,69 +239,85 @@ static GtkWidget *create_rates_dialog(void)
   setup_dialog(shell, toplevel);
   gtk_dialog_set_default_response(GTK_DIALOG(shell), GTK_RESPONSE_OK);
   gtk_window_set_position(GTK_WINDOW(shell), GTK_WIN_POS_MOUSE);
+  content = gtk_dialog_get_content_area(GTK_DIALOG(shell));
 
   rates_gov_label = gtk_label_new("");
-  gtk_box_pack_start( GTK_BOX( GTK_DIALOG( shell )->vbox ), rates_gov_label, TRUE, TRUE, 5 );
+  gtk_box_pack_start( GTK_BOX( content ), rates_gov_label, TRUE, TRUE, 5 );
 
   frame = gtk_frame_new( _("Tax") );
-  gtk_box_pack_start( GTK_BOX( GTK_DIALOG( shell )->vbox ), frame, TRUE, TRUE, 5 );
+  gtk_box_pack_start( GTK_BOX( content ), frame, TRUE, TRUE, 5 );
 
-  hbox = gtk_hbox_new( FALSE, 10 );
-  gtk_container_add( GTK_CONTAINER( frame ), hbox );
+  hgrid = gtk_grid_new();
+  gtk_grid_set_column_spacing(GTK_GRID(hgrid), 10);
+  gtk_container_add(GTK_CONTAINER(frame), hgrid);
 
-  rates_tax_adj = gtk_adjustment_new( 0.0, 0.0, 11.0, 1.0, 1.0, 1.0 );
-  scale = gtk_hscale_new( GTK_ADJUSTMENT( rates_tax_adj ) );
-  gtk_widget_set_size_request(scale, 300, 40);
-  gtk_scale_set_digits( GTK_SCALE( scale ), 0 );
-  gtk_scale_set_draw_value( GTK_SCALE( scale ), FALSE );
-  gtk_box_pack_start( GTK_BOX( hbox ), scale, TRUE, TRUE, 0 );
+  rates_tax_scale =
+    gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, 0, 10, 1);
+  gtk_range_set_increments(GTK_RANGE(rates_tax_scale), 1, 1);
+  for (i = 0; i <= 10; i++) {
+    gtk_scale_add_mark(GTK_SCALE(rates_tax_scale), i, GTK_POS_TOP, NULL);
+  }
+  gtk_widget_set_size_request(rates_tax_scale, 300, 40);
+  gtk_scale_set_digits(GTK_SCALE(rates_tax_scale), 0);
+  gtk_scale_set_draw_value(GTK_SCALE(rates_tax_scale), FALSE);
+  gtk_container_add(GTK_CONTAINER(hgrid), rates_tax_scale);
 
   rates_tax_label = gtk_label_new("  0%");
-  gtk_box_pack_start( GTK_BOX( hbox ), rates_tax_label, TRUE, TRUE, 0 );
+  gtk_container_add(GTK_CONTAINER(hgrid), rates_tax_label);
   gtk_widget_set_size_request(rates_tax_label, 40, -1);
 
   rates_tax_toggle = gtk_check_button_new_with_label( _("Lock") );
-  gtk_box_pack_start( GTK_BOX( hbox ), rates_tax_toggle, TRUE, TRUE, 0 );
+  gtk_container_add(GTK_CONTAINER(hgrid), rates_tax_toggle);
 
   frame = gtk_frame_new( _("Luxury") );
-  gtk_box_pack_start( GTK_BOX( GTK_DIALOG( shell )->vbox ), frame, TRUE, TRUE, 5 );
+  gtk_box_pack_start( GTK_BOX( content ), frame, TRUE, TRUE, 5 );
 
-  hbox = gtk_hbox_new( FALSE, 10 );
-  gtk_container_add( GTK_CONTAINER( frame ), hbox );
+  hgrid = gtk_grid_new();
+  gtk_grid_set_column_spacing(GTK_GRID(hgrid), 10);
+  gtk_container_add(GTK_CONTAINER(frame), hgrid);
 
-  rates_lux_adj = gtk_adjustment_new( 0.0, 0.0, 11.0, 1.0, 1.0, 1.0 );
-  scale = gtk_hscale_new( GTK_ADJUSTMENT( rates_lux_adj ) );
-  gtk_widget_set_size_request(scale, 300, 40);
-  gtk_scale_set_digits( GTK_SCALE( scale ), 0 );
-  gtk_scale_set_draw_value( GTK_SCALE( scale ), FALSE );
-  gtk_box_pack_start( GTK_BOX( hbox ), scale, TRUE, TRUE, 0 );
+  rates_lux_scale =
+    gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, 0, 10, 1);
+  gtk_range_set_increments(GTK_RANGE(rates_lux_scale), 1, 1);
+  for (i = 0; i <= 10; i++) {
+    gtk_scale_add_mark(GTK_SCALE(rates_lux_scale), i, GTK_POS_TOP, NULL);
+  }
+  gtk_widget_set_size_request(rates_lux_scale, 300, 40);
+  gtk_scale_set_digits(GTK_SCALE(rates_lux_scale), 0);
+  gtk_scale_set_draw_value(GTK_SCALE(rates_lux_scale), FALSE);
+  gtk_container_add(GTK_CONTAINER(hgrid), rates_lux_scale);
 
   rates_lux_label = gtk_label_new("  0%");
-  gtk_box_pack_start( GTK_BOX( hbox ), rates_lux_label, TRUE, TRUE, 0 );
+  gtk_container_add(GTK_CONTAINER(hgrid), rates_lux_label);
   gtk_widget_set_size_request(rates_lux_label, 40, -1);
 
   rates_lux_toggle = gtk_check_button_new_with_label( _("Lock") );
-  gtk_box_pack_start( GTK_BOX( hbox ), rates_lux_toggle, TRUE, TRUE, 0 );
+  gtk_container_add(GTK_CONTAINER(hgrid), rates_lux_toggle);
 
   frame = gtk_frame_new( _("Science") );
-  gtk_box_pack_start( GTK_BOX( GTK_DIALOG( shell )->vbox ), frame, TRUE, TRUE, 5 );
+  gtk_box_pack_start( GTK_BOX( content ), frame, TRUE, TRUE, 5 );
 
-  hbox = gtk_hbox_new( FALSE, 10 );
-  gtk_container_add( GTK_CONTAINER( frame ), hbox );
+  hgrid = gtk_grid_new();
+  gtk_grid_set_column_spacing(GTK_GRID(hgrid), 10);
+  gtk_container_add(GTK_CONTAINER(frame), hgrid);
 
-  rates_sci_adj = gtk_adjustment_new( 0.0, 0.0, 11.0, 1.0, 1.0, 1.0 );
-  scale = gtk_hscale_new( GTK_ADJUSTMENT( rates_sci_adj ) );
-  gtk_widget_set_size_request(scale, 300, 40);
-  gtk_scale_set_digits( GTK_SCALE( scale ), 0 );
-  gtk_scale_set_draw_value( GTK_SCALE( scale ), FALSE );
-  gtk_box_pack_start( GTK_BOX( hbox ), scale, TRUE, TRUE, 0 );
+  rates_sci_scale =
+    gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, 0, 10, 1);
+  gtk_range_set_increments(GTK_RANGE(rates_sci_scale), 1, 1);
+  for (i = 0; i <= 10; i++) {
+    gtk_scale_add_mark(GTK_SCALE(rates_sci_scale), i, GTK_POS_TOP, NULL);
+  }
+  gtk_widget_set_size_request(rates_sci_scale, 300, 40);
+  gtk_scale_set_digits(GTK_SCALE(rates_sci_scale), 0);
+  gtk_scale_set_draw_value(GTK_SCALE(rates_sci_scale), FALSE);
+  gtk_container_add(GTK_CONTAINER(hgrid), rates_sci_scale);
 
   rates_sci_label = gtk_label_new("  0%");
-  gtk_box_pack_start( GTK_BOX( hbox ), rates_sci_label, TRUE, TRUE, 0 );
+  gtk_container_add(GTK_CONTAINER(hgrid), rates_sci_label);
   gtk_widget_set_size_request(rates_sci_label, 40, -1);
 
   rates_sci_toggle = gtk_check_button_new_with_label( _("Lock") );
-  gtk_box_pack_start( GTK_BOX( hbox ), rates_sci_toggle, TRUE, TRUE, 0 );
+  gtk_container_add(GTK_CONTAINER(hgrid), rates_sci_toggle);
 
 
   g_signal_connect(shell, "response",
@@ -310,23 +325,23 @@ static GtkWidget *create_rates_dialog(void)
   g_signal_connect(shell, "destroy",
 		   G_CALLBACK(gtk_widget_destroyed), &rates_dialog_shell);
 
-  gtk_widget_show_all( GTK_DIALOG( shell )->vbox );
-  gtk_widget_show_all( GTK_DIALOG( shell )->action_area );
+  gtk_widget_show_all(content);
+  gtk_widget_show_all(gtk_dialog_get_action_area(GTK_DIALOG(shell)));
 
   rates_tax_value=-1;
   rates_lux_value=-1;
   rates_sci_value=-1;
 
   rates_tax_sig =
-    g_signal_connect_after(rates_tax_adj, "value_changed",
+    g_signal_connect_after(rates_tax_scale, "value-changed",
 			   G_CALLBACK(rates_changed_callback), NULL);
 
   rates_lux_sig =
-    g_signal_connect_after(rates_lux_adj, "value_changed",
+    g_signal_connect_after(rates_lux_scale, "value-changed",
 			   G_CALLBACK(rates_changed_callback), NULL);
 
   rates_sci_sig =
-    g_signal_connect_after(rates_sci_adj, "value_changed",
+    g_signal_connect_after(rates_sci_scale, "value-changed",
 			   G_CALLBACK(rates_changed_callback), NULL);
 
   rates_set_values(client.conn.playing->economic.tax, 0,
@@ -337,7 +352,7 @@ static GtkWidget *create_rates_dialog(void)
 
 
 /****************************************************************
-... 
+  Popup rates dialog
 *****************************************************************/
 void popup_rates_dialog(void)
 {
@@ -358,6 +373,15 @@ void popup_rates_dialog(void)
       government_name_for_player(client.conn.playing),
       get_player_bonus(client.conn.playing, EFT_MAX_RATES));
   gtk_label_set_text(GTK_LABEL(rates_gov_label), buf);
-  
+  gtk_range_set_fill_level(GTK_RANGE(rates_tax_scale),
+                           get_player_bonus(client.conn.playing,
+                                            EFT_MAX_RATES)/10);
+  gtk_range_set_fill_level(GTK_RANGE(rates_lux_scale),
+                           get_player_bonus(client.conn.playing,
+                                            EFT_MAX_RATES)/10);
+  gtk_range_set_fill_level(GTK_RANGE(rates_sci_scale),
+                           get_player_bonus(client.conn.playing,
+                                            EFT_MAX_RATES)/10);
+
   gtk_window_present(GTK_WINDOW(rates_dialog_shell));
 }
