@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <gtk/gtk.h>
@@ -35,7 +35,7 @@ static void tileset_suggestion_callback(GtkWidget *dlg, gint arg)
 {
   if (arg == GTK_RESPONSE_YES) {
     /* User accepted tileset loading */
-    tilespec_reread(game.control.prefered_tileset);
+    tilespec_reread(game.control.prefered_tileset, FALSE);
   }
 }
 
@@ -66,7 +66,7 @@ void popup_tileset_suggestion_dialog(void)
           game.control.prefered_tileset, tileset_get_name(tileset));
 
   label = gtk_label_new(buf);
-  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), label);
+  gtk_container_add(GTK_CONTAINER(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), label);
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_CENTER);
   gtk_widget_show(label);
 
