@@ -1,6 +1,6 @@
 /*
     SDL_ttf:  A companion library to SDL for working with TrueType (tm) fonts
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997-2004 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -14,14 +14,12 @@
 
     You should have received a copy of the GNU Library General Public
     License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
     slouken@libsdl.org
 */
 
-/* $Id: SDL_ttf.c 7167 2004-01-05 17:55:18Z jdorje $ */
-
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -42,21 +40,11 @@
 #include <ft2build.h>
 #include FT_FREETYPE_H
 #include FT_OUTLINE_H
-#include FT_INTERNAL_OBJECTS_H
+#include FT_TRUETYPE_IDS_H
 
 #include "SDL.h"
 #include "SDL_endian.h"
 #include "SDL_ttf.h"
-#include "graphics.h"
-
-#ifdef HAVE_MMX1
-#include "mmx.h"
-#endif
-
-/* fix freetype header bug */
-#ifndef FT_OPEN_STREAM
-#define FT_OPEN_STREAM	ft_open_stream
-#endif
 
 /* FIXME: Right now we assume the gray-scale renderer Freetype is using
    supports 256 shades of gray, but we should instead key off of num_grays
@@ -71,6 +59,13 @@
 #define CACHED_BITMAP	0x01
 #define CACHED_PIXMAP	0x02
 
+/*
+ * Allows building with early libfreetype 2.1.x (x < 3)
+ */
+#ifndef FT_PIXEL_MODE_MONO
+#define FT_PIXEL_MODE_MONO ft_pixel_mode_mono
+#endif
+
 /* Cached glyph information */
 typedef struct cached_glyph {
 	int stored;
@@ -159,7 +154,7 @@ static __inline__ void UNICODE_strcpy(Uint16 *dst, const Uint16 *src, int swap)
 const SDL_version *TTF_Linked_Version(void)
 {
 	static SDL_version linked_version;
-	TTF_VERSION(&linked_version);
+	SDL_TTF_VERSION(&linked_version);
 	return(&linked_version);
 }
 
@@ -275,7 +270,7 @@ TTF_Font* TTF_OpenFontIndexRW( SDL_RWops *src, int freesrc, int ptsize, long ind
 	}
 	memset(stream, 0, sizeof(*stream));
 
-	stream->memory = library->memory;
+	stream->memory = NULL;  /* set by FT_Open_Face */
 	stream->read = RWread;
 	stream->descriptor.pointer = src;
 	stream->pos = (unsigned long)position;
@@ -297,18 +292,23 @@ TTF_Font* TTF_OpenFontIndexRW( SDL_RWops *src, int freesrc, int ptsize, long ind
 	/* Make sure that our font face is scalable (global metrics) */
 	if ( FT_IS_SCALABLE(face) ) {
 
-	  	/* Set the character size and use default DPI (72) */
-	  	error = FT_Set_Char_Size( font->face, 0, ptsize * 64, 0, 0 );
+		/* Set the character size and use default DPI (72) */
+		error = FT_Set_Char_Size( font->face, 0, ptsize * 64, 0, 0 );
 			if( error ) {
-	    	TTF_SetFTError( "Couldn't set font size", error );
-	    	TTF_CloseFont( font );
-	    	return NULL;
+		TTF_SetFTError( "Couldn't set font size", error );
+		TTF_CloseFont( font );
+		return NULL;
 	  }
 
 	  /* Get the scalable font metrics for this font */
 	  scale = face->size->metrics.y_scale;
-	  font->ascent  = FT_CEIL(FT_MulFix(face->bbox.yMax, scale));
-	  font->descent = FT_CEIL(FT_MulFix(face->bbox.yMin, scale));
+	  /* This broke with fonts containing large boxes that were not
+	   * actually used in the output: */
+	  /* font->ascent  = FT_CEIL(FT_MulFix(face->bbox.yMax, scale)); */
+	  /* font->descent = FT_CEIL(FT_MulFix(face->bbox.yMin, scale)); */
+	  /* fix suggested by Stepan Roh: */
+	  font->ascent  = FT_CEIL(FT_MulFix(face->ascender, scale));
+	  font->descent = FT_CEIL(FT_MulFix(face->descender, scale));
 	  font->height  = font->ascent - font->descent + /* baseline */ 1;
 	  font->lineskip = FT_CEIL(FT_MulFix(face->height, scale));
 	  font->underline_offset = FT_FLOOR(FT_MulFix(face->underline_position, scale));
@@ -325,17 +325,17 @@ TTF_Font* TTF_OpenFontIndexRW( SDL_RWops *src, int freesrc, int ptsize, long ind
 		error = FT_Set_Pixel_Sizes( face,
 				face->available_sizes[ptsize].height,
 				face->available_sizes[ptsize].width );
-	  	/* With non-scalale fonts, Freetype2 likes to fill many of the
+		/* With non-scalale fonts, Freetype2 likes to fill many of the
 		 * font metrics with the value of 0.  The size of the
 		 * non-scalable fonts must be determined differently
 		 * or sometimes cannot be determined.
 		 * */
-	  	font->ascent = face->available_sizes[ptsize].height;
-	  	font->descent = 0;
-	  	font->height = face->available_sizes[ptsize].height;
-	  	font->lineskip = FT_CEIL(font->ascent);
-	  	font->underline_offset = FT_FLOOR(face->underline_position);
-	  	font->underline_height = FT_FLOOR(face->underline_thickness);
+		font->ascent = face->available_sizes[ptsize].height;
+		font->descent = 0;
+		font->height = face->available_sizes[ptsize].height;
+		font->lineskip = FT_CEIL(font->ascent);
+		font->underline_offset = FT_FLOOR(face->underline_position);
+		font->underline_height = FT_FLOOR(face->underline_thickness);
 	}
 
 	if ( font->underline_height < 1 ) {
@@ -370,7 +370,7 @@ TTF_Font* TTF_OpenFontRW( SDL_RWops *src, int freesrc, int ptsize )
 TTF_Font* TTF_OpenFontIndex( const char *file, int ptsize, long index )
 {
 	SDL_RWops *rw = SDL_RWFromFile(file, "rb");
-	if(!rw) {
+	if ( rw == NULL ) {
 		TTF_SetError(SDL_GetError());
 		return NULL;
 	}
@@ -396,7 +396,7 @@ static void Flush_Glyph( c_glyph* glyph )
 	}
 	glyph->cached = 0;
 }
-	
+
 static void Flush_Cache( TTF_Font* font )
 {
 	int i;
@@ -466,7 +466,7 @@ static FT_Error Load_Glyph( TTF_Font* font, Uint16 ch, c_glyph* cached, int want
 			cached->yoffset = 0;
 			cached->advance = FT_CEIL(metrics->horiAdvance);
 		}
-		
+
 		/* Adjust for bold and italic text */
 		if( font->style & TTF_STYLE_BOLD ) {
 			cached->maxx += font->glyph_overhang;
@@ -521,7 +521,7 @@ static FT_Error Load_Glyph( TTF_Font* font, Uint16 ch, c_glyph* cached, int want
 		 * a freetype2 bug because it is inconsistent with the
 		 * freetype2 documentation under FT_Render_Mode section.
 		 * */
-		if ( mono || !FT_IS_SCALABLE(face) ) {
+		if ( mono || !FT_IS_SCALABLE(face) || dst->pixel_mode == FT_PIXEL_MODE_MONO ) {
 			dst->pitch *= 8;
 		}
 
@@ -538,7 +538,7 @@ static FT_Error Load_Glyph( TTF_Font* font, Uint16 ch, c_glyph* cached, int want
 		}
 
 		if (dst->rows != 0) {
-			dst->buffer = malloc( dst->pitch * dst->rows );
+			dst->buffer = (unsigned char *)malloc( dst->pitch * dst->rows );
 			if( !dst->buffer ) {
 				return FT_Err_Out_Of_Memory;
 			}
@@ -547,27 +547,17 @@ static FT_Error Load_Glyph( TTF_Font* font, Uint16 ch, c_glyph* cached, int want
 			for( i = 0; i < src->rows; i++ ) {
 				int soffset = i * src->pitch;
 				int doffset = i * dst->pitch;
-				if ( mono ) {
+				if ( mono || dst->pixel_mode == FT_PIXEL_MODE_MONO ) {
 					unsigned char *srcp = src->buffer + soffset;
 					unsigned char *dstp = dst->buffer + doffset;
 					int j;
 					for ( j = 0; j < src->width; j += 8 ) {
 						unsigned char ch = *srcp++;
-						*dstp++ = (ch&0x80) >> 7;
-						ch <<= 1;
-						*dstp++ = (ch&0x80) >> 7;
-						ch <<= 1;
-						*dstp++ = (ch&0x80) >> 7;
-						ch <<= 1;
-						*dstp++ = (ch&0x80) >> 7;
-						ch <<= 1;
-						*dstp++ = (ch&0x80) >> 7;
-						ch <<= 1;
-						*dstp++ = (ch&0x80) >> 7;
-						ch <<= 1;
-						*dstp++ = (ch&0x80) >> 7;
-						ch <<= 1;
-						*dstp++ = (ch&0x80) >> 7;
+						int k;
+						for ( k = 0; k < 8; ++k) {
+							*dstp++ = ch & 0x80 ? 0xff : 0;
+							ch <<= 1;
+						}
 					}
 				} else if ( !FT_IS_SCALABLE(face) ) {
 					/* This special case wouldn't
@@ -577,7 +567,7 @@ static FT_Error Load_Glyph( TTF_Font* font, Uint16 ch, c_glyph* cached, int want
 					 * shades of gray.  Instead, it
 					 * returns a black and white surface
 					 * and we have to translate it back
-					 * to a 256 gray shaded surface. 
+					 * to a 256 gray shaded surface.
 					 * */
 					unsigned char *srcp = src->buffer + soffset;
 					unsigned char *dstp = dst->buffer + doffset;
@@ -697,12 +687,12 @@ static Uint16 *UTF8_to_UNICODE(Uint16 *unicode, const char *utf8, int len)
 			ch |=  (Uint16)(utf8[++i]&0x3F);
 		} else
 		if ( ch >= 0xE0 ) {
-			ch  =  (Uint16)(utf8[i]&0x3F) << 12;
+			ch  =  (Uint16)(utf8[i]&0x0F) << 12;
 			ch |=  (Uint16)(utf8[++i]&0x3F) << 6;
 			ch |=  (Uint16)(utf8[++i]&0x3F);
 		} else
 		if ( ch >= 0xC0 ) {
-			ch  =  (Uint16)(utf8[i]&0x3F) << 6;
+			ch  =  (Uint16)(utf8[i]&0x1F) << 6;
 			ch |=  (Uint16)(utf8[++i]&0x3F);
 		}
 		unicode[j] = ch;
@@ -719,7 +709,7 @@ int TTF_FontHeight(TTF_Font *font)
 
 int TTF_FontAscent(TTF_Font *font)
 {
-       return(font->ascent);
+	return(font->ascent);
 }
 
 int TTF_FontDescent(TTF_Font *font)
@@ -768,6 +758,9 @@ int TTF_GlyphMetrics(TTF_Font *font, Uint16 ch,
 	}
 	if ( maxx ) {
 		*maxx = font->current->maxx;
+		if( font->style & TTF_STYLE_BOLD ) {
+			*maxx += font->glyph_overhang;
+		}
 	}
 	if ( miny ) {
 		*miny = font->current->miny;
@@ -777,7 +770,7 @@ int TTF_GlyphMetrics(TTF_Font *font, Uint16 ch,
 	}
 	if ( advance ) {
 		*advance = font->current->advance;
-	  	if ( font->style & TTF_STYLE_BOLD ) {
+		if( font->style & TTF_STYLE_BOLD ) {
 			*advance += font->glyph_overhang;
 		}
 	}
@@ -842,6 +835,8 @@ int TTF_SizeUNICODE(TTF_Font *font, const Uint16 *text, int *w, int *h)
 	int miny, maxy;
 	c_glyph *glyph;
 	FT_Error error;
+	FT_Long use_kerning;
+	FT_UInt prev_index = 0;
 
 	/* Initialize everything to 0 */
 	if ( ! TTF_initialized ) {
@@ -853,6 +848,9 @@ int TTF_SizeUNICODE(TTF_Font *font, const Uint16 *text, int *w, int *h)
 	miny = maxy = 0;
 	swapped = TTF_byteswapped;
 
+	/* check kerning */
+	use_kerning = FT_HAS_KERNING( font->face );
+
 	/* Load each character and sum it's bounding box */
 	x= 0;
 	for ( ch=text; *ch; ++ch ) {
@@ -881,6 +879,14 @@ int TTF_SizeUNICODE(TTF_Font *font, const Uint16 *text, int *w, int *h)
 		}
 		glyph = font->current;
 
+		/* handle kerning */
+		if ( use_kerning && prev_index && glyph->index ) {
+			FT_Vector delta;
+			FT_Get_Kerning( font->face, prev_index, glyph->index, ft_kerning_default, &delta );
+			x += delta.x >> 6;
+		}
+
+#if 0
 		if ( (ch == text) && (glyph->minx < 0) ) {
 		/* Fixes the texture wrapping bug when the first letter
 		 * has a negative minx value or horibearing value.  The entire
@@ -896,9 +902,10 @@ int TTF_SizeUNICODE(TTF_Font *font, const Uint16 *text, int *w, int *h)
 		 * work out correctly.
 		 * */
 			z -= glyph->minx;
-			
+
 		}
-		
+#endif
+
 		z = x + glyph->minx;
 		if ( minx > z ) {
 			minx = z;
@@ -922,6 +929,7 @@ int TTF_SizeUNICODE(TTF_Font *font, const Uint16 *text, int *w, int *h)
 		if ( glyph->maxy > maxy ) {
 			maxy = glyph->maxy;
 		}
+		prev_index = glyph->index;
 	}
 
 	/* Fill the bounds rectangle */
@@ -1003,11 +1011,15 @@ SDL_Surface *TTF_RenderUNICODE_Solid(TTF_Font *font,
 	const Uint16* ch;
 	Uint8* src;
 	Uint8* dst;
+	Uint8 *dst_check;
 	int swapped;
 	int row, col;
 	c_glyph *glyph;
+
 	FT_Bitmap *current;
 	FT_Error error;
+	FT_Long use_kerning;
+	FT_UInt prev_index = 0;
 
 	/* Get the dimensions of the text surface */
 	if( ( TTF_SizeUNICODE(font, text, &width, NULL) < 0 ) || !width ) {
@@ -1022,6 +1034,10 @@ SDL_Surface *TTF_RenderUNICODE_Solid(TTF_Font *font,
 		return NULL;
 	}
 
+	/* Adding bound checking to avoid all kinds of memory corruption errors
+	   that may occur. */
+	dst_check = (Uint8*)textbuf->pixels + textbuf->pitch * textbuf->h;
+
 	/* Fill the palette with the foreground color */
 	palette = textbuf->format->palette;
 	palette->colors[0].r = 255 - fg.r;
@@ -1032,6 +1048,9 @@ SDL_Surface *TTF_RenderUNICODE_Solid(TTF_Font *font,
 	palette->colors[1].b = fg.b;
 	SDL_SetColorKey( textbuf, SDL_SRCCOLORKEY, 0 );
 
+	/* check kerning */
+	use_kerning = FT_HAS_KERNING( font->face );
+
 	/* Load and render each character */
 	xstart = 0;
 	swapped = TTF_byteswapped;
@@ -1062,13 +1081,29 @@ SDL_Surface *TTF_RenderUNICODE_Solid(TTF_Font *font,
 		}
 		glyph = font->current;
 		current = &glyph->bitmap;
+		/* Ensure the width of the pixmap is correct. On some cases,
+		 * freetype may report a larger pixmap than possible.*/
+		width = current->width;
+		if (width > glyph->maxx - glyph->minx) {
+			width = glyph->maxx - glyph->minx;
+		}
+		/* do kerning, if possible AC-Patch */
+		if ( use_kerning && prev_index && glyph->index ) {
+			FT_Vector delta;
+			FT_Get_Kerning( font->face, prev_index, glyph->index, ft_kerning_default, &delta );
+			xstart += delta.x >> 6;
+		}
 		/* Compensate for wrap around bug with negative minx's */
 		if ( (ch == text) && (glyph->minx < 0) ) {
 			xstart -= glyph->minx;
 		}
-		
+
 		for( row = 0; row < current->rows; ++row ) {
-			/* Make sure we don't go over the limit */
+			/* Make sure we don't go either over, or under the
+			 * limit */
+			if ( row+glyph->yoffset < 0 ) {
+				continue;
+			}
 			if ( row+glyph->yoffset >= textbuf->h ) {
 				continue;
 			}
@@ -1077,7 +1112,7 @@ SDL_Surface *TTF_RenderUNICODE_Solid(TTF_Font *font,
 				xstart + glyph->minx;
 			src = current->buffer + row * current->pitch;
 
-			for ( col=current->width; col>0; --col ) {
+			for ( col=width; col>0 && dst < dst_check; --col ) {
 				*dst++ |= *src++;
 			}
 		}
@@ -1086,6 +1121,7 @@ SDL_Surface *TTF_RenderUNICODE_Solid(TTF_Font *font,
 		if ( font->style & TTF_STYLE_BOLD ) {
 			xstart += font->glyph_overhang;
 		}
+		prev_index = glyph->index;
 	}
 
 	/* Handle the underline style */
@@ -1236,11 +1272,14 @@ SDL_Surface* TTF_RenderUNICODE_Shaded( TTF_Font* font,
 	const Uint16* ch;
 	Uint8* src;
 	Uint8* dst;
+	Uint8* dst_check;
 	int swapped;
 	int row, col;
 	FT_Bitmap* current;
 	c_glyph *glyph;
 	FT_Error error;
+	FT_Long use_kerning;
+	FT_UInt prev_index = 0;
 
 	/* Get the dimensions of the text surface */
 	if( ( TTF_SizeUNICODE(font, text, &width, NULL) < 0 ) || !width ) {
@@ -1255,6 +1294,10 @@ SDL_Surface* TTF_RenderUNICODE_Shaded( TTF_Font* font,
 		return NULL;
 	}
 
+	/* Adding bound checking to avoid all kinds of memory corruption errors
+	   that may occur. */
+	dst_check = (Uint8*)textbuf->pixels + textbuf->pitch * textbuf->h;
+
 	/* Fill the palette with NUM_GRAYS levels of shading from bg to fg */
 	palette = textbuf->format->palette;
 	rdiff = fg.r - bg.r;
@@ -1267,6 +1310,9 @@ SDL_Surface* TTF_RenderUNICODE_Shaded( TTF_Font* font,
 		palette->colors[index].b = bg.b + (index*bdiff) / (NUM_GRAYS-1);
 	}
 
+	/* check kerning */
+	use_kerning = FT_HAS_KERNING( font->face );
+
 	/* Load and render each character */
 	xstart = 0;
 	swapped = TTF_byteswapped;
@@ -1296,14 +1342,30 @@ SDL_Surface* TTF_RenderUNICODE_Shaded( TTF_Font* font,
 			return NULL;
 		}
 		glyph = font->current;
+		/* Ensure the width of the pixmap is correct. On some cases,
+		 * freetype may report a larger pixmap than possible.*/
+		width = glyph->pixmap.width;
+		if (width > glyph->maxx - glyph->minx) {
+			width = glyph->maxx - glyph->minx;
+		}
+		/* do kerning, if possible AC-Patch */
+		if ( use_kerning && prev_index && glyph->index ) {
+			FT_Vector delta;
+			FT_Get_Kerning( font->face, prev_index, glyph->index, ft_kerning_default, &delta );
+			xstart += delta.x >> 6;
+		}
 		/* Compensate for the wrap around with negative minx's */
 		if ( (ch == text) && (glyph->minx < 0) ) {
 			xstart -= glyph->minx;
 		}
-		
+
 		current = &glyph->pixmap;
 		for( row = 0; row < current->rows; ++row ) {
-			/* Make sure we don't go over the limit */
+			/* Make sure we don't go either over, or under the
+			 * limit */
+			if ( row+glyph->yoffset < 0 ) {
+				continue;
+			}
 			if ( row+glyph->yoffset >= textbuf->h ) {
 				continue;
 			}
@@ -1311,7 +1373,7 @@ SDL_Surface* TTF_RenderUNICODE_Shaded( TTF_Font* font,
 				(row+glyph->yoffset) * textbuf->pitch +
 				xstart + glyph->minx;
 			src = current->buffer + row * current->pitch;
-			for ( col=current->width; col>0; --col ) {
+			for ( col=width; col>0 && dst < dst_check; --col ) {
 				*dst++ |= *src++;
 			}
 		}
@@ -1320,6 +1382,7 @@ SDL_Surface* TTF_RenderUNICODE_Shaded( TTF_Font* font,
 		if( font->style & TTF_STYLE_BOLD ) {
 			xstart += font->glyph_overhang;
 		}
+		prev_index = glyph->index;
 	}
 
 	/* Handle the underline style */
@@ -1470,10 +1533,13 @@ SDL_Surface *TTF_RenderUNICODE_Blended(TTF_Font *font,
 	const Uint16 *ch;
 	Uint8 *src;
 	Uint32 *dst;
+	Uint32 *dst_check;
 	int swapped;
 	int row, col;
 	c_glyph *glyph;
 	FT_Error error;
+	FT_Long use_kerning;
+	FT_UInt prev_index = 0;
 
 	/* Get the dimensions of the text surface */
 	if ( (TTF_SizeUNICODE(font, text, &width, NULL) < 0) || !width ) {
@@ -1483,15 +1549,24 @@ SDL_Surface *TTF_RenderUNICODE_Blended(TTF_Font *font,
 	height = font->height;
 
 	textbuf = SDL_AllocSurface(SDL_SWSURFACE, width, height, 32,
-                  0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000);
+	                           0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000);
 	if ( textbuf == NULL ) {
 		return(NULL);
 	}
 
+	/* Adding bound checking to avoid all kinds of memory corruption errors
+	   that may occur. */
+	dst_check = (Uint32*)textbuf->pixels + textbuf->pitch/4 * textbuf->h;
+
+	/* check kerning */
+	use_kerning = FT_HAS_KERNING( font->face );
+
 	/* Load and render each character */
 	xstart = 0;
 	swapped = TTF_byteswapped;
 	pixel = (fg.r<<16)|(fg.g<<8)|fg.b;
+	SDL_FillRect(textbuf, NULL, pixel);     /* Initialize with fg and 0 alpha */
+
 	for ( ch=text; *ch; ++ch ) {
 		Uint16 c = *ch;
 		if ( c == UNICODE_BOM_NATIVE ) {
@@ -1517,25 +1592,42 @@ SDL_Surface *TTF_RenderUNICODE_Blended(TTF_Font *font,
 			return NULL;
 		}
 		glyph = font->current;
+		/* Ensure the width of the pixmap is correct. On some cases,
+		 * freetype may report a larger pixmap than possible.*/
 		width = glyph->pixmap.width;
+		if (width > glyph->maxx - glyph->minx) {
+			width = glyph->maxx - glyph->minx;
+		}
+		/* do kerning, if possible AC-Patch */
+		if ( use_kerning && prev_index && glyph->index ) {
+			FT_Vector delta;
+			FT_Get_Kerning( font->face, prev_index, glyph->index, ft_kerning_default, &delta );
+			xstart += delta.x >> 6;
+		}
+
 		/* Compensate for the wrap around bug with negative minx's */
 		if ( (ch == text) && (glyph->minx < 0) ) {
 			xstart -= glyph->minx;
 		}
 
 		for ( row = 0; row < glyph->pixmap.rows; ++row ) {
-			/* Make sure we don't go over the limit */
+			/* Make sure we don't go either over, or under the
+			 * limit */
+			if ( row+glyph->yoffset < 0 ) {
+				continue;
+			}
 			if ( row+glyph->yoffset >= textbuf->h ) {
 				continue;
 			}
 			dst = (Uint32*) textbuf->pixels +
 				(row+glyph->yoffset) * textbuf->pitch/4 +
 				xstart + glyph->minx;
+
 			/* Added code to adjust src pointer for pixmaps to
 			 * account for pitch.
 			 * */
 			src = (Uint8*) (glyph->pixmap.buffer + glyph->pixmap.pitch * row);
-			for ( col = width; col>0; --col) {
+			for ( col = width; col>0 && dst < dst_check; --col) {
 				alpha = *src++;
 				*dst++ |= pixel | (alpha << 24);
 			}
@@ -1545,6 +1637,7 @@ SDL_Surface *TTF_RenderUNICODE_Blended(TTF_Font *font,
 		if ( font->style & TTF_STYLE_BOLD ) {
 			xstart += font->glyph_overhang;
 		}
+		prev_index = glyph->index;
 	}
 
 	/* Handle the underline style */
@@ -1592,6 +1685,8 @@ SDL_Surface *TTF_RenderGlyph_Blended(TTF_Font *font, Uint16 ch, SDL_Color fg)
 
 	/* Copy the character from the pixmap */
 	pixel = (fg.r<<16)|(fg.g<<8)|fg.b;
+	SDL_FillRect(textbuf, NULL, pixel);     /* Initialize with fg and 0 alpha */
+
 	for ( row=0; row<textbuf->h; ++row ) {
 		/* Changed src to take pitch into account, not just width */
 		src = glyph->pixmap.buffer + row * glyph->pixmap.pitch;
@@ -1620,336 +1715,6 @@ SDL_Surface *TTF_RenderGlyph_Blended(TTF_Font *font, Uint16 ch, SDL_Color fg)
 	return(textbuf);
 }
 
-/* Convert the Latin-1 text to UNICODE and render it
-*/
-SDL_Surface *TTF_RenderText_Blended_Shaded(TTF_Font *font,
-			const char *text, SDL_Color fg, SDL_Color bg)
-{
-	SDL_Surface *textbuf;
-	Uint16 *unicode_text;
-	int unicode_len;
-
-	/* Copy the Latin-1 text to a UNICODE text buffer */
-	unicode_len = strlen(text);
-	unicode_text = (Uint16 *)ALLOCA((1+unicode_len+1)*(sizeof *unicode_text));
-	if ( unicode_text == NULL ) {
-		TTF_SetError("Out of memory");
-		return(NULL);
-	}
-	*unicode_text = UNICODE_BOM_NATIVE;
-	LATIN1_to_UNICODE(unicode_text+1, text, unicode_len);
-
-	/* Render the new text */
-	textbuf = TTF_RenderUNICODE_Blended_Shaded(font, unicode_text, fg, bg);
-
-	/* Free the text buffer and return */
-	FREEA(unicode_text);
-	return(textbuf);
-}
-
-/* Convert the UTF-8 text to UNICODE and render it
-*/
-SDL_Surface *TTF_RenderUTF8_Blended_Shaded(TTF_Font *font,
-		const char *text, SDL_Color fg, SDL_Color bg)
-{
-	SDL_Surface *textbuf;
-	Uint16 *unicode_text;
-	int unicode_len;
-
-	/* Copy the UTF-8 text to a UNICODE text buffer */
-	unicode_len = strlen(text);
-	unicode_text = (Uint16 *)ALLOCA((1+unicode_len+1)*(sizeof *unicode_text));
-	if ( unicode_text == NULL ) {
-		TTF_SetError("Out of memory");
-		return(NULL);
-	}
-	*unicode_text = UNICODE_BOM_NATIVE;
-	UTF8_to_UNICODE(unicode_text+1, text, unicode_len);
-
-	/* Render the new text */
-	textbuf = TTF_RenderUNICODE_Blended_Shaded(font, unicode_text, fg, bg);
-
-	/* Free the text buffer and return */
-	FREEA(unicode_text);
-	return(textbuf);
-}
-
-SDL_Surface *TTF_RenderUNICODE_Blended_Shaded(TTF_Font *font,
-			const Uint16 *text, SDL_Color fg, SDL_Color bg)
-{
-	int xstart;
-	int width, height;
-	SDL_Surface *textbuf;
-	Uint32 alpha;
-	Uint32 dst_pixel;
-	Uint32 pixel;
-#ifndef HAVE_MMX1  
-	Uint32 buf1, buf2;
-#endif  
-	const Uint16 *ch;
-	Uint8 *src;
-	Uint32 *dst;
-	int row, col;
-	int swapped;
-	c_glyph *glyph;
-	FT_Error error;
-
-	if(!bg.unused) { /* background ALPHA = 0 */
-	  return TTF_RenderUNICODE_Blended(font, text, fg);
-	}
-
-	/* Get the dimensions of the text surface */
-	if ( (TTF_SizeUNICODE(font, text, &width, NULL) < 0) || !width ) {
-		TTF_SetError("Text has zero width");
-		return(NULL);
-	}
-	height = font->height;
-
-	textbuf = SDL_AllocSurface(SDL_SWSURFACE, width, height, 32,
-                  0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000);
-	if ( textbuf == NULL ) {
-		return(NULL);
-	}
-
-	/* Load and render each character */
-	
-	swapped = TTF_byteswapped;
-	pixel = (fg.r<<16)|(fg.g<<8)|fg.b|0xFF000000;
-	dst_pixel = (bg.unused<<24)|(bg.r<<16)|(bg.g<<8)|bg.b;
-
-	SDL_FillRect(textbuf, NULL , dst_pixel);
-	
-#ifdef HAVE_MMX1
-	xstart = 0x00FF00FF;
-	movd_m2r(*(&xstart), mm7);
-	punpckldq_r2r(mm7, mm7); /* mm7(00000F0F) -> mm7(0F0F0F0F) */
-	movd_m2r(*(&pixel), mm0); /* src -> mm0(0000ARGB) */
-	punpcklbw_r2r(mm0, mm0); /* mm7(0000ARGB) -> mm7(AARRGGBB) */
-	pand_r2r(mm7, mm0); /* mm0 & mask -> mm0(0A0R0G0B) */
-	movd_m2r(*(&dst_pixel), mm1); /* dst -> mm1(0000ARGB) */
-	punpcklbw_r2r(mm1, mm1); /* mm1(0000ARGB) -> mm1(AARRGGBB) */
-	pand_r2r(mm7, mm1); /* mm1 & mask -> mm1(0A0R0G0B) */
-	psubw_r2r(mm1, mm0); /* src - dst -> mm1 */
-#else
-	buf1 = (pixel & 0x00FF00FF) - (dst_pixel & 0x00FF00FF);
-	buf2 = ((pixel & 0xFF00FF00) >> 8) - ((dst_pixel & 0xFF00FF00) >> 8);
-#endif	
-	xstart = 0;
-	for ( ch=text; *ch; ++ch ) {
-		Uint16 c = *ch;
-		if ( c == UNICODE_BOM_NATIVE ) {
-			swapped = 0;
-			if ( text == ch ) {
-				++text;
-			}
-			continue;
-		}
-		if ( c == UNICODE_BOM_SWAPPED ) {
-			swapped = 1;
-			if ( text == ch ) {
-				++text;
-			}
-			continue;
-		}
-		if ( swapped ) {
-			c = SDL_Swap16(c);
-		}
-	
-		error = Find_Glyph(font, *ch, CACHED_METRICS|CACHED_PIXMAP);
-		if( error ) {
-			SDL_FreeSurface( textbuf );
-			return NULL;
-		}
-		glyph = font->current;
-		width = glyph->pixmap.width;
-		
-		/* Compensate for the wrap around bug with negative minx's */
-		if ( (ch == text) && (glyph->minx < 0) ) {
-			xstart -= glyph->minx;
-		}
-
-		for ( row = 0; row < glyph->pixmap.rows; ++row ) {
-			/* Make sure we don't go over the limit */
-			if ( row+glyph->yoffset >= textbuf->h ) {
-				continue;
-			}
-			dst = (Uint32*) textbuf->pixels +
-				(row+glyph->yoffset) * textbuf->pitch/4 +
-				xstart + glyph->minx;
-			/* Added code to adjust src pointer for pixmaps to
-			 * account for pitch.
-			 * */
-			src = (Uint8*) (glyph->pixmap.buffer + glyph->pixmap.pitch * row);
-		
-			for ( col=width; col>0; --col ) {
-				alpha = *src++;
-				if(alpha)
-				{
-				  if(alpha == SDL_ALPHA_OPAQUE)
-				  {
-				    *dst = pixel;
-				  } else {
-#ifdef HAVE_MMX1
-				    movd_m2r(*(&alpha), mm2); /* alpha -> mm2 */
-	                            punpcklwd_r2r(mm2, mm2); /* mm2(0000000A) -> mm2(00000A0A) */
-				    movq_r2r(mm0, mm3); /* src - dst -> mm3 */
-				    punpckldq_r2r(mm2, mm2); /* mm2(00000A0A) -> mm2(0A0A0A0A) */
-				    pmullw_r2r(mm2, mm3); /* mm3 * alpha -> mm3 */
-				    psrlw_i2r(8, mm3); /* mm3 >> 8 -> mm3 */
-	                            paddw_r2r(mm1, mm3); /* mm3 + mm1(dst) -> mm3 */
-				    pand_r2r(mm7, mm3); /* mm3 & mask -> mm3 */
-				    packuswb_r2r(mm3, mm3);  /* ARGBARGB -> mm3 */
-				    movd_r2m(mm3, *dst); /* mm3 -> dst */
-#else
-				    *dst = (((dst_pixel & 0x00FF00FF) +
-					(buf1 * alpha >> 8)) & 0x00FF00FF) |
-					(((((dst_pixel & 0xFF00FF00) >> 8) +
-					(buf2 * alpha >> 8)) & 0x00FF00FF) << 8);
-#endif				    
-				  } 
-				}
-				dst++;
-			}
-		}
-
-		xstart += glyph->advance;
-		if ( font->style & TTF_STYLE_BOLD ) {
-			xstart += font->glyph_overhang;
-		}
-	}
-#ifdef HAVE_MMX1
-	emms();
-#endif
-	/* Handle the underline style */
-	if( font->style & TTF_STYLE_UNDERLINE ) {
-		row = font->ascent - font->underline_offset - 1;
-		if ( row >= textbuf->h) {
-			row = (textbuf->h-1) - font->underline_height;
-		}
-		dst = (Uint32 *)textbuf->pixels + row * textbuf->pitch/4;
-		pixel |= 0xFF000000; /* Amask */
-		for ( row=font->underline_height; row>0; --row ) {
-			for ( col=0; col < textbuf->w; ++col ) {
-				dst[col] = pixel;
-			}
-			dst += textbuf->pitch/4;
-		}
-	}
-	return(textbuf);
-}
-
-SDL_Surface *TTF_RenderGlyph_Blended_Shaded(TTF_Font *font,
-			     Uint16 ch, SDL_Color fg, SDL_Color bg)
-{
-	SDL_Surface *textbuf;
-	Uint32 alpha;
-	Uint32 dst_pixel;
-	Uint32 pixel;
-#ifndef HAVE_MMX1
-	Uint32 buf1, buf2;
-#endif  
-	Uint8 *src;
-	Uint32 *dst;
-	int row, col;
-	FT_Error error;
-	c_glyph *glyph;
-
-	if(!bg.unused) {
-	  return TTF_RenderGlyph_Blended(font, ch, fg);
-	}
-	
-	/* Get the glyph itself */
-	error = Find_Glyph(font, ch, CACHED_METRICS|CACHED_PIXMAP);
-	if ( error ) {
-		return(NULL);
-	}
-	glyph = font->current;
-
-	textbuf = SDL_CreateRGBSurface(SDL_SWSURFACE,
-	              glyph->pixmap.width, glyph->pixmap.rows, 32,
-                  0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000);
-	if ( ! textbuf ) {
-		return(NULL);
-	}
-
-	/* Copy the character from the pixmap */
-	pixel = (fg.r<<16)|(fg.g<<8)|fg.b|0xFF000000;
-	dst_pixel = (bg.unused<<24)|(bg.r<<16)|(bg.g<<8)|bg.b;
-
-	SDL_FillRect(textbuf, NULL , dst_pixel);
-	
-#ifdef HAVE_MMX1
-	alpha = 0x00FF00FF;
-	movd_m2r(*(&alpha), mm7);
-	punpckldq_r2r(mm7, mm7); /* mm7(00000F0F) -> mm7(0F0F0F0F) */
-	movd_m2r(*(&pixel), mm0);
-	punpcklbw_r2r(mm0, mm0); /* mm7(0000ARGB) -> mm7(AARRGGBB) */
-	pand_r2r(mm7, mm0);
-	movd_m2r(*(&dst_pixel), mm1);
-	punpcklbw_r2r(mm1, mm1); /* mm1(0000ARGB) -> mm1(AARRGGBB) */
-	pand_r2r(mm7, mm1);
-	psubw_r2r(mm1, mm0); /* src - dst -> mm0 */
-#else
-	buf1 = (pixel & 0x00FF00FF) - (dst_pixel & 0x00FF00FF);
-	buf2 = ((pixel & 0xFF00FF00) >> 8) - ((dst_pixel & 0xFF00FF00) >> 8);
-#endif	
-	
-	for ( row=0; row<textbuf->h; ++row ) {
-		/* Changed src to take pitch into account, not just width */
-		src = glyph->pixmap.buffer + row * glyph->pixmap.pitch;
-		dst = (Uint32 *)textbuf->pixels + row * textbuf->pitch/4;
-		for ( col=0; col<glyph->pixmap.width; ++col ) {
-			alpha = *src++;
-			if(alpha)
-			{
-			  if(alpha == SDL_ALPHA_OPAQUE)
-			  {
-			    *dst = pixel;
-			  } else {
-#ifdef HAVE_MMX1
-			    movd_m2r(*(&alpha), mm2); /* alpha -> mm2 */
-                            punpcklwd_r2r(mm2, mm2); /* mm2(0000000A) -> mm2(00000A0A) */
-			    movq_r2r(mm0, mm3); /* src - dst -> mm3 */
-			    punpckldq_r2r(mm2, mm2); /* mm2(00000A0A) -> mm2(0A0A0A0A) */
-			    pmullw_r2r(mm2, mm3); /* mm3 * alpha -> mm3 */
-			    psrlw_i2r(8, mm3); /* mm3 >> 8 -> mm3 */
-                            paddw_r2r(mm1, mm3); /* mm3 + mm1(dst) -> mm3 */
-			    pand_r2r(mm7, mm3); /* mm3 & mask -> mm3 */
-			    packuswb_r2r(mm3, mm3);  /* ARGBARGB -> mm3 */
-			    movd_r2m(mm3, *dst); /* mm3 -> dst */
-#else
-			    *dst = (((dst_pixel & 0x00FF00FF) +
-				(buf1 * alpha >> 8)) & 0x00FF00FF) |
-				(((((dst_pixel & 0xFF00FF00) >> 8) +
-				(buf2 * alpha >> 8)) & 0x00FF00FF) << 8);
-#endif				    
-			  }    
-			}
-			dst++;
-		}
-	}
-#ifdef HAVE_MMX1
-	emms();
-#endif
-	/* Handle the underline style */
-	if( font->style & TTF_STYLE_UNDERLINE ) {
-		row = font->ascent - font->underline_offset - 1;
-		if ( row >= textbuf->h) {
-			row = (textbuf->h-1) - font->underline_height;
-		}
-		dst = (Uint32 *)textbuf->pixels + row * textbuf->pitch/4;
-		pixel |= 0xFF000000; /* Amask */
-		for ( row=font->underline_height; row>0; --row ) {
-			for ( col=0; col < textbuf->w; ++col ) {
-				dst[col] = pixel;
-			}
-			dst += textbuf->pitch/4;
-		}
-	}
-	return(textbuf);
-}
-
-
 void TTF_SetFontStyle( TTF_Font* font, int style )
 {
 	font->style = style;
