@@ -12,10 +12,11 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 /* utility */
+#include "astring.h"
 #include "fcintl.h"
 #include "log.h"
 
@@ -335,9 +336,16 @@ static int spy_steal_popup(struct widget *pWidget)
   } advance_index_iterate_end;
   
   /* Get Spy tech to use for "At Spy's Discretion" -- this will have the
-   * side effect of display the unit's icon */
+   * side effect of displaying the unit's icon */
   i = advance_number(unit_type(game_unit_by_number(id))->require_advance);
-  copy_chars_to_string16(pStr, _("At Spy's Discretion"));
+  {
+    struct astring str = ASTRING_INIT;
+    /* TRANS: %s is a unit name, e.g., Spy */
+    astr_set(&str, _("At %s's Discretion"),
+             unit_name_translation(game_unit_by_number(id)));
+    copy_chars_to_string16(pStr, astr_str(&str));
+    astr_free(&str);
+  }
   pSurf = create_sellect_tech_icon(pStr, i, FULL_MODE);
 	
   pBuf = create_icon2(pSurf, pWindow->dst,
@@ -553,13 +561,12 @@ void popup_diplomat_dialog(struct unit *pUnit, struct tile *ptile)
   /* window */
   if (pCity)
   {
-    if(spy) {
-      pStr = create_str16_from_char(_("Choose Your Spy's Strategy") , adj_font(12));
-    }
-    else
-    {
-      pStr = create_str16_from_char(_("Choose Your Diplomat's Strategy"), adj_font(12));
-    }
+    struct astring title = ASTRING_INIT;
+    /* TRANS: %s is a unit name, e.g., Spy */
+    astr_set(&title, _("Choose Your %s's Strategy"),
+             unit_name_translation(pUnit));
+    pStr = create_str16_from_char(astr_str(&title), adj_font(12));
+    astr_free(&title);
   }
   else
   {
@@ -755,7 +762,7 @@ void popup_diplomat_dialog(struct unit *pUnit, struct tile *ptile)
   
   auto_center_on_focus_unit();
   put_window_near_map_tile(pWindow, pWindow->size.w, pWindow->size.h,
-                           pUnit->tile);
+                           unit_tile(pUnit));
  
   /* setup widget size and start position */
     
@@ -932,8 +939,14 @@ void popup_sabotage_dialog(struct city *pCity)
   /* ------------------ */
   }
   
-  create_active_iconlabel(pBuf, pWindow->dst, pStr, _("At Spy's Discretion"),
-			  sabotage_impr_callback);
+  {
+    struct astring str = ASTRING_INIT;
+    /* TRANS: %s is a unit name, e.g., Spy */
+    astr_set(&str, _("At %s's Discretion"), unit_name_translation(pUnit));
+    create_active_iconlabel(pBuf, pWindow->dst, pStr, astr_str(&str),
+                            sabotage_impr_callback);
+    astr_free(&str);
+  }
   pBuf->data.cont = pCont;  
   set_wstate(pBuf, FC_WS_NORMAL);
   
@@ -971,7 +984,7 @@ void popup_sabotage_dialog(struct city *pCity)
   
   auto_center_on_focus_unit();
   put_window_near_map_tile(pWindow, pWindow->size.w, pWindow->size.h,
-                           pUnit->tile);        
+                           unit_tile(pUnit));
   
   w = area.w;
   
@@ -1012,7 +1025,7 @@ void popup_sabotage_dialog(struct city *pCity)
       area2.y = pBuf->size.h / 2 - 1;
       area2.w = pBuf->size.w - adj_size(20);
       
-      SDL_FillRect(pBuf->theme , &area2, map_rgba(pBuf->theme->format, *get_game_colorRGB(COLOR_THEME_SABOTAGEDLG_SEPARATOR)));
+      SDL_FillRect(pBuf->theme , &area2, map_rgba(pBuf->theme->format, *get_theme_color(COLOR_THEME_SABOTAGEDLG_SEPARATOR)));
     }
     
     if (pBuf == pLast) {
@@ -1492,8 +1505,8 @@ void popup_bribe_dialog(struct unit *pUnit, int cost)
   
   auto_center_on_focus_unit();
   put_window_near_map_tile(pWindow, pWindow->size.w, pWindow->size.h,
-                           pDiplomatUnit->tile);      
-  
+                           unit_tile(pDiplomatUnit));
+
   /* setup widget size and start position */
   pBuf = pWindow;
   
