@@ -33,6 +33,8 @@ enum GUI_ID {
   ID_ICON,
   ID_EDIT,
   ID_CHECKBOX,
+  ID_COMBO,
+  ID_COMBO_MENU,
   ID_CLIENT_OPTIONS,
   ID_WAITING_LABEL,
   ID_START_NEW_GAME,
@@ -65,72 +67,14 @@ enum GUI_ID {
   ID_NATION_WIZARD_PREV_LEADER_NAME_BUTTON,
   ID_NATION_WIZARD_CHANGE_SEX_BUTTON,
   ID_OPTIONS_WINDOW,
-  ID_OPTIONS_VIDEO_BUTTON,
-  ID_OPTIONS_RESOLUTION_LABEL,
-  ID_OPTIONS_TOGGLE_FULLSCREEN_CHECKBOX,
-  ID_OPTIONS_FULLSCREEN_LABEL,
-  ID_OPTIONS_SOUND_BUTTON,
-  ID_OPTIONS_LOCAL_BUTTON,
-  ID_OPTIONS_LOCAL_SOUND_LABEL,
-  ID_OPTIONS_LOCAL_SOUND_CHECKBOX,
-  ID_OPTIONS_LOCAL_MOVE_LABEL,
-  ID_OPTIONS_LOCAL_MOVE_CHECKBOX,
-  ID_OPTIONS_LOCAL_MOVE_STEP_LABEL,
-  ID_OPTIONS_LOCAL_MOVE_STEP_EDIT,
-  ID_OPTIONS_LOCAL_COMBAT_LABEL,
-  ID_OPTIONS_LOCAL_COMBAT_CHECKBOX,
-  ID_OPTIONS_LOCAL_ACENTER_LABEL,
-  ID_OPTIONS_LOCAL_ACENTER_CHECKBOX,
-  ID_OPTIONS_LOCAL_COMBAT_CENTER_LABEL,
-  ID_OPTIONS_LOCAL_COMBAT_CENTER_CHECKBOX,
-  ID_OPTIONS_LOCAL_ACTIVE_UNITS_LABEL,
-  ID_OPTIONS_LOCAL_ACTIVE_UNITS_CHECKBOX,
-  ID_OPTIONS_LOCAL_CITY_CENTER_LABEL,
-  ID_OPTIONS_LOCAL_CITY_CENTER_CHECKBOX,
-  ID_OPTIONS_LOCAL_END_TURN_LABEL,
-  ID_OPTIONS_LOCAL_END_TURN_CHECKBOX,
-  ID_OPTIONS_MAP_BUTTON,
-  ID_OPTIONS_MAP_GRID_LABEL,
-  ID_OPTIONS_MAP_GRID_CHECKBOX,
-  ID_OPTIONS_MAP_CITY_NAMES_LABEL,
-  ID_OPTIONS_MAP_CITY_NAMES_CHECKBOX,
-  ID_OPTIONS_MAP_CITY_PROD_LABEL,
-  ID_OPTIONS_MAP_CITY_PROD_CHECKBOX,
-  ID_OPTIONS_MAP_CITY_OUTPUT_CHECKBOX,
-  ID_OPTIONS_MAP_CITY_OUTPUT_LABEL,
-  ID_OPTIONS_MAP_CITY_TRADE_ROUTES_CHECKBOX,
-  ID_OPTIONS_MAP_CITY_TRADE_ROUTES_LABEL,
-  ID_OPTIONS_MAP_BORDERS_CHECKBOX,
-  ID_OPTIONS_MAP_BORDERS_LABEL,
-  ID_OPTIONS_MAP_CITY_CIV3_TEXT_STYLE_LABEL,
-  ID_OPTIONS_MAP_CITY_CIV3_TEXT_STYLE_CHECKBOX,
-  ID_OPTIONS_MAP_TERRAIN_LABEL,
-  ID_OPTIONS_MAP_TERRAIN_CHECKBOX,
-  ID_OPTIONS_MAP_TERRAIN_COAST_LABEL,
-  ID_OPTIONS_MAP_TERRAIN_COAST_CHECKBOX,
-  ID_OPTIONS_MAP_TERRAIN_INPR_LABEL,
-  ID_OPTIONS_MAP_TERRAIN_RR_LABEL,
-  ID_OPTIONS_MAP_TERRAIN_RR_CHECKBOX,
-  ID_OPTIONS_MAP_TERRAIN_IR_LABEL,
-  ID_OPTIONS_MAP_TERRAIN_IR_CHECKBOX,
-  ID_OPTIONS_MAP_TERRAIN_M_LABEL,
-  ID_OPTIONS_MAP_TERRAIN_M_CHECKBOX,
-  ID_OPTIONS_MAP_TERRAIN_FA_LABEL,
-  ID_OPTIONS_MAP_TERRAIN_FA_CHECKBOX,
-  ID_OPTIONS_MAP_TERRAIN_SPEC_LABEL,
-  ID_OPTIONS_MAP_TERRAIN_SPEC_CHECKBOX,
-  ID_OPTIONS_MAP_TERRAIN_POLL_LABEL,
-  ID_OPTIONS_MAP_TERRAIN_POLL_CHECKBOX,
-  ID_OPTIONS_MAP_TERRAIN_CITY_LABEL,
-  ID_OPTIONS_MAP_TERRAIN_CITY_CHECKBOX,
-  ID_OPTIONS_MAP_TERRAIN_UNITS_LABEL,
-  ID_OPTIONS_MAP_TERRAIN_UNITS_CHECKBOX,
-  ID_OPTIONS_MAP_TERRAIN_FOG_LABEL,
-  ID_OPTIONS_MAP_TERRAIN_FOG_CHECKBOX,
+  ID_OPTIONS_CLIENT_BUTTON,
+  ID_OPTIONS_SERVER_BUTTON,
   ID_OPTIONS_WORKLIST_BUTTON,
+  ID_OPTIONS_SAVE_BUTTON,
   ID_OPTIONS_SAVE_GAME_BUTTON,
   ID_OPTIONS_DISC_BUTTON,
   ID_OPTIONS_BACK_BUTTON,
+  ID_OPTIONS_APPLY_BUTTON,
   ID_OPTIONS_EXIT_BUTTON,
   ID_UNITS_WINDOW,
   ID_TOGGLE_UNITS_WINDOW_BUTTON,
@@ -168,6 +112,7 @@ enum GUI_ID {
   ID_UNIT_ORDER_AIRLIFT,
   ID_UNIT_ORDER_RETURN,
   ID_UNIT_ORDER_UPGRADE,
+  ID_UNIT_ORDER_CONVERT,
   ID_UNIT_ORDER_DISBAND,
   ID_UNIT_ORDER_DIPLOMAT_DLG,
   ID_UNIT_ORDER_NUKE,
