@@ -20,7 +20,7 @@
  **********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <errno.h>
@@ -50,11 +50,13 @@
 
 /* client */
 #include "client_main.h"
+#include "climisc.h"
 #include "clinet.h"
 #include "editgui_g.h"
 #include "ggzclient.h"
 #include "ggz_g.h"
 #include "tilespec.h"
+#include "update_queue.h"
 
 /* gui-sdl */
 #include "chatline.h"
@@ -102,7 +104,6 @@ bool RSHIFT;
 bool LCTRL;
 bool RCTRL;
 bool LALT;
-bool do_focus_animation = TRUE;
 int city_names_font_size = 12;
 int city_productions_font_size = 12;
 
@@ -229,21 +230,21 @@ static Uint16 main_key_down_handler(SDL_keysym Key, void *pData)
               struct unit *pUnit;
               struct city *pCity;
               if (NULL != (pUnit = head_of_units_in_focus()) && 
-                (pCity = tile_city(pUnit->tile)) != NULL &&
+                (pCity = tile_city(unit_tile(pUnit))) != NULL &&
                 city_owner(pCity) == client.conn.playing) {
                 popup_city_dialog(pCity);
               }
 	    }
           return ID_ERROR;
 
-	  case SDLK_F2:
-	    popup_activeunits_report_dialog(FALSE);
-	  return ID_ERROR;
-	   
-	  case SDLK_F4:
-            popup_city_report_dialog(FALSE);
-	  return ID_ERROR;
-	    
+          case SDLK_F2:
+            units_report_dialog_popup(FALSE);
+          return ID_ERROR;
+
+          case SDLK_F4:
+            city_report_dialog_popup(FALSE);
+            return ID_ERROR;
+
 	  case SDLK_F7:
             send_report_request(REPORT_WONDERS_OF_THE_WORLD);
           return ID_ERROR;
@@ -251,16 +252,16 @@ static Uint16 main_key_down_handler(SDL_keysym Key, void *pData)
           case SDLK_F8:
             send_report_request(REPORT_TOP_5_CITIES);
           return ID_ERROR;
-	    
-	  case SDLK_F9:
-            if(is_meswin_open()) {
-              popdown_meswin_dialog();
+
+          case SDLK_F9:
+            if (meswin_dialog_is_open()) {
+              meswin_dialog_popdown();
             } else {
-              popup_meswin_dialog(true);
+              meswin_dialog_popup(TRUE);
             }
-	    flush_dirty();
-          return ID_ERROR;
-	    	        
+            flush_dirty();
+            return ID_ERROR;
+
 	  case SDLK_F11:
             send_report_request(REPORT_DEMOGRAPHIC);
           return ID_ERROR;
@@ -357,8 +358,7 @@ static Uint16 main_mouse_motion_handler(SDL_MouseMotionEvent *pMotionEvent, void
    * hold state or above */
   if (button_behavior.counting && (button_behavior.hold_state >= MB_HOLD_MEDIUM)) {
     ptile = canvas_pos_to_tile(pMotionEvent->x, pMotionEvent->y);
-    if ((ptile->x != button_behavior.ptile->x)
-        || (ptile->y != button_behavior.ptile->y)) {
+    if (tile_index(ptile) != tile_index(button_behavior.ptile)) {
       button_behavior.counting = FALSE;
     }
   }
@@ -631,8 +631,8 @@ Uint16 gui_event_loop(void *pData,
         case SDL_KEYDOWN:
           switch(Main.event.key.keysym.sym) {
             case SDLK_PRINT:
-              my_snprintf(schot, sizeof(schot), "fc_%05d.bmp", schot_nr++);
-              freelog(LOG_NORMAL, _("Making screenshot %s"), schot);
+              fc_snprintf(schot, sizeof(schot), "fc_%05d.bmp", schot_nr++);
+              log_normal(_("Making screenshot %s"), schot);
               SDL_SaveBMP(Main.screen, schot);
             break;
             
@@ -729,7 +729,7 @@ Uint16 gui_event_loop(void *pData,
     if (ID == ID_ERROR) {
       if (callbacks && callback_list_size(callbacks) > 0) {
         struct callback *cb = callback_list_get(callbacks, 0);
-        callback_list_unlink(callbacks, cb);
+        callback_list_remove(callbacks, cb);
         (cb->callback)(cb->data);
         free(cb);
       }
@@ -763,19 +763,20 @@ void ui_init(void)
   putenv((char *)"SDL_VIDEO_CENTERED=yes");
   
   init_sdl(iSDL_Flags);
-  
-  freelog(LOG_NORMAL, _("Using Video Output: %s"),
-	  SDL_VideoDriverName(device, sizeof(device)));
+
+  log_normal(_("Using Video Output: %s"),
+             SDL_VideoDriverName(device, sizeof(device)));
   
   /* create splash screen */  
 #ifdef SMALL_SCREEN
   {
-    SDL_Surface *pTmpSurf = load_surf(datafilename("misc/intro.png"));
+    SDL_Surface *pTmpSurf = load_surf(fileinfoname(get_data_dirs(),
+                                                   "misc/intro.png"));
     pBgd = zoomSurface(pTmpSurf, DEFAULT_ZOOM, DEFAULT_ZOOM, 0);
     FREESURFACE(pTmpSurf);
   }
 #else
-  pBgd = load_surf(datafilename("misc/intro.png"));
+  pBgd = load_surf(fileinfoname(get_data_dirs(), "misc/intro.png"));
 #endif
   
   if(pBgd && SDL_GetVideoInfo()->wm_available) {
@@ -837,11 +838,73 @@ void ui_init(void)
 }
 
 /****************************************************************************
-  Extra initializers for client options.
+  Really resize the main window.
+****************************************************************************/
+static void real_resize_window_callback(void *data)
+{
+  struct widget *widget;
+  Uint32 flags = Main.screen->flags;
+
+  if (gui_sdl_fullscreen) {
+    flags |= SDL_FULLSCREEN;
+  } else {
+    flags &= ~SDL_FULLSCREEN;
+  }
+  set_video_mode(gui_sdl_screen.width, gui_sdl_screen.height, flags);
+
+  if (C_S_RUNNING == client_state()) {
+    /* Move units window to botton-right corner. */
+    set_new_unitinfo_window_pos();
+    /* Move minimap window to botton-left corner. */
+    set_new_minimap_window_pos();
+
+    /* Move cooling/warming icons to botton-right corner. */
+    widget = get_widget_pointer_form_main_list(ID_WARMING_ICON);
+    widget_set_position(widget, (Main.screen->w - adj_size(10)
+                                 - (widget->size.w * 2)), widget->size.y);
+
+    widget = get_widget_pointer_form_main_list(ID_COOLING_ICON);
+    widget_set_position(widget, (Main.screen->w - adj_size(10)
+                                 - widget->size.w), widget->size.y);
+
+    map_canvas_resized(Main.screen->w, Main.screen->h);
+    update_info_label();
+    update_unit_info_label(get_units_in_focus());
+    center_on_something();      /* With redrawing full map. */
+    update_order_widgets();
+  } else {
+    draw_intro_gfx();
+    dirty_all();
+  }
+  flush_all();
+}
+
+/****************************************************************************
+  Resize the main window.
+****************************************************************************/
+static void resize_window_callback(struct option *poption)
+{
+  update_queue_add(real_resize_window_callback, NULL);
+}
+
+/****************************************************************************
+  Extra initializers for client options. Here we make set the callback
+  for the specific gui-sdl options.
 ****************************************************************************/
 void gui_options_extra_init(void)
 {
-  /* Nothing to do. */
+  struct option *poption;
+
+#define option_var_set_callback(var, callback)                              \
+  if ((poption = optset_option_by_name(client_optset, #var))) {             \
+    option_set_changed_callback(poption, callback);                         \
+  } else {                                                                  \
+    log_error("Didn't find option %s!", #var);                              \
+  }
+
+  option_var_set_callback(gui_sdl_fullscreen, resize_window_callback);
+  option_var_set_callback(gui_sdl_screen, resize_window_callback);
+#undef option_var_set_callback
 }
 
 /**************************************************************************
@@ -851,9 +914,8 @@ static void clear_double_messages_call(void)
 {
   int i;
   /* clear double call */
-  for(i=0; i<E_LAST; i++) {
-    if (messages_where[i] & MW_MESSAGES)
-    {
+  for(i = 0; i <= event_type_max(); i++) {
+    if (messages_where[i] & MW_MESSAGES) {
       messages_where[i] &= ~MW_OUTPUT;
     }
   }
@@ -947,15 +1009,15 @@ void ui_main(int argc, char *argv[])
     #ifdef SMALL_SCREEN
       #ifdef UNDER_CE
         /* set 320x240 fullscreen */
-        set_video_mode(gui_sdl_screen_width, gui_sdl_screen_height,
+        set_video_mode(gui_sdl_screen.width, gui_sdl_screen.height,
                        SDL_SWSURFACE | SDL_ANYFORMAT | SDL_FULLSCREEN);
       #else
         /* small screen on desktop -> don't set 320x240 fullscreen mode */
-        set_video_mode(gui_sdl_screen_width, gui_sdl_screen_height,
+        set_video_mode(gui_sdl_screen.width, gui_sdl_screen.height,
                        SDL_SWSURFACE | SDL_ANYFORMAT);
       #endif
     #else
-      set_video_mode(gui_sdl_screen_width, gui_sdl_screen_height,
+      set_video_mode(gui_sdl_screen.width, gui_sdl_screen.height,
                      SDL_SWSURFACE | SDL_ANYFORMAT | SDL_FULLSCREEN);
     #endif
     
@@ -963,14 +1025,14 @@ void ui_main(int argc, char *argv[])
     
     #ifdef SMALL_SCREEN
       #ifdef UNDER_CE    
-      set_video_mode(gui_sdl_screen_width, gui_sdl_screen_height,
+      set_video_mode(gui_sdl_screen.width, gui_sdl_screen.height,
                      SDL_SWSURFACE | SDL_ANYFORMAT);
       #else
-      set_video_mode(gui_sdl_screen_width, gui_sdl_screen_height,
+      set_video_mode(gui_sdl_screen.width, gui_sdl_screen.height,
                      SDL_SWSURFACE | SDL_ANYFORMAT);
       #endif
     #else
-    set_video_mode(gui_sdl_screen_width, gui_sdl_screen_height,
+    set_video_mode(gui_sdl_screen.width, gui_sdl_screen.height,
       SDL_SWSURFACE | SDL_ANYFORMAT);
     #endif
     
@@ -985,8 +1047,8 @@ void ui_main(int argc, char *argv[])
   /* SDL_WM_SetCaption(_("SDL Client for Freeciv"), _("Freeciv")); */
 
   /* this need correct Main.screen size */
-  init_mapcanvas_and_overview();    
-  
+  init_mapcanvas_and_overview();
+
   set_client_state(C_S_DISCONNECTED);
 
   /* Main game loop */
@@ -1001,11 +1063,13 @@ void ui_main(int argc, char *argv[])
 **************************************************************************/
 void ui_exit()
 {
-  
+
 #if defined UNDER_CE && defined SMALL_SCREEN
   /* change back to window mode to restore the title bar */
   set_video_mode(320, 240, SDL_SWSURFACE | SDL_ANYFORMAT);
 #endif
+
+  free_mapcanvas_and_overview();
   
   free_auxiliary_tech_icons();
   free_intro_radar_sprites();
@@ -1013,14 +1077,14 @@ void ui_exit()
   diplomacy_dialog_done();
   intel_dialog_done();  
 
-  callback_list_free(callbacks);
+  callback_list_destroy(callbacks);
   
   unload_cursors();
 
   FC_FREE(button_behavior.event);
 
-  popdown_meswin_dialog();
-        
+  meswin_dialog_popdown();
+
   del_main_list();
   
   free_font_system();
@@ -1043,7 +1107,7 @@ enum gui_type get_gui_type(void)
 **************************************************************************/
 void sound_bell(void)
 {
-  freelog(LOG_DEBUG, "sound_bell : PORT ME");
+  log_debug("sound_bell : PORT ME");
 }
 
 /**************************************************************************
@@ -1069,7 +1133,7 @@ void disable_focus_animation(void)
 **************************************************************************/
 void add_net_input(int sock)
 {
-  freelog(LOG_DEBUG, "Connection UP (%d)", sock);
+  log_debug("Connection UP (%d)", sock);
   net_socket = sock;
   autoconnect = FALSE;
   enable_focus_animation();
@@ -1080,7 +1144,7 @@ void add_net_input(int sock)
 **************************************************************************/
 void remove_net_input(void)
 {
-  freelog(LOG_DEBUG, "Connection DOWN... ");
+  log_debug("Connection DOWN... ");
   net_socket = (-1);
   disable_focus_animation();
   draw_goto_patrol_lines = FALSE;
@@ -1092,7 +1156,7 @@ void remove_net_input(void)
 **************************************************************************/
 void add_ggz_input(int sock)
 {
-  freelog(LOG_DEBUG, "GGZ Connection UP (%d)", sock);
+  log_debug("GGZ Connection UP (%d)", sock);
   ggz_socket = sock;
 }
 
@@ -1102,7 +1166,7 @@ void add_ggz_input(int sock)
 **************************************************************************/
 void remove_ggz_input(void)
 {
-  freelog(LOG_DEBUG, "GGZ Connection DOWN... ");
+  log_debug("GGZ Connection DOWN... ");
   ggz_socket = (-1);
 }
 
