@@ -20,7 +20,7 @@
  *********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 /* #define SDL_CVS */
@@ -404,7 +404,7 @@ void update_info_label(void)
 
   /* set text settings */
   pText->style |= TTF_STYLE_BOLD;
-  pText->fgcol = *get_game_colorRGB(COLOR_THEME_MAPVIEW_INFO_TEXT);
+  pText->fgcol = *get_theme_color(COLOR_THEME_MAPVIEW_INFO_TEXT);
   pText->bgcol = (SDL_Color) {0, 0, 0, 0};
 
   if (NULL != client.conn.playing) {
@@ -441,18 +441,18 @@ void update_info_label(void)
     /* Horizontal lines */
     putline(Main.gui->surface,
                area.x + 1, area.y, area.x + area.w - 2, area.y,
-               get_game_colorRGB(COLOR_THEME_MAPVIEW_INFO_FRAME));
+               get_theme_color(COLOR_THEME_MAPVIEW_INFO_FRAME));
     putline(Main.gui->surface,
                area.x + 1, area.y + area.h - 1, area.x + area.w - 2, area.y + area.h - 1,
-               get_game_colorRGB(COLOR_THEME_MAPVIEW_INFO_FRAME));
+               get_theme_color(COLOR_THEME_MAPVIEW_INFO_FRAME));
   
     /* vertical lines */
     putline(Main.gui->surface,
                area.x + area.w - 1, area.y + 1, area.x + area.w - 1, area.y + area.h - 2,
-               get_game_colorRGB(COLOR_THEME_MAPVIEW_INFO_FRAME));
+               get_theme_color(COLOR_THEME_MAPVIEW_INFO_FRAME));
     putline(Main.gui->surface,
                area.x, area.y + 1, area.x, area.y + area.h - 2,
-               get_game_colorRGB(COLOR_THEME_MAPVIEW_INFO_FRAME));
+               get_theme_color(COLOR_THEME_MAPVIEW_INFO_FRAME));
   
     /* blit text to screen */  
     blit_entire_src(pTmp, Main.gui->surface, area.x + adj_size(5), area.y + adj_size(2));
@@ -480,7 +480,7 @@ static int fucus_units_info_callback(struct widget *pWidget)
     struct unit *pUnit = pWidget->data.unit;
     if (pUnit) {
       request_new_unit_activity(pUnit, ACTIVITY_IDLE);
-      set_unit_focus(pUnit);
+      unit_focus_set(pUnit);
     }
   }
   return -1;
@@ -513,7 +513,7 @@ void redraw_unit_info_label(struct unit_list *punitlist)
       int sy, y, width, height, n;
       bool right;
       char buffer[512];
-      struct tile *pTile = pUnit->tile;
+      struct tile *pTile = unit_tile(pUnit);
 
       /* get and draw unit name (with veteran status) */
       pStr = create_str16_from_char(unit_name_translation(pUnit), adj_font(12));
@@ -535,9 +535,9 @@ void redraw_unit_info_label(struct unit_list *punitlist)
       if(pUnit->veteran) {
 	copy_chars_to_string16(pStr, _("veteran"));
         change_ptsize16(pStr, adj_font(10));
-	pStr->fgcol = *get_game_colorRGB(COLOR_THEME_MAPVIEW_UNITINFO_VETERAN_TEXT);
+	pStr->fgcol = *get_theme_color(COLOR_THEME_MAPVIEW_UNITINFO_VETERAN_TEXT);
         pVet_Name = create_text_surf_from_str16(pStr);
-        pStr->fgcol = *get_game_colorRGB(COLOR_THEME_MAPVIEW_UNITINFO_TEXT);
+        pStr->fgcol = *get_theme_color(COLOR_THEME_MAPVIEW_UNITINFO_TEXT);
       }
 
       /* get and draw other info (MP, terran, city, etc.) */
@@ -594,7 +594,6 @@ void redraw_unit_info_label(struct unit_list *punitlist)
             /* Look at city owner, not tile owner (the two should be the same, if
              * borders are in use). */
             struct player *pOwner = city_owner(pCity);
-            bool citywall;
 /*            bool barrack = FALSE, airport = FALSE, port = FALSE;*/
 	    const char *diplo_city_adjectives[DS_LAST] =
     			{Q_("?city:Neutral"), Q_("?city:Hostile"),
@@ -604,8 +603,6 @@ void redraw_unit_info_label(struct unit_list *punitlist)
 	    cat_snprintf(buffer, sizeof(buffer),
 			 _("\nCity of %s"),
 			 city_name(pCity));
-
-	    citywall = pCity->client.walls;
                           
 #if 0       
             /* This has hardcoded assumption that EFT_LAND_REGEN is always
@@ -660,7 +657,7 @@ void redraw_unit_info_label(struct unit_list *punitlist)
 	if (pInfo_Window->size.h > 
             4 * h + (DEFAULT_UNITS_H + (pInfo_Window->size.h - pInfo_Window->area.h)) || right) {
           cat_snprintf(buffer, sizeof(buffer), _("\nFood/Prod/Trade: %s"),
-				get_tile_output_text(pUnit->tile));
+				get_tile_output_text(unit_tile(pUnit)));
 	}
 	
 	copy_chars_to_string16(pStr, buffer);
@@ -704,7 +701,7 @@ void redraw_unit_info_label(struct unit_list *punitlist)
       FREESURFACE(pName);
       
       /* draw unit sprite */
-      pTmpSurf = ResizeSurfaceBox(get_unittype_surface(unit_type(pUnit)),
+      pTmpSurf = ResizeSurfaceBox(get_unittype_surface(unit_type(pUnit), pUnit->facing),
                                   adj_size(80), adj_size(80), 1, TRUE, TRUE);
       pBuf_Surf = blend_surface(pTmpSurf, 32);
       FREESURFACE(pTmpSurf);
@@ -768,10 +765,11 @@ void redraw_unit_info_label(struct unit_list *punitlist)
 	    
 	  pUType = unit_type(aunit);
           pHome_City = game_city_by_number(aunit->homecity);
-          fc_snprintf(buffer, sizeof(buffer), "%s (%d,%d,%d)%s\n%s\n(%d/%d)\n%s",
+          fc_snprintf(buffer, sizeof(buffer), "%s (%d,%d,%s)%s\n%s\n(%d/%d)\n%s",
 		utype_name_translation(pUType),
 		pUType->attack_strength,
-		pUType->defense_strength, pUType->move_rate / SINGLE_MOVE,
+                pUType->defense_strength,
+                move_points_text(pUType->move_rate, NULL, NULL, FALSE),
                 (aunit->veteran ? _("\nveteran") : ""),
                 unit_activity_text(aunit),
 		aunit->hp, pUType->hp,
@@ -1049,7 +1047,7 @@ void get_overview_area_dimensions(int *width, int *height)
   int overview_tile_size_bak = OVERVIEW_TILE_SIZE;
   
   int xfact = MAP_IS_ISOMETRIC ? 2 : 1;
-  int shift = (MAP_IS_ISOMETRIC && !topo_has_flag(TF_WRAPX)) ? -1 : 0;
+  int shift = (MAP_IS_ISOMETRIC && !current_topo_has_flag(TF_WRAPX)) ? -1 : 0;
   
   OVERVIEW_TILE_SIZE = MIN((DEFAULT_OVERVIEW_W - 1) / (map.xsize * xfact),
                            (DEFAULT_OVERVIEW_H - 1) / map.ysize) + 1;
@@ -1114,7 +1112,7 @@ void draw_selection_rectangle(int canvas_x, int canvas_y, int w, int h)
   /* PORTME */
   putframe(Main.map,
            canvas_x, canvas_y, canvas_x + w, canvas_y + h,
-           get_game_colorRGB(COLOR_THEME_SELECTIONRECTANGLE));
+           get_theme_color(COLOR_THEME_SELECTIONRECTANGLE));
 }
 
 /**************************************************************************
