@@ -20,7 +20,7 @@
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include "SDL.h"
@@ -31,6 +31,7 @@
 
 /* common */
 #include "game.h"
+#include "road.h"
 #include "unitlist.h"
 
 /* client */
@@ -539,7 +540,10 @@ void create_units_order_widgets(void)
   /* --------- */
 
   /* Connect road */
-  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Connect With Road"), "Shift+R");
+  fc_snprintf(cBuf, sizeof(cBuf),
+              _("Connect With %s (%s)"),
+              road_name_translation(road_by_number(ROAD_ROAD)),
+              "Shift+R");
   pBuf = create_themeicon(pTheme->OAutoConnect_Icon, Main.gui,
                           WF_HIDDEN | WF_RESTORE_BACKGROUND
                           | WF_WIDGET_HAS_INFO_LABEL);
@@ -552,7 +556,10 @@ void create_units_order_widgets(void)
   /* --------- */
 
   /* Connect railroad */
-  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Connect With Rail"), "Shift+L");
+  fc_snprintf(cBuf, sizeof(cBuf),
+              _("Connect With %s (%s)"),
+              road_name_translation(road_by_number(ROAD_RAILROAD)),
+              "Shift+L");
   pBuf = create_themeicon(pTheme->OAutoConnect_Icon, Main.gui,
                           WF_HIDDEN | WF_RESTORE_BACKGROUND
                           | WF_WIDGET_HAS_INFO_LABEL);
@@ -812,13 +819,15 @@ void create_units_order_widgets(void)
   /* --------- */    
 
   /* Build (Rail-)Road */
-  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
-			_("Build Railroad"), "R", 999, 
-			PL_("turn", "turns", 999));
+  /* TRANS: "Build Railroad (R) 999 turns" */
+  fc_snprintf(cBuf, sizeof(cBuf), _("Build %s (%s) %d %s"),
+              road_name_translation(road_by_number(ROAD_RAILROAD)), "R", 999, 
+              PL_("turn", "turns", 999));
   len = strlen(cBuf);
-  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
-			_("Build Road"), "R", 999, 
-			PL_("turn", "turns", 999));
+  /* TRANS: "Build Road (R) 999 turns" */
+  fc_snprintf(cBuf, sizeof(cBuf), _("Build %s (%s) %d %s"),
+              road_name_translation(road_by_number(ROAD_ROAD)), "R", 999, 
+              PL_("turn", "turns", 999));
   len = MAX(len, strlen(cBuf));
   
   pBuf = create_themeicon(pTheme->ORoad_Icon, Main.gui,
@@ -1025,7 +1034,7 @@ void real_menus_update(void)
     if (pUnit && !pUnit->ai_controlled) {
       struct city *pHomecity;
       int time;
-      struct tile *pTile = pUnit->tile;
+      struct tile *pTile = unit_tile(pUnit);
       struct city *pCity = tile_city(pTile);
       struct terrain *pTerrain = tile_terrain(pTile);
       struct base_type *pbase;
@@ -1063,16 +1072,20 @@ void real_menus_update(void)
       time = can_unit_do_activity(pUnit, ACTIVITY_RAILROAD);
       if (can_unit_do_activity(pUnit, ACTIVITY_ROAD) || time) {
 	if(time) {
-	  time = tile_activity_time(ACTIVITY_RAILROAD, pUnit->tile);
-	  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
-			_("Build Railroad"), "R", time , 
-			PL_("turn", "turns", time));
+	  time = tile_activity_time(ACTIVITY_RAILROAD, unit_tile(pUnit));
+          /* TRANS: "Build Railroad (R) 3 turns" */
+	  fc_snprintf(cBuf, sizeof(cBuf), _("Build %s (%s) %d %s"),
+                      road_name_translation(road_by_number(ROAD_RAILROAD)),
+                      "R", time, 
+                      PL_("turn", "turns", time));
 	  pOrder_Road_Button->theme = pTheme->ORailRoad_Icon;
 	} else {
-	  time = tile_activity_time(ACTIVITY_ROAD, pUnit->tile);
-	  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
-			_("Build Road"), "R", time , 
-			PL_("turn", "turns", time));
+	  time = tile_activity_time(ACTIVITY_ROAD, unit_tile(pUnit));
+          /* TRANS: "Build Road (R) 1 turn" */
+	  fc_snprintf(cBuf, sizeof(cBuf), _("Build %s (%s) %d %s"),
+		      road_name_translation(road_by_number(ROAD_ROAD)),
+                      "R", time,
+                      PL_("turn", "turns", time));
 	  pOrder_Road_Button->theme = pTheme->ORoad_Icon;
 	}
         copy_chars_to_string16(pOrder_Road_Button->info_label, cBuf);
@@ -1107,7 +1120,7 @@ void real_menus_update(void)
       }
 
       if (can_unit_do_activity(pUnit, ACTIVITY_IRRIGATE)) {
-	time = tile_activity_time(ACTIVITY_IRRIGATE, pUnit->tile);
+	time = tile_activity_time(ACTIVITY_IRRIGATE, unit_tile(pUnit));
 
         if (!strcmp(terrain_rule_name(pTerrain), "Forest") ||
           !strcmp(terrain_rule_name(pTerrain), "Jungle")) {
@@ -1138,7 +1151,7 @@ void real_menus_update(void)
       }
 
       if (can_unit_do_activity(pUnit, ACTIVITY_MINE)) {
-	time = tile_activity_time(ACTIVITY_MINE, pUnit->tile);
+	time = tile_activity_time(ACTIVITY_MINE, unit_tile(pUnit));
 
 	/* FIXME: THIS CODE IS WRONG */
    if (!strcmp(terrain_rule_name(pTerrain), "Forest")) {  
@@ -1173,7 +1186,7 @@ void real_menus_update(void)
       }
 
       if (can_unit_do_activity(pUnit, ACTIVITY_TRANSFORM)) {
-	time = tile_activity_time(ACTIVITY_TRANSFORM, pUnit->tile);
+	time = tile_activity_time(ACTIVITY_TRANSFORM, unit_tile(pUnit));
 	fc_snprintf(cBuf, sizeof(cBuf),"%s %s (%s) %d %s",
 	  _("Transform to"),
 	  terrain_name_translation(pTerrain->transform_result),
@@ -1185,7 +1198,7 @@ void real_menus_update(void)
 	set_wflag(pOrder_Transform_Button, WF_HIDDEN);
       }
 
-      pbase = get_base_by_gui_type(BASE_GUI_FORTRESS, pUnit, pUnit->tile);
+      pbase = get_base_by_gui_type(BASE_GUI_FORTRESS, pUnit, unit_tile(pUnit));
       if (!pCity && pbase) {
 	local_show(ID_UNIT_ORDER_FORTRESS);
       } else {
@@ -1198,7 +1211,7 @@ void real_menus_update(void)
 	local_hide(ID_UNIT_ORDER_FORTIFY);
       }
 
-      pbase = get_base_by_gui_type(BASE_GUI_AIRBASE, pUnit, pUnit->tile);
+      pbase = get_base_by_gui_type(BASE_GUI_AIRBASE, pUnit, unit_tile(pUnit));
       if (!pCity && pbase) {
 	local_show(ID_UNIT_ORDER_AIRBASE);
       } else {
@@ -1242,7 +1255,7 @@ void real_menus_update(void)
 	local_hide(ID_UNIT_ORDER_HOMECITY);
       }
 
-      if (pUnit->occupy && get_transporter_occupancy(pUnit) > 0) {
+      if (pUnit->client.occupied) {
 	local_show(ID_UNIT_ORDER_UNLOAD_TRANSPORTER);
       } else {
 	local_hide(ID_UNIT_ORDER_UNLOAD_TRANSPORTER);
@@ -1260,7 +1273,7 @@ void real_menus_update(void)
         local_hide(ID_UNIT_ORDER_UNLOAD);
       }
       
-      if (is_unit_activity_on_tile(ACTIVITY_SENTRY, pUnit->tile)) {
+      if (is_unit_activity_on_tile(ACTIVITY_SENTRY, unit_tile(pUnit))) {
 	local_show(ID_UNIT_ORDER_WAKEUP_OTHERS);
       } else {
 	local_hide(ID_UNIT_ORDER_WAKEUP_OTHERS);
@@ -1312,7 +1325,7 @@ void real_menus_update(void)
       }
 
       if (is_diplomat_unit(pUnit) &&
-	  diplomat_can_do_action(pUnit, DIPLOMAT_ANY_ACTION, pUnit->tile)) {
+	  diplomat_can_do_action(pUnit, DIPLOMAT_ANY_ACTION, unit_tile(pUnit))) {
 	local_show(ID_UNIT_ORDER_DIPLOMAT_DLG);
       } else {
 	local_hide(ID_UNIT_ORDER_DIPLOMAT_DLG);
