@@ -12,12 +12,14 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
+/* common */
 #include "game.h"
 
-#include "civclient.h"
+/* client */
+#include "client_main.h"
 #include "control.h"
 
 #include "gotodlg.h"
