@@ -13,7 +13,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 /* utility */
@@ -152,7 +152,7 @@ static void xaw_key_end_turn(Widget w, XEvent *event, String *argv, Cardinal *ar
 
 static void xaw_key_focus_to_next_unit(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
-  advance_unit_focus();
+  unit_focus_advance();
 }
 
 static void xaw_key_map_grid_toggle(Widget w, XEvent *event, String *argv, Cardinal *argc)
@@ -169,6 +169,15 @@ static void xaw_key_map_borders_toggle(Widget w, XEvent *event,
   key_map_borders_toggle();
 }
 
+/*************************************************************************
+  Called when the key to toggle native tile display is pressed.
+**************************************************************************/
+static void xaw_key_map_native_toggle(Widget w, XEvent *event,
+                                      String *argv, Cardinal *argc)
+{
+  key_map_native_toggle();
+}
+
 static void xaw_key_move_north(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
   key_unit_move(DIR8_NORTH);
@@ -477,7 +486,7 @@ static void xaw_key_unit_fortify_or_fortress(Widget w, XEvent *event, String *ar
 {
   unit_list_iterate(get_units_in_focus(), punit) {
     struct base_type *pbase = get_base_by_gui_type(BASE_GUI_FORTRESS,
-                                                   punit, punit->tile);
+                                                   punit, unit_tile(punit));
     if (pbase != NULL) {
       key_unit_fortress();
     } else {
@@ -866,6 +875,7 @@ static XtActionsRec Actions[] = {
   { "key-focus-to-next-unit", xaw_key_focus_to_next_unit },
   { "key-map-grid-toggle", xaw_key_map_grid_toggle },
   { "key-map-borders-toggle", xaw_key_map_borders_toggle },
+  { "key-map-native-toggle", xaw_key_map_native_toggle },
   { "key-move-north", xaw_key_move_north },
   { "key-move-north-east", xaw_key_move_north_east },
   { "key-move-east", xaw_key_move_east },
