@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <stdarg.h>
@@ -53,7 +53,7 @@
 #include "cityrep.h"    /* city_report_dialog_popdown() */
 #include "client_main.h"
 #include "climisc.h"
-#include "control.h" /* request_xxx and set_unit_focus */
+#include "control.h" /* request_xxx and unit_focus_set */
 #include "graphics.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
@@ -594,7 +594,7 @@ void popup_pillage_dialog(struct unit *punit,
     BV_CLR_ALL(what_base);
 
     if (what > S_LAST) {
-      BV_SET(what_base, what - S_LAST - 1);
+      BV_SET(what_base, what % (S_LAST + 1));
     } else {
       BV_SET(what_bv, what);
     }
@@ -774,7 +774,7 @@ static int number_of_rows(int n)
 /****************************************************************
 popup the dialog 10% inside the main-window 
 *****************************************************************/
-void popup_unit_select_dialog(struct tile *ptile)
+void unit_select_dialog_popup(struct tile *ptile)
 {
   int i,n,r;
   char buffer[512];
@@ -887,6 +887,14 @@ void popup_unit_select_dialog(struct tile *ptile)
   XtPopup(unit_select_dialog_shell, XtGrabNone);
 }
 
+/**************************************************************************
+  Update the dialog window to select units on a particular tile.
+**************************************************************************/
+void unit_select_dialog_update_real(void)
+{
+  /* PORTME */
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -902,7 +910,7 @@ void unit_select_all_callback(Widget w, XtPointer client_data,
     struct unit *punit = player_unit_by_number(client_player(),
                                                unit_select_ids[i]);
     if(punit) {
-      set_unit_focus(punit);
+      unit_focus_set(punit);
     }
   }
 }
@@ -924,7 +932,7 @@ void unit_select_callback(Widget w, XtPointer client_data,
       struct unit *punit = player_unit_by_number(client_player(),
                                                  unit_select_ids[i]);
       if(punit) {
-	set_unit_focus(punit);
+	unit_focus_set(punit);
       }
       return;
     }
