@@ -13,6 +13,10 @@
 #ifndef FC__HELPDATA_H
 #define FC__HELPDATA_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 #include <stddef.h>		/* size_t */
 
 #include "improvement.h" 	/* Impr_type_id */
@@ -63,4 +67,8 @@ char *helptext_unit_upkeep_str(struct unit_type *punittype);
         while((pitem=help_iter_next())) {   
 #define help_items_iterate_end }}
 
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
 #endif  /* FC__HELPDATA_H */
