@@ -13,16 +13,20 @@
 #ifndef FC__CITYREP_G_H
 #define FC__CITYREP_G_H
 
-#include "shared.h"		/* bool type */
+#include "support.h"            /* bool type */
 
 #include "fc_types.h"
 
-void popup_city_report_dialog(bool raise);
-void city_report_dialog_update(void);
-void city_report_dialog_update_city(struct city *pcity);
+void city_report_dialog_popup(bool raise);
+void real_city_report_dialog_update(void);
+void real_city_report_update_city(struct city *pcity);
 
 /* Area Selection on canvas */
 void hilite_cities_from_canvas(void);
 void toggle_city_hilite(struct city *pcity, bool on_off);
 
+/* Actually defined in update_queue.c */
+void city_report_dialog_update_city(struct city *pcity);
+void city_report_dialog_update(void);
+
 #endif  /* FC__CITYREP_G_H */
