@@ -13,26 +13,11 @@
 #ifndef FC__COLORS_G_H
 #define FC__COLORS_G_H
 
-enum color_std { 
-  COLOR_STD_BLACK, COLOR_STD_WHITE, COLOR_STD_RED,
-  COLOR_STD_YELLOW, COLOR_STD_CYAN,
-  COLOR_STD_GROUND, COLOR_STD_OCEAN,
-  COLOR_STD_BACKGROUND,
-  COLOR_STD_RACE0, COLOR_STD_RACE1, COLOR_STD_RACE2,
-  COLOR_STD_RACE3, COLOR_STD_RACE4, COLOR_STD_RACE5,
-  COLOR_STD_RACE6, COLOR_STD_RACE7, COLOR_STD_RACE8,
-  COLOR_STD_RACE9, COLOR_STD_RACE10, COLOR_STD_RACE11,
-  COLOR_STD_RACE12, COLOR_STD_RACE13,
+#include "colors_common.h"
 
-  COLOR_STD_LAST
-};
+struct color;
 
-enum Display_color_type {
-  BW_DISPLAY, GRAYSCALE_DISPLAY, COLOR_DISPLAY
-};
-
-enum Display_color_type get_visual(void);
-void init_color_system(void);
-void color_error(void);
+struct color *color_alloc(int r, int g, int b);
+void color_free(struct color *color);
 
 #endif  /* FC__COLORS_G_H */
