@@ -16,6 +16,9 @@
 #include "plrdlg_common.h"
 
 void popup_players_dialog(bool raise);
-void update_players_dialog(void);
+void real_players_dialog_update(void);
+
+/* Actually defined in update_queue.c */
+void players_dialog_update(void);
 
 #endif  /* FC__PLRDLG_G_H */
