@@ -14,6 +14,10 @@
 #ifndef FC__MAPCTRL_COMMON_H
 #define FC__MAPCTRL_COMMON_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 /* utility */
 #include "support.h"            /* bool type */
 
@@ -67,4 +71,8 @@ void fill_tile_unit_list(const struct tile *ptile, struct unit **unit_list);
 
 extern struct city *city_workers_display;
 
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
 #endif /* FC__MAPCTRL_COMMON_H */
