@@ -9,7 +9,7 @@
 
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 /* common */
@@ -47,18 +47,14 @@ bool client_handle_packet(enum packet_type type, const void *packet)
     handle_server_shutdown();
     return TRUE;
 
-  case PACKET_ENDGAME_REPORT_OLD:
-    handle_endgame_report_old(packet);
+  case PACKET_ENDGAME_REPORT:
+    handle_endgame_report(packet);
     return TRUE;
 
   case PACKET_ENDGAME_PLAYER:
     handle_endgame_player(packet);
     return TRUE;
 
-  case PACKET_ENDGAME_REPORT_NEW:
-    handle_endgame_report_new(packet);
-    return TRUE;
-
   case PACKET_TILE_INFO:
     handle_tile_info(packet);
     return TRUE;
