@@ -10,6 +10,10 @@
 #ifndef FC__PACKHAND_GEN_H
 #define FC__PACKHAND_GEN_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 /* utility */
 #include "shared.h"
 
@@ -23,12 +27,10 @@ void handle_processing_finished(void);
 void handle_server_join_reply(bool you_can_join, const char *message, const char *capability, const char *challenge_file, int conn_id);
 void handle_authentication_req(enum authentication_type type, const char *message);
 void handle_server_shutdown(void);
-struct packet_endgame_report_old;
-void handle_endgame_report_old(const struct packet_endgame_report_old *packet);
+struct packet_endgame_report;
+void handle_endgame_report(const struct packet_endgame_report *packet);
 struct packet_endgame_player;
 void handle_endgame_player(const struct packet_endgame_player *packet);
-struct packet_endgame_report_new;
-void handle_endgame_report_new(const struct packet_endgame_report_new *packet);
 struct packet_tile_info;
 void handle_tile_info(const struct packet_tile_info *packet);
 struct packet_game_info;
@@ -146,4 +148,8 @@ struct packet_edit_startpos_full;
 void handle_edit_startpos_full(const struct packet_edit_startpos_full *packet);
 void handle_edit_object_created(int tag, int id);
 
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
 #endif /* FC__PACKHAND_GEN_H */
