@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <string.h>
@@ -29,6 +29,7 @@
 /* client */
 #include "client_main.h"
 #include "climisc.h"
+#include "options.h"
 #include "text.h"
 
 /* client/include */
@@ -177,13 +178,39 @@ const char *plrdlg_col_state(const struct player *plr)
     /* TRANS: Dead -- Rest In Peace -- Reqia In Pace */
     return _("R.I.P.");
   } else if (!plr->is_connected) {
-    return "";
-  } else if (!is_player_phase(plr, game.info.phase)) {
-    return _("waiting");
-  } else if (plr->phase_done) {
-    return _("done");
+    struct option *opt;
+    bool consider_tb = FALSE;
+
+    if (plr->ai_controlled) {
+      return "";
+    }
+
+    opt = optset_option_by_name(server_optset, "turnblock");
+    if (opt != NULL) {
+      consider_tb = option_bool_get(opt);
+    }
+
+    if (!consider_tb) {
+      /* TRANS: No connection */
+      return _("noconn");
+    }
+
+    if (!is_player_phase(plr, game.info.phase)) {
+      return _("waiting");
+    } else if (plr->phase_done) {
+      return _("done");
+    } else {
+      /* TRANS: Turnblocking & player not connected */
+      return _("blocking");
+    }
   } else {
-    return _("moving");
+    if (!is_player_phase(plr, game.info.phase)) {
+      return _("waiting");
+    } else if (plr->phase_done) {
+      return _("done");
+    } else {
+      return _("moving");
+    }
   }
 }
 
@@ -247,7 +274,7 @@ struct player_dlg_column player_dlg_columns[] = {
 const int num_player_dlg_columns = ARRAY_SIZE(player_dlg_columns);
 
 /******************************************************************
- ...
+  Return default player dlg sorting column.
 *******************************************************************/
 int player_dlg_default_sort_column(void)
 {
