@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <errno.h>
@@ -55,6 +55,9 @@
 #include <winsock.h>
 #endif
 
+/* dependencies */
+#include "cvercmp.h"
+
 /* utility */
 #include "fcintl.h"
 #include "log.h"
@@ -72,6 +75,7 @@
 #include "version.h"
 
 /* client */
+#include "chatline_common.h"
 #include "client_main.h"
 #include "servers.h"
 
@@ -109,12 +113,38 @@ static struct server_list *parse_metaserver_data(fz_FILE *f)
   struct server_list *server_list;
   struct section_file *file;
   int nservers, i, j;
+  const char *latest_ver;
 
   /* This call closes f. */
   if (!(file = secfile_from_stream(f, TRUE))) {
     return NULL;
   }
 
+  latest_ver = secfile_lookup_str_default(file, NULL, "versions." FOLLOWTAG);
+
+  if (latest_ver != NULL) {
+    const char *my_comparable = fc_comparable_version();
+    char vertext[2048];
+
+    log_verbose("Metaserver says latest '%s' version is '%s'; we have '%s'",
+                FOLLOWTAG, latest_ver, my_comparable);
+    if (cvercmp_greater(latest_ver, my_comparable)) {
+      const char *const followtag = "?vertag:" FOLLOWTAG;
+      fc_snprintf(vertext, sizeof(vertext),
+                  /* TRANS: Type is version tag name like "stable", "S2_4",
+                   * "win32" (which can also be localised -- msgids start
+                   * '?vertag:') */
+                  _("Latest %s release of Freeciv is %s, this is %s."),
+                  Q_(followtag), latest_ver, my_comparable);
+    } else {
+      fc_snprintf(vertext, sizeof(vertext),
+                  _("There is no newer %s release of Freeciv available."),
+                  FOLLOWTAG);
+    }
+
+    output_window_append(ftc_client, vertext);
+  }
+
   server_list = server_list_new();
   nservers = secfile_lookup_int_default(file, 0, "main.nservers");
 
@@ -278,9 +308,9 @@ static void my_uname(char *buf, size_t len)
     fc_snprintf(buf, len, "%s %ld.%ld [%s]",
                 osname, osvi.dwMajorVersion, osvi.dwMinorVersion, cpuname);
   }
-#else
+#else  /* WIN32_NATIVE */
   fc_snprintf(buf, len, "unknown unknown [unknown]");
-#endif
+#endif /* WIN32_NATIVE */
 #endif /* HAVE_UNAME */
 }
 
@@ -339,7 +369,7 @@ static void meta_read_response(struct server_scan *scan)
     scan->meta.fp = fc_fopen(filename, "w+b");
 #else
     scan->meta.fp = tmpfile();
-#endif
+#endif /* WIN32_NATIVE */
 
     if (!scan->meta.fp) {
       scan->error_func(scan, _("Could not open temp file."));
@@ -414,8 +444,8 @@ static void meta_read_response(struct server_scan *scan)
 ****************************************************************************/
 static bool begin_metaserver_scan(struct server_scan *scan)
 {
-  union fc_sockaddr names[2];
-  int i, name_count;
+  struct fc_sockaddr_list *list;
+  int name_count;
   int s = -1;
 
   scan->meta.urlpath = fc_lookup_httpd(scan->meta.name, &scan->meta.port,
@@ -427,34 +457,31 @@ static bool begin_metaserver_scan(struct server_scan *scan)
     return FALSE;
   }
 
-  if (!net_lookup_service(scan->meta.name, scan->meta.port,
-                          &names[0], FALSE)) {
+  name_count = 0;
+
+#ifdef IPV6_SUPPORT
+  list = net_lookup_service(scan->meta.name, scan->meta.port, FC_ADDR_ANY);
+#else  /* IPV6_SUPPORT */
+  list = net_lookup_service(scan->meta.name, scan->meta.port, FC_ADDR_IPV4);
+#endif /* IPV6_SUPPORT */
+
+  name_count = fc_sockaddr_list_size(list);
+
+  if (name_count <= 0) {
     scan->error_func(scan, _("Failed looking up metaserver's host"));
     return FALSE;
   }
-  name_count = 1;
-#ifdef IPV6_SUPPORT
-  if (names[0].saddr.sa_family == AF_INET6) {
-    /* net_lookup_service() prefers IPv6 address.
-     * Check if there is also IPv4 address.
-     * TODO: This would be easier using getaddrinfo() */
-    if (net_lookup_service(scan->meta.name, scan->meta.port,
-                           &names[1], TRUE /* force IPv4 */)) {
-      name_count = 2;
-    }
-  }
-#endif
 
-  /* Try all (IPv4, IPv6, ...) addresses until we have a connection. */  
-  for (i = 0; i < name_count; i++) {
-    if ((s = socket(names[i].saddr.sa_family, SOCK_STREAM, 0)) == -1) {
+  /* Try all addresses until we have a connection. */  
+  fc_sockaddr_list_iterate(list, paddr) {
+    if ((s = socket(paddr->saddr.sa_family, SOCK_STREAM, 0)) == -1) {
       /* Probably EAFNOSUPPORT or EPROTONOSUPPORT. */
       continue;
     }
 
     fc_nonblock(s);
   
-    if (fc_connect(s, &names[i].saddr, sockaddr_size(&names[i])) == -1) {
+    if (fc_connect(s, &paddr->saddr, sockaddr_size(paddr)) == -1) {
       if (errno == EINPROGRESS) {
         /* With non-blocking sockets this is the expected result. */
         scan->meta.state = META_CONNECTING;
@@ -472,7 +499,9 @@ static bool begin_metaserver_scan(struct server_scan *scan)
       meta_send_request(scan);
       break;
     }
-  }
+  } fc_sockaddr_list_iterate_end;
+
+  fc_sockaddr_list_destroy(list);
 
   if (s == -1) {
     scan->error_func(scan, fc_strerror(fc_get_errno()));
@@ -602,7 +631,7 @@ static bool begin_lanserver_scan(struct server_scan *scan)
   if (announce == ANNOUNCE_IPV6) {
     family = AF_INET6;
   } else
-#endif
+#endif /* IPv6 support */
   {
     family = AF_INET;
   }
@@ -623,7 +652,7 @@ static bool begin_lanserver_scan(struct server_scan *scan)
   memset(&addr, 0, sizeof(addr));
 
 #ifndef IPV6_SUPPORT
-  {
+  if (family == AF_INET) {
 #ifdef HAVE_INET_ATON
     inet_aton(group, &addr.saddr_in4.sin_addr);
 #else  /* HAVE_INET_ATON */
@@ -634,11 +663,17 @@ static bool begin_lanserver_scan(struct server_scan *scan)
     addr.saddr.sa_family = AF_INET6;
     inet_pton(AF_INET6, group, &addr.saddr_in6.sin6_addr);
     addr.saddr_in6.sin6_port = htons(SERVER_LAN_PORT);
-  } else {
+  } else if (family == AF_INET) {
     inet_pton(AF_INET, group, &addr.saddr_in4.sin_addr);
 #endif /* IPv6 support */
     addr.saddr.sa_family = AF_INET;
     addr.saddr_in4.sin_port = htons(SERVER_LAN_PORT);
+  } else {
+    fc_assert(FALSE);
+
+    log_error("Unsupported address family in begin_lanserver_scan()");
+
+    return FALSE;
   }
 
 /* this setsockopt call fails on Windows 98, so we stick with the default
@@ -699,10 +734,17 @@ static bool begin_lanserver_scan(struct server_scan *scan)
     addr.saddr_in6.sin6_addr = in6addr_any;
   } else
 #endif /* IPv6 support */
-  {
+  if (family == AF_INET) {
     addr.saddr.sa_family = AF_INET;
     addr.saddr_in4.sin_port = htons(SERVER_LAN_PORT + 1);
     addr.saddr_in4.sin_addr.s_addr = htonl(INADDR_ANY);
+  } else {
+    /* This is not only error situation worth assert() This
+     * is error situation that has check (with assert) against
+     * earlier already. */
+    fc_assert(FALSE);
+
+    return FALSE;
   }
 
   if (bind(scan->sock, &addr.saddr, sockaddr_size(&addr)) < 0) {
@@ -761,6 +803,7 @@ get_lan_server_list(struct server_scan *scan)
   char version[256];
   char status[256];
   char players[256];
+  char humans[256];
   char message[1024];
   bool found_new = FALSE;
 
@@ -788,6 +831,7 @@ get_lan_server_list(struct server_scan *scan)
     dio_get_string(&din, version, sizeof(version));
     dio_get_string(&din, status, sizeof(status));
     dio_get_string(&din, players, sizeof(players));
+    dio_get_string(&din, humans, sizeof(humans));
     dio_get_string(&din, message, sizeof(message));
 
     if (!fc_strcasecmp("none", servername)) {
@@ -804,9 +848,15 @@ get_lan_server_list(struct server_scan *scan)
         if (fromend.saddr.sa_family == AF_INET6) {
           inet_ntop(AF_INET6, &fromend.saddr_in6.sin6_addr,
                     dst, sizeof(dst));
-        } else {
+        } else if (fromend.saddr.sa_family == AF_INET) {
           inet_ntop(AF_INET, &fromend.saddr_in4.sin_addr, dst, sizeof(dst));;
-        }
+        } else {
+	  fc_assert(FALSE);
+
+	  log_error("Unsupported address family in get_lan_server_list()");
+
+	  fc_snprintf(dst, sizeof(dst), "Unknown");
+	}
       }
 #else  /* IPv6 support */
       const char *dst = NULL;
@@ -847,7 +897,7 @@ get_lan_server_list(struct server_scan *scan)
     pserver->version = fc_strdup(version);
     pserver->state = fc_strdup(status);
     pserver->nplayers = atoi(players);
-    pserver->humans = -1;
+    pserver->humans = atoi(humans);
     pserver->message = fc_strdup(message);
     pserver->players = NULL;
     found_new = TRUE;
