@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <stdarg.h>
@@ -150,12 +150,15 @@ const char *popup_info_text(struct tile *ptile)
   static struct astring str = ASTRING_INIT;
   char username[MAX_LEN_NAME + 32];
   char nation[2 * MAX_LEN_NAME + 32];
+  int tile_x, tile_y, nat_x, nat_y;
 
   astr_clear(&str);
+  index_to_map_pos(&tile_x, &tile_y, tile_index(ptile));
   astr_add_line(&str, _("Location: (%d, %d) [%d]"),
-                ptile->x, ptile->y, tile_continent(ptile));
+                tile_x, tile_y, tile_continent(ptile));
+  index_to_native_pos(&nat_x, &nat_y, tile_index(ptile));
   astr_add_line(&str, _("Native coordinates: (%d, %d)"),
-                ptile->nat_x, ptile->nat_y);
+                nat_x, nat_y);
 
   if (client_tile_get_known(ptile) == TILE_UNKNOWN) {
     astr_add(&str, _("Unknown"));
@@ -365,14 +368,29 @@ const char *popup_info_text(struct tile *ptile)
 
     /* TRANS: A is attack power, D is defense power, FP is firepower,
      * HP is hitpoints (current and max). */
-    astr_add_line(&str, _("A:%d D:%d FP:%d HP:%d/%d (%s)"),
-		  ptype->attack_strength, 
-		  ptype->defense_strength, ptype->firepower, punit->hp, 
-		  ptype->hp,
-                  name_translation(&ptype->veteran[punit->veteran].name));
+    astr_add_line(&str, _("A:%d D:%d FP:%d HP:%d/%d"),
+                  ptype->attack_strength, ptype->defense_strength,
+                  ptype->firepower, punit->hp, ptype->hp);
+    {
+      const char *veteran_name =
+        utype_veteran_name_translation(ptype, punit->veteran);
+      if (veteran_name) {
+        astr_add(&str, " (%s)", veteran_name);
+      }
+    }
+
+    if (unit_owner(punit) == client_player()
+        || client_is_global_observer()) {
+      /* Show bribe cost for own units. */
+      astr_add_line(&str, _("Bribe cost: %d"), unit_bribe_cost(punit));
+    } else {
+      /* We can only give an (lower) boundary for units of other players. */
+      astr_add_line(&str, _("Estimated bribe cost: > %d"),
+                    unit_bribe_cost(punit));
+    }
 
     if ((NULL == client.conn.playing || owner == client.conn.playing)
-	&& unit_list_size(ptile->units) >= 2) {
+        && unit_list_size(ptile->units) >= 2) {
       /* TRANS: "5 more" units on this tile */
       astr_add(&str, _("  (%d more)"), unit_list_size(ptile->units) - 1);
     }
@@ -524,8 +542,12 @@ const char *unit_description(struct unit *punit)
 
   astr_add(&str, "%s", utype_name_translation(ptype));
 
-  if (rule_name(&ptype->veteran[punit->veteran].name)[0] != '\0') {
-    astr_add(&str, " (%s)", name_translation(&ptype->veteran[punit->veteran].name));
+  {
+    const char *veteran_name =
+      utype_veteran_name_translation(ptype, punit->veteran);
+    if (veteran_name) {
+      astr_add(&str, " (%s)", veteran_name);
+    }
   }
 
   if (pplayer == unit_owner(punit)) {
@@ -707,7 +729,7 @@ const char *science_dialog_text(void)
   bool team;
   int ours, theirs, perturn, upkeep;
   static struct astring str = ASTRING_INIT;
-  char ourbuf[1024] = "", theirbuf[1024] = "";
+  struct astring ourbuf = ASTRING_INIT, theirbuf = ASTRING_INIT;
   struct player_research *research;
 
   astr_clear(&str);
@@ -746,16 +768,19 @@ const char *science_dialog_text(void)
       astr_add(&str, _("Progress: none"));
     }
   }
-  fc_snprintf(ourbuf, sizeof(ourbuf),
-              PL_("%d bulb/turn", "%d bulbs/turn", ours), ours);
+  astr_set(&ourbuf, PL_("%d bulb/turn", "%d bulbs/turn", ours), ours);
   if (team) {
     /* Techpool version */
-    fc_snprintf(theirbuf, sizeof(theirbuf),
-                /* TRANS: This is appended to "%d bulb/turn" text */
-                PL_(", %d bulb/turn from team",
-                    ", %d bulbs/turn from team", theirs), theirs);
+    astr_set(&theirbuf,
+             /* TRANS: This is appended to "%d bulb/turn" text */
+             PL_(", %d bulb/turn from team",
+                 ", %d bulbs/turn from team", theirs), theirs);
+  } else {
+    astr_clear(&theirbuf);
   }
-  astr_add(&str, " (%s%s)", ourbuf, theirbuf);
+  astr_add(&str, " (%s%s)", astr_str(&ourbuf), astr_str(&theirbuf));
+  astr_free(&ourbuf);
+  astr_free(&theirbuf);
 
   if (game.info.tech_upkeep_style == 1) {
     /* perturn is defined as: (bulbs produced) - upkeep */
@@ -829,9 +854,11 @@ const char *get_science_goal_text(Tech_type_id goal)
   int bulbs_needed = total_bulbs_required_for_goal(client.conn.playing, goal);
   int turns;
   int perturn = get_bulbs_per_turn(NULL, NULL, NULL);
-  char buf1[256], buf2[256], buf3[256];
   struct player_research* research = player_research_get(client_player());
   static struct astring str = ASTRING_INIT;
+  struct astring buf1 = ASTRING_INIT,
+                 buf2 = ASTRING_INIT,
+                 buf3 = ASTRING_INIT;
 
   if (!research) {
     return "-";
@@ -845,18 +872,22 @@ const char *get_science_goal_text(Tech_type_id goal)
     bulbs_needed -= research->bulbs_researched;
   }
 
-  fc_snprintf(buf1, sizeof(buf1),
-              PL_("%d step", "%d steps", steps), steps);
-  fc_snprintf(buf2, sizeof(buf2),
-              PL_("%d bulb", "%d bulbs", bulbs_needed), bulbs_needed);
+  astr_set(&buf1,
+           PL_("%d step", "%d steps", steps), steps);
+  astr_set(&buf2,
+           PL_("%d bulb", "%d bulbs", bulbs_needed), bulbs_needed);
   if (perturn > 0) {
     turns = (bulbs_needed + perturn - 1) / perturn;
-    fc_snprintf(buf3, sizeof(buf3),
-                PL_("%d turn", "%d turns", turns), turns);
+    astr_set(&buf3,
+             PL_("%d turn", "%d turns", turns), turns);
   } else {
-    fc_snprintf(buf3, sizeof(buf3), _("never"));
+    astr_set(&buf3, _("never"));
   }
-  astr_add_line(&str, "(%s - %s - %s)", buf1, buf2, buf3);
+  astr_add_line(&str, "(%s - %s - %s)",
+                astr_str(&buf1), astr_str(&buf2), astr_str(&buf3));
+  astr_free(&buf1);
+  astr_free(&buf2);
+  astr_free(&buf3);
 
   return astr_str(&str);
 }
@@ -931,6 +962,9 @@ const char *get_info_label_text_popup(void)
   astr_add_line(&str, _("Turn: %d"), game.info.turn);
 
   if (NULL != client.conn.playing) {
+    int perturn = get_bulbs_per_turn(NULL, NULL, NULL);
+    int upkeep = player_research_get(client_player())->tech_upkeep;
+
     astr_add_line(&str, _("Gold: %d"),
 		  client.conn.playing->economic.gold);
     astr_add_line(&str, _("Net Income: %d"),
@@ -943,13 +977,13 @@ const char *get_info_label_text_popup(void)
     astr_add_line(&str, _("Researching %s: %s"),
 		  advance_name_researching(client.conn.playing),
 		  get_science_target_text(NULL));
+    /* perturn is defined as: (bulbs produced) - upkeep */
     if (game.info.tech_upkeep_style == 1) {
-      int perturn = get_bulbs_per_turn(NULL, NULL, NULL);
-      int upkeep = player_research_get(client_player())->tech_upkeep;
-
-      /* perturn is defined as: (bulbs produced) - upkeep */
       astr_add_line(&str, _("Bulbs per turn: %d - %d = %d"), perturn + upkeep,
                     upkeep, perturn);
+    } else {
+      fc_assert(upkeep == 0);
+      astr_add_line(&str, _("Bulbs per turn: %d"), perturn);
     }
   }
 
@@ -1058,10 +1092,12 @@ const char *get_unit_info_label_text2(struct unit_list *punits, int linebreaks)
     struct city *pcity = player_city_by_number(unit_owner(punit),
                                                punit->homecity);
 
-    astr_add_line(&str, "%s", tile_get_info_text(punit->tile, linebreaks));
+    astr_add_line(&str, "%s", tile_get_info_text(unit_tile(punit),
+                                                 linebreaks));
     {
-      const char *infratext = get_infrastructure_text(punit->tile->special,
-                                                      punit->tile->bases);
+      const char *infratext
+        = get_infrastructure_text(unit_tile(punit)->special,
+                                  unit_tile(punit)->bases);
       if (*infratext != '\0') {
         astr_add_line(&str, "%s", infratext);
       } else {
@@ -1149,7 +1185,7 @@ const char *get_unit_info_label_text2(struct unit_list *punits, int linebreaks)
   } else {
     astr_add_line(&str, " ");
   }
-#endif
+#endif /* DEBUG */
 
   return astr_str(&str);
 }
@@ -1279,11 +1315,36 @@ const char *get_bulb_tooltip(void)
     if (research->researching == A_UNSET) {
       astr_add_line(&str, _("no research target."));
     } else {
+      int turns = 0;
+      int perturn = get_bulbs_per_turn(NULL, NULL, NULL);
+      int done = research->bulbs_researched;
+      int total = total_bulbs_required(client_player());
+      struct astring buf1 = ASTRING_INIT, buf2 = ASTRING_INIT;
+
+      if (perturn > 0) {
+        turns = MAX(1, ceil((double) (total - done) / perturn));
+      } else if (perturn < 0 ) {
+        turns = ceil((double) done / -perturn);
+      }
+
+      if (turns == 0) {
+        astr_set(&buf1, _("No progress"));
+      } else {
+        astr_set(&buf1, PL_("%d turn", "%d turns", turns), turns);
+      }
+
+      /* TRANS: <perturn> bulbs/turn */
+      astr_set(&buf2, PL_("%d bulb/turn", "%d bulbs/turn", perturn), perturn);
+
       /* TRANS: <tech>: <amount>/<total bulbs> */
-      astr_add_line(&str, _("%s: %d/%d."),
-		    advance_name_researching(client.conn.playing),
-		    research->bulbs_researched,
-		    total_bulbs_required(client.conn.playing));
+      astr_add_line(&str, _("%s: %d/%d (%s, %s)."),
+                    advance_name_researching(client.conn.playing),
+                    research->bulbs_researched,
+                    total_bulbs_required(client.conn.playing),
+                    astr_str(&buf1), astr_str(&buf2));
+      
+      astr_free(&buf1);
+      astr_free(&buf2);
     }
   }
   return astr_str(&str);
@@ -1532,13 +1593,13 @@ const char *get_report_title(const char *report_name)
                   nation_adjective_for_player(pplayer),
                   government_name_for_player(pplayer));
 
-    /* TRANS: Just happending 2 strings, using the correct localized
+    /* TRANS: Just appending 2 strings, using the correct localized
      * syntax. */
     astr_add_line(&str, _("%s - %s"),
                   ruler_title_for_player(pplayer, buf, sizeof(buf)),
                   textyear(game.info.year));
   } else {
-    /* TRANS: "Observer: 1985" */
+    /* TRANS: "Observer - 1985 AD" */
     astr_add_line(&str, _("Observer - %s"),
 		  textyear(game.info.year));
   }
