@@ -12,20 +12,21 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <stdio.h>
 #include <string.h>
 
-#include "city.h"
-#include "game.h"
-#ifdef DEBUG
+/* utility */
 #include "log.h"
-#endif
-#include "map.h"
 #include "mem.h"
 #include "support.h"
+
+/* common */
+#include "city.h"
+#include "game.h"
+#include "map.h"
 #include "unit.h"
 #include "unitlist.h"
 #include "unittype.h"
@@ -86,7 +87,7 @@ void citymap_turn_init(struct player *pplayer)
 
   unit_list_iterate(pplayer->units, punit) {
     if (unit_has_type_flag(punit, F_CITIES)
-        && punit->server.adv->role == AIUNIT_BUILD_CITY) {
+        && punit->server.adv->task == AUT_BUILD_CITY) {
 
       /* use default (squared) city radius */
       city_tile_iterate(CITY_MAP_DEFAULT_RADIUS_SQ, punit->goto_tile,
