@@ -271,7 +271,10 @@
 enum tile_behavior {
   TB_IGNORE = 0,        /* This one will be ignored. */
   TB_DONT_LEAVE,        /* Paths can lead _to_ such tile, but are not
-                         * allowed to go _through_. */
+                         * allowed to go _through_. This move cost is
+                         * always evaluated to a constant single move cost,
+                         * prefering straight paths because we don't need
+                         * moves left to leave this tile. */
   TB_NORMAL             /* Well, normal .*/
 };
 
@@ -441,7 +444,7 @@ void pf_path_print_real(const struct pf_path *path, enum log_level level,
                         const char *file, const char *function, int line);
 #define pf_path_print(path, level)                                          \
   if (log_do_output_for_level(level)) {                                     \
-    pf_path_print_real(path, level, __FILE__, __FUNCTION__, __LINE__);      \
+    pf_path_print_real(path, level, __FILE__, __FUNCTION__, __FC_LINE__);   \
   }
 
 
