@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <errno.h>
@@ -100,7 +100,8 @@ void connection_close(struct connection *pconn, const char *reason)
 
 
 /**************************************************************************
-...
+  Make sure that there is at least extra_space bytes free space in buffer,
+  allocating more memory if needed.
 **************************************************************************/
 static bool buffer_ensure_free_extra_space(struct socket_packet_buffer *buf,
 					   int extra_space)
@@ -515,7 +516,7 @@ int get_next_request_id(int old_request_id)
 }
 
 /**************************************************************************
- ...
+  Free compression queue for given connection.
 **************************************************************************/
 void free_compression_queue(struct connection *pc)
 {
@@ -525,7 +526,7 @@ void free_compression_queue(struct connection *pc)
 }
 
 /**************************************************************************
- ...
+  Allocate and initialize packet hashs for given connection.
 **************************************************************************/
 static void init_packet_hashs(struct connection *pc)
 {
@@ -543,7 +544,7 @@ static void init_packet_hashs(struct connection *pc)
 }
 
 /**************************************************************************
- ...
+  Free packet hash resources from given connection.
 **************************************************************************/
 static void free_packet_hashes(struct connection *pc)
 {
@@ -576,7 +577,8 @@ static void free_packet_hashes(struct connection *pc)
 }
 
 /**************************************************************************
- ...
+  Initialize common part of connection structure. This is used by
+  both server and client.
 **************************************************************************/
 void connection_common_init(struct connection *pconn)
 {
@@ -597,7 +599,7 @@ void connection_common_init(struct connection *pconn)
 }
 
 /**************************************************************************
- ...
+   Connection closing part common to server and client.
 **************************************************************************/
 void connection_common_close(struct connection *pconn)
 {
@@ -886,3 +888,12 @@ struct conn_pattern *conn_pattern_from_string(const char *pattern,
 
   return  conn_pattern_new(type, p);
 }
+
+/**************************************************************************
+  Returns TRUE if the connection is valid, i.e. not NULL, not closed, not
+  closing, etc.
+**************************************************************************/
+bool conn_is_valid(const struct connection *pconn)
+{
+  return (pconn && pconn->used && !pconn->server.is_closing);
+}
