@@ -20,7 +20,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <limits.h>
@@ -61,13 +61,14 @@
 
 #include "dataio.h"
 
-/**************************************************************************
-...
-**************************************************************************/
+static bool get_conv(char *dst, size_t ndst, const char *src,
+		     size_t nsrc);
+
 static DIO_PUT_CONV_FUN put_conv_callback = NULL;
+static DIO_GET_CONV_FUN get_conv_callback = get_conv;
 
 /**************************************************************************
-...
+  Sets string conversion callback to be used when putting text.
 **************************************************************************/
 void dio_set_put_conv_callback(DIO_PUT_CONV_FUN fun)
 {
@@ -76,7 +77,7 @@ void dio_set_put_conv_callback(DIO_PUT_CONV_FUN fun)
 
 /**************************************************************************
  Returns FALSE if the destination isn't large enough or the source was
- bad.
+ bad. This is default get_conv_callback.
 **************************************************************************/
 static bool get_conv(char *dst, size_t ndst, const char *src,
 		     size_t nsrc)
@@ -96,12 +97,7 @@ static bool get_conv(char *dst, size_t ndst, const char *src,
 }
 
 /**************************************************************************
-...
-**************************************************************************/
-static DIO_GET_CONV_FUN get_conv_callback = get_conv;
-
-/**************************************************************************
-...
+  Sets string conversion callback to use when getting text.
 **************************************************************************/
 void dio_set_get_conv_callback(DIO_GET_CONV_FUN fun)
 {
@@ -176,6 +172,7 @@ void dio_input_init(struct data_in *din, const void *src, size_t src_size)
   din->src_size = src_size;
   din->current = 0;
   din->too_short = FALSE;
+  din->bad_boolean = FALSE;
   din->bad_string = FALSE;
   din->bad_bit_string = FALSE;
 }
@@ -198,7 +195,20 @@ size_t dio_input_remaining(struct data_in *din)
 }
 
 /**************************************************************************
-...
+   Skips 'n' bytes.
+**************************************************************************/
+bool dio_input_skip(struct data_in *din, size_t size)
+{
+  if (enough_data(din, size)) {
+    din->current += size;
+    return TRUE;
+  } else {
+    return FALSE;
+  }
+}
+
+/**************************************************************************
+  Insert value using 8 bits. May overflow.
 **************************************************************************/
 void dio_put_uint8(struct data_out *dout, int value)
 {
@@ -212,7 +222,7 @@ void dio_put_uint8(struct data_out *dout, int value)
 }
 
 /**************************************************************************
-...
+  Insert value using 16 bits. May overflow.
 **************************************************************************/
 void dio_put_uint16(struct data_out *dout, int value)
 {
@@ -226,7 +236,7 @@ void dio_put_uint16(struct data_out *dout, int value)
 }
 
 /**************************************************************************
-...
+  Insert value using 16 bits.
 **************************************************************************/
 void dio_put_uint32(struct data_out *dout, int value)
 {
@@ -240,7 +250,7 @@ void dio_put_uint32(struct data_out *dout, int value)
 }
 
 /**************************************************************************
-...
+  Insert value 0 or 1 using 8 bits.
 **************************************************************************/
 void dio_put_bool8(struct data_out *dout, bool value)
 {
@@ -253,7 +263,7 @@ void dio_put_bool8(struct data_out *dout, bool value)
 }
 
 /**************************************************************************
-...
+  Insert value 0 or 1 using 32 bits.
 **************************************************************************/
 void dio_put_bool32(struct data_out *dout, bool value)
 {
@@ -266,7 +276,9 @@ void dio_put_bool32(struct data_out *dout, bool value)
 }
 
 /**************************************************************************
-...
+  Insert number of values brefore stop_value using 8 bits. Then
+  insert values using 8 bits for each. stop_value is not required to
+  fit in 8 bits. Actual values may overflow.
 **************************************************************************/
 void dio_put_uint8_vec8(struct data_out *dout, int *values, int stop_value)
 {
@@ -288,7 +300,9 @@ void dio_put_uint8_vec8(struct data_out *dout, int *values, int stop_value)
 }
 
 /**************************************************************************
-...
+  Insert number of values brefore stop_value using 8 bits. Then
+  insert values using 16 bits for each. stop_value is not required to
+  fit in 16 bits. Actual values may overflow.
 **************************************************************************/
 void dio_put_uint16_vec8(struct data_out *dout, int *values, int stop_value)
 {
@@ -310,7 +324,7 @@ void dio_put_uint16_vec8(struct data_out *dout, int *values, int stop_value)
 }
 
 /**************************************************************************
-...
+  Insert block directly from memory.
 **************************************************************************/
 void dio_put_memory(struct data_out *dout, const void *value, size_t size)
 {
@@ -321,7 +335,7 @@ void dio_put_memory(struct data_out *dout, const void *value, size_t size)
 }
 
 /**************************************************************************
-...
+  Insert NULL-terminated string. Conversion callback is used if set.
 **************************************************************************/
 void dio_put_string(struct data_out *dout, const char *value)
 {
@@ -339,7 +353,9 @@ void dio_put_string(struct data_out *dout, const char *value)
 }
 
 /**************************************************************************
-...
+  Insert number of bits as 16 bit value and then insert bits. In incoming
+  value string each bit is represented by one character, value '1' indicating
+  TRUE bit.
 **************************************************************************/
 void dio_put_bit_string(struct data_out *dout, const char *value)
 {
@@ -374,7 +390,8 @@ void dio_put_bit_string(struct data_out *dout, const char *value)
 }
 
 /**************************************************************************
-...
+  Insert tech numbers from value array as 8 bit values until there is value
+  A_LAST or MAX_NUM_TECH_LIST tech numbers have been inserted.
 **************************************************************************/
 void dio_put_tech_list(struct data_out *dout, const int *value)
 {
@@ -389,7 +406,40 @@ void dio_put_tech_list(struct data_out *dout, const int *value)
 }
 
 /**************************************************************************
-...
+  Insert unit type numbers from value array as 8 bit values until there is
+  value U_LAST or MAX_NUM_UNIT_LIST numbers have been inserted.
+**************************************************************************/
+void dio_put_unit_list(struct data_out *dout, const int *value)
+{
+  int i;
+
+  for (i = 0; i < MAX_NUM_UNIT_LIST; i++) {
+    dio_put_uint8(dout, value[i]);
+    if (value[i] == U_LAST) {
+      break;
+    }
+  }
+}
+
+/**************************************************************************
+  Insert building type numbers from value array as 8 bit values until there
+  is value B_LAST or MAX_NUM_BUILDING_LIST numbers have been inserted.
+**************************************************************************/
+void dio_put_building_list(struct data_out *dout, const int *value)
+{
+  int i;
+
+  for (i = 0; i < MAX_NUM_BUILDING_LIST; i++) {
+    dio_put_uint8(dout, value[i]);
+    if (value[i] == B_LAST) {
+      break;
+    }
+  }
+}
+
+/**************************************************************************
+  Insert number of worklist items as 8 bit value and then insert
+  8 bit kind and 8 bit number for each worklist item.
 **************************************************************************/
 void dio_put_worklist(struct data_out *dout, const struct worklist *pwl)
 {
@@ -405,185 +455,160 @@ void dio_put_worklist(struct data_out *dout, const struct worklist *pwl)
 }
 
 /**************************************************************************
- Receive uint8 value to dest. In case of failure, value stored to dest
- will be zero. Note that zero is legal value even when there is no failure.
+ Receive uint8 value to dest.
 **************************************************************************/
 bool dio_get_uint8(struct data_in *din, int *dest)
 {
-  if (enough_data(din, 1)) {
-    if (dest) {
-      uint8_t x;
-
-      fc_assert(sizeof(x) == 1);
-      memcpy(&x, ADD_TO_POINTER(din->src, din->current), 1);
-      *dest = x;
-    }
-    din->current++;
-  } else if (dest) {
-    *dest = 0;
+  uint8_t x;
 
+  if (!enough_data(din, 1)) {
     return FALSE;
   }
 
+  fc_assert(sizeof(x) == 1);
+  memcpy(&x, ADD_TO_POINTER(din->src, din->current), 1);
+  *dest = x;
+  din->current++;
   return TRUE;
 }
 
 /**************************************************************************
- Receive uint16 value to dest. In case of failure, value stored to dest
- will be zero. Note that zero is legal value even when there is no failure.
+ Receive uint16 value to dest.
 **************************************************************************/
 bool dio_get_uint16(struct data_in *din, int *dest)
 {
-  if (enough_data(din, 2)) {
-    if (dest) {
-      uint16_t x;
-
-      fc_assert(sizeof(x) == 2);
-      memcpy(&x, ADD_TO_POINTER(din->src, din->current), 2);
-      *dest = ntohs(x);
-    }
-    din->current += 2;
-  } else if (dest) {
-    *dest = 0;
+  uint16_t x;
 
+  if (!enough_data(din, 2)) {
     return FALSE;
   }
 
+  fc_assert(sizeof(x) == 2);
+  memcpy(&x, ADD_TO_POINTER(din->src, din->current), 2);
+  *dest = ntohs(x);
+  din->current += 2;
   return TRUE;
 }
 
 /**************************************************************************
- Receive uint32 value to dest. In case of failure, value stored to dest
- will be zero. Note that zero is legal value even when there is no failure.
+ Receive uint32 value to dest.
 **************************************************************************/
 bool dio_get_uint32(struct data_in *din, int *dest)
 {
-  if (enough_data(din, 4)) {
-    if (dest) {
-      uint32_t x;
-
-      fc_assert(sizeof(x) == 4);
-      memcpy(&x, ADD_TO_POINTER(din->src, din->current), 4);
-      *dest = ntohl(x);
-    }
-    din->current += 4;
-  } else if (dest) {
-    *dest = 0;
+  uint32_t x;
 
+  if (!enough_data(din, 4)) {
     return FALSE;
   }
 
+  fc_assert(sizeof(x) == 4);
+  memcpy(&x, ADD_TO_POINTER(din->src, din->current), 4);
+  *dest = ntohl(x);
+  din->current += 4;
   return TRUE;
 }
 
 /**************************************************************************
-...
+  Take boolean value from 8 bits.
 **************************************************************************/
-bool dio_get_bool8(struct data_in *din, bool * dest)
+bool dio_get_bool8(struct data_in *din, bool *dest)
 {
   int ival;
-  bool retval;
 
-  retval = dio_get_uint8(din, &ival);
+  if (!dio_get_uint8(din, &ival)) {
+    return FALSE;
+  }
 
   if (ival != 0 && ival != 1) {
-    log_error("Received value isn't boolean: %d", ival);
-    ival = 1;
+    din->bad_boolean = TRUE;
+    return FALSE;
   }
 
   *dest = (ival != 0);
-
-  return retval;
+  return TRUE;
 }
 
 /**************************************************************************
-...
+  Take boolean value from 32 bits.
 **************************************************************************/
 bool dio_get_bool32(struct data_in *din, bool * dest)
 {
-  int ival = 0;
-  bool retval;
+  int ival;
 
-  retval = dio_get_uint32(din, &ival);
+  if (!dio_get_uint32(din, &ival)) {
+    return FALSE;
+  }
 
   if (ival != 0 && ival != 1) {
-    log_error("Received value isn't boolean: %d", ival);
-    ival = 1;
+    din->bad_boolean = TRUE;
+    return FALSE;
   }
 
   *dest = (ival != 0);
-
-  return retval;
+  return TRUE;
 }
 
 /**************************************************************************
-...
+  Take value from 8 bits.
 **************************************************************************/
 bool dio_get_sint8(struct data_in *din, int *dest)
 {
   int tmp;
-  bool retval;
 
-  retval = dio_get_uint8(din, &tmp);
-  if (dest) {
-    if (tmp > 0x7f) {
-      tmp -= 0x100;
-    }
-    *dest = tmp;
-  }
-
-  return retval;
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-bool dio_get_sint16(struct data_in *din, int *dest)
-{
-  int tmp = 0;
-  bool retval;
-
-  retval = dio_get_uint16(din, &tmp);
-  if (dest) {
-    if (tmp > 0x7fff) {
-      tmp -= 0x10000;
-    }
-    *dest = tmp;
-  }
-
-  return retval;
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-bool dio_get_memory(struct data_in *din, void *dest, size_t dest_size)
-{
-  if (enough_data(din, dest_size)) {
-    if (dest) {
-      memcpy(dest, ADD_TO_POINTER(din->src, din->current), dest_size);
-    }
-    din->current += dest_size;
-  } else {
+  if (!dio_get_uint8(din, &tmp)) {
     return FALSE;
   }
 
+  if (tmp > 0x7f) {
+    tmp -= 0x100;
+  }
+  *dest = tmp;
   return TRUE;
 }
 
 /**************************************************************************
-...
+  Take value from 16 bits.
+**************************************************************************/
+bool dio_get_sint16(struct data_in *din, int *dest)
+{
+  int tmp = 0;
+
+  if (!dio_get_uint16(din, &tmp)) {
+    return FALSE;
+  }
+
+  if (tmp > 0x7fff) {
+    tmp -= 0x10000;
+  }
+  *dest = tmp;
+  return TRUE;
+}
+
+/**************************************************************************
+  Take memory block directly.
+**************************************************************************/
+bool dio_get_memory(struct data_in *din, void *dest, size_t dest_size)
+{
+  if (!enough_data(din, dest_size)) {
+    return FALSE;
+  }
+
+  memcpy(dest, ADD_TO_POINTER(din->src, din->current), dest_size);
+  din->current += dest_size;
+  return TRUE;
+}
+
+/**************************************************************************
+  Take string. Conversion callback is used.
 **************************************************************************/
 bool dio_get_string(struct data_in *din, char *dest, size_t max_dest_size)
 {
   char *c;
-  size_t ps_len;		/* length in packet, not including null */
   size_t offset, remaining;
 
-  fc_assert(max_dest_size > 0 || dest == NULL);
+  fc_assert(max_dest_size > 0);
 
   if (!enough_data(din, 1)) {
-    dest[0] = '\0';
     return FALSE;
   }
 
@@ -596,54 +621,50 @@ bool dio_get_string(struct data_in *din, char *dest, size_t max_dest_size)
   }
 
   if (offset >= remaining) {
-    ps_len = remaining;
     din->too_short = TRUE;
     din->bad_string = TRUE;
-  } else {
-    ps_len = offset;
+    return FALSE;
   }
 
-  if (dest && !(*get_conv_callback) (dest, max_dest_size, c, ps_len)) {
+  if (!(*get_conv_callback) (dest, max_dest_size, c, offset)) {
     din->bad_string = TRUE;
+    return FALSE;
   }
 
-  if (!din->too_short) {
-    din->current += (ps_len + 1);	/* past terminator */
-  }
-
+  din->current += offset + 1;
   return TRUE;
 }
 
 /**************************************************************************
-...
+  Take bits and produce string containing chars '0' and '1'
 **************************************************************************/
 bool dio_get_bit_string(struct data_in *din, char *dest,
 			size_t max_dest_size)
 {
   int npack = 0;		/* number claimed in packet */
   int i;			/* iterate the bytes */
-  bool retval;
 
-  fc_assert(dest != NULL && max_dest_size > 0);
+  fc_assert(max_dest_size > 0);
 
-  if (!enough_data(din, 1)) {
-    dest[0] = '\0';
+  if (!dio_get_uint16(din, &npack)) {
+    din->bad_bit_string = TRUE;
     return FALSE;
   }
 
-  retval = dio_get_uint16(din, &npack);
   if (npack >= max_dest_size) {
       log_error("Have size for %lu, got %d",
                 (unsigned long) max_dest_size, npack);
     din->bad_bit_string = TRUE;
-    dest[0] = '\0';
     return FALSE;
   }
 
   for (i = 0; i < npack;) {
     int bit, byte_value;
 
-    retval = dio_get_uint8(din, &byte_value);
+    if (!dio_get_uint8(din, &byte_value)) {
+      din->bad_bit_string = TRUE;
+      return FALSE;
+    }
     for (bit = 0; bit < 8 && i < npack; bit++, i++) {
       if (TEST_BIT(byte_value, bit)) {
 	dest[i] = '1';
@@ -654,24 +675,21 @@ bool dio_get_bit_string(struct data_in *din, char *dest,
   }
 
   dest[npack] = '\0';
-
-  if (din->too_short) {
-    din->bad_bit_string = TRUE;
-  }
-
-  return retval;
+  return TRUE;
 }
 
 /**************************************************************************
-...
+  Take tech numbers until A_LAST encountered, or MAX_NUM_TECH_LIST techs
+  retrieved.
 **************************************************************************/
 bool dio_get_tech_list(struct data_in *din, int *dest)
 {
   int i;
-  bool retval;
 
   for (i = 0; i < MAX_NUM_TECH_LIST; i++) {
-    retval = dio_get_uint8(din, &dest[i]);
+    if (!dio_get_uint8(din, &dest[i])) {
+      return FALSE;
+    }
     if (dest[i] == A_LAST) {
       break;
     }
@@ -681,53 +699,110 @@ bool dio_get_tech_list(struct data_in *din, int *dest)
     dest[i] = A_LAST;
   }
 
-  return retval;
+  return TRUE;
 }
 
 /**************************************************************************
-...
+  Take unit type numbers until U_LAST encountered, or MAX_NUM_UNIT_LIST
+  types retrieved.
+**************************************************************************/
+bool dio_get_unit_list(struct data_in *din, int *dest)
+{
+  int i;
+
+  for (i = 0; i < MAX_NUM_UNIT_LIST; i++) {
+    if (!dio_get_uint8(din, &dest[i])) {
+      return FALSE;
+    }
+    if (dest[i] == U_LAST) {
+      break;
+    }
+  }
+
+  for (; i < MAX_NUM_UNIT_LIST; i++) {
+    dest[i] = U_LAST;
+  }
+
+  return TRUE;
+}
+
+/**************************************************************************
+  Take building type numbers until B_LAST encountered, or
+  MAX_NUM_BUILDING_LIST types retrieved.
+**************************************************************************/
+bool dio_get_building_list(struct data_in *din, int *dest)
+{
+  int i;
+
+  for (i = 0; i < MAX_NUM_BUILDING_LIST; i++) {
+    if (!dio_get_uint8(din, &dest[i])) {
+      return FALSE;
+    }
+    if (dest[i] == B_LAST) {
+      break;
+    }
+  }
+
+  for (; i < MAX_NUM_BUILDING_LIST; i++) {
+    dest[i] = B_LAST;
+  }
+
+  return TRUE;
+}
+
+/**************************************************************************
+  Take worklist item count and then kind and number for each item, and
+  put them to provided worklist.
 **************************************************************************/
 bool dio_get_worklist(struct data_in *din, struct worklist *pwl)
 {
   int i, length;
-  bool retval;
 
   worklist_init(pwl);
 
-  retval = dio_get_uint8(din, &length);
+  if (!dio_get_uint8(din, &length)) {
+    return FALSE;
+  }
+
   for (i = 0; i < length; i++) {
     int identifier;
     int kind;
 
-    dio_get_uint8(din, &kind);
-    retval = dio_get_uint8(din, &identifier);
+    if (!dio_get_uint8(din, &kind)
+        || !dio_get_uint8(din, &identifier)) {
+      return FALSE;
+    }
 
     worklist_append(pwl, universal_by_number(kind, identifier));
   }
 
-  return retval;
+  return TRUE;
 }
 
 /**************************************************************************
-...
+  Take vector of 8 bit values and insert stop_value after them. stop_value
+  does not need to fit in 8 bits.
 **************************************************************************/
 bool dio_get_uint8_vec8(struct data_in *din, int **values, int stop_value)
 {
   int count, inx;
-  bool retval;
+  int *vec;
 
-  retval = dio_get_uint8(din, &count);
-  if (values) {
-    *values = fc_calloc((count + 1), sizeof(**values));
+  if (!dio_get_uint8(din, &count)) {
+    return FALSE;
   }
+
+  vec = fc_calloc(count + 1, sizeof(*vec));
   for (inx = 0; inx < count; inx++) {
-    retval = dio_get_uint8(din, values ? &((*values)[inx]) : NULL);
-  }
-  if (values) {
-    (*values)[inx] = stop_value;
+    if (!dio_get_uint8(din, vec + inx)) {
+      free (vec);
+      return FALSE;
+    }
   }
+  vec[inx] = stop_value;
+  *values = vec;
 
-  return retval;
+  return TRUE;
 }
 
 /**************************************************************************
@@ -736,21 +811,23 @@ bool dio_get_uint8_vec8(struct data_in *din, int **values, int stop_value)
 bool dio_get_uint16_vec8(struct data_in *din, int **values, int stop_value)
 {
   int count, inx;
-  bool retval;
+  int *vec;
 
-  retval = dio_get_uint8(din, &count);
-
-  if (values) {
-    *values = fc_calloc((count + 1), sizeof(**values));
+  if (!dio_get_uint8(din, &count)) {
+    return FALSE;
   }
+
+  vec = fc_calloc(count + 1, sizeof(*vec));
   for (inx = 0; inx < count; inx++) {
-    dio_get_uint16(din, values ? &((*values)[inx]) : NULL);
-  }
-  if (values) {
-    (*values)[inx] = stop_value;
+    if (!dio_get_uint16(din, vec + inx)) {
+      free (vec);
+      return FALSE;
+    }
   }
+  vec[inx] = stop_value;
+  *values = vec;
 
-  return retval;
+  return TRUE;
 }
 
 /**************************************************************************
@@ -760,17 +837,18 @@ bool dio_get_requirement(struct data_in *din, struct requirement *preq)
 {
   int type, range, value;
   bool survives, negated;
-  bool retval;
 
-  dio_get_uint8(din, &type);
-  dio_get_sint32(din, &value);
-  dio_get_uint8(din, &range);
-  dio_get_bool8(din, &survives);
-  retval = dio_get_bool8(din, &negated);
+  if (!dio_get_uint8(din, &type)
+      || !dio_get_sint32(din, &value)
+      || !dio_get_uint8(din, &range)
+      || !dio_get_bool8(din, &survives)
+      || !dio_get_bool8(din, &negated)) {
+    return FALSE;
+  }
 
   *preq = req_from_values(type, range, survives, negated, value);
 
-  return retval;
+  return TRUE;
 }
 
 /**************************************************************************
@@ -789,4 +867,3 @@ void dio_put_requirement(struct data_out *dout, const struct requirement *preq)
   dio_put_bool8(dout, survives);
   dio_put_bool8(dout, negated);
 }
-
