@@ -12,12 +12,15 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
-#include "game.h"
+/* utility */
 #include "log.h"
 #include "mem.h"
+
+/* common */
+#include "game.h"
 #include "player.h"
 
 #include "diptreaty.h"
@@ -79,7 +82,7 @@ bool could_intel_with_player(const struct player *pplayer,
 }
 
 /****************************************************************
-...
+  Initialize treaty structure between two players.
 *****************************************************************/
 void init_treaty(struct Treaty *ptreaty, 
 		 struct player *plr0, struct player *plr1)
@@ -103,7 +106,7 @@ void clear_treaty(struct Treaty *ptreaty)
 }
 
 /****************************************************************
-...
+  Remove clause from treaty
 *****************************************************************/
 bool remove_clause(struct Treaty *ptreaty, struct player *pfrom, 
 		  enum clause_type type, int val)
@@ -126,7 +129,7 @@ bool remove_clause(struct Treaty *ptreaty, struct player *pfrom,
 
 
 /****************************************************************
-...
+  Add clause to treaty.
 *****************************************************************/
 bool add_clause(struct Treaty *ptreaty, struct player *pfrom, 
 		enum clause_type type, int val)
