@@ -349,7 +349,7 @@ void send_ruleset_cache(struct conn_list *dest)
       packet.source_value = value;
       packet.range = range;
       packet.survives = survives;
-      packet.negated = FALSE;
+      packet.negated = negated;
 
       lsend_packet_ruleset_effect_req(dest, &packet);
     } requirement_list_iterate_end;
@@ -366,7 +366,7 @@ void send_ruleset_cache(struct conn_list *dest)
       packet.source_value = value;
       packet.range = range;
       packet.survives = survives;
-      packet.negated = FALSE;
+      packet.negated = negated;
 
       lsend_packet_ruleset_effect_req(dest, &packet);
     } requirement_list_iterate_end;
@@ -985,7 +985,12 @@ void get_effect_req_text(struct effect *peffect, char *buf, size_t buf_len)
 {
   buf[0] = '\0';
 
+  /* FIXME: should we do something for nreqs and negated reqs?
+   * Currently we just ignore them. */
   requirement_list_iterate(peffect->reqs, preq) {
+    if (preq->negated) {
+      continue;
+    }
     if (buf[0] != '\0') {
       fc_strlcat(buf, "+", buf_len);
     }
