@@ -13,19 +13,27 @@
 #ifndef FC__FC_INTERFACE_H
 #define FC__FC_INTERFACE_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 #include "support.h"
 
 struct player;
 struct tile;
 enum vision_layer;
+struct color;
+struct base_type;
 
 /* The existence of each function should be checked in interface_init()! */
 struct functions {
+  void (*destroy_base)(struct tile *ptile, struct base_type *pbase);
   /* Returns iff the player 'pplayer' has the vision in the layer
      'vision' at tile given by 'ptile'. */
   bool (*player_tile_vision_get)(const struct tile *ptile,
                                  const struct player *pplayer,
                                  enum vision_layer vision);
+  void (*gui_color_free)(struct color *pcolor);
 };
 
 const extern struct functions *fc_funcs;
@@ -33,4 +41,8 @@ const extern struct functions *fc_funcs;
 struct functions *fc_interface_funcs(void);
 void fc_interface_init(void);
 
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
 #endif  /* FC__FC_INTERFACE_H */
