@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 /* utility */
@@ -113,7 +113,7 @@ int civ_population(const struct player *pplayer)
 
 
 /**************************************************************************
-...
+  Find city with given name from any player.
 **************************************************************************/
 struct city *game_city_by_name(const char *name)
 {
@@ -168,7 +168,7 @@ void game_remove_unit(struct unit *punit)
 
     log_debug("game_remove_unit()"
               " at (%d,%d) unit %d, %s %s home (%d,%d) city %d, %s %s",
-              TILE_XY(punit->tile),
+              TILE_XY(unit_tile(punit)),
               punit->id, 
               nation_rule_name(nation_of_unit(punit)),
               unit_rule_name(punit),
@@ -178,21 +178,21 @@ void game_remove_unit(struct unit *punit)
               city_name(pcity));
   } else if (IDENTITY_NUMBER_ZERO == punit->homecity) {
     log_debug("game_remove_unit() at (%d,%d) unit %d, %s %s home %d",
-              TILE_XY(punit->tile),
+              TILE_XY(unit_tile(punit)),
               punit->id, 
               nation_rule_name(nation_of_unit(punit)),
               unit_rule_name(punit),
               punit->homecity);
   } else {
     log_error("game_remove_unit() at (%d,%d) unit %d, %s %s home %d invalid",
-              TILE_XY(punit->tile),
+              TILE_XY(unit_tile(punit)),
               punit->id, 
               nation_rule_name(nation_of_unit(punit)),
               unit_rule_name(punit),
               punit->homecity);
   }
 
-  unit_list_remove(punit->tile->units, punit);
+  unit_list_remove(unit_tile(punit)->units, punit);
   unit_list_remove(unit_owner(punit)->units, punit);
 
   idex_unregister_unit(punit);
@@ -200,11 +200,11 @@ void game_remove_unit(struct unit *punit)
   if (game.callbacks.unit_deallocate) {
     (game.callbacks.unit_deallocate)(punit->id);
   }
-  destroy_unit_virtual(punit);
+  unit_virtual_destroy(punit);
 }
 
 /**************************************************************************
-...
+  Remove city from game.
 **************************************************************************/
 void game_remove_city(struct city *pcity)
 {
@@ -320,6 +320,12 @@ static void game_defaults(void)
   game.scenario.name[0] = '\0';
   game.scenario.players = TRUE;
 
+  /* Veteran system. */
+  game.veteran = NULL;
+
+  /* player colors */
+  game.plr_bg_color = NULL;
+
   if (is_server()) {
     /* All settings only used by the server (./server/ and ./ai/ */
     sz_strlcpy(game.server.allow_take, GAME_DEFAULT_ALLOW_TAKE);
@@ -365,6 +371,7 @@ static void game_defaults(void)
     game.server.migration         = GAME_DEFAULT_MIGRATION;
     game.server.min_players       = GAME_DEFAULT_MIN_PLAYERS;
     game.server.natural_city_names = GAME_DEFAULT_NATURALCITYNAMES;
+    game.server.plrcolormode      = GAME_DEFAULT_PLRCOLORMODE;
     game.server.netwait           = GAME_DEFAULT_NETWAIT;
     game.server.occupychance      = GAME_DEFAULT_OCCUPYCHANCE;
     game.server.onsetbarbarian    = GAME_DEFAULT_ONSETBARBARIAN;
@@ -372,6 +379,7 @@ static void game_defaults(void)
     game.server.pingtime          = GAME_DEFAULT_PINGTIME;
     game.server.pingtimeout       = GAME_DEFAULT_PINGTIMEOUT;
     game.server.razechance        = GAME_DEFAULT_RAZECHANCE;
+    game.server.revealmap         = GAME_DEFAULT_REVEALMAP;
     game.server.revolution_length = GAME_DEFAULT_REVOLUTION_LENGTH;
     sz_strlcpy(game.server.rulesetdir, GAME_DEFAULT_RULESETDIR);
     game.server.save_compress_level = GAME_DEFAULT_COMPRESS_LEVEL;
@@ -401,6 +409,7 @@ static void game_defaults(void)
     game.server.timeoutintinc     = GAME_DEFAULT_TIMEOUTINTINC;
     game.server.turnblock         = GAME_DEFAULT_TURNBLOCK;
     game.server.unitwaittime      = GAME_DEFAULT_UNITWAITTIME;
+    game.server.plr_colors        = NULL;
   }
 
   terrain_control.river_help_text[0] = '\0';
@@ -446,6 +455,7 @@ void game_free(void)
   map_free();
   idex_free();
   team_slots_free();
+  set_allowed_nation_groups(NULL);
   game_ruleset_free();
   cm_free();
 }
@@ -494,6 +504,13 @@ void game_ruleset_init(void)
 ***************************************************************/
 void game_ruleset_free(void)
 {
+  /* Clear main structures which can points to the ruleset dependent
+   * structures. */
+  players_iterate(pplayer) {
+    player_ruleset_close(pplayer);
+  } players_iterate_end;
+  game.government_during_revolution = NULL;
+
   specialists_free();
   techs_free();
   governments_free();
@@ -507,11 +524,14 @@ void game_ruleset_free(void)
   terrains_free();
   ruleset_cache_free();
   nation_groups_free();
+
+  /* Destroy the default veteran system. */
+  veteran_system_destroy(game.veteran);
+  game.veteran = NULL;
 }
 
-
 /***************************************************************
-...
+  Initialize wonder information.
 ***************************************************************/
 void initialize_globals(void)
 {
@@ -634,22 +654,6 @@ const char *population_to_text(int thousand_citizen)
   return big_int_to_text(thousand_citizen, 3);
 }
 
-/****************************************************************************
-  Returns gui name string
-****************************************************************************/
-const char *gui_name(enum gui_type gui)
-{
-  /* This must be in same order as enum gui_type in fc_types.h */
-  const char *gui_names[] = {
-    "stub", "gtk2", "sdl", "xaw", "win32", "ftwl" };
-
-  if (gui < GUI_LAST) {
-    return gui_names[gui];
-  } else {
-    return "Unknown";
-  }
-}
-
 /****************************************************************************
   Produce a statically allocated textual representation of the given
   year.
@@ -722,11 +726,11 @@ int generate_save_name(const char *format, char *buf, int buflen,
                        const char *reason)
 {
   struct cf_sequence sequences[] = {
-    CF_STR_SEQ('R', (reason == NULL) ? "auto" : reason),
-    CF_STR_SEQ('S', year_suffix()),
-    CF_INT_SEQ('T', game.info.turn),
-    CF_INT_SEQ('Y', game.info.year),
-    CF_END
+    cf_str_seq('R', (reason == NULL) ? "auto" : reason),
+    cf_str_seq('S', year_suffix()),
+    cf_int_seq('T', game.info.turn),
+    cf_int_seq('Y', game.info.year),
+    cf_end()
   };
 
   fc_vsnprintcf(buf, buflen, format, sequences, -1);
