@@ -13,9 +13,18 @@
 #ifndef FC__MAP_H
 #define FC__MAP_H
 
-#include <assert.h>
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 #include <math.h> /* sqrt */
 
+/* utility */
+#include "bitvector.h"
+#include "iterator.h"
+#include "log.h"                /* fc_assert */
+
+/* common */
 #include "fc_types.h"
 
 #include "tile.h"
@@ -33,7 +42,35 @@ miscellaneous terrain information
 *****************************************************************/
 #define terrain_misc packet_ruleset_terrain_control
 
-struct hash_table;
+/* Some types used below. */
+struct nation_hash;
+struct nation_type;
+struct packet_edit_startpos_full;
+struct startpos;
+struct startpos_hash;
+
+enum mapsize_type {
+  MAPSIZE_FULLSIZE = 0, /* Using the number of tiles / 1000. */
+  MAPSIZE_PLAYER,       /* Define the number of (land) tiles per player;
+                         * the setting 'landmass' and the number of players
+                         * are used to calculate the map size. */
+  MAPSIZE_XYSIZE        /* 'xsize' and 'ysize' are defined. */
+};
+
+enum map_generator {
+  MAPGEN_SCENARIO = 0,
+  MAPGEN_RANDOM,
+  MAPGEN_FRACTAL,
+  MAPGEN_ISLAND
+};
+
+enum map_startpos {
+  MAPSTARTPOS_DEFAULT = 0,      /* Generator's choice. */
+  MAPSTARTPOS_SINGLE,           /* One player per continent. */
+  MAPSTARTPOS_2or3,             /* Two on three players per continent. */
+  MAPSTARTPOS_ALL,              /* All players on a single continent. */
+  MAPSTARTPOS_VARIABLE,         /* Depending on size of continents. */
+};
 
 struct civ_map {
   int topology_id;
@@ -45,7 +82,7 @@ struct civ_map {
   int num_continents;
   int num_oceans;               /* not updated at the client */
   struct tile *tiles;
-  struct hash_table *startpos_table;
+  struct startpos_hash *startpos_table;
 
   union {
     struct {
@@ -53,29 +90,25 @@ struct civ_map {
     } client;
 
     struct {
+      enum mapsize_type mapsize; /* how the map size is defined */
       int size; /* used to calculate [xy]size */
+      int tilesperplayer; /* tiles per player; used to calculate size */
       int seed;
       int riches;
       int huts;
       int landpercent;
-      int generator;
-      int startpos;
+      enum map_generator generator;
+      enum map_startpos startpos;
       bool tinyisles;
       bool separatepoles;
       bool alltemperate;
       int temperature;
       int wetness;
       int steepness;
-      int num_start_positions;
       bool have_resources;
       bool ocean_resources;         /* Resources in the middle of the ocean */
       bool have_huts;
       bool have_rivers_overlay;	/* only applies if !have_resources */
-
-      struct start_position {
-        struct tile *tile;
-        struct nation_type *nation; /* May be NO_NATION_SELECTED. */
-      } *start_positions;	/* allocated at runtime */
     } server;
   };
 };
@@ -95,11 +128,12 @@ static const bool C_CARDINAL = TRUE;
 static const bool C_NUMBER = FALSE;
 static const bool C_PERCENT = TRUE;
 
-#define MAP_IS_ISOMETRIC (topo_has_flag(TF_ISO) || topo_has_flag(TF_HEX))
+#define MAP_IS_ISOMETRIC (current_topo_has_flag(TF_ISO) || current_topo_has_flag(TF_HEX))
 
 #define CURRENT_TOPOLOGY (map.topology_id)
 
-#define topo_has_flag(flag) ((CURRENT_TOPOLOGY & (flag)) != 0)
+#define topo_has_flag(topo, flag) (((topo) & (flag)) != 0)
+#define current_topo_has_flag(flag) topo_has_flag((CURRENT_TOPOLOGY), (flag))
 
 bool map_is_empty(void);
 void map_init(void);
@@ -120,34 +154,90 @@ bool base_get_direction_for_step(const struct tile *src_tile,
 int get_direction_for_step(const struct tile *src_tile,
 			   const struct tile *dst_tile);
 
-bool map_startpositions_set(void);
-void map_set_startpos(const struct tile *ptile,
-                      const struct nation_type *pnation);
-bool map_has_startpos(const struct tile *ptile);
-const struct nation_type *map_get_startpos(const struct tile *ptile);
-void map_clear_startpos(const struct tile *ptile);
+
+/* Specific functions for start positions. */
+struct startpos *map_startpos_by_number(int id);
+int startpos_number(const struct startpos *psp);
+
+bool startpos_allow(struct startpos *psp, struct nation_type *pnation);
+bool startpos_disallow(struct startpos *psp, struct nation_type *pnation);
+
+struct tile *startpos_tile(const struct startpos *psp);
+bool startpos_nation_allowed(const struct startpos *psp,
+                             const struct nation_type *pnation);
+bool startpos_allows_all(const struct startpos *psp);
+
+bool startpos_pack(const struct startpos *psp,
+                   struct packet_edit_startpos_full *packet);
+bool startpos_unpack(struct startpos *psp,
+                     const struct packet_edit_startpos_full *packet);
+
+/* See comment in "common/map.c". */
+bool startpos_is_excluding(const struct startpos *psp);
+const struct nation_hash *startpos_raw_nations(const struct startpos *psp);
+
+/****************************************************************************
+  Iterate over all nations at the start position for which the function
+  startpos_nation_allowed() would return TRUE. This automatically takes into
+  account the value of startpos_is_excluding() and startpos_allows_all() to
+  iterate over the correct set of nations.
+****************************************************************************/
+struct startpos_iter;
+size_t startpos_iter_sizeof(void);
+struct iterator *startpos_iter_init(struct startpos_iter *it,
+                                    const struct startpos *psp);
+#define startpos_nations_iterate(ARG_psp, NAME_pnation)                     \
+  generic_iterate(struct startpos_iter, const struct nation_type *,         \
+                  NAME_pnation, startpos_iter_sizeof,                       \
+                  startpos_iter_init, (ARG_psp))
+#define startpos_nations_iterate_end generic_iterate_end
+
+
+/* General map start positions functions. */
+int map_startpos_count(void);
+struct startpos *map_startpos_new(struct tile *ptile);
+struct startpos *map_startpos_get(const struct tile *ptile);
+bool map_startpos_remove(struct tile *ptile);
+
+/****************************************************************************
+  Iterate over all start positions placed on the map.
+****************************************************************************/
+struct map_startpos_iter;
+size_t map_startpos_iter_sizeof(void);
+struct iterator *map_startpos_iter_init(struct map_startpos_iter *iter);
+
+#define map_startpos_iterate(NAME_psp)                                      \
+  generic_iterate(struct map_startpos_iter, struct startpos *,              \
+                  NAME_psp, map_startpos_iter_sizeof, map_startpos_iter_init)
+#define map_startpos_iterate_end generic_iterate_end
 
 
 /* Number of index coordinates (for sanity checks and allocations) */
 #define MAP_INDEX_SIZE (map.xsize * map.ysize)
 
 #ifdef DEBUG
-#define CHECK_MAP_POS(x,y) assert(is_normal_map_pos((x),(y)))
-#define CHECK_NATIVE_POS(x, y) assert((x) >= 0 && (x) < map.xsize \
-				      && (y) >= 0 && (y) < map.ysize)
-#define CHECK_INDEX(index) assert((index) >= 0 && (index) < MAP_INDEX_SIZE)
+#define CHECK_MAP_POS(x,y) \
+  fc_assert(is_normal_map_pos((x),(y)))
+#define CHECK_NATIVE_POS(x, y) \
+  fc_assert((x) >= 0 && (x) < map.xsize && (y) >= 0 && (y) < map.ysize)
+#define CHECK_INDEX(index) \
+  fc_assert((index) >= 0 && (index) < MAP_INDEX_SIZE)
 #else
 #define CHECK_MAP_POS(x,y) ((void)0)
 #define CHECK_NATIVE_POS(x, y) ((void)0)
 #define CHECK_INDEX(index) ((void)0)
 #endif
 
-#define native_pos_to_index(nat_x, nat_y)                                   \
-  (CHECK_NATIVE_POS((nat_x), (nat_y)),					    \
+#define native_pos_to_index(nat_x, nat_y)                                    \
+  (CHECK_NATIVE_POS((nat_x), (nat_y)),                                       \
    (nat_x) + (nat_y) * map.xsize)
-#define index_to_native_pos(pnat_x, pnat_y, index)                          \
-  (*(pnat_x) = (index) % map.xsize,                                         \
-   *(pnat_y) = (index) / map.xsize)
+#define index_to_native_pos(pnat_x, pnat_y, index)                           \
+  (*(pnat_x) = index_to_native_pos_x(index),                                 \
+   *(pnat_y) = index_to_native_pos_y(index))
+#define index_to_native_pos_x(index)                                         \
+  ((index) % map.xsize) 
+#define index_to_native_pos_y(index)                                         \
+  ((index) / map.xsize)
 
 /* Obscure math.  See explanation in doc/HACKING. */
 #define NATIVE_TO_MAP_POS(pmap_x, pmap_y, nat_x, nat_y)                     \
@@ -217,7 +307,7 @@ void map_clear_startpos(const struct tile *ptile);
   (MAP_IS_ISOMETRIC ? (map.xsize + map.ysize / 2) : map.xsize)
 #define MAP_HEIGHT \
   (MAP_IS_ISOMETRIC ? (map.xsize + map.ysize / 2) : map.ysize)
-  
+
 static inline int map_pos_to_index(int map_x, int map_y);
 
 /* index_to_map_pos(int *, int *, int) inverts map_pos_to_index */
@@ -225,6 +315,8 @@ static inline int map_pos_to_index(int map_x, int map_y);
   (CHECK_INDEX(index),                          \
    index_to_native_pos(pmap_x, pmap_y, index),  \
    NATIVE_TO_MAP_POS(pmap_x, pmap_y, *(pmap_x), *(pmap_y)))
+static inline int index_to_map_pos_x(int index);
+static inline int index_to_map_pos_y(int index);
 
 #define DIRSTEP(dest_x, dest_y, dir)	\
 (    (dest_x) = DIR_DX[(dir)],      	\
@@ -255,6 +347,7 @@ void base_map_distance_vector(int *dx, int *dy,
 void map_distance_vector(int *dx, int *dy, const struct tile *ptile0,
 			 const struct tile *ptile1);
 int map_num_tiles(void);
+#define map_size_checked()  MAX(map_num_tiles() / 1000, 1)
 
 struct tile *rand_neighbour(const struct tile *ptile);
 struct tile *rand_map_pos(void);
@@ -262,13 +355,16 @@ struct tile *rand_map_pos_filtered(void *data,
 				   bool (*filter)(const struct tile *ptile,
 						  const void *data));
 
-bool is_water_adjacent_to_tile(const struct tile *ptile);
+bool can_be_irrigated(const struct tile *ptile,
+                      const struct unit *punit);
 bool is_tiles_adjacent(const struct tile *ptile0, const struct tile *ptile1);
 bool is_move_cardinal(const struct tile *src_tile,
 		      const struct tile *dst_tile);
 int map_move_cost_unit(struct unit *punit, const struct tile *ptile);
-int map_move_cost_ai(const struct tile *tile0, const struct tile *tile1);
-int map_move_cost(const struct tile *src_tile, const struct tile *dst_tile);
+int map_move_cost_ai(const struct player *pplayer, const struct tile *tile0,
+                     const struct tile *tile1);
+int map_move_cost(const struct player *pplayer, const struct tile *src_tile,
+                  const struct tile *dst_tile);
 bool is_safe_ocean(const struct tile *ptile);
 bool is_cardinally_adj_to_ocean(const struct tile *ptile);
 bv_special get_tile_infrastructure_set(const struct tile *ptile,
@@ -290,12 +386,13 @@ extern struct terrain_misc terrain_control;
  * See also iterate_outward() */
 #define iterate_outward_dxy(start_tile, max_dist, _tile, _x, _y)	    \
 {									    \
-  int _x, _y, _tile##_x, _tile##_y;					    \
+  int _x, _y, _tile##_x, _tile##_y, _start##_x, _start##_y;                 \
   struct tile *_tile;							    \
   const struct tile *_tile##_start = (start_tile);			    \
   int _tile##_max = (max_dist);						    \
   bool _tile##_is_border = is_border_tile(_tile##_start, _tile##_max);	    \
   int _tile##_index = 0;						    \
+  index_to_map_pos(&_start##_x, &_start##_y, tile_index(_tile##_start));    \
   for (;								    \
        _tile##_index < map.num_iterate_outwards_indices;		    \
        _tile##_index++) { 						    \
@@ -304,8 +401,8 @@ extern struct terrain_misc terrain_control;
     }									    \
     _x = map.iterate_outwards_indices[_tile##_index].dx;		    \
     _y = map.iterate_outwards_indices[_tile##_index].dy;		    \
-    _tile##_x = _x + _tile##_start->x;					    \
-    _tile##_y = _y + _tile##_start->y;					    \
+    _tile##_x = _x + _start##_x;                                            \
+    _tile##_y = _y + _start##_y;                                            \
     if (_tile##_is_border && !normalize_map_pos(&_tile##_x, &_tile##_y)) {  \
       continue;								    \
     }									    \
@@ -391,6 +488,13 @@ extern struct terrain_misc terrain_control;
 
 #define adjc_dir_iterate_end adjc_dirlist_iterate_end
 
+#define adjc_dir_base_iterate(center_tile, dir_itr)                            \
+  adjc_dirlist_base_iterate(center_tile, dir_itr,                              \
+                            map.valid_dirs, map.num_valid_dirs)
+
+#define adjc_dir_base_iterate_end                                              \
+  adjc_dirlist_base_iterate_end
+
 #define cardinal_adjc_iterate(center_tile, itr_tile)			    \
   adjc_dirlist_iterate(center_tile, itr_tile, _dir_itr,			    \
 		       map.cardinal_dirs, map.num_cardinal_dirs)
@@ -403,6 +507,13 @@ extern struct terrain_misc terrain_control;
 
 #define cardinal_adjc_dir_iterate_end adjc_dirlist_iterate_end
 
+#define cardinal_adjc_dir_base_iterate(center_tile, dir_itr)                   \
+  adjc_dirlist_base_iterate(center_tile, dir_itr,                              \
+                            map.cardinal_dirs, map.num_cardinal_dirs)
+
+#define cardinal_adjc_dir_base_iterate_end                                     \
+  adjc_dirlist_base_iterate_end
+
 /* Iterate through all tiles adjacent to a tile using the given list of
  * directions.  _dir is the directional value, (center_x, center_y) is
  * the center tile (which must be normalized).
@@ -413,18 +524,19 @@ extern struct terrain_misc terrain_control;
 			     dirlist, dircount)				    \
 {									    \
   enum direction8 _dir;							    \
-  int _tile##_x, _tile##_y;						    \
+  int _tile##_x, _tile##_y, _center##_x, _center##_y;                       \
   struct tile *_tile;							    \
   const struct tile *_tile##_center = (center_tile);			    \
   bool _tile##_is_border = is_border_tile(_tile##_center, 1);		    \
   int _tile##_index = 0;						    \
+  index_to_map_pos(&_center##_x, &_center##_y, tile_index(_tile##_center)); \
   for (;								    \
        _tile##_index < (dircount);					    \
        _tile##_index++) {						    \
     _dir = dirlist[_tile##_index];					    \
     DIRSTEP(_tile##_x, _tile##_y, _dir);				    \
-    _tile##_x += _tile##_center->x;					    \
-    _tile##_y += _tile##_center->y;					    \
+    _tile##_x += _center##_x;                                               \
+    _tile##_y += _center##_y;                                               \
     if (_tile##_is_border && !normalize_map_pos(&_tile##_x, &_tile##_y)) {  \
       continue;								    \
     }									    \
@@ -434,6 +546,30 @@ extern struct terrain_misc terrain_control;
     }									    \
 }
 
+/* Same as above but without setting the tile. */
+#define adjc_dirlist_base_iterate(center_tile, _dir, dirlist, dircount)        \
+{                                                                              \
+  enum direction8 _dir;                                                        \
+  int _tile##_x, _tile##_y, _center##_x, _center##_y;                          \
+  const struct tile *_tile##_center = (center_tile);                           \
+  bool _tile##_is_border = is_border_tile(_tile##_center, 1);                  \
+  int _tile##_index = 0;                                                       \
+  index_to_map_pos(&_center##_x, &_center##_y, tile_index(_tile##_center));    \
+  for (;                                                                       \
+       _tile##_index < (dircount);                                             \
+       _tile##_index++) {                                                      \
+    _dir = dirlist[_tile##_index];                                             \
+    DIRSTEP(_tile##_x, _tile##_y, _dir);                                       \
+    _tile##_x += _center##_x;                                                  \
+    _tile##_y += _center##_y;                                                  \
+    if (_tile##_is_border && !normalize_map_pos(&_tile##_x, &_tile##_y)) {     \
+      continue;                                                                \
+    }
+
+#define adjc_dirlist_base_iterate_end                                          \
+  }                                                                            \
+}
+
 /* Iterate over all positions on the globe.
  * Use index positions for cache efficiency. */
 #define whole_map_iterate(_tile)					    \
@@ -470,25 +606,31 @@ extern const int DIR_DY[8];
 #define MAP_MIN_HUTS             0
 #define MAP_MAX_HUTS             500
 
-/* Size of the map in thousands of tiles.  Setting the maximal size over
- * than 30 is dangerous, because some parts of the code (e.g. path finding)
- * assume the tile index is of type (signed short int). */
-#define MAP_DEFAULT_SIZE         4
-#define MAP_MIN_SIZE             1
-#define MAP_MAX_SIZE             30
+#define MAP_DEFAULT_MAPSIZE     MAPSIZE_FULLSIZE
 
-/* This defines the maximum linear size in _map_ coordinates.
- * This must be smaller than 255 because of the way coordinates are sent
- * across the network. */
-#define MAP_MAX_LINEAR_SIZE      254
-#define MAP_MIN_LINEAR_SIZE      8
-#define MAP_MAX_WIDTH            MAP_MAX_LINEAR_SIZE
-#define MAP_MAX_HEIGHT           MAP_MAX_LINEAR_SIZE
+/* Size of the map in thousands of tiles. If MAP_MAX_SIZE is increased, 
+ * MAX_DBV_LENGTH in bitvector.c must be checked; see the static assertion
+ * below. */
+#define MAP_DEFAULT_SIZE         4
+#define MAP_MIN_SIZE             0
+#define MAP_MAX_SIZE             2048
+
+FC_STATIC_ASSERT(MAP_MAX_SIZE * 1000 <= MAX_DBV_LENGTH,
+                 map_too_big_for_bitvector);
+
+#define MAP_DEFAULT_TILESPERPLAYER      100
+#define MAP_MIN_TILESPERPLAYER            1
+#define MAP_MAX_TILESPERPLAYER         1000
+
+/* This defines the maximum linear size in _map_ coordinates. */
+#define MAP_DEFAULT_LINEAR_SIZE  64
+/* 32 * 1024 is 2^15; thus, x*y is <= 2^15 * 2^15 = 2^30. This can be
+ * represented by an signed int as required by the network protocol. */
+#define MAP_MAX_LINEAR_SIZE      (32 * 1024)
+#define MAP_MIN_LINEAR_SIZE      16
 
 #define MAP_ORIGINAL_TOPO        TF_WRAPX
-#define MAP_DEFAULT_TOPO         TF_WRAPX
-#define MAP_MIN_TOPO             0
-#define MAP_MAX_TOPO             15
+#define MAP_DEFAULT_TOPO         (TF_WRAPX|TF_ISO)
 
 #define MAP_DEFAULT_SEED         0
 #define MAP_MIN_SEED             0
@@ -510,13 +652,9 @@ extern const int DIR_DY[8];
 #define MAP_MIN_WETNESS          0
 #define MAP_MAX_WETNESS          100
 
-#define MAP_DEFAULT_GENERATOR    1
-#define MAP_MIN_GENERATOR        1
-#define MAP_MAX_GENERATOR        3
+#define MAP_DEFAULT_GENERATOR    MAPGEN_RANDOM
 
-#define MAP_DEFAULT_STARTPOS     0
-#define MAP_MIN_STARTPOS         0
-#define MAP_MAX_STARTPOS         4
+#define MAP_DEFAULT_STARTPOS     MAPSTARTPOS_DEFAULT
 
 #define MAP_DEFAULT_TINYISLES    FALSE
 #define MAP_MIN_TINYISLES        FALSE
@@ -549,6 +687,24 @@ static inline int map_pos_to_index(int map_x, int map_y)
   return native_pos_to_index(nat_x, nat_y);
 }
 
+static inline int index_to_map_pos_x(int index)
+{
+  /* Note: writing this as a macro is hard; it needs temp variables. */
+  int map_x, map_y;
+
+  index_to_map_pos(&map_x, &map_y, index);
+  return map_x;
+}
+
+static inline int index_to_map_pos_y(int index)
+{
+  /* Note: writing this as a macro is hard; it needs temp variables. */
+  int map_x, map_y;
+
+  index_to_map_pos(&map_x, &map_y, index);
+  return map_y;
+}
+
 /****************************************************************************
   A "border position" is any map position that _may have_ positions within
   real map distance dist that are non-normal.  To see its correctness,
@@ -561,11 +717,21 @@ static inline bool is_border_tile(const struct tile *ptile, int dist)
    * one tile away. */
   int xdist = dist;
   int ydist = (MAP_IS_ISOMETRIC ? (2 * dist) : dist);
+  int nat_x, nat_y;
 
-  return (ptile->nat_x < xdist 
-	  || ptile->nat_y < ydist
-	  || ptile->nat_x >= map.xsize - xdist
-	  || ptile->nat_y >= map.ysize - ydist);
+  index_to_native_pos(&nat_x, &nat_y, tile_index(ptile));
+
+  return (nat_x < xdist 
+          || nat_y < ydist
+          || nat_x >= map.xsize - xdist
+          || nat_y >= map.ysize - ydist);
 }
 
+enum direction8 rand_direction(void);
+enum direction8 opposite_direction(enum direction8 dir);
+
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
 #endif  /* FC__MAP_H */
