@@ -12,7 +12,7 @@
 ****************************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 /* utility */
@@ -23,6 +23,7 @@
 #include "support.h"
 
 /* common */
+#include "astring.h"
 #include "base.h"
 #include "effects.h"
 #include "fc_types.h"
@@ -35,10 +36,6 @@
 
 #include "movement.h"
 
-static const char *move_type_names[] = {
-  "Land", "Sea", "Both"
-};
-
 /****************************************************************************
   This function calculates the move rate of the unit, taking into 
   account the penalty for reduced hitpoints (affects sea and land 
@@ -48,12 +45,20 @@ static const char *move_type_names[] = {
 int unit_move_rate(const struct unit *punit)
 {
   int move_rate = 0;
-  int base_move_rate = unit_type(punit)->move_rate 
-    + unit_type(punit)->veteran[punit->veteran].move_bonus;
-  struct unit_class *pclass = unit_class(punit);
+  int base_move_rate;
+  struct unit_class *pclass;
+  const struct veteran_level *vlevel;
 
+  fc_assert_ret_val(punit != NULL, 0);
+
+  vlevel = utype_veteran_level(unit_type(punit), punit->veteran);
+  fc_assert_ret_val(vlevel != NULL, 0);
+
+  base_move_rate = unit_type(punit)->move_rate + vlevel->move_bonus;
   move_rate = base_move_rate;
 
+  pclass = unit_class(punit);
+
   if (uclass_has_flag(pclass, UCF_DAMAGE_SLOWS)) {
     /* Scale the MP based on how many HP the unit has. */
     move_rate = (move_rate * punit->hp) / unit_type(punit)->hp;
@@ -80,10 +85,13 @@ int unit_move_rate(const struct unit *punit)
 ****************************************************************************/
 bool unit_can_defend_here(const struct unit *punit)
 {
+  struct unit *ptrans = unit_transport_get(punit);
+
   /* Do not just check if unit is transported.
    * Even transported units may step out from transport to fight,
    * if this is their native terrain. */
-  return can_unit_exist_at_tile(punit, punit->tile);
+  return (can_unit_exist_at_tile(punit, unit_tile(punit))
+          && (ptrans == NULL || can_unit_unload(punit, ptrans)));
 }
 
 /****************************************************************************
@@ -92,7 +100,7 @@ bool unit_can_defend_here(const struct unit *punit)
 ****************************************************************************/
 bool can_attack_non_native(const struct unit_type *utype)
 {
-  return (utype_class(utype)->move_type == SEA_MOVING
+  return (utype_class(utype)->move_type == UMT_SEA
           && !utype_has_flag(utype, F_NO_LAND_ATTACK));
 }
 
@@ -102,13 +110,13 @@ bool can_attack_non_native(const struct unit_type *utype)
 ****************************************************************************/
 bool can_attack_from_non_native(const struct unit_type *utype)
 {
-  /* It's clear that LAND_MOVING should not be able to attack from
-   * non-native (unless F_MARINES) and it's clear that SEA_MOVING
+  /* It's clear that UMT_LAND should not be able to attack from
+   * non-native (unless F_MARINES) and it's clear that UMT_SEA
    * should be able to attack from non-native. It's not clear what to do
-   * with BOTH_MOVING. At the moment we return FALSE for
+   * with UMT_BOTH. At the moment we return FALSE for
    * them. One can always give "Marines" flag for them. This should be
    * generalized for unit_classes anyway. */
-  return (utype_class(utype)->move_type == SEA_MOVING
+  return (utype_class(utype)->move_type == UMT_SEA
           || utype_has_flag(utype, F_MARINES));
 }
 
@@ -117,7 +125,7 @@ bool can_attack_from_non_native(const struct unit_type *utype)
 ****************************************************************************/
 bool is_sailing_unit(const struct unit *punit)
 {
-  return (uclass_move_type(unit_class(punit)) == SEA_MOVING);
+  return (uclass_move_type(unit_class(punit)) == UMT_SEA);
 }
 
 
@@ -126,7 +134,7 @@ bool is_sailing_unit(const struct unit *punit)
 ****************************************************************************/
 bool is_ground_unit(const struct unit *punit)
 {
-  return (uclass_move_type(unit_class(punit)) == LAND_MOVING);
+  return (uclass_move_type(unit_class(punit)) == UMT_LAND);
 }
 
 
@@ -135,7 +143,7 @@ bool is_ground_unit(const struct unit *punit)
 ****************************************************************************/
 bool is_sailing_unittype(const struct unit_type *punittype)
 {
-  return (utype_move_type(punittype) == SEA_MOVING);
+  return (utype_move_type(punittype) == UMT_SEA);
 }
 
 
@@ -144,14 +152,15 @@ bool is_sailing_unittype(const struct unit_type *punittype)
 ****************************************************************************/
 bool is_ground_unittype(const struct unit_type *punittype)
 {
-  return (utype_move_type(punittype) == LAND_MOVING);
+  return (utype_move_type(punittype) == UMT_LAND);
 }
 
 /****************************************************************************
   Check for a city channel.
 ****************************************************************************/
 bool is_city_channel_tile(const struct unit_class *punitclass,
-                          const struct tile *ptile)
+                          const struct tile *ptile,
+                          const struct tile *pexclude)
 {
   struct dbv tile_processed;
   struct tile_list *process_queue = tile_list_new();
@@ -163,11 +172,13 @@ bool is_city_channel_tile(const struct unit_class *punitclass,
     adjc_iterate(ptile, piter) {
       if (dbv_isset(&tile_processed, tile_index(piter))) {
         continue;
-      } else if (is_native_to_class(punitclass, tile_terrain(piter),
-                                    piter->special, piter->bases)) {
+      } else if (piter != pexclude
+                 && is_native_to_class(punitclass, tile_terrain(piter),
+                                       piter->special, piter->bases)) {
         found = TRUE;
         break;
-      } else if (NULL != tile_city(piter)) {
+      } else if (piter != pexclude
+                 && NULL != tile_city(piter)) {
         tile_list_append(process_queue, piter);
       } else {
         dbv_set(&tile_processed, tile_index(piter));
@@ -201,10 +212,8 @@ bool can_exist_at_tile(const struct unit_type *utype,
   if (NULL != tile_city(ptile)
       && (uclass_has_flag(utype_class(utype), UCF_BUILD_ANYWHERE)
           || is_native_near_tile(utype_class(utype), ptile)
-          || ((1 == game.info.citymindist
-               || (0 == game.info.citymindist
-                   && 1 == game.info.min_dist_bw_cities))
-              && is_city_channel_tile(utype_class(utype), ptile)))) {
+          || (1 == game.info.citymindist
+              && is_city_channel_tile(utype_class(utype), ptile, NULL)))) {
     return TRUE;
   }
 
@@ -413,7 +422,7 @@ static bool zoc_ok_move_gen(const struct unit *punit,
 ****************************************************************************/
 bool zoc_ok_move(const struct unit *punit, const struct tile *dst_tile)
 {
-  return zoc_ok_move_gen(punit, punit->tile, dst_tile);
+  return zoc_ok_move_gen(punit, unit_tile(punit), dst_tile);
 }
 
 
@@ -497,7 +506,7 @@ unit_move_to_tile_test(const struct unit_type *punittype,
     }
 
     /* 6) */
-    if (utype_move_type(punittype) == LAND_MOVING
+    if (utype_move_type(punittype) == UMT_LAND
         && is_ocean_tile(src_tile)      /* Moving from ocean */
         && !utype_has_flag(punittype, F_MARINES)) {
       /* Most ground units can't move into cities from ships. (Note this
@@ -571,22 +580,6 @@ bool can_unit_type_transport(const struct unit_type *transporter,
   return BV_ISSET(transporter->cargo, uclass_index(transported));
 }
 
-/**************************************************************************
-  Convert move type names to enum; case insensitive;
-  returns MOVETYPE_LAST if can't match.
-**************************************************************************/
-enum unit_move_type move_type_from_str(const char *s)
-{
-  int i;
-
-  for (i = 0; i < MOVETYPE_LAST; i++) {
-    if (fc_strcasecmp(move_type_names[i], s)==0) {
-      return i;
-    }
-  }
-  return MOVETYPE_LAST;
-}
-
 /**************************************************************************
   Search transport suitable for given unit from tile. It has to have
   free space in it.
@@ -625,3 +618,55 @@ int unit_class_transporter_capacity(const struct tile *ptile,
 
   return availability;
 }
+
+/****************************************************************************
+  Render movement points as text, including fractional movement points,
+  scaled by SINGLE_MOVE. Returns a pointer to a static buffer.
+  'prefix' is a string put in front of all numeric output.
+  'none' is the string to display in place of the integer part if no
+  movement points (or NULL to just say 0).
+  'align' controls whether this is for a fixed-width table, in which case
+  padding spaces will be included to make all such strings line up when
+  right-aligned.
+****************************************************************************/
+const char *move_points_text(int mp, const char *prefix, const char *none,
+                             bool align)
+{
+  static struct astring str = ASTRING_INIT;
+  static int denomlen = 0;
+  int pad1, pad2;
+  if (denomlen == 0) {
+    /* String length of denominator for fractional representation of
+     * movement points, for padding */
+    char denomstr[10];
+    fc_snprintf(denomstr, sizeof(denomstr), "%d", SINGLE_MOVE);
+    denomlen = strlen(denomstr);
+  }
+  if (align) {
+    pad1 = denomlen;     /* numerator or denominator */
+    pad2 = denomlen*2+2; /* everything right of integer part */
+  } else {
+    pad1 = pad2 = 0;
+  }
+  if (!prefix) {
+    prefix = "";
+  }
+  astr_clear(&str);
+  if (mp == 0 && none) {
+    /* No movement points, special representation */
+    astr_add(&str, "%s%*s", none, pad2, "");
+  } else if ((mp % SINGLE_MOVE) == 0) {
+    /* Integer move bonus */
+    astr_add(&str, "%s%d%*s", prefix, mp / SINGLE_MOVE, pad2, "");
+  } else if (mp < SINGLE_MOVE) {
+    /* Fractional move bonus */
+    astr_add(&str, "%s%*d/%*d", prefix,
+             pad1, mp % SINGLE_MOVE, pad1, SINGLE_MOVE);
+  } else {
+    /* Integer + fractional move bonus */
+    astr_add(&str,
+             "%s%d %*d/%*d", prefix, mp / SINGLE_MOVE,
+             pad1, mp % SINGLE_MOVE, pad1, SINGLE_MOVE);
+  }
+  return astr_str(&str);
+}
