@@ -6,6 +6,11 @@
  *                       DO NOT CHANGE THIS FILE                             *
  ****************************************************************************/
 
+
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 struct packet_processing_started {
   char __dummy;			/* to avoid malloc(0); */
 };
@@ -57,13 +62,10 @@ struct packet_player_ready {
   bool is_ready;
 };
 
-struct packet_endgame_report_old {
+struct packet_endgame_report {
   int category_num;
   char category_name[32][MAX_LEN_NAME];
   int player_num;
-  int player_id[MAX_NUM_PLAYERS];
-  int score[MAX_NUM_PLAYERS];
-  int category_score[32][MAX_NUM_PLAYERS];
 };
 
 struct packet_endgame_player {
@@ -73,12 +75,6 @@ struct packet_endgame_player {
   int category_score[32];
 };
 
-struct packet_endgame_report_new {
-  int category_num;
-  char category_name[32][MAX_LEN_NAME];
-  int player_num;
-};
-
 struct packet_tile_info {
   int tile;
   Continent_id continent;
@@ -90,6 +86,7 @@ struct packet_tile_info {
   bool special[S_LAST];
   bv_bases bases;
   char spec_sprite[MAX_LEN_NAME];
+  char label[MAX_LEN_NAME];
 };
 
 struct packet_game_info {
@@ -102,9 +99,11 @@ struct packet_game_info {
   int border_city_radius_sq;
   int border_size_effect;
   enum borders_mode borders;
+  int base_bribe_cost;
   bool calendar_skip_0;
   int celebratesize;
   bool changable_tax;
+  bool citizen_nationality;
   int citymindist;
   int cooling;
   int coolinglevel;
@@ -140,7 +139,6 @@ struct packet_game_info {
   int killcitizen;
   bool killstack;
   int min_city_center_output[O_LAST];
-  int min_dist_bw_cities;
   char negative_year_label[MAX_LEN_NAME];
   int notradesize;
   bool nuclear_winter;
@@ -154,7 +152,6 @@ struct packet_game_info {
   bool unreachable_protects;
   int sciencebox;
   float seconds_to_phasedone;
-  float seconds_to_phasedone2;
   int shieldbox;
   int skill_level;
   bool slow_invasions;
@@ -218,12 +215,15 @@ struct packet_city_info {
   int owner;
   int size;
   int city_radius_sq;
-  int ppl_happy[5];
-  int ppl_content[5];
-  int ppl_unhappy[5];
-  int ppl_angry[5];
+  int ppl_happy[FEELING_LAST];
+  int ppl_content[FEELING_LAST];
+  int ppl_unhappy[FEELING_LAST];
+  int ppl_angry[FEELING_LAST];
   int specialists_size;
   int specialists[SP_MAX];
+  int nationalities_count;
+  int nation_id[MAX_NUM_PLAYER_SLOTS];
+  int nation_citizens[MAX_NUM_PLAYER_SLOTS];
   int surplus[O_LAST];
   int waste[O_LAST];
   int unhappy_penalty[O_LAST];
@@ -378,7 +378,11 @@ struct packet_player_info {
   bv_player gives_shared_vision;
   char inventions[A_LAST+1];
   int love[MAX_NUM_PLAYER_SLOTS];
+  int color_red;
+  int color_green;
+  int color_blue;
   int wonders[B_LAST];
+  int wonders_new[B_LAST];
 };
 
 struct packet_player_phase_done {
@@ -432,11 +436,13 @@ struct packet_unit_info {
   int id;
   int owner;
   int tile;
+  enum direction8 facing;
   int homecity;
   int upkeep[O_LAST];
   int veteran;
   bool ai;
   bool paradropped;
+  bool occupied;
   bool transported;
   bool done_moving;
   Unit_type_id type;
@@ -444,9 +450,10 @@ struct packet_unit_info {
   int movesleft;
   int hp;
   int fuel;
-  int activity_count;
-  int changed_from_count;
-  int occupy;
+  int activity_count_old;
+  int changed_from_count_old;
+  int activity_count_new;
+  int changed_from_count_new;
   int goto_tile;
   enum unit_activity activity;
   enum tile_special_type activity_target;
@@ -470,6 +477,7 @@ struct packet_unit_short_info {
   int id;
   int owner;
   int tile;
+  enum direction8 facing;
   Unit_type_id type;
   int veteran;
   bool occupied;
@@ -705,6 +713,7 @@ struct packet_conn_pong {
 
 struct packet_client_info {
   enum gui_type gui;
+  char distribution[MAX_LEN_NAME];
 };
 
 struct packet_end_phase {
@@ -797,8 +806,9 @@ struct packet_ruleset_unit {
   int paratroopers_range;
   int paratroopers_mr_req;
   int paratroopers_mr_sub;
+  int veteran_levels;
   char veteran_name[MAX_VET_LEVELS][MAX_LEN_NAME];
-  float power_fact[MAX_VET_LEVELS];
+  int power_fact[MAX_VET_LEVELS];
   int move_bonus[MAX_VET_LEVELS];
   int bombard_rate;
   int city_size;
@@ -812,8 +822,14 @@ struct packet_ruleset_unit {
 struct packet_ruleset_game {
   int default_specialist;
   int global_init_techs[MAX_NUM_TECH_LIST];
-  int work_veteran_chance[MAX_VET_LEVELS];
-  int veteran_chance[MAX_VET_LEVELS];
+  int global_init_buildings[MAX_NUM_BUILDING_LIST];
+  int veteran_levels;
+  char veteran_name[MAX_VET_LEVELS][MAX_LEN_NAME];
+  int power_fact[MAX_VET_LEVELS];
+  int move_bonus[MAX_VET_LEVELS];
+  int background_red;
+  int background_green;
+  int background_blue;
 };
 
 struct packet_ruleset_specialist {
@@ -902,6 +918,10 @@ struct packet_ruleset_nation {
   int barbarian_type;
   int ngroups;
   int groups[MAX_NUM_NATION_GROUPS];
+  int init_government_id;
+  int init_techs[MAX_NUM_TECH_LIST];
+  int init_units[MAX_NUM_UNIT_LIST];
+  int init_buildings[MAX_NUM_BUILDING_LIST];
 };
 
 struct packet_ruleset_city {
@@ -965,6 +985,9 @@ struct packet_ruleset_terrain {
   int rail_time;
   int clean_pollution_time;
   int clean_fallout_time;
+  int color_red;
+  int color_green;
+  int color_blue;
   char helptext[MAX_LEN_PACKET];
 };
 
@@ -1230,6 +1253,7 @@ struct packet_edit_tile {
   Resource_type_id resource;
   Terrain_type_id terrain;
   Nation_type_id startpos_nation;
+  char label[MAX_LEN_NAME];
 };
 
 struct packet_edit_unit_create {
@@ -1261,7 +1285,8 @@ struct packet_edit_unit {
   int veteran;
   int fuel;
   enum unit_activity activity;
-  int activity_count;
+  int activity_count_old;
+  int activity_count_new;
   Base_type_id activity_base;
   bool debug;
   bool moved;
@@ -1380,7 +1405,7 @@ enum packet_type {
   PACKET_SERVER_SHUTDOWN,
   PACKET_NATION_SELECT_REQ = 10,         /* 10 */
   PACKET_PLAYER_READY,
-  PACKET_ENDGAME_REPORT_OLD,
+  PACKET_ENDGAME_REPORT,
   PACKET_TILE_INFO = 15,
   PACKET_GAME_INFO,
   PACKET_MAP_INFO,
@@ -1528,9 +1553,8 @@ enum packet_type {
   PACKET_EDIT_PLAYER_VISION,
   PACKET_EDIT_GAME,
   PACKET_EDIT_OBJECT_CREATED,
-  PACKET_RULESETS_READY = 221,
-  PACKET_ENDGAME_PLAYER = 223,
-  PACKET_ENDGAME_REPORT_NEW,
+  PACKET_ENDGAME_PLAYER,                 /* 220 */
+  PACKET_RULESETS_READY,
 
   PACKET_LAST  /* leave this last */
 };
@@ -1567,18 +1591,14 @@ struct packet_player_ready *receive_packet_player_ready(struct connection *pc, e
 int send_packet_player_ready(struct connection *pc, const struct packet_player_ready *packet);
 int dsend_packet_player_ready(struct connection *pc, int player_no, bool is_ready);
 
-struct packet_endgame_report_old *receive_packet_endgame_report_old(struct connection *pc, enum packet_type type);
-int send_packet_endgame_report_old(struct connection *pc, const struct packet_endgame_report_old *packet);
-void lsend_packet_endgame_report_old(struct conn_list *dest, const struct packet_endgame_report_old *packet);
+struct packet_endgame_report *receive_packet_endgame_report(struct connection *pc, enum packet_type type);
+int send_packet_endgame_report(struct connection *pc, const struct packet_endgame_report *packet);
+void lsend_packet_endgame_report(struct conn_list *dest, const struct packet_endgame_report *packet);
 
 struct packet_endgame_player *receive_packet_endgame_player(struct connection *pc, enum packet_type type);
 int send_packet_endgame_player(struct connection *pc, const struct packet_endgame_player *packet);
 void lsend_packet_endgame_player(struct conn_list *dest, const struct packet_endgame_player *packet);
 
-struct packet_endgame_report_new *receive_packet_endgame_report_new(struct connection *pc, enum packet_type type);
-int send_packet_endgame_report_new(struct connection *pc, const struct packet_endgame_report_new *packet);
-void lsend_packet_endgame_report_new(struct conn_list *dest, const struct packet_endgame_report_new *packet);
-
 struct packet_tile_info *receive_packet_tile_info(struct connection *pc, enum packet_type type);
 int send_packet_tile_info(struct connection *pc, const struct packet_tile_info *packet);
 void lsend_packet_tile_info(struct conn_list *dest, const struct packet_tile_info *packet);
@@ -2167,3 +2187,7 @@ int dsend_packet_edit_object_created(struct connection *pc, int tag, int id);
 void delta_stats_report(void);
 void delta_stats_reset(void);
 void *get_packet_from_connection_helper(struct connection *pc, enum packet_type type);
+
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
