@@ -359,6 +359,7 @@ struct requirement req_from_str(const char *type, const char *range,
 {
   struct requirement req;
   bool invalid = TRUE;
+  const char *error = NULL;
 
   req.source = universal_by_rule_name(type, value);
 
@@ -404,7 +405,9 @@ struct requirement req_from_str(const char *type, const char *range,
   req.negated = negated;
 
   /* These checks match what combinations are supported inside
-   * is_req_active(). */
+   * is_req_active(). However, it's only possible to do basic checks,
+   * not anything that might depend on the rest of the ruleset which
+   * might not have been lodaed yet. */
   switch (req.source.kind) {
   case VUT_SPECIAL:
   case VUT_TERRAIN:
@@ -421,12 +424,6 @@ struct requirement req_from_str(const char *type, const char *range,
   case VUT_AI_LEVEL:
     invalid = (req.range != REQ_RANGE_PLAYER);
     break;
-  case VUT_IMPROVEMENT:
-    invalid = ((req.range == REQ_RANGE_WORLD
-		&& !is_great_wonder(req.source.value.building))
-	       || (req.range > REQ_RANGE_CITY
-		   && !is_wonder(req.source.value.building)));
-    break;
   case VUT_MINSIZE:
     invalid = (req.range != REQ_RANGE_CITY);
     break;
@@ -451,6 +448,11 @@ struct requirement req_from_str(const char *type, const char *range,
   case VUT_MINYEAR:
     invalid = (req.range != REQ_RANGE_WORLD);
     break;
+  case VUT_IMPROVEMENT:
+    /* Valid ranges depend on the building genus (wonder/improvement),
+     * which might not have been loaded from the ruleset yet.
+     * So we allow anything here, and do a proper check once ruleset
+     * loading is complete, in sanity_check_req_individual(). */
   case VUT_NONE:
     invalid = FALSE;
     break;
@@ -459,9 +461,50 @@ struct requirement req_from_str(const char *type, const char *range,
   }
 
   if (invalid) {
-    log_error("Invalid requirement %s | %s | %s | %s | %s",
+    error = "bad range";
+  } else {
+    /* Check 'survives'. */
+    switch (req.source.kind) {
+    case VUT_IMPROVEMENT:
+      /* See count_buildings_in_range(). */
+      invalid = survives && req.range <= REQ_RANGE_CONTINENT;
+      break;
+    case VUT_NATION:
+      invalid = survives && req.range != REQ_RANGE_WORLD;
+      break;
+    case VUT_ADVANCE:
+    case VUT_GOVERNMENT:
+    case VUT_TERRAIN:
+    case VUT_SPECIAL:
+    case VUT_UTYPE:
+    case VUT_UTFLAG:
+    case VUT_UCLASS:
+    case VUT_UCFLAG:
+    case VUT_OTYPE:
+    case VUT_SPECIALIST:
+    case VUT_MINSIZE:
+    case VUT_AI_LEVEL:
+    case VUT_TERRAINCLASS:
+    case VUT_BASE:
+    case VUT_MINYEAR:
+    case VUT_TERRAINALTER:
+    case VUT_CITYTILE:
+      /* Most requirements don't support 'survives'. */
+      invalid = survives;
+      break;
+    case VUT_NONE:
+    case VUT_COUNT:
+      break;
+    }
+    if (invalid) {
+      error = "bad 'survives'";
+    }
+  }
+
+  if (invalid) {
+    log_error("Invalid requirement %s | %s | %s | %s | %s: %s",
               type, range, survives ? "survives" : "",
-              negated ? "negated" : "", value);
+              negated ? "negated" : "", value, error);
     req.source.kind = universals_n_invalid();
   }
 
@@ -534,9 +577,7 @@ static int num_world_buildings_total(const struct impr_type *building)
     return (great_wonder_is_built(building)
             || great_wonder_is_destroyed(building) ? 1 : 0);
   } else {
-    /* TRANS: Obscure ruleset error. */
-    log_error(_("World-ranged requirements are only supported "
-                "for wonders."));
+    log_error("World-ranged requirements are only supported for wonders.");
     return 0;
   }
 }
@@ -549,9 +590,7 @@ static int num_world_buildings(const struct impr_type *building)
   if (is_great_wonder(building)) {
     return (great_wonder_is_built(building) ? 1 : 0);
   } else {
-    /* TRANS: Obscure ruleset error. */
-    log_error(_("World-ranged requirements are only supported "
-                "for wonders."));
+    log_error("World-ranged requirements are only supported for wonders.");
     return 0;
   }
 }
@@ -574,9 +613,7 @@ static bool player_has_ever_built(const struct player *pplayer,
     return (wonder_is_built(pplayer, building)
             || wonder_is_lost(pplayer, building) ? TRUE : FALSE);
   } else {
-    /* TRANS: Obscure ruleset error. */
-    log_error(_("Player-ranged requirements are only supported "
-                "for wonders."));
+    log_error("Player-ranged requirements are only supported for wonders.");
     return FALSE;
   }
 }
@@ -590,9 +627,7 @@ static int num_player_buildings(const struct player *pplayer,
   if (is_wonder(building)) {
     return (wonder_is_built(pplayer, building) ? 1 : 0);
   } else {
-    /* TRANS: Obscure ruleset error. */
-    log_error(_("Player-ranged requirements are only supported "
-                "for wonders."));
+    log_error("Player-ranged requirements are only supported for wonders.");
     return 0;
   }
 }
@@ -612,9 +647,7 @@ static int num_continent_buildings(const struct player *pplayer,
       return 1;
     }
   } else {
-    /* TRANS: Obscure ruleset error. */
-    log_error(_("Island-ranged requirements are only supported "
-                "for wonders."));
+    log_error("Island-ranged requirements are only supported for wonders.");
   }
   return 0;
 }
@@ -629,7 +662,8 @@ static int num_city_buildings(const struct city *pcity,
 }
 
 /****************************************************************************
-  How many of the source building are there within range of the target?
+  Are there any source buildings within range of the target that are not
+  obsolete?
 
   The target gives the type of the target.  The exact target is a player,
   city, or building specified by the target_xxx arguments.
@@ -641,69 +675,94 @@ static int num_city_buildings(const struct city *pcity,
   living buildings are counted.
 
   source gives the building type of the source in question.
-
-  Note that this function does a lookup into the source caches to find
-  the number of available sources.  However not all source caches exist: if
-  the cache doesn't exist then we return 0.
 ****************************************************************************/
-static int count_buildings_in_range(const struct player *target_player,
-				    const struct city *target_city,
-				    const struct impr_type *target_building,
-				    enum req_range range,
-				    bool survives,
-				    const struct impr_type *source)
+static enum fc_tristate
+is_building_in_range(const struct player *target_player,
+                     const struct city *target_city,
+                     const struct impr_type *target_building,
+                     enum req_range range,
+                     bool survives,
+                     const struct impr_type *source)
 {
+  /* Check if it's certain that the building is obsolete given the
+   * specification we have */
   if (improvement_obsolete(target_player, source)) {
-    return 0;
+    return TRI_NO;
   }
 
   if (survives) {
-    if (range == REQ_RANGE_WORLD) {
-      return num_world_buildings_total(source);
-    } else if (range == REQ_RANGE_PLAYER) {
-      return player_has_ever_built(target_player, source);
-    } else {
+
+    /* Check whether condition has ever held, using cached information. */
+    switch (range) {
+    case REQ_RANGE_WORLD:
+      return BOOL_TO_TRISTATE(num_world_buildings_total(source) > 0);
+    case REQ_RANGE_PLAYER:
+      if (target_player == NULL) {
+        return TRI_MAYBE;
+      }
+      return BOOL_TO_TRISTATE(player_has_ever_built(target_player, source));
+    case REQ_RANGE_CONTINENT:
+    case REQ_RANGE_CITY:
+    case REQ_RANGE_LOCAL:
+    case REQ_RANGE_CADJACENT:
+    case REQ_RANGE_ADJACENT:
       /* There is no sources cache for this. */
-      /* TRANS: Obscure ruleset error. */
-      log_error(_("Surviving requirements are only supported "
-                  "at world and player ranges."));
-      return 0;
+      log_error("Surviving requirements are only supported at "
+                "world and player ranges.");
+      return TRI_NO;
+    case REQ_RANGE_COUNT:
+      break;
     }
-  }
 
-  switch (range) {
-  case REQ_RANGE_WORLD:
-    return num_world_buildings(source);
-  case REQ_RANGE_PLAYER:
-    return target_player ? num_player_buildings(target_player, source) : 0;
-  case REQ_RANGE_CONTINENT:
-    if (target_player && target_city) {
-      int continent = tile_continent(target_city->tile);
+  } else {
 
-      return num_continent_buildings(target_player, continent, source);
-    } else {
+    /* Non-surviving requirement. */
+    switch (range) {
+    case REQ_RANGE_WORLD:
+      return BOOL_TO_TRISTATE(num_world_buildings(source) > 0);
+    case REQ_RANGE_PLAYER:
+      if (target_player == NULL) {
+        return TRI_MAYBE;
+      }
+      return BOOL_TO_TRISTATE(num_player_buildings(target_player, source) > 0);
+    case REQ_RANGE_CONTINENT:
       /* At present, "Continent" effects can affect only
        * cities and units in cities. */
-      return 0;
+      if (target_player && target_city) {
+        int continent = tile_continent(target_city->tile);
+        return BOOL_TO_TRISTATE(num_continent_buildings(target_player,
+                                                        continent, source) > 0);
+      } else {
+        return TRI_MAYBE;
+      }
+    case REQ_RANGE_CITY:
+      if (target_city) {
+        return BOOL_TO_TRISTATE(num_city_buildings(target_city, source) > 0);
+      } else {
+        return TRI_MAYBE;
+      }
+    case REQ_RANGE_LOCAL:
+      if (target_building) {
+        if (target_building == source) {
+          return BOOL_TO_TRISTATE(num_city_buildings(target_city, source) > 0);
+        } else {
+          return TRI_NO;
+        }
+      } else {
+        /* TODO: other local targets */
+        return TRI_MAYBE;
+      }
+    case REQ_RANGE_CADJACENT:
+    case REQ_RANGE_ADJACENT:
+      return TRI_NO;
+    case REQ_RANGE_COUNT:
+      break;
     }
-  case REQ_RANGE_CITY:
-    return target_city ? num_city_buildings(target_city, source) : 0;
-  case REQ_RANGE_LOCAL:
-    if (target_building && target_building == source) {
-      return num_city_buildings(target_city, source);
-    } else {
-      /* TODO: other local targets */
-      return 0;
-    }
-  case REQ_RANGE_CADJACENT:
-  case REQ_RANGE_ADJACENT:
-    return 0;
-  case REQ_RANGE_COUNT:
-    break;
+
   }
 
   fc_assert_msg(FALSE, "Invalid range %d.", range);
-  return 0;
+  return TRI_NO;
 }
 
 /****************************************************************************
@@ -906,6 +965,10 @@ static enum fc_tristate is_nation_in_range(const struct player *target_player,
     }
     return BOOL_TO_TRISTATE(nation_of_player(target_player) == nation);
   case REQ_RANGE_WORLD:
+    /* NB: if a player is ever removed outright from the game
+     * (e.g. via /remove), rather than just dying, this 'survives'
+     * requirement will stop being true for their nation.
+     * create_command_newcomer() can also cause this to happen. */
     return BOOL_TO_TRISTATE(NULL != nation->player
                             && (survives || nation->player->is_alive));
   case REQ_RANGE_LOCAL:
@@ -1096,21 +1159,10 @@ bool is_req_active(const struct player *target_player,
     }
     break;
   case VUT_IMPROVEMENT:
-    /* The requirement is filled if there's at least one of the building
-     * in the city.  (This is a slightly nonstandard use of
-     * count_sources_in_range.) */
-    /* With NULL target_player count_buildings_in_range() would
-     * return 0, which would cause evaluation to return FALSE
-     * even for RPT_POSSIBLE */
-    if ((req->range == REQ_RANGE_PLAYER && target_player == NULL)
-        || (req->range == REQ_RANGE_CITY && target_city == NULL)) {
-      eval = TRI_MAYBE;
-    } else {
-      eval = BOOL_TO_TRISTATE(count_buildings_in_range(target_player, target_city,
-                                                       target_building,
-                                                       req->range, req->survives,
-                                                       req->source.value.building) > 0);
-    }
+    eval = is_building_in_range(target_player, target_city,
+                                target_building,
+                                req->range, req->survives,
+                                req->source.value.building);
     break;
   case VUT_SPECIAL:
     if (target_tile == NULL) {
