@@ -12,7 +12,7 @@
    GNU General Public License for more details.
 ****************************************************************************/
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 /* utility */
