@@ -14,6 +14,10 @@
 #ifndef FC__TEAM_H
 #define FC__TEAM_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 #include "fc_types.h"
 
 #include "tech.h"
@@ -80,4 +84,8 @@ void team_remove_player(struct player *pplayer);
 #define teams_iterate_end                                                   \
   } team_slots_iterate_end;
 
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
 #endif /* FC__TEAM_H */
