@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 /* utility */
@@ -829,10 +829,6 @@ struct tile *tile_virtual_new(const struct tile *ptile)
   vtile = fc_calloc(1, sizeof(*vtile));
 
   /* initialise some values */
-  vtile->x = -1;
-  vtile->y = -1;
-  vtile->nat_x = -1;
-  vtile->nat_y = -1;
   vtile->index = -1;
   vtile->continent = -1;
 
@@ -848,9 +844,8 @@ struct tile *tile_virtual_new(const struct tile *ptile)
 
   if (ptile) {
     /* Used by is_city_center to give virtual tiles the output bonuses
-     * they deserve */
-    vtile->x = ptile->x;
-    vtile->y = ptile->y;
+     * they deserve. */
+    vtile->index = tile_index(ptile);
 
     /* Copy all but the unit list. */
     tile_special_type_iterate(spe) {
@@ -898,7 +893,7 @@ void tile_virtual_destroy(struct tile *vtile)
   if (vtile->units) {
     unit_list_iterate(vtile->units, vunit) {
       if (unit_is_virtual(vunit)) {
-        destroy_unit_virtual(vunit);
+        unit_virtual_destroy(vunit);
       }
     } unit_list_iterate_end;
     unit_list_destroy(vtile->units);
@@ -928,12 +923,6 @@ bool tile_virtual_check(struct tile *vtile)
   }
 
   tindex = tile_index(vtile);
-
-  if (tindex == -1) {
-    /* This is a virtual tile. */
-    return TRUE;
-  }
-
   fc_assert_ret_val(0 <= tindex && tindex < map_num_tiles(), FALSE);
 
   return (vtile != map.tiles + tindex);
@@ -951,3 +940,32 @@ void *tile_hash_key(const struct tile *ptile)
 
   return key;
 }
+
+/****************************************************************************
+  Sets label for tile. Returns whether label changed.
+****************************************************************************/
+bool tile_set_label(struct tile *ptile, const char *label)
+{
+  bool changed = FALSE;
+
+  /* Handle empty label as NULL label */
+  if (label != NULL && label[0] == '\0') {
+    label = NULL;
+  }
+
+  if (ptile->label != NULL) {
+    if (label == NULL) {
+      changed = TRUE;
+    }
+    FC_FREE(ptile->label);
+    ptile->label = NULL;
+  } else if (label != NULL) {
+    changed = TRUE;
+  }
+
+  if (label != NULL) {
+    ptile->label = fc_strdup(label);
+  }
+
+  return changed;
+}
