@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 /* utility */
@@ -33,6 +33,7 @@
 #include "movement.h"
 #include "packets.h"
 #include "player.h"
+#include "road.h"
 #include "tech.h"
 #include "unitlist.h"
 
@@ -59,8 +60,8 @@ bool diplomat_can_do_action(const struct unit *pdiplomat,
     return FALSE;
   }
 
-  if (!is_tiles_adjacent(pdiplomat->tile, ptile)
-      && !same_pos(pdiplomat->tile, ptile)) {
+  if (!is_tiles_adjacent(unit_tile(pdiplomat), ptile)
+      && !same_pos(unit_tile(pdiplomat), ptile)) {
     return FALSE;
   }
 
@@ -83,13 +84,13 @@ bool is_diplomat_action_available(const struct unit *pdiplomat,
   struct city *pcity=tile_city(ptile);
 
   if (action != DIPLOMAT_MOVE
-      && !can_unit_exist_at_tile(pdiplomat, pdiplomat->tile)) {
+      && !can_unit_exist_at_tile(pdiplomat, unit_tile(pdiplomat))) {
     return FALSE;
   }
 
   if (pcity) {
     if (city_owner(pcity) != unit_owner(pdiplomat)
-       && real_map_distance(pdiplomat->tile, pcity->tile) <= 1) {
+       && real_map_distance(unit_tile(pdiplomat), pcity->tile) <= 1) {
       if(action==DIPLOMAT_SABOTAGE)
 	return pplayers_at_war(unit_owner(pdiplomat), city_owner(pcity));
       if(action==DIPLOMAT_MOVE)
@@ -100,10 +101,11 @@ bool is_diplomat_action_available(const struct unit *pdiplomat,
                                       city_owner(pcity))) {
         return TRUE;
       }
-      if(action==SPY_POISON &&
-	 pcity->size>1 &&
-	 unit_has_type_flag(pdiplomat, F_SPY))
-	return pplayers_at_war(unit_owner(pdiplomat), city_owner(pcity));
+      if(action == SPY_POISON
+         && city_size_get(pcity) > 1
+         && unit_has_type_flag(pdiplomat, F_SPY)) {
+        return pplayers_at_war(unit_owner(pdiplomat), city_owner(pcity));
+      }
       if(action==DIPLOMAT_INVESTIGATE)
         return TRUE;
       if (action == DIPLOMAT_STEAL && !is_barbarian(city_owner(pcity))) {
@@ -155,7 +157,7 @@ enum unit_airlift_result
                              const struct unit *punit,
                              const struct city *pdest_city)
 {
-  const struct city *psrc_city = tile_city(punit->tile);
+  const struct city *psrc_city = tile_city(unit_tile(punit));
   const struct player *punit_owner;
   enum unit_airlift_result ok_result = AR_OK;
 
@@ -286,8 +288,8 @@ bool unit_has_orders(const struct unit *punit)
 bool unit_can_help_build_wonder(const struct unit *punit,
 				const struct city *pcity)
 {
-  if (!is_tiles_adjacent(punit->tile, pcity->tile)
-      && !same_pos(punit->tile, pcity->tile)) {
+  if (!is_tiles_adjacent(unit_tile(punit), pcity->tile)
+      && !same_pos(unit_tile(punit), pcity->tile)) {
     return FALSE;
   }
 
@@ -306,7 +308,7 @@ bool unit_can_help_build_wonder(const struct unit *punit,
 **************************************************************************/
 bool unit_can_help_build_wonder_here(const struct unit *punit)
 {
-  struct city *pcity = tile_city(punit->tile);
+  struct city *pcity = tile_city(unit_tile(punit));
 
   return pcity && unit_can_help_build_wonder(punit, pcity);
 }
@@ -321,7 +323,7 @@ bool unit_can_est_trade_route_here(const struct unit *punit)
   struct city *phomecity, *pdestcity;
 
   return (unit_has_type_flag(punit, F_TRADE_ROUTE)
-          && (pdestcity = tile_city(punit->tile))
+          && (pdestcity = tile_city(unit_tile(punit)))
           && (phomecity = game_city_by_number(punit->homecity))
           && can_cities_trade(phomecity, pdestcity));
 }
@@ -382,7 +384,10 @@ bool is_square_threatened(const struct player *pplayer,
 {
   square_iterate(ptile, 2, ptile1) {
     unit_list_iterate(ptile1->units, punit) {
-      if (is_ground_threat(pplayer, punit)) {
+      if (((pplayer->ai_controlled
+            && !ai_handicap(pplayer, H_MAP))
+           || can_player_see_unit_at(pplayer, punit, ptile1))
+          && is_ground_threat(pplayer, punit)) {
 	return TRUE;
       }
     } unit_list_iterate_end;
@@ -408,10 +413,10 @@ bool is_field_unit(const struct unit *punit)
 **************************************************************************/
 bool is_hiding_unit(const struct unit *punit)
 {
-  struct unit *transporter = game_unit_by_number(punit->transported_by);
-
   return (unit_has_type_flag(punit, F_PARTIAL_INVIS)
-	  || (transporter && unit_has_type_flag(transporter, F_PARTIAL_INVIS)));
+          || (unit_transported(punit)
+              && unit_has_type_flag(unit_transport_get(punit),
+                                    F_PARTIAL_INVIS)));
 }
 
 /**************************************************************************
@@ -500,7 +505,7 @@ unit_add_or_build_city_test(const struct unit *punit)
   }
 
   fc_assert(unit_pop_value(punit) > 0);
-  new_pop = pcity->size + unit_pop_value(punit);
+  new_pop = city_size_get(pcity) + unit_pop_value(punit);
 
   if (new_pop > game.info.add_to_size_limit) {
     return UAB_TOO_BIG;
@@ -520,7 +525,7 @@ unit_add_or_build_city_test(const struct unit *punit)
 bool can_unit_change_homecity_to(const struct unit *punit,
 				 const struct city *pcity)
 {
-  struct city *acity = tile_city(punit->tile);
+  struct city *acity = tile_city(unit_tile(punit));
 
   /* Requirements to change homecity:
    *
@@ -541,7 +546,7 @@ bool can_unit_change_homecity_to(const struct unit *punit,
 **************************************************************************/
 bool can_unit_change_homecity(const struct unit *punit)
 {
-  return can_unit_change_homecity_to(punit, tile_city(punit->tile));
+  return can_unit_change_homecity_to(punit, tile_city(unit_tile(punit)));
 }
 
 /**************************************************************************
@@ -554,7 +559,12 @@ bool can_unit_change_homecity(const struct unit *punit)
 **************************************************************************/
 int get_activity_rate(const struct unit *punit)
 {
-  double fact = unit_type(punit)->veteran[punit->veteran].power_fact;
+  const struct veteran_level *vlevel;
+
+  fc_assert_ret_val(punit != NULL, 0);
+
+  vlevel = utype_veteran_level(unit_type(punit), punit->veteran);
+  fc_assert_ret_val(vlevel != NULL, 0);
 
   /* The speed of the settler depends on its base move_rate, not on
    * the number of moves actually remaining or the adjusted move rate.
@@ -564,7 +574,9 @@ int get_activity_rate(const struct unit *punit)
   int move_rate = unit_type(punit)->move_rate;
 
   /* All settler actions are multiplied by ACTIVITY_COUNT. */
-  return ACTIVITY_FACTOR * fact * move_rate / SINGLE_MOVE;
+  return ACTIVITY_FACTOR
+         * (float)vlevel->power_fact / 100
+         * move_rate / SINGLE_MOVE;
 }
 
 /**************************************************************************
@@ -658,10 +670,11 @@ bool can_unit_do_autosettlers(const struct unit *punit)
 bool is_real_activity(enum unit_activity activity)
 {
   /* ACTIVITY_FORTRESS and ACTIVITY_AIRBASE are deprecated */
-  return activity != ACTIVITY_FORTRESS
-    && activity != ACTIVITY_AIRBASE
-    && activity != ACTIVITY_UNKNOWN
-    && activity != ACTIVITY_PATROL_UNUSED;
+  return (0 <= activity && activity < ACTIVITY_LAST)
+          && activity != ACTIVITY_FORTRESS
+          && activity != ACTIVITY_AIRBASE
+          && activity != ACTIVITY_UNKNOWN
+          && activity != ACTIVITY_PATROL_UNUSED;
 }
 
 /**************************************************************************
@@ -678,7 +691,8 @@ const char *get_activity_text(enum unit_activity activity)
   case ACTIVITY_POLLUTION:
     return _("Pollution");
   case ACTIVITY_ROAD:
-    return _("Road");
+  case ACTIVITY_RAILROAD:
+    return road_name_translation(road_by_activity(activity));
   case ACTIVITY_MINE:
     return _("Mine");
   case ACTIVITY_IRRIGATE:
@@ -691,8 +705,6 @@ const char *get_activity_text(enum unit_activity activity)
     return _("Fortress");
   case ACTIVITY_SENTRY:
     return _("Sentry");
-  case ACTIVITY_RAILROAD:
-    return _("Railroad");
   case ACTIVITY_PILLAGE:
     return _("Pillage");
   case ACTIVITY_GOTO:
@@ -734,12 +746,14 @@ bool could_unit_load(const struct unit *pcargo, const struct unit *ptrans)
   }
 
   /* Only top-level transporters may be loaded or loaded into. */
-  if (pcargo->transported_by != -1 || ptrans->transported_by != -1) {
+  if (unit_transported(pcargo)
+      || unit_transported(ptrans)) {
     return FALSE;
   }
 
-  /* Recursive transporting is not allowed (for now). */
-  if (get_transporter_occupancy(pcargo) > 0) {
+  /* Check iff this is a valid transport. */
+  if (unit_transported(pcargo)
+      && !unit_transport_check(pcargo, ptrans)) {
     return FALSE;
   }
 
@@ -748,8 +762,10 @@ bool could_unit_load(const struct unit *pcargo, const struct unit *ptrans)
     return FALSE;
   }
 
-  /* Transporter must be native to the tile it is on. */
-  if (!can_unit_exist_at_tile(ptrans, ptrans->tile)) {
+  /* Transporter must be native to the tile it is on (or it itself is
+   * transported). */
+  if (!can_unit_exist_at_tile(ptrans, unit_tile(ptrans))
+      && !unit_transported(ptrans)) {
     return FALSE;
   }
 
@@ -767,7 +783,7 @@ bool can_unit_load(const struct unit *pcargo, const struct unit *ptrans)
 
   /* Check positions of the units.  Of course you can't load a unit onto
    * a transporter on a different tile... */
-  if (!same_pos(pcargo->tile, ptrans->tile)) {
+  if (!same_pos(unit_tile(pcargo), unit_tile(ptrans))) {
     return FALSE;
   }
 
@@ -790,14 +806,14 @@ bool can_unit_unload(const struct unit *pcargo, const struct unit *ptrans)
   }
 
   /* Make sure the unit's transporter exists and is known. */
-  if (pcargo->transported_by != ptrans->id) {
+  if (unit_transport_get(pcargo) != ptrans) {
     return FALSE;
   }
 
   /* Only top-level transporters may be unloaded.  However the unit being
    * unloaded may be transporting other units (well, at least it's allowed
    * here: elsewhere this may be disallowed). */
-  if (ptrans->transported_by != -1) {
+  if (unit_transport_get(ptrans)) {
     return FALSE;
   }
 
@@ -824,13 +840,13 @@ bool can_unit_paradrop(const struct unit *punit)
   if(punit->moves_left < utype->paratroopers_mr_req)
     return FALSE;
 
-  if (tile_has_base_flag(punit->tile, BF_PARADROP_FROM)) {
+  if (tile_has_base_flag(unit_tile(punit), BF_PARADROP_FROM)) {
     /* Paradrop has to be possible from non-native base.
      * Paratroopers are "Land" units, but they can paradrom from Airbase. */
     return TRUE;
   }
 
-  if (!tile_city(punit->tile)) {
+  if (!tile_city(unit_tile(punit))) {
     return FALSE;
   }
 
@@ -848,7 +864,7 @@ bool can_unit_bombard(const struct unit *punit)
     return FALSE;
   }
 
-  if (punit->transported_by != -1) {
+  if (unit_transported(punit)) {
     return FALSE;
   }
 
@@ -913,7 +929,7 @@ bool can_unit_do_activity_targeted(const struct unit *punit,
                                    Base_type_id base)
 {
   return can_unit_do_activity_targeted_at(punit, activity, target,
-					  punit->tile, base);
+					  unit_tile(punit), base);
 }
 
 /**************************************************************************
@@ -992,7 +1008,7 @@ bool can_unit_do_activity_targeted_at(const struct unit *punit,
 	    || (!tile_has_special(ptile, S_FARMLAND)
 		&& player_knows_techs_with_flag(pplayer, TF_FARMLAND)))
 	&& ((pterrain == pterrain->irrigation_result
-	     && is_water_adjacent_to_tile(ptile))
+             && can_be_irrigated(ptile, punit))
 	    || (pterrain != pterrain->irrigation_result
 		&& pterrain->irrigation_result != T_NONE
 		&& (!is_ocean(pterrain)
@@ -1026,8 +1042,8 @@ bool can_unit_do_activity_targeted_at(const struct unit *punit,
     return can_build_base(punit, pbase, ptile);
 
   case ACTIVITY_SENTRY:
-    if (!can_unit_survive_at_tile(punit, punit->tile)
-	&& punit->transported_by == -1) {
+    if (!can_unit_survive_at_tile(punit, unit_tile(punit))
+        && !unit_transported(punit)) {
       /* Don't let units sentry on tiles they will die on. */
       return FALSE;
     }
@@ -1305,39 +1321,19 @@ void unit_activity_astr(const struct unit *punit, struct astring *astr)
   case ACTIVITY_IDLE:
     if (utype_fuel(unit_type(punit))) {
       int rate, f;
-      rate = unit_type(punit)->move_rate / SINGLE_MOVE;
+      rate = unit_type(punit)->move_rate;
       f = ((punit->fuel) - 1);
 
-      if ((punit->moves_left % SINGLE_MOVE) != 0) {
-        if (punit->moves_left / SINGLE_MOVE > 0) {
-          astr_add_line(astr, "%s: (%d)%d %d/%d", _("Moves"),
-                        ((rate * f) + (punit->moves_left / SINGLE_MOVE)),
-                        punit->moves_left / SINGLE_MOVE,
-                        punit->moves_left % SINGLE_MOVE, SINGLE_MOVE);
-        } else {
-          astr_add_line(astr, "%s: (%d)%d/%d", _("Moves"),
-                        ((rate * f) + (punit->moves_left / SINGLE_MOVE)),
-                        punit->moves_left % SINGLE_MOVE, SINGLE_MOVE);
-        }
-      } else {
-        astr_add_line(astr, "%s: (%d)%d", _("Moves"),
-                      rate * f + punit->moves_left / SINGLE_MOVE,
-                      punit->moves_left / SINGLE_MOVE);
-      }
+      /* Add in two parts as move_points_text() returns ptr to static
+       * End result: "Moves: (fuel)moves_left" */
+      astr_add_line(astr, "%s: (%s)", _("Moves"),
+                    move_points_text((rate * f) + punit->moves_left,
+                                     NULL, NULL, FALSE));
+      astr_add(astr, "%s",
+               move_points_text(punit->moves_left, NULL, NULL, FALSE));
     } else {
-      if ((punit->moves_left % SINGLE_MOVE) != 0) {
-        if (punit->moves_left / SINGLE_MOVE > 0) {
-          astr_add_line(astr, "%s: %d %d/%d", _("Moves"),
-                        punit->moves_left / SINGLE_MOVE,
-                        punit->moves_left % SINGLE_MOVE, SINGLE_MOVE);
-        } else {
-          astr_add_line(astr, "%s: %d/%d", _("Moves"),
-                        punit->moves_left % SINGLE_MOVE, SINGLE_MOVE);
-        }
-      } else {
-        astr_add_line(astr, "%s: %d", _("Moves"),
-                      punit->moves_left / SINGLE_MOVE);
-      }
+      astr_add_line(astr, "%s: %s", _("Moves"),
+                    move_points_text(punit->moves_left, NULL, NULL, FALSE));
     }
     return;
   case ACTIVITY_POLLUTION:
@@ -1428,6 +1424,15 @@ struct tile *unit_tile(const struct unit *punit)
   return punit->tile;
 }
 
+/*****************************************************************************
+  Set the tile location of the unit. Tile can be NULL (for transported units.
+*****************************************************************************/
+void unit_tile_set(struct unit *punit, struct tile *ptile)
+{
+  fc_assert_ret(NULL != punit);
+  punit->tile = ptile;
+}
+
 /**************************************************************************
 Returns true if the tile contains an allied unit and only allied units.
 (ie, if your nation A is allied with B, and B is allied with C, a tile
@@ -1585,15 +1590,15 @@ bool unit_being_aggressive(const struct unit *punit)
   if (!is_attack_unit(punit)) {
     return FALSE;
   }
-  if (tile_city(punit->tile)) {
+  if (tile_city(unit_tile(punit))) {
     return FALSE;
   }
   if (BORDERS_DISABLED != game.info.borders
       && game.info.happyborders
-      && tile_owner(punit->tile) == unit_owner(punit)) {
+      && tile_owner(unit_tile(punit)) == unit_owner(punit)) {
     return FALSE;
   }
-  if (tile_has_base_flag_for_unit(punit->tile,
+  if (tile_has_base_flag_for_unit(unit_tile(punit),
                                   unit_type(punit),
                                   BF_NOT_AGGRESSIVE)) {
     return !is_unit_near_a_friendly_city (punit);
@@ -1628,7 +1633,7 @@ bool is_build_or_clean_activity(enum unit_activity activity)
   Create a virtual unit skeleton. pcity can be NULL, but then you need
   to set tile and homecity yourself.
 **************************************************************************/
-struct unit *create_unit_virtual(struct player *pplayer, struct city *pcity,
+struct unit *unit_virtual_create(struct player *pplayer, struct city *pcity,
                                  struct unit_type *punittype,
                                  int veteran_level)
 {
@@ -1646,17 +1651,19 @@ struct unit *create_unit_virtual(struct player *pplayer, struct city *pcity,
   fc_assert_ret_val(NULL != pplayer, NULL);     /* No unowned units! */
   punit->owner = pplayer;
 
+  punit->facing = rand_direction();
+
   if (pcity) {
-    punit->tile = pcity->tile;
+    unit_tile_set(punit, pcity->tile);
     punit->homecity = pcity->id;
   } else {
-    punit->tile = NULL;
+    unit_tile_set(punit, NULL);
     punit->homecity = IDENTITY_NUMBER_ZERO;
   }
 
   memset(punit->upkeep, 0, O_LAST * sizeof(*punit->upkeep));
   punit->goto_tile = NULL;
-  max_vet_lvl = punittype->veteran_levels - 1;
+  max_vet_lvl = utype_veteran_levels(punittype) - 1;
   punit->veteran = MIN(veteran_level, max_vet_lvl);
   /* A unit new and fresh ... */
   punit->fuel = utype_fuel(unit_type(punit));
@@ -1668,9 +1675,10 @@ struct unit *create_unit_virtual(struct player *pplayer, struct city *pcity,
   punit->paradropped = FALSE;
   punit->done_moving = FALSE;
 
-  punit->transported_by = -1;
+  punit->transporter = NULL;
+  punit->transporting = unit_list_new();
+
   set_unit_activity(punit, ACTIVITY_IDLE);
-  punit->occupy = 0;
   punit->battlegroup = BATTLEGROUP_NONE;
   punit->has_orders = FALSE;
 
@@ -1701,10 +1709,27 @@ struct unit *create_unit_virtual(struct player *pplayer, struct city *pcity,
   Free the memory used by virtual unit. By the time this function is
   called, you should already have unregistered it everywhere.
 **************************************************************************/
-void destroy_unit_virtual(struct unit *punit)
+void unit_virtual_destroy(struct unit *punit)
 {
   free_unit_orders(punit);
 
+  /* Unload unit if transported. */
+  unit_transport_unload(punit);
+  fc_assert(!unit_transported(punit));
+
+  /* Check for transported units. Use direct access to the list. */
+  if (unit_list_size(punit->transporting) != 0) {
+    /* Unload all units. */
+    unit_list_iterate_safe(punit->transporting, pcargo) {
+      unit_transport_unload(pcargo);
+    } unit_list_iterate_safe_end;
+  }
+  fc_assert(unit_list_size(punit->transporting) == 0);
+
+  if (punit->transporting) {
+    unit_list_destroy(punit->transporting);
+  }
+
   CALL_PLR_AI_FUNC(unit_lost, punit->owner, punit);
   CALL_FUNC_EACH_AI(unit_free, punit);
 
@@ -1730,19 +1755,13 @@ void free_unit_orders(struct unit *punit)
 }
 
 /****************************************************************************
-  Expensive function to check how many units are in the transport.
+  Return how many units are in the transport.
 ****************************************************************************/
 int get_transporter_occupancy(const struct unit *ptrans)
 {
-  int occupied = 0;
+  fc_assert_ret_val(ptrans, -1);
 
-  unit_list_iterate(ptrans->tile->units, pcargo) {
-    if (pcargo->transported_by == ptrans->id) {
-      occupied++;
-    }
-  } unit_list_iterate_end;
-
-  return occupied;
+  return unit_list_size(ptrans->transporting);
 }
 
 /****************************************************************************
@@ -1750,7 +1769,7 @@ int get_transporter_occupancy(const struct unit *ptrans)
 ****************************************************************************/
 struct unit *transporter_for_unit(const struct unit *pcargo)
 {
-  struct tile *ptile = pcargo->tile;
+  struct tile *ptile = unit_tile(pcargo);
 
   unit_list_iterate(ptile->units, ptrans) {
     if (can_unit_load(pcargo, ptrans)) {
@@ -1772,9 +1791,8 @@ static bool can_type_transport_units_cargo(const struct unit_type *utype,
     return FALSE;
   }
 
-  unit_list_iterate(unit_tile(punit)->units, pcargo) {
-    if (pcargo->transported_by == punit->id
-        && !can_unit_type_transport(utype, unit_class(pcargo))) {
+  unit_list_iterate(punit->transporting, pcargo) {
+    if (!can_unit_type_transport(utype, unit_class(pcargo))) {
       return FALSE;
     }
   } unit_list_iterate_end;
@@ -1809,7 +1827,7 @@ enum unit_upgrade_result unit_upgrade_test(const struct unit *punit,
       return UU_NO_MONEY;
     }
 
-    pcity = tile_city(punit->tile);
+    pcity = tile_city(unit_tile(punit));
     if (!pcity) {
       return UU_NOT_IN_CITY;
     }
@@ -1994,3 +2012,204 @@ void unit_set_ai_data(struct unit *punit, const struct ai_type *ai,
 {
   punit->server.ais[ai_type_number(ai)] = data;
 }
+
+/*****************************************************************************
+  Calculate how expensive it is to bribe the unit. The cost depends on the
+  distance to the capital, the owner's treasury, and the build cost of the
+  unit. For a damaged unit the price is reduced. For a veteran unit, it is
+  increased.
+
+  The bribe cost for settlers are halved.
+**************************************************************************/
+int unit_bribe_cost(struct unit *punit)
+{
+  int cost, default_hp, dist = 0;
+  struct city *capital;
+
+  fc_assert_ret_val(punit != NULL, 0);
+
+  default_hp = unit_type(punit)->hp;
+  cost = unit_owner(punit)->economic.gold + game.info.base_bribe_cost;
+  capital = player_capital(unit_owner(punit));
+
+  /* Consider the distance to the capital. */
+  if (capital != NULL) {
+    dist = MIN(GAME_UNIT_BRIBE_DIST_MAX,
+               map_distance(capital->tile, unit_tile(punit)));
+  } else {
+    dist = GAME_UNIT_BRIBE_DIST_MAX;
+  }
+  cost /= dist + 2;
+
+  /* Consider the build cost. */
+  cost *= unit_build_shield_cost(punit) / 10;
+
+  /* FIXME: This is a weird one - should be replaced. */
+  if (unit_has_type_flag(punit, F_CITIES)) {
+    cost /= 2;
+  }
+
+  /* Veterans are not cheap. */
+  {
+    const struct veteran_level *vlevel
+      = utype_veteran_level(unit_type(punit), punit->veteran);
+    fc_assert_ret_val(vlevel != NULL, 0);
+    cost = cost * vlevel->power_fact / 100;
+    if (unit_type(punit)->move_rate > 0) {
+      cost += cost * vlevel->move_bonus / unit_type(punit)->move_rate;
+    } else {
+      cost += cost * vlevel->move_bonus / SINGLE_MOVE;
+    }
+  }
+
+  /* Cost now contains the basic bribe cost.  We now reduce it by:
+   *    bribecost = cost/2 + cost/2 * damage/hp
+   *              = cost/2 * (1 + damage/hp) */
+  return ((float)cost / 2 * (1.0 + punit->hp / default_hp));
+}
+
+/*****************************************************************************
+  Load pcargo onto ptrans. Returns TRUE on success.
+*****************************************************************************/
+bool unit_transport_load(struct unit *pcargo, struct unit *ptrans, bool force)
+{
+  fc_assert_ret_val(ptrans != NULL, FALSE);
+  fc_assert_ret_val(pcargo != NULL, FALSE);
+
+  fc_assert_ret_val(!unit_list_search(ptrans->transporting, pcargo), FALSE);
+
+  if (force || can_unit_load(pcargo, ptrans)) {
+    pcargo->transporter = ptrans;
+    unit_list_append(ptrans->transporting, pcargo);
+
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
+/*****************************************************************************
+  Unload pcargo from ptrans. Returns TRUE on success.
+*****************************************************************************/
+bool unit_transport_unload(struct unit *pcargo)
+{
+  struct unit *ptrans;
+
+  fc_assert_ret_val(pcargo != NULL, FALSE);
+
+  if (!unit_transported(pcargo)) {
+    /* 'pcargo' is not transported. */
+    return FALSE;
+  }
+
+  /* Get the transporter; must not be defined on the client! */
+  ptrans = unit_transport_get(pcargo);
+  if (ptrans) {
+    bool success;
+
+    /* 'pcargo' and 'ptrans' should be on the same tile. */
+    fc_assert_ret_val(same_pos(unit_tile(pcargo), unit_tile(ptrans)), FALSE);
+    /* It is an error if 'pcargo' can not be removed from the 'ptrans'. */
+    success = unit_list_remove(ptrans->transporting, pcargo);
+    fc_assert(success);
+  }
+
+  /* For the server (also safe for the client). */
+  pcargo->transporter = NULL;
+
+  return TRUE;
+}
+
+/*****************************************************************************
+  Returns TRUE iff the unit is transported.
+*****************************************************************************/
+bool unit_transported(const struct unit *pcargo)
+{
+  fc_assert_ret_val(pcargo != NULL, FALSE);
+
+  /* The unit is transported if a transporter unit is set or, (for the client)
+   * if the transported_by field is set. */
+  if (pcargo->transporter != NULL
+      || (!is_server() && pcargo->client.transported_by != -1)) {
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
+/*****************************************************************************
+  Returns the transporter of the unit or NULL if it is not transported.
+*****************************************************************************/
+struct unit *unit_transport_get(const struct unit *pcargo)
+{
+  fc_assert_ret_val(pcargo != NULL, NULL);
+
+  if (pcargo->transporter == NULL) {
+    return NULL;
+  } else {
+    /* Need this as the return value is not 'const'! */
+    return game_unit_by_number(pcargo->transporter->id);
+  }
+}
+
+/*****************************************************************************
+  Returns the list of cargo units.
+*****************************************************************************/
+struct unit_list *unit_transport_cargo(const struct unit *ptrans)
+{
+  fc_assert_ret_val(ptrans != NULL, NULL);
+  fc_assert_ret_val(ptrans->transporting != NULL, NULL);
+
+  return ptrans->transporting;
+}
+
+/*****************************************************************************
+  Returns if pcargo in ptrans is a valid transport.
+*****************************************************************************/
+bool unit_transport_check(const struct unit *pcargo,
+                          const struct unit *ptrans)
+{
+  struct unit_list *ptrans_recursive = unit_list_new();
+  /* Get !const unit struct for pcargo. */
+  struct unit *plevel = game_unit_by_number(pcargo->id);
+  bool transport_ok = TRUE;
+
+  /* Recursive loop over all transporters up to max level. */
+  while (transport_ok && plevel && unit_list_size(ptrans_recursive)
+                                   < GAME_TRANSPORT_MAX_RECURSIVE) {
+    /* Check if the unit can be transported. If the unit can be transported
+     * by any of the previous transporters, fail. THis disallows to carry
+     * units of one type within the same unit type. */
+    unit_list_iterate(ptrans_recursive, pcargo_recursive) {
+      if (can_unit_transport(pcargo_recursive, plevel)) {
+        transport_ok = FALSE;
+        break;
+      }
+    } unit_list_iterate_end;
+
+    if (!transport_ok) {
+#ifdef DEBUG
+      char buf[512] = "";
+      fc_snprintf(buf, sizeof(buf), "%s [Error] ",
+                  unit_name_translation(plevel));
+      unit_list_iterate(ptrans_recursive, pcargo_recursive) {
+        cat_snprintf(buf, sizeof(buf), " in %s",
+                     unit_name_translation(pcargo_recursive));
+      } unit_list_iterate_end;
+      log_error("Invalid transport at level %d (%s).",
+                unit_list_size(ptrans_recursive), buf);
+#endif /* DEBUG */
+      break;
+    }
+
+    /* Insert cargo at the beginning of the list. */
+    unit_list_prepend(ptrans_recursive, plevel);
+
+    /* Check for next level. */
+    plevel = unit_transport_get(plevel);
+  }
+
+  unit_list_destroy(ptrans_recursive);
+
+  return transport_ok;
+}
