@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <string.h>
@@ -126,7 +126,7 @@ Unit_type_id utype_number(const struct unit_type *punittype)
 **************************************************************************/
 struct unit_type *utype_by_number(const Unit_type_id id)
 {
-  if (id < 0 || id > game.control.num_unit_types) {
+  if (id < 0 || id >= game.control.num_unit_types) {
     return NULL;
   }
   return &unit_types[id];
@@ -305,7 +305,7 @@ int unit_disband_shields(const struct unit *punit)
 }
 
 /**************************************************************************
-...
+  How much city shrinks when it builds unit of this type.
 **************************************************************************/
 int utype_pop_value(const struct unit_type *punittype)
 {
@@ -313,7 +313,7 @@ int utype_pop_value(const struct unit_type *punittype)
 }
 
 /**************************************************************************
-...
+  How much population is put to building this unit.
 **************************************************************************/
 int unit_pop_value(const struct unit *punit)
 {
@@ -373,29 +373,29 @@ const char *unit_rule_name(const struct unit *punit)
 }
 
 /**************************************************************************
-...
+  Return string describing unit type values.
+  String is static buffer that gets reused when function is called again.
 **************************************************************************/
 const char *utype_values_string(const struct unit_type *punittype)
 {
   static char buffer[256];
 
+  /* Print in two parts as move_points_text() returns a static buffer */
+  fc_snprintf(buffer, sizeof(buffer), "%d/%d/%s",
+              punittype->attack_strength,
+              punittype->defense_strength,
+              move_points_text(punittype->move_rate, NULL, NULL, FALSE));
   if (utype_fuel(punittype)) {
-    fc_snprintf(buffer, sizeof(buffer), "%d/%d/%d(%d)",
-                punittype->attack_strength,
-                punittype->defense_strength,
-                punittype->move_rate / 3,
-                punittype->move_rate / 3 * utype_fuel(punittype));
-  } else {
-    fc_snprintf(buffer, sizeof(buffer), "%d/%d/%d",
-                punittype->attack_strength,
-                punittype->defense_strength,
-                punittype->move_rate / 3);
+    cat_snprintf(buffer, sizeof(buffer), "(%s)",
+                 move_points_text(punittype->move_rate * utype_fuel(punittype),
+                                  NULL, NULL, FALSE));
   }
   return buffer;
 }
 
 /**************************************************************************
-...
+  Return string with translated unit name and list of its values.
+  String is static buffer that gets reused when function is called again.
 **************************************************************************/
 const char *utype_values_translation(const struct unit_type *punittype)
 {
@@ -966,6 +966,7 @@ void unit_types_init(void)
    * num_unit_types isn't known yet. */
   for (i = 0; i < ARRAY_SIZE(unit_types); i++) {
     unit_types[i].item_number = i;
+    unit_types[i].veteran = NULL;
   }
 }
 
@@ -978,6 +979,8 @@ static void unit_type_free(struct unit_type *punittype)
     strvec_destroy(punittype->helptext);
     punittype->helptext = NULL;
   }
+
+  veteran_system_destroy(punittype->veteran);
 }
 
 /***************************************************************
@@ -1097,3 +1100,130 @@ void unit_classes_init(void)
     unit_classes[i].item_number = i;
   }
 }
+
+/****************************************************************************
+  Return veteran system used for this unit type.
+****************************************************************************/
+const struct veteran_system *
+  utype_veteran_system(const struct unit_type *punittype)
+{
+  fc_assert_ret_val(punittype != NULL, NULL);
+
+  if (punittype->veteran) {
+    return punittype->veteran;
+  }
+
+  fc_assert_ret_val(game.veteran != NULL, NULL);
+  return game.veteran;
+}
+
+/****************************************************************************
+  Return veteran level properties of given unit in given veterancy level.
+****************************************************************************/
+const struct veteran_level *
+  utype_veteran_level(const struct unit_type *punittype, int level)
+{
+  const struct veteran_system *vsystem = utype_veteran_system(punittype);
+
+  fc_assert_ret_val(vsystem != NULL, NULL);
+  fc_assert_ret_val(vsystem->definitions != NULL, NULL);
+  fc_assert_ret_val(vsystem->levels > level, NULL);
+
+  return (vsystem->definitions + level);
+}
+
+/****************************************************************************
+  Return translated name of the given veteran level.
+  NULL if this unit type doesn't have different veteran levels.
+****************************************************************************/
+const char *utype_veteran_name_translation(const struct unit_type *punittype,
+                                           int level)
+{
+  if (utype_has_flag(punittype, F_NO_VETERAN)
+      || utype_veteran_levels(punittype) <= 1) {
+    return NULL;
+  } else {
+    const struct veteran_level *vlvl = utype_veteran_level(punittype, level);
+    return name_translation(&vlvl->name);
+  }
+}
+
+/****************************************************************************
+  Return veteran levels of the given unit type.
+****************************************************************************/
+int utype_veteran_levels(const struct unit_type *punittype)
+{
+  const struct veteran_system *vsystem = utype_veteran_system(punittype);
+
+  fc_assert_ret_val(vsystem != NULL, 1);
+
+  return vsystem->levels;
+}
+
+/****************************************************************************
+  Return whether this unit type's veteran system, if any, confers a power
+  factor bonus at any level (it could just add extra moves).
+****************************************************************************/
+bool utype_veteran_has_power_bonus(const struct unit_type *punittype)
+{
+  int i, initial_power_fact = utype_veteran_level(punittype, 0)->power_fact;
+  for (i = 1; i < utype_veteran_levels(punittype); i++) {
+    if (utype_veteran_level(punittype, i)->power_fact > initial_power_fact) {
+      return TRUE;
+    }
+  }
+  return FALSE;
+}
+
+/****************************************************************************
+  Allocate new veteran system structure with given veteran level count.
+****************************************************************************/
+struct veteran_system *veteran_system_new(int count)
+{
+  struct veteran_system *vsystem;
+
+  /* There must be at least one level. */
+  fc_assert_ret_val(count > 0, NULL);
+
+  vsystem = fc_calloc(1, sizeof(*vsystem));
+  vsystem->levels = count;
+  vsystem->definitions = fc_calloc(vsystem->levels,
+                                   sizeof(*vsystem->definitions));
+
+  return vsystem;
+}
+
+/****************************************************************************
+  Free veteran system
+****************************************************************************/
+void veteran_system_destroy(struct veteran_system *vsystem)
+{
+  if (vsystem) {
+    if (vsystem->definitions) {
+      free(vsystem->definitions);
+    }
+    free(vsystem);
+  }
+}
+
+/****************************************************************************
+  Fill veteran level in given veteran system with given information.
+****************************************************************************/
+void veteran_system_definition(struct veteran_system *vsystem, int level,
+                               const char *vlist_name, int vlist_power,
+                               int vlist_move, int vlist_raise,
+                               int vlist_wraise)
+{
+  struct veteran_level *vlevel;
+
+  fc_assert_ret(vsystem != NULL);
+  fc_assert_ret(vsystem->levels > level);
+
+  vlevel = vsystem->definitions + level;
+
+  names_set(&vlevel->name, vlist_name, NULL);
+  vlevel->power_fact = vlist_power;
+  vlevel->move_bonus = vlist_move;
+  vlevel->raise_chance = vlist_raise;
+  vlevel->work_raise_chance = vlist_wraise;
+}
