@@ -12,14 +12,17 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
+/* utility */
 #include "fcintl.h"
-#include "fc_types.h"
 #include "shared.h"
 #include "support.h"
 
+/* common */
+#include "fc_types.h"
+
 #include "version.h"
 
 #ifdef SVNREV
@@ -28,7 +31,7 @@
 
 
 /**********************************************************************
-  ...
+  Return string containing both name of Freeciv and version.
 ***********************************************************************/
 const char *freeciv_name_version(void)
 {
@@ -49,7 +52,7 @@ const char *freeciv_name_version(void)
 }
 
 /**********************************************************************
-  ...
+  Return string describing version type.
 ***********************************************************************/
 const char *word_version(void)
 {
@@ -73,6 +76,19 @@ const char *fc_svn_revision(void)
 #endif /* FC_SVNREV_OFF */
 }
 
+/**********************************************************************
+  Returns version string that can be used to compare two freeciv builds.
+***********************************************************************/
+const char *fc_comparable_version(void)
+{
+#ifdef FC_SVNREV_ON
+  /* Sane revision number in FC_SVNREV */
+  return VERSION_STRING "-" FC_SVNREV;
+#else  /* FC_SVNREV_ON */
+  return VERSION_STRING;
+#endif
+}
+
 /**********************************************************************
   Return the BETA message.
   If returns NULL, not a beta version.
@@ -97,11 +113,19 @@ const char *beta_message(void)
     N_("November"),
     N_("December")
   };
-  fc_snprintf (msgbuf, sizeof (msgbuf),
-               /* TRANS: No full stop after the URL, could cause confusion. */
-               _("THIS IS A BETA VERSION\n"
-                 "Freeciv %s will be released in %s, at %s"),
-               NEXT_STABLE_VERSION, _(NEXT_RELEASE_MONTH), WIKI_URL);
+
+  if (RELEASE_MONTH > 0) {
+    fc_snprintf(msgbuf, sizeof(msgbuf),
+                /* TRANS: No full stop after the URL, could cause confusion. */
+                _("THIS IS A BETA VERSION\n"
+                  "Freeciv %s will be released in %s, at %s"),
+                NEXT_STABLE_VERSION, _(NEXT_RELEASE_MONTH), WIKI_URL);
+  } else {
+    fc_snprintf(msgbuf, sizeof(msgbuf),
+                _("THIS IS A BETA VERSION\n"
+                  "Freeciv %s will be released at %s"),
+                NEXT_STABLE_VERSION, WIKI_URL);
+  }
   return msgbuf;
 #else
   return NULL;
