@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 /* utility */
@@ -68,7 +68,7 @@ bool vision_reveal_tiles(struct vision *vision, bool reveal_tiles)
 /****************************************************************************
   Frees vision site structure.
 ****************************************************************************/
-void free_vision_site(struct vision_site *psite)
+void vision_site_destroy(struct vision_site *psite)
 {
   free(psite);
 }
@@ -76,8 +76,8 @@ void free_vision_site(struct vision_site *psite)
 /****************************************************************************
   Returns the basic structure.
 ****************************************************************************/
-struct vision_site *create_vision_site(int identity, struct tile *location,
-				       struct player *owner)
+struct vision_site *vision_site_new(int identity, struct tile *location,
+                                    struct player *owner)
 {
   struct vision_site *psite = fc_calloc(1, sizeof(*psite));
 
@@ -91,12 +91,12 @@ struct vision_site *create_vision_site(int identity, struct tile *location,
 /****************************************************************************
   Returns the basic structure filled with initial elements.
 ****************************************************************************/
-struct vision_site *create_vision_site_from_city(const struct city *pcity)
+struct vision_site *vision_site_new_from_city(const struct city *pcity)
 {
   struct vision_site *psite =
-    create_vision_site(pcity->id, city_tile(pcity), city_owner(pcity));
+    vision_site_new(pcity->id, city_tile(pcity), city_owner(pcity));
 
-  psite->size = pcity->size;
+  vision_site_size_set(psite, city_size_get(pcity));
   sz_strlcpy(psite->name, city_name(pcity));
 
   return psite;
@@ -105,12 +105,36 @@ struct vision_site *create_vision_site_from_city(const struct city *pcity)
 /****************************************************************************
   Returns the basic structure filled with current elements.
 ****************************************************************************/
-void update_vision_site_from_city(struct vision_site *psite,
-				  const struct city *pcity)
+void vision_site_update_from_city(struct vision_site *psite,
+                                  const struct city *pcity)
 {
   /* should be same identity and location */
+  fc_assert_ret(psite->identity = pcity->id);
+  fc_assert_ret(psite->location = pcity->tile);
+
   psite->owner = city_owner(pcity);
 
-  psite->size = pcity->size;
+  vision_site_size_set(psite, city_size_get(pcity));
   sz_strlcpy(psite->name, city_name(pcity));
 }
+
+/*****************************************************************************
+  Get the city size.
+*****************************************************************************/
+citizens vision_site_size_get(const struct vision_site *psite)
+{
+  fc_assert_ret_val(psite != NULL, 0);
+
+  return psite->size;
+}
+
+/*****************************************************************************
+  Set the city size.
+*****************************************************************************/
+void vision_site_size_set(struct vision_site *psite, citizens size)
+{
+  fc_assert_ret(psite != NULL);
+  fc_assert_ret(size <= MAX_CITY_SIZE);
+
+  psite->size = size;
+}
