@@ -14,7 +14,7 @@
 #define FC__LOCALLUACONF_H
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #ifdef HAVE_MKSTEMP
