@@ -41,6 +41,7 @@
 #include "fc_cmdhelp.h"
 #include "fc_types.h" /* LINE_BREAK */
 #include "game.h"
+#include "government.h"
 #include "improvement.h"
 #include "map.h"
 #include "player.h"
@@ -71,6 +72,7 @@ enum manuals {
   MANUAL_TERRAIN,
   MANUAL_BUILDINGS,
   MANUAL_WONDERS,
+  MANUAL_GOVS,
   MANUAL_COUNT
 };
 
@@ -98,6 +100,8 @@ enum manuals {
 /* Needed for "About Freeciv" help */
 const char *client_string = "freeciv-manual";
 
+static char *ruleset = NULL;
+
 /**************************************************************************
   Replace html special characters ('&', '<' and '>').
 **************************************************************************/
@@ -382,11 +386,15 @@ static bool manual_command(void)
                 pimprove->upkeep);
 
         requirement_vector_iterate(&pimprove->reqs, req) {
-          char text[512];
-          fprintf(doc, "%s<br/>",
-                  VUT_NONE != req->source.kind
-                  ? universal_name_translation(&req->source, text, sizeof(text))
-                  : _("None"));
+          char text[512], text2[512];
+          fc_snprintf(text2, sizeof(text2),
+                      /* TRANS: improvement requires a feature to be absent. */
+                      req->negated ? _("no %s") : "%s",
+                      VUT_NONE != req->source.kind
+                      ? universal_name_translation(&req->source,
+                                                   text, sizeof(text))
+                      : _("None"));
+          fprintf(doc, "%s<br/>", text2);
         } requirement_vector_iterate_end;
 
         fprintf(doc, "<em>%s</em></td>\n",
@@ -397,6 +405,18 @@ static bool manual_command(void)
       } improvement_iterate_end;
       break;
 
+    case MANUAL_GOVS:
+      /* FIXME: this doesn't resemble the wiki manual at all. */
+      fprintf(doc, _("<h1>Freeciv %s governments help</h1>\n\n"), VERSION_STRING);
+      governments_iterate(pgov) {
+        char buf[64000];
+        fprintf(doc, "%s%s%s\n\n", SECTION_BEGIN,
+                government_name_translation(pgov), SECTION_END);
+        helptext_government(buf, sizeof(buf), NULL, NULL, pgov);
+        fprintf(doc, "%s\n\n", buf);
+      } governments_iterate_end;
+      break;
+
     case MANUAL_COUNT:
       break;
 
@@ -430,8 +450,12 @@ int main(int argc, char **argv)
   inx = 1;
   while (inx < argc) {
     if ((option = get_option_malloc("--ruleset", argv, &inx, argc))) {
-      sz_strlcpy(game.server.rulesetdir, option);
-      free(option);
+      if (ruleset != NULL) {
+        log_error(_("Multiple rulesets requested. Only one ruleset at time supported"));
+        free(option);
+      } else {
+        ruleset = option;
+      }
     } else if (is_option("--help", argv[inx])) {
       showhelp = TRUE;
       break;
@@ -458,6 +482,9 @@ int main(int argc, char **argv)
         break;
       }
       free(option);
+    } else {
+      fc_fprintf(stderr, _("Unrecognized option: \"%s\"\n"), argv[inx]);
+      exit(EXIT_FAILURE);
     }
     inx++;
   }
@@ -475,6 +502,11 @@ int main(int argc, char **argv)
   /* Initialize game with default values */
   game_init();
 
+  /* Set ruleset user requested in to use */
+  if (ruleset != NULL) {
+    sz_strlcpy(game.server.rulesetdir, ruleset);
+  }
+
   settings_init(FALSE);
 
   if (showvers && !showhelp) {
@@ -524,6 +556,9 @@ int main(int argc, char **argv)
   }
 
   manual_command();
+
+  FC_FREE(ruleset);
+
   con_log_close();
 
   return EXIT_SUCCESS;
