@@ -1,5 +1,5 @@
 /**********************************************************************
- Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
+< Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <stdlib.h>
@@ -31,9 +31,16 @@
 #include "download.h"
 #include "mpcmdline.h"
 
-#define MODPACK_LIST_URL "http://download.gna.org/freeciv/modinst/" DATASUBDIR "/modpack.list"
+#include "modinst.h"
+
+#if IS_DEVEL_VERSION
+#define MODPACK_LIST_URL  "http://www.cazfi.net/freeciv/modinst/" DATASUBDIR "/modpack.list"
+#define DEFAULT_URL_START "http://www.cazfi.net/freeciv/modinst/" DATASUBDIR "/"
+#else  /* IS_DEVEL_VERSION */
+#define MODPACK_LIST_URL  "http://modpack.freeciv.org/" DATASUBDIR "/modpack.list"
+#define DEFAULT_URL_START "http://modpack.freeciv.org/" DATASUBDIR "/"
+#endif /* IS_DEVEL_VERSION */
 
-#define DEFAULT_URL_START "http://download.gna.org/freeciv/modinst/" DATASUBDIR "/"
 #define EXAMPLE_URL DEFAULT_URL_START "ancients.modpack"
 
 static GtkWidget *statusbar;
@@ -43,12 +50,18 @@ static GtkListStore *main_store;
 static GtkWidget *URL_input;
 static gboolean downloading = FALSE;
 
+struct fcmp_params fcmp = {
+  .list_url = MODPACK_LIST_URL,
+  .inst_prefix = NULL
+};
+
 static gboolean quit_dialog_callback(void);
 
 #define ML_COL_NAME 0
 #define ML_COL_VER  1
 #define ML_COL_TYPE 2
-#define ML_COL_URL  3
+#define ML_COL_LIC  3
+#define ML_COL_URL  4
 
 /****************************************************************
   freeciv-modpack quit
@@ -73,29 +86,37 @@ static void quit_dialog_response(GtkWidget *dialog, gint response)
 }
 
 /****************************************************************
-  Popups the quit dialog.
+  Popups the quit dialog if 
 ****************************************************************/
 static gboolean quit_dialog_callback(void)
 {
-  static GtkWidget *dialog;
+  if (downloading) {
+    /* Download in progress. Confirm quit from user. */
+    static GtkWidget *dialog;
 
-  if (!dialog) {
-    dialog = gtk_message_dialog_new(NULL,
-	0,
-	GTK_MESSAGE_WARNING,
-	GTK_BUTTONS_YES_NO,
-	_("Are you sure you want to quit?"));
+    if (!dialog) {
+      dialog = gtk_message_dialog_new(NULL,
+                                      0,
+                                      GTK_MESSAGE_WARNING,
+                                      GTK_BUTTONS_YES_NO,
+                                      _("Are you sure you want to quit?"));
 
-    gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE);
+      gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE);
 
-    g_signal_connect(dialog, "response", 
-	G_CALLBACK(quit_dialog_response), NULL);
-    g_signal_connect(dialog, "destroy",
-	G_CALLBACK(gtk_widget_destroyed), &dialog);
+      g_signal_connect(dialog, "response", 
+                       G_CALLBACK(quit_dialog_response), NULL);
+      g_signal_connect(dialog, "destroy",
+                       G_CALLBACK(gtk_widget_destroyed), &dialog);
+    }
+
+    gtk_window_present(GTK_WINDOW(dialog));
+
+  } else {
+    /* User loses no work by quitting, so let's not annoy him/her
+     * with confirmation dialog. */
+    modinst_quit();
   }
 
-  gtk_window_present(GTK_WINDOW(dialog));
-
   /* Stop emission of event. */
   return TRUE;
 }
@@ -123,7 +144,7 @@ static gpointer download_thread(gpointer data)
 {
   const char *errmsg;
 
-  errmsg = download_modpack(data, msg_callback, pbar_callback);
+  errmsg = download_modpack(data, &fcmp, msg_callback, pbar_callback);
 
   if (errmsg == NULL) {
     gtk_label_set_text(GTK_LABEL(statusbar), _("Ready"));
@@ -192,10 +213,12 @@ static void URL_return(GtkEntry *w, gpointer data)
   Build main modpack list view
 **************************************************************************/
 static void setup_modpack_list(const char *name, const char *URL,
-                               const char *version, enum modpack_type type)
+                               const char *version, const char *license,
+                               enum modpack_type type)
 {
   GtkTreeIter iter;
   const char *type_str;
+  const char *lic_str;
 
   if (modpack_type_is_valid(type)) {
     type_str = _(modpack_type_name(type));
@@ -204,11 +227,19 @@ static void setup_modpack_list(const char *name, const char *URL,
     type_str = _("?");
   }
 
+  if (license != NULL) {
+    lic_str = license;
+  } else {
+    /* TRANS: License of modpack is not known */
+    lic_str = Q_("?license:Unknown");
+  }
+
   gtk_list_store_append(main_store, &iter);
   gtk_list_store_set(main_store, &iter,
                      ML_COL_NAME, name,
                      ML_COL_VER, version,
                      ML_COL_TYPE, type_str,
+                     ML_COL_LIC, lic_str,
                      ML_COL_URL, URL,
                      -1);
 }
@@ -259,12 +290,19 @@ static void modinst_setup_widgets(GtkWidget *toplevel)
   renderer = gtk_cell_renderer_text_new();
   gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(main_list),
                                               ML_COL_TYPE,
-                                              _("Type"), renderer, "text", 2,
+                                              Q_("?modpack:Type"),
+                                              renderer, "text", 2,
+                                              NULL);
+  renderer = gtk_cell_renderer_text_new();
+  gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(main_list),
+                                              ML_COL_LIC,
+                                              /* TRANS: noun */
+                                              _("License"), renderer, "text", 3,
                                               NULL);
   renderer = gtk_cell_renderer_text_new();
   gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(main_list),
                                               ML_COL_URL,
-                                              _("URL"), renderer, "text", 3,
+                                              _("URL"), renderer, "text", 4,
                                               NULL);
   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(main_list));
   g_signal_connect(selection, "changed", G_CALLBACK(select_from_list), NULL);
@@ -303,10 +341,9 @@ static void modinst_setup_widgets(GtkWidget *toplevel)
 
   gtk_container_add(GTK_CONTAINER(toplevel), mbox);
 
-  main_store = gtk_list_store_new(4, G_TYPE_STRING, G_TYPE_STRING,
-                                  G_TYPE_STRING, G_TYPE_STRING);
-  errmsg = download_modpack_list(MODPACK_LIST_URL, setup_modpack_list,
-                                 msg_callback);
+  main_store = gtk_list_store_new(5, G_TYPE_STRING, G_TYPE_STRING,
+                                  G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+  errmsg = download_modpack_list(&fcmp, setup_modpack_list, msg_callback);
   gtk_tree_view_set_model(GTK_TREE_VIEW(main_list), GTK_TREE_MODEL(main_store));
 
   g_object_unref(main_store);
@@ -329,7 +366,7 @@ int main(int argc, char *argv[])
   init_character_encodings(FC_DEFAULT_DATA_ENCODING, FALSE);
 
   fc_init_network();
-  
+
   g_thread_init(NULL);
 
   log_init(NULL, loglevel, NULL, NULL, -1);
@@ -337,6 +374,19 @@ int main(int argc, char *argv[])
   /* This modifies argv! */
   ui_options = fcmp_parse_cmdline(argc, argv);
 
+  if (fcmp.inst_prefix == NULL) {
+    const char *home = user_home_dir();
+
+    if (home == NULL) {
+      log_error("Cannot determine user home directory");
+    } else {
+      static char pfx_buf[500];
+
+      snprintf(pfx_buf, sizeof(pfx_buf), "%s/.freeciv", home);
+      fcmp.inst_prefix = pfx_buf;
+    }
+  }
+
   if (ui_options != -1) {
 
     /* Process GTK arguments */
@@ -369,5 +419,7 @@ int main(int argc, char *argv[])
     gtk_main();
   }
 
+  log_close();
+
   return EXIT_SUCCESS;
 }
