@@ -21,7 +21,7 @@ then
 else
   PROGRAM_NAME="setup_auth_server.sh"
 fi
-PROGRAM_VERSION="0.6"
+PROGRAM_VERSION="0.9"
 
 #############################################################################
 #
@@ -163,18 +163,20 @@ fi
 echo "This script helps in setup of Freeciv server with player authentication."
 echo "Most users do not need player authentication."
 echo
-echo "Authentication uses MySQL database to store player information."
-echo "Before running this script you should have MySQL server running"
-echo "and that server should have"
-echo " - user account to be used by civserver"
-echo " - empty database this script will populate"
-echo "Password for that account will be stored to config file,"
-echo "so you want to create special user account just for civserver use."
-echo "This script needs MySQL user account that can create tables in to"
-echo "that database. Generated config file for Freeciv server will"
-echo "contain MySQL username used by this script, but it's easy to change"
+echo "As set up by this script, authentication uses a MySQL database to store"
+echo "player information."
+echo "Before running this script you should have a MySQL server running"
+echo "and that server should have:"
+echo " - a user account to be used by freeciv-server"
+echo " - an empty database, which this script will populate"
+echo "The password for that account will be stored in a Freeciv server"
+echo "config file, so you want to create special database user account just"
+echo "for freeciv-server use."
+echo "This script needs a MySQL user account that can create tables in to"
+echo "that database. The generated config file for the Freeciv server will"
+echo "contain the MySQL username used by this script, but it's easy to change"
 echo "afterwards if you don't want use same account from this script and"
-echo "later from Freeciv server."
+echo "later from the Freeciv server."
 
 if ask_yes_no "\nDo you want to continue with this script now?"
 then
@@ -183,7 +185,7 @@ then
 fi
 
 echo
-echo "First we populate player database at MySQL server"
+echo "First we populate the player database on the MySQL server"
 
 if ! which mysql >/dev/null
 then
@@ -251,7 +253,7 @@ do
   fi
 done
 
-echo "Next we select that database"
+echo "Next we select the database"
 
 MYSQL_DATABASE="Freeciv"
 DATABASE_SELECTED=no
@@ -283,7 +285,7 @@ do
   # Start lines with " -"
   echo "$DBLIST" | sed 's/^/ -/'
   echo
-  echo "Please tell which one to use."
+  echo "Please select which one to use."
   echo -n "($MYSQL_DATABASE_TMP)> "
   read MYSQL_DATABASE_NEW
 
@@ -315,8 +317,8 @@ do
 done
 
 # These are hardcoded here, and not prompted
-TABLE_AUTH="auth"
-TABLE_LOGINLOG="loginlog"
+TABLE_USER="auth"
+TABLE_LOG="loginlog"
 
 TABLELIST="$(make_query 'show tables' | grep -v '^Tables_in_')"
 
@@ -324,37 +326,37 @@ if test "x$TABLELIST" != "x"
 then
   echo "This database already contains some tables."
 
-  echo "$TABLELIST" | grep "$TABLE_AUTH" > /dev/null
+  echo "$TABLELIST" | grep "$TABLE_USER" > /dev/null
   GREPRESULT=$?
 
   if test $GREPRESULT -eq 0
   then
-    AUTH_TABLE_PRESENT=yes
+    USER_TABLE_PRESENT=yes
   else
-    AUTH_TABLE_PRESENT=no
+    USER_TABLE_PRESENT=no
   fi
 
-  echo "$TABLELIST" | grep "$TABLE_LOGINLOG" > /dev/null
+  echo "$TABLELIST" | grep "$TABLE_LOG" > /dev/null
   GREPRESULT=$?
 
   if test $GREPRESULT -eq 0
   then
-    LOGINLOG_TABLE_PRESENT=yes
+    LOG_TABLE_PRESENT=yes
   else
-    LOGINLOG_TABLE_PRESENT=no
+    LOG_TABLE_PRESENT=no
   fi
 
-  if test $LOGINLOG_TABLE_PRESENT = yes ||
-     test $AUTH_TABLE_PRESENT = yes
+  if test $LOG_TABLE_PRESENT = yes ||
+     test $USER_TABLE_PRESENT = yes
   then
-    echo "There is even tables with names Freeciv would use."
-    if test $AUTH_TABLE_PRESENT = yes
+    echo "There are even tables with the names Freeciv would use."
+    if test $USER_TABLE_PRESENT = yes
     then
-      echo " -$TABLE_AUTH"
+      echo " -$TABLE_USER"
     fi
-    if test $LOGINLOG_TABLE_PRESENT = yes
+    if test $LOG_TABLE_PRESENT = yes
     then
-      echo " -$TABLE_LOGINLOG"
+      echo " -$TABLE_LOG"
     fi
 
     echo "Maybe you have already attempted to create Freeciv tables?"
@@ -371,20 +373,20 @@ then
     fi
 
     # Drop tables
-    if test $AUTH_TABLE_PRESENT = yes
+    if test $USER_TABLE_PRESENT = yes
     then
-      if ! make_query "drop table $TABLE_AUTH"
+      if ! make_query "drop table $TABLE_USER"
       then
-        echo "Dropping table $TABLE_AUTH failed!"
+        echo "Dropping table $TABLE_USER failed!"
         echo "Aborting!"
         exit 1
       fi
     fi
-    if test $LOGINLOG_TABLE_PRESENT = yes
+    if test $LOG_TABLE_PRESENT = yes
     then
-      if ! make_query "drop table $TABLE_LOGINLOG"
+      if ! make_query "drop table $TABLE_LOG"
       then
-        echo "Dropping table $TABLE_AUTH failed!"
+        echo "Dropping table $TABLE_LOG failed!"
         echo "Aborting!"
         exit 1
       fi
@@ -417,10 +419,11 @@ echo "Now we create the Freeciv tables."
 
 # We have embedded table creation SQL to this script.
 # Maybe we should read it from separate file in the future.
+# The tables here are as the supplied data/database.lua expects to find them.
 (echo \
- "CREATE TABLE $TABLE_AUTH ( \
+ "CREATE TABLE $TABLE_USER ( \
    id int(11) NOT NULL auto_increment, \
-   name varchar(32) default NULL, \
+   name varchar(48) default NULL, \
    password varchar(32) default NULL, \
    email varchar(128) default NULL, \
    createtime int(11) default NULL, \
@@ -432,9 +435,9 @@ echo "Now we create the Freeciv tables."
    UNIQUE KEY name (name) \
  ) TYPE=MyISAM;"
  echo \
- "CREATE TABLE $TABLE_LOGINLOG ( \
+ "CREATE TABLE $TABLE_LOG ( \
    id int(11) NOT NULL auto_increment, \
-   name varchar(32) default NULL, \
+   name varchar(48) default NULL, \
    logintime int(11) default NULL, \
    address varchar(255) default NULL, \
    succeed enum('S','F') default 'S', \
@@ -472,12 +475,12 @@ do
   TRY_FILE=yes
   if test -e "$CONFIG_FILE"
   then
-    echo "$CONFIG_FILE already exist"
+    echo "$CONFIG_FILE already exists"
     # Default is not to test overwriting
     TRY_FILE=no
     if test -d "$CONFIG_FILE"
     then
-      echo "and it's directory. Can't overwrite with file."
+      echo "and it's a directory. Can't overwrite with file."
     else
       if ! ask_yes_no "\nOK to overwrite?"
       then
@@ -493,7 +496,7 @@ do
       ACCEPTABLE_FILE=yes
     else
       echo "Can't write to that file"
-      if ask_yes_no "\nRetry with other file?"
+      if ask_yes_no "\nRetry with another file?"
       then
         echo "Aborting!"
         exit 1
@@ -527,8 +530,10 @@ fi
  echo "; Configuration file for Freeciv server player authentication"
  echo "; Generated by \"$PROGRAM_NAME\" version $PROGRAM_VERSION"
  echo
- echo "[auth]"
- echo "; How to connect MySQL server"
+ echo "[fcdb]"
+ echo "; Use MySQL server"
+ echo "backend=\"mysql\""
+ echo "; How to connect to MySQL server"
  echo "host=\"$MYSQL_SERVER\""
  echo "port=\"$MYSQL_PORT\" ; This is handled as string!"
  echo "user=\"$MYSQL_USER\""
@@ -543,9 +548,11 @@ fi
  echo "database=\"$MYSQL_DATABASE\""
  echo
  echo "; Table names"
- echo "table=\"$TABLE_AUTH\""
- echo "login_table=\"$TABLE_LOGINLOG\""
+ echo "table_user=\"$TABLE_USER\""
+ echo "table_log=\"$TABLE_LOG\""
 ) > $CONFIG_FILE
 
 echo "Config file generated."
 echo "Auth server setup finished."
+echo "To use the newly created database, run"
+echo "  freeciv-server --Database $CONFIG_FILE --auth"
