@@ -17,4 +17,7 @@ struct player;
 
 void building_advisor(struct player *pplayer);
 
+void advisor_choose_build(struct player *pplayer, struct city *pcity);
+void building_advisor_choose(struct city *pcity, struct adv_choice *choice);
+
 #endif   /* FC__BUILDINGADV_H */
