@@ -18,7 +18,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <stdio.h>
@@ -44,6 +44,7 @@
 #include "unitlist.h"
 
 /* server */
+#include "aiiface.h"
 #include "citytools.h"
 #include "gamehand.h"
 #include "maphand.h"
@@ -117,11 +118,10 @@ static struct player *create_barbarian_player(enum barbarian_type type)
   } players_iterate_end;
 
   /* make a new player, or not */
-  barbarians = server_create_player(-1);
+  barbarians = server_create_player(-1, default_ai_type_name(), NULL);
   if (!barbarians) {
     return NULL;
   }
-
   server_player_init(barbarians, TRUE, TRUE);
 
   nation = pick_a_nation(NULL, FALSE, TRUE, type);
@@ -129,11 +129,10 @@ static struct player *create_barbarian_player(enum barbarian_type type)
   sz_strlcpy(barbarians->name, pick_random_player_name(nation));
 
   server.nbarbarians++;
-  game.server.max_players = MAX(player_count(), game.server.max_players);
 
   sz_strlcpy(barbarians->username, ANON_USER_NAME);
   barbarians->is_connected = FALSE;
-  barbarians->government = nation->server.init_government;
+  barbarians->government = nation->init_government;
   fc_assert(barbarians->revolution_finishes < 0);
   barbarians->server.capital = FALSE;
   barbarians->economic.gold = 100;
@@ -156,6 +155,8 @@ static struct player *create_barbarian_player(enum barbarian_type type)
     }
   } players_iterate_end;
 
+  CALL_PLR_AI_FUNC(gained_control, barbarians, barbarians);
+
   log_verbose("Created barbarian %s, player %d", player_name(barbarians),
               player_number(barbarians));
   notify_player(NULL, NULL, E_UPRISING, ftc_server,
@@ -258,7 +259,7 @@ bool unleash_barbarians(struct tile *ptile)
       || game.info.turn < game.server.onsetbarbarian
       || num_role_units(L_BARBARIAN) == 0) {
     unit_list_iterate_safe((ptile)->units, punit) {
-      wipe_unit(punit, TRUE, NULL);
+      wipe_unit(punit, ULR_BARB_UNLEASH, NULL);
     } unit_list_iterate_safe_end;
     return FALSE;
   }
@@ -268,7 +269,8 @@ bool unleash_barbarians(struct tile *ptile)
     return FALSE;
   }
 
-  ai_data_phase_init(barbarians, TRUE);
+  adv_data_phase_init(barbarians, TRUE);
+  CALL_PLR_AI_FUNC(phase_begin, barbarians, barbarians, TRUE);
 
   unit_cnt = 3 + fc_rand(4);
   for (i = 0; i < unit_cnt; i++) {
@@ -405,7 +407,7 @@ bool unleash_barbarians(struct tile *ptile)
     /* There's barbarian in this village! Kill the explorer. */
     unit_list_iterate_safe((ptile)->units, punit2) {
       if (unit_owner(punit2) != barbarians) {
-        wipe_unit(punit2, TRUE, NULL);
+        wipe_unit(punit2, ULR_BARB_UNLEASH, NULL);
         alive = FALSE;
       } else {
         send_unit_info(NULL, punit2);
@@ -489,7 +491,7 @@ static void try_summon_barbarians(void)
   }
 
   if (!(pc = find_closest_city(ptile, NULL, NULL, FALSE, FALSE, FALSE, FALSE,
-                               FALSE))) {
+                               FALSE, NULL))) {
     /* any city */
     return;
   }
@@ -568,7 +570,6 @@ static void try_summon_barbarians(void)
     if (!barbarians) {
       return;
     }
-
     /* Setup data phase if it's not already set up. Created ferries may
        need that data.
        We don't know if create_barbarian_player() above created completely
@@ -576,9 +577,12 @@ static void try_summon_barbarians(void)
        one, phase has already been set up at turn begin and will be closed
        at turn end. If this is completely new player, we have to take care
        of both opening and closing the data phase. Return value of
-       ai_data_phase_init() tells us if data phase was already initialized
+       adv_data_phase_init() tells us if data phase was already initialized
        at turn beginning. */
-    miniphase = ai_data_phase_init(barbarians, TRUE);
+    miniphase = adv_data_phase_init(barbarians, TRUE);
+    if (miniphase) {
+      CALL_PLR_AI_FUNC(phase_begin, barbarians, barbarians, TRUE);
+    }
 
     boat = find_a_unit_type(L_BARBARIAN_BOAT,-1);
 
@@ -611,7 +615,8 @@ static void try_summon_barbarians(void)
     }
 
     if (miniphase) {
-      ai_data_phase_done(barbarians);
+      CALL_PLR_AI_FUNC(phase_finished, barbarians, barbarians);
+      adv_data_phase_done(barbarians);
     }
   }
 
