@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <stdarg.h>
@@ -24,6 +24,7 @@
 #endif
 
 /* utility */
+#include "fcbacktrace.h"
 #include "fciconv.h"
 #include "fcintl.h"
 #include "log.h"
@@ -95,10 +96,10 @@ static void con_update_prompt(void)
   } else {
     rl_forced_update_display();
   }
-#else
+#else  /* HAVE_LIBREADLINE */
   con_dump(C_READY,"> ");
   con_flush();
-#endif
+#endif /* HAVE_LIBREADLINE */
 
   console_prompt_is_showing = TRUE;
 }
@@ -111,6 +112,8 @@ static void con_update_prompt(void)
 static const char *log_prefix(void)
 {
   static char buf[128];
+
+#ifdef LOG_TIMERS
   char timestr[32];
   time_t timestamp;
 
@@ -120,6 +123,10 @@ static const char *log_prefix(void)
 
   fc_snprintf(buf, sizeof(buf), "T%03d - %s", game.info.turn, timestr);
 
+#else  /* LOG_TIMERS */
+  fc_snprintf(buf, sizeof(buf), "T%03d", game.info.turn);
+#endif /* LOG_TIMERS */
+
   return buf;
 }
 #endif /* DEBUG */
@@ -137,6 +144,17 @@ void con_log_init(const char *log_filename, enum log_level level,
   log_init(log_filename, level, con_handle_log, NULL,
            fatal_assertions);
 #endif /* DEBUG */
+  backtrace_init();
+}
+
+/************************************************************************
+  Deinitialize logging
+************************************************************************/
+void con_log_close(void)
+{
+  backtrace_deinit();
+
+  log_close();
 }
 
 #ifndef HAVE_LIBREADLINE
@@ -163,7 +181,7 @@ static int con_dump(enum rfc_status rfc_status, const char *message, ...)
   console_prompt_is_showing = FALSE;
   return (int) strlen(buf);
 }
-#endif
+#endif /* HAVE_LIBREADLINE */
 
 /************************************************************************
 Write to console and add line-break, and show prompt if required.
