@@ -12,7 +12,7 @@
 ***********************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <stdio.h>
@@ -515,8 +515,10 @@ static void place_terrain(struct tile *ptile, int diff,
   (*to_be_placed)--;
 
   cardinal_adjc_iterate(ptile, tile1) {
-    int Delta = (abs(map_colatitude(tile1) - map_colatitude(ptile)) / L_UNIT
-                 + abs(hmap(tile1) - (hmap(ptile))) /  H_UNIT);
+    /* Check L_UNIT and H_UNIT against 0. */
+    int Delta = (abs(map_colatitude(tile1) - map_colatitude(ptile))
+                 / MAX(L_UNIT, 1)
+                 + abs(hmap(tile1) - (hmap(ptile))) / MAX(H_UNIT, 1));
     if (not_placed(tile1)
         && tmap_is(tile1, tc)
         && test_wetness(tile1, wc)
@@ -910,9 +912,9 @@ static bool make_river(struct river_map *privermap, struct tile *ptile)
 
     /* Mark all available cardinal directions as available. */
     memset(rd_direction_is_valid, 0, sizeof(rd_direction_is_valid));
-    cardinal_adjc_dir_iterate(ptile, tile1, dir) {
+    cardinal_adjc_dir_base_iterate(ptile, dir) {
       rd_direction_is_valid[dir] = TRUE;
-    } cardinal_adjc_dir_iterate_end;
+    } cardinal_adjc_dir_base_iterate_end;
 
     /* Test series that selects a direction for the river. */
     for (func_num = 0; func_num < NUM_TEST_FUNCTIONS; func_num++) {
@@ -939,23 +941,23 @@ static bool make_river(struct river_map *privermap, struct tile *ptile)
       }
 
       /* mark the less attractive directions as invalid */
-      cardinal_adjc_dir_iterate(ptile, tile1, dir) {
+      cardinal_adjc_dir_base_iterate(ptile, dir) {
 	if (rd_direction_is_valid[dir]) {
 	  if (rd_comparison_val[dir] != best_val) {
 	    rd_direction_is_valid[dir] = FALSE;
 	  }
 	}
-      } cardinal_adjc_dir_iterate_end;
+      } cardinal_adjc_dir_base_iterate_end;
     }
 
     /* Directions evaluated with all functions. Now choose the best
        direction before going to the next iteration of the while loop */
     num_valid_directions = 0;
-    cardinal_adjc_dir_iterate(ptile, tile1, dir) {
+    cardinal_adjc_dir_base_iterate(ptile, dir) {
       if (rd_direction_is_valid[dir]) {
 	num_valid_directions++;
       }
-    } cardinal_adjc_dir_iterate_end;
+    } cardinal_adjc_dir_base_iterate_end;
 
     if (num_valid_directions == 0) {
       return FALSE; /* river aborted */
@@ -1160,7 +1162,7 @@ static void make_land(void)
         }
       } adjc_iterate_end;
 
-      depth += 30 * (ocean - land) / (ocean + land);
+      depth += 30 * (ocean - land) / MAX(1, (ocean + land));
 
       depth = MIN(depth, TERRAIN_OCEAN_DEPTH_MAXIMUM);
 
@@ -1206,11 +1208,13 @@ static bool is_tiny_island(struct tile *ptile)
     return FALSE;
   }
 
-  cardinal_adjc_iterate(ptile, tile1) {
+  adjc_iterate(ptile, tile1) {
+    /* This was originally a cardinal_adjc_iterate, which seemed to cause
+     * two or three problems. /MSS */
     if (!is_ocean_tile(tile1)) {
       return FALSE;
     }
-  } cardinal_adjc_iterate_end;
+  } adjc_iterate_end;
 
   return TRUE;
 }
@@ -1297,7 +1301,7 @@ FIXME: Some continent numbers are unused at the end of this function, fx
   based on the map.server.size server parameter and the specified topology.
   If not map.xsize and map.ysize will be used.
 **************************************************************************/
-void map_fractal_generate(bool autosize, struct unit_type *initial_unit)
+bool map_fractal_generate(bool autosize, struct unit_type *initial_unit)
 {
   /* save the current random state: */
   RANDOM_STATE rstate = fc_rand_state();
@@ -1315,7 +1319,10 @@ void map_fractal_generate(bool autosize, struct unit_type *initial_unit)
   /* also, don't delete (the handcrafted!) tiny islands in a scenario */
   if (map.server.generator != MAPGEN_SCENARIO) {
     generator_init_topology(autosize);
-    map_allocate();
+    /* Map can be already allocated, if we failed first map generation */
+    if (map_is_empty()) {
+      map_allocate();
+    }
     adjust_terrain_param();
     /* if one mapgenerator fails, it will choose another mapgenerator */
     /* with a lower number to try again */
@@ -1402,8 +1409,7 @@ void map_fractal_generate(bool autosize, struct unit_type *initial_unit)
    * provides them. */
   if (0 == map_startpos_count()) {
     enum map_startpos mode = MAPSTARTPOS_ALL;
-    bool success;
-    
+
     switch (map.server.generator) {
     case MAPGEN_SCENARIO:
     case MAPGEN_RANDOM:
@@ -1435,13 +1441,15 @@ void map_fractal_generate(bool autosize, struct unit_type *initial_unit)
       }
       break;
     }
-    
+
     for(;;) {
+      bool success;
+
       success = create_start_positions(mode, initial_unit);
       if (success) {
         break;
       }
-      
+
       switch(mode) {
         case MAPSTARTPOS_SINGLE:
           log_verbose("Falling back to startpos=2or3");
@@ -1456,8 +1464,9 @@ void map_fractal_generate(bool autosize, struct unit_type *initial_unit)
           mode = MAPSTARTPOS_VARIABLE;
           break;
         default:
-          fc_assert_exit_msg(FALSE, "The server couldn't allocate "
-                             "starting positions.");
+          log_error(_("The server couldn't allocate starting positions."));
+          destroy_tmap();
+          return FALSE;
       }
     }
   }
@@ -1466,6 +1475,8 @@ void map_fractal_generate(bool autosize, struct unit_type *initial_unit)
   destroy_tmap();
 
   print_mapgen_map();
+
+  return TRUE;
 }
 
 /**************************************************************************
@@ -1581,7 +1592,7 @@ static void add_resources(int prob)
       }
     }
   } whole_map_iterate_end;
-  
+
   map.server.have_resources = TRUE;
 }
 
@@ -1606,7 +1617,7 @@ static struct tile *get_random_map_position_from_state(
 }
 
 /**************************************************************************
-  ...
+  Allocate and initialize new terrain_select structure.
 **************************************************************************/
 static struct terrain_select *tersel_new(int weight,
                                          enum mapgen_terrain_property target,
@@ -1628,7 +1639,7 @@ static struct terrain_select *tersel_new(int weight,
 }
 
 /**************************************************************************
-  ...
+  Free resources allocated for terrain_select structure.
 **************************************************************************/
 static void tersel_free(struct terrain_select *ptersel)
 {
@@ -1823,18 +1834,19 @@ static long int checkmass;
 **************************************************************************/
 static bool place_island(struct gen234_state *pstate)
 {
-  int i=0, xn, yn;
+  int i=0, xn, yn, nat_x, nat_y;
   struct tile *ptile;
 
   ptile = rand_map_pos();
+  index_to_native_pos(&nat_x, &nat_y, tile_index(ptile));
 
   /* this helps a lot for maps with high landmass */
   for (yn = pstate->n, xn = pstate->w;
        yn < pstate->s && xn < pstate->e;
        yn++, xn++) {
     struct tile *tile0 = native_pos_to_tile(xn, yn);
-    struct tile *tile1 = native_pos_to_tile(xn + ptile->nat_x - pstate->w,
-					    yn + ptile->nat_y - pstate->n);
+    struct tile *tile1 = native_pos_to_tile(xn + nat_x - pstate->w,
+                                            yn + nat_y - pstate->n);
 
     if (!tile0 || !tile1) {
       return FALSE;
@@ -1847,8 +1859,8 @@ static bool place_island(struct gen234_state *pstate)
   for (yn = pstate->n; yn < pstate->s; yn++) {
     for (xn = pstate->w; xn < pstate->e; xn++) {
       struct tile *tile0 = native_pos_to_tile(xn, yn);
-      struct tile *tile1 = native_pos_to_tile(xn + ptile->nat_x - pstate->w,
-					      yn + ptile->nat_y - pstate->n);
+      struct tile *tile1 = native_pos_to_tile(xn + nat_x - pstate->w,
+                                              yn + nat_y - pstate->n);
 
       if (!tile0 || !tile1) {
 	return FALSE;
@@ -1862,9 +1874,8 @@ static bool place_island(struct gen234_state *pstate)
   for (yn = pstate->n; yn < pstate->s; yn++) {
     for (xn = pstate->w; xn < pstate->e; xn++) {
       if (hmap(native_pos_to_tile(xn, yn)) != 0) {
-	struct tile *tile1
-	  = native_pos_to_tile(xn + ptile->nat_x - pstate->w,
-			       yn + ptile->nat_y - pstate->n);
+        struct tile *tile1 = native_pos_to_tile(xn + nat_x - pstate->w,
+                                                yn + nat_y - pstate->n);
 
 	checkmass--; 
 	if (checkmass <= 0) {
@@ -1880,10 +1891,12 @@ static bool place_island(struct gen234_state *pstate)
       }
     }
   }
-  pstate->s += ptile->nat_y - pstate->n;
-  pstate->e += ptile->nat_x - pstate->w;
-  pstate->n = ptile->nat_y;
-  pstate->w = ptile->nat_x;
+
+  pstate->s += nat_y - pstate->n;
+  pstate->e += nat_x - pstate->w;
+  pstate->n = nat_y;
+  pstate->w = nat_x;
+
   return i != 0;
 }
 
@@ -1908,36 +1921,39 @@ static int count_card_adjc_elevated_tiles(struct tile *ptile)
 **************************************************************************/
 static bool create_island(int islemass, struct gen234_state *pstate)
 {
-  int i;
+  int i, nat_x, nat_y;
   long int tries=islemass*(2+islemass/20)+99;
   bool j;
   struct tile *ptile = native_pos_to_tile(map.xsize / 2, map.ysize / 2);
 
   memset(height_map, '\0', MAP_INDEX_SIZE * sizeof(*height_map));
   hmap(native_pos_to_tile(map.xsize / 2, map.ysize / 2)) = 1;
-  pstate->n = ptile->nat_y - 1;
-  pstate->w = ptile->nat_x - 1;
-  pstate->s = ptile->nat_y + 2;
-  pstate->e = ptile->nat_x + 2;
+
+  index_to_native_pos(&nat_x, &nat_y, tile_index(ptile));
+  pstate->n = nat_y - 1;
+  pstate->w = nat_x - 1;
+  pstate->s = nat_y + 2;
+  pstate->e = nat_x + 2;
   i = islemass - 1;
   while (i > 0 && tries-->0) {
     ptile = get_random_map_position_from_state(pstate);
+    index_to_native_pos(&nat_x, &nat_y, tile_index(ptile));
 
     if ((!near_singularity(ptile) || fc_rand(50) < 25 ) 
-	&& hmap(ptile) == 0 && count_card_adjc_elevated_tiles(ptile) > 0) {
+        && hmap(ptile) == 0 && count_card_adjc_elevated_tiles(ptile) > 0) {
       hmap(ptile) = 1;
       i--;
-      if (ptile->nat_y >= pstate->s - 1 && pstate->s < map.ysize - 2) {
-	pstate->s++;
+      if (nat_y >= pstate->s - 1 && pstate->s < map.ysize - 2) {
+        pstate->s++;
       }
-      if (ptile->nat_x >= pstate->e - 1 && pstate->e < map.xsize - 2) {
-	pstate->e++;
+      if (nat_x >= pstate->e - 1 && pstate->e < map.xsize - 2) {
+        pstate->e++;
       }
-      if (ptile->nat_y <= pstate->n && pstate->n > 2) {
-	pstate->n--;
+      if (nat_y <= pstate->n && pstate->n > 2) {
+        pstate->n--;
       }
-      if (ptile->nat_x <= pstate->w && pstate->w > 2) {
-	pstate->w--;
+      if (nat_x <= pstate->w && pstate->w > 2) {
+        pstate->w--;
       }
     }
     if (i < islemass / 10) {
@@ -2403,7 +2419,7 @@ static void mapgenerator3(void)
 }
 
 /**************************************************************************
-...
+  Generator for placing a couple of players to each island.
 **************************************************************************/
 static void mapgenerator4(void)
 {
