@@ -11,7 +11,7 @@
    GNU General Public License for more details.
 ***********************************************************************/
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <math.h> /* sqrt, HUGE_VAL */
@@ -27,6 +27,7 @@
 /* server */
 #include "maphand.h"
 
+/* server/generator */
 #include "mapgen_topology.h"
 #include "startpos.h"
 #include "temperature_map.h"
@@ -217,6 +218,13 @@ bool create_start_positions(enum map_startpos mode,
   bool failure = FALSE;
   bool is_tmap = temperature_is_initialized();
 
+  if (map.num_continents < 1) {
+    /* Currently we can only place starters on land terrain, so fail
+     * immediately if there isn't any on the map. */
+    log_verbose("Map has no land, so cannot assign start positions!");
+    return FALSE;
+  }
+
   if (!is_tmap) {
     /* The temperature map has already been destroyed by the time start
      * positions have been placed.  We check for this and then create a
@@ -264,11 +272,13 @@ bool create_start_positions(enum map_startpos mode,
 
   initialize_isle_data();
 
-  /* oceans are not good for starters; discard them */
+  /* Only consider tiles marked as 'starter terrains' by ruleset */
   whole_map_iterate(ptile) {
     if (!filter_starters(ptile, NULL)) {
       tile_value[tile_index(ptile)] = 0;
     } else {
+      /* Oceanic terrain cannot be starter terrain currently */
+      fc_assert_action(tile_continent(ptile) > 0, continue);
       islands[tile_continent(ptile)].goodies += tile_value[tile_index(ptile)];
       total_goodies += tile_value[tile_index(ptile)];
     }
@@ -361,7 +371,7 @@ bool create_start_positions(enum map_startpos mode,
 
       if (MAPSTARTPOS_VARIABLE == mode && 0 < to_place) {
         islands[nr].starters = MAX(1, islands[nr].goodies
-                                   / min_goodies_per_player);
+                                   / MAX(1, min_goodies_per_player));
         to_place -= islands[nr].total = islands[nr].starters;
       }
     }
@@ -390,17 +400,14 @@ bool create_start_positions(enum map_startpos mode,
     } else {
       data.min_value *= 0.95;
       if (data.min_value <= 10) {
-        log_error(_("The server appears to have gotten into an infinite "
-                    "loop in the allocation of starting positions.\n Maybe "
-                    "the number of players is too high for this map."));
-        /* TRANS: No full stop after the URL, could cause confusion. */
-        log_error(_("Please report this message at %s"), BUG_URL);
+        log_normal(_("The server appears to have gotten into an infinite "
+                     "loop in the allocation of starting positions.\nMaybe "
+                     "the number of players is too high for this map."));
         failure = TRUE;
         break;
       }
     }
   }
-  fc_assert(player_count() == map_startpos_count());
 
   free(islands);
   free(islands_index);
