@@ -11,7 +11,7 @@
    GNU General Public License for more details.
 ***********************************************************************/
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 /* utility */
@@ -171,6 +171,9 @@ void adjust_int_map_filtered(int *int_map, int int_map_max, void *data,
   }
 }
 
+/****************************************************************************
+  Is given native position normal position
+****************************************************************************/
 bool is_normal_nat_pos(int x, int y)
 {
   NATIVE_TO_MAP_POS(&x, &y, x, y);
