@@ -260,9 +260,12 @@ bool really_gives_vision(struct player *me, struct player *them)
 static void buffer_shared_vision(struct player *pplayer)
 {
   players_iterate(pplayer2) {
-    if (really_gives_vision(pplayer, pplayer2))
+    if (really_gives_vision(pplayer, pplayer2)) {
+      conn_list_compression_freeze(pplayer2->connections);
       conn_list_do_buffer(pplayer2->connections);
+    }
   } players_iterate_end;
+  conn_list_compression_freeze(pplayer->connections);
   conn_list_do_buffer(pplayer->connections);
 }
 
@@ -272,10 +275,13 @@ static void buffer_shared_vision(struct player *pplayer)
 static void unbuffer_shared_vision(struct player *pplayer)
 {
   players_iterate(pplayer2) {
-    if (really_gives_vision(pplayer, pplayer2))
+    if (really_gives_vision(pplayer, pplayer2)) {
       conn_list_do_unbuffer(pplayer2->connections);
+      conn_list_compression_thaw(pplayer2->connections);
+    }
   } players_iterate_end;
   conn_list_do_unbuffer(pplayer->connections);
+  conn_list_compression_thaw(pplayer->connections);
 }
 
 /**************************************************************************
@@ -841,10 +847,10 @@ void map_change_seen(struct player *pplayer,
 
     /* Fog the tile. */
     update_player_tile_last_seen(pplayer, ptile);
-    send_tile_info(pplayer->connections, ptile, FALSE);
     if (game.server.foggedborders) {
       plrtile->owner = tile_owner(ptile);
     }
+    send_tile_info(pplayer->connections, ptile, FALSE);
   }
 
   if ((revealing_tile && 0 < plrtile->seen_count[V_MAIN])
@@ -1000,45 +1006,78 @@ void player_map_init(struct player *pplayer)
   dbv_init(&pplayer->tile_known, MAP_INDEX_SIZE);
 }
 
-/***************************************************************
- frees a player's private map.
-***************************************************************/
+/**************************************************************************
+  Free a player's private map.
+**************************************************************************/
 void player_map_free(struct player *pplayer)
 {
   if (!pplayer->server.private_map) {
     return;
   }
 
-  /* only after removing borders! */
-  whole_map_iterate(ptile) {
-    /* Clear player vision. */
-    change_playertile_site(map_get_player_tile(ptile, pplayer), NULL);
-
-    /* Clear other players knowledge about the removed one. */
-    players_iterate(aplayer) {
-      struct player_tile *aplrtile = map_get_player_tile(ptile, aplayer);
-
-      if (aplrtile && aplrtile->site &&
-          vision_site_owner(aplrtile->site) == pplayer) {
-        change_playertile_site(aplrtile, NULL);
-      }
-    } players_iterate_end;
-
-    /* clear players knowledge */
-    map_clear_known(ptile, pplayer);
-
-    /* Free all claimed tiles. */
-    if (tile_owner(ptile) == pplayer) {
-      tile_set_owner(ptile, NULL, NULL);
-    }
-  } whole_map_iterate_end;
-
   free(pplayer->server.private_map);
   pplayer->server.private_map = NULL;
 
   dbv_free(&pplayer->tile_known);
 }
 
+/**************************************************************************
+  Remove all knowledge of a player from main map and other players'
+  private maps, and send updates to connected clients.
+  Frees all vision_sites associated with that player.
+**************************************************************************/
+void remove_player_from_maps(struct player *pplayer)
+{
+  /* only after removing borders! */
+  conn_list_do_buffer(game.est_connections);
+  whole_map_iterate(ptile) {
+    /* Clear all players' knowledge about the removed player, and free
+     * data structures (including those in removed player's player map). */
+    players_iterate(aplayer) {
+      struct player_tile *aplrtile;
+      bool changed = FALSE;
+
+      if (!aplayer->server.private_map) {
+        continue;
+      }
+      aplrtile = map_get_player_tile(ptile, aplayer);
+
+      /* Free vision sites (cities) for removed and other players */
+      if (aplrtile && aplrtile->site &&
+          vision_site_owner(aplrtile->site) == pplayer) {
+        change_playertile_site(aplrtile, NULL);
+        changed = TRUE;
+      }
+
+      /* Remove references to player from others' maps */
+      if (aplrtile->owner == pplayer) {
+        aplrtile->owner = NULL;
+        changed = TRUE;
+      }
+
+      /* Must ensure references to dying player are gone from clients
+       * before player is destroyed */
+      if (changed) {
+        /* This will use player tile if fogged */
+        send_tile_info(pplayer->connections, ptile, FALSE);
+      }
+    } players_iterate_end;
+
+    /* Clear removed player's knowledge */
+    if (pplayer->tile_known.vec) {
+      map_clear_known(ptile, pplayer);
+    }
+
+    /* Free all claimed tiles. */
+    if (tile_owner(ptile) == pplayer) {
+      tile_set_owner(ptile, NULL, NULL);
+      /* Update anyone who can see the tile (e.g. global observers) */
+      send_tile_info(NULL, ptile, FALSE);
+    }
+  } whole_map_iterate_end;
+  conn_list_do_unbuffer(game.est_connections);
+}
+
 /***************************************************************
   We need to use fogofwar_old here, so the player's tiles get
   in the same state as the other players' tiles.
@@ -1106,20 +1145,23 @@ struct player_tile *map_get_player_tile(const struct tile *ptile,
 bool update_player_tile_knowledge(struct player *pplayer, struct tile *ptile)
 {
   struct player_tile *plrtile = map_get_player_tile(ptile, pplayer);
-  struct player *owner = (game.server.foggedborders
-                          && !map_is_known_and_seen(ptile, pplayer, V_MAIN)
-                          ? plrtile->owner
-                          : tile_owner(ptile));
+  bool plrtile_owner_valid = game.server.foggedborders
+                             && !map_is_known_and_seen(ptile, pplayer, V_MAIN);
+  struct player *owner = plrtile_owner_valid
+                         ? plrtile->owner
+                         : tile_owner(ptile);
 
   if (plrtile->terrain != ptile->terrain
       || !BV_ARE_EQUAL(plrtile->special, ptile->special)
       || plrtile->resource != ptile->resource
-      || plrtile->owner != owner
+      || owner != tile_owner(ptile)
       || !BV_ARE_EQUAL(plrtile->bases, ptile->bases)) {
     plrtile->terrain = ptile->terrain;
     plrtile->special = ptile->special;
     plrtile->resource = ptile->resource;
-    plrtile->owner = owner;
+    if (plrtile_owner_valid) {
+      plrtile->owner = tile_owner(ptile);
+    }
     plrtile->bases = ptile->bases;
     return TRUE;
   }
