@@ -1501,6 +1501,10 @@ void server_remove_player(struct player *pplayer)
   /* We have to clear all player data before the ai memory is freed because
    * some function may depend on it. */
   player_clear(pplayer, TRUE);
+
+  if (!map_is_empty()) {
+    remove_player_from_maps(pplayer);
+  }
   player_map_free(pplayer);
 
   /* Destroy advisor and ai data. */
@@ -1814,7 +1818,7 @@ void make_contact(struct player *pplayer1, struct player *pplayer2,
     if (pplayer1->ai_controlled) {
       call_first_contact(pplayer1, pplayer2);
     }
-    if (pplayer2->ai_controlled && !pplayer1->ai_controlled) {
+    if (pplayer2->ai_controlled) {
       call_first_contact(pplayer2, pplayer1);
     }
     return;
@@ -2176,7 +2180,6 @@ static struct player *split_player(struct player *pplayer)
 
   cplayer->ai_controlled = TRUE;
   cplayer->ai_common.maxbuycost = pplayer->ai_common.maxbuycost;
-  cplayer->ai_common.handicaps = pplayer->ai_common.handicaps;
   cplayer->ai_common.warmth = pplayer->ai_common.warmth;
   cplayer->ai_common.frost = pplayer->ai_common.frost;
   set_ai_level_direct(cplayer, game.info.skill_level);
