@@ -16,12 +16,20 @@
 struct section_file;
 struct connection;
 struct conn_list;
+struct rgbcolor;
+struct player;
 
 enum plr_info_level { INFO_MINIMUM, INFO_MEETING, INFO_EMBASSY, INFO_FULL };
 
-void server_player_init(struct player *pplayer,
-			bool initmap, bool needs_team);
-struct player *server_create_player(int player_id);
+struct player *server_create_player(int player_id, const char *ai_type,
+                                    struct rgbcolor *prgbcolor);
+const struct rgbcolor *player_preferred_color(struct player *pplayer);
+void assign_player_colors(void);
+void server_player_set_color(struct player *pplayer,
+                             const struct rgbcolor *prgbcolor);
+const char *player_color_ftstr(struct player *pplayer);
+void server_player_init(struct player *pplayer, bool initmap,
+                        bool needs_team);
 void server_remove_player(struct player *pplayer);
 void kill_player(struct player *pplayer);
 void update_revolution(struct player *pplayer);
@@ -46,6 +54,9 @@ void make_contact(struct player *pplayer1, struct player *pplayer2,
 void maybe_make_contact(struct tile *ptile, struct player *pplayer);
 void enter_war(struct player *pplayer, struct player *pplayer2);
 
+void player_info_freeze(void);
+void player_info_thaw(void);
+
 void send_player_all_c(struct player *src, struct conn_list *dest);
 void send_player_info_c(struct player *src, struct conn_list *dest);
 void send_player_diplstate_c(struct player *src, struct conn_list *dest);
@@ -62,7 +73,7 @@ do {\
   int MY_i;\
   struct player *NAME_pplayer;\
   log_debug("shuffled_players_iterate @ %s line %d",\
-            __FILE__, __LINE__);\
+            __FILE__, __FC_LINE__);\
   for (MY_i = 0; MY_i < player_slot_count(); MY_i++) {\
     NAME_pplayer = shuffled_player(MY_i);\
     if (NAME_pplayer != NULL) {\
@@ -82,8 +93,10 @@ do {\
   } shuffled_players_iterate_end;\
 } while (0)
 
+bool civil_war_possible(struct player *pplayer, bool conquering_city,
+                        bool honour_server_option);
 bool civil_war_triggered(struct player *pplayer);
-void civil_war(struct player *pplayer);
+struct player *civil_war(struct player *pplayer);
 
 void update_players_after_alliance_breakup(struct player* pplayer,
                                           struct player* pplayer2);
@@ -96,4 +109,20 @@ void player_status_add(struct player *plr, enum player_status status);
 bool player_status_check(struct player *plr, enum player_status status);
 void player_status_reset(struct player *plr);
 
+const char *player_delegation_get(const struct player *pplayer);
+void player_delegation_set(struct player *pplayer, const char *username);
+bool player_delegation_active(const struct player *pplayer);
+
+void send_delegation_info(const struct connection *pconn);
+
+struct player *player_by_user_delegated(const char *name);
+
+/* player colors */
+void playercolor_init(void);
+void playercolor_free(void);
+
+int playercolor_count(void);
+void playercolor_add(struct rgbcolor *prgbcolor);
+struct rgbcolor *playercolor_get(int id);
+
 #endif  /* FC__PLRHAND_H */
