@@ -13,22 +13,23 @@
 #ifndef FC__REPORT_H
 #define FC__REPORT_H
 
-#include "shared.h"		/* bool type */
+#include "support.h"            /* bool type */
 
 struct connection;
 struct conn_list;
 
-void page_conn(struct conn_list *dest, const char *caption, const char *headline,
-	       const char *lines);
+void page_conn(struct conn_list *dest, const char *caption,
+               const char *headline, const char *lines);
+
+void log_civ_score_init(void);
+void log_civ_score_free(void);
+void log_civ_score_now(void);
 
 void make_history_report(void);
 void report_wonders_of_the_world(struct conn_list *dest);
 void report_top_five_cities(struct conn_list *dest);
-bool is_valid_demography(const char *demography, const char **error_message);
+bool is_valid_demography(const char *demography, int *error);
 void report_demographics(struct connection *pconn);
-void report_progress_scores(void);
-void report_final_scores(void);
-
-/* See also report_server_options() in stdinhand.h */
+void report_final_scores(struct conn_list *dest);
 
 #endif  /* FC__REPORT_H */
