@@ -25,6 +25,6 @@ bool unit_move_handling(struct unit *punit, struct tile *pdesttile,
                         bool igzoc, bool move_diplomat_city);
 
 void city_add_or_build_error(struct player *pplayer, struct unit *punit,
-                             enum add_build_city_result res);
+                             enum unit_add_build_city_result res);
 
 #endif  /* FC__UNITHAND_H */
