@@ -18,13 +18,67 @@
 #include "packets.h"		/* enum unit_info_use */
 #include "unitlist.h"
 
+#define SPECENUM_NAME unit_loss_reason
+#define SPECENUM_VALUE0 ULR_KILLED
+#define SPECENUM_VALUE0NAME "killed"
+#define SPECENUM_VALUE1 ULR_RETIRED
+#define SPECENUM_VALUE1NAME "retired"
+#define SPECENUM_VALUE2 ULR_DISBANDED
+#define SPECENUM_VALUE2NAME "disbanded"
+#define SPECENUM_VALUE3 ULR_BARB_UNLEASH
+#define SPECENUM_VALUE3NAME "barb_unleash"
+#define SPECENUM_VALUE4 ULR_CITY_LOST
+#define SPECENUM_VALUE4NAME "city_lost"
+#define SPECENUM_VALUE5 ULR_STARVED
+#define SPECENUM_VALUE5NAME "starved"
+#define SPECENUM_VALUE6 ULR_SOLD
+#define SPECENUM_VALUE6NAME "sold"
+#define SPECENUM_VALUE7 ULR_USED
+#define SPECENUM_VALUE7NAME "used"
+#define SPECENUM_VALUE8 ULR_EXECUTED
+#define SPECENUM_VALUE8NAME "executed"
+#define SPECENUM_VALUE9 ULR_ELIMINATED
+#define SPECENUM_VALUE9NAME "eliminated"
+#define SPECENUM_VALUE10 ULR_EDITOR
+#define SPECENUM_VALUE10NAME "editor"
+#define SPECENUM_VALUE11 ULR_NONNATIVE_TERR
+#define SPECENUM_VALUE11NAME "nonnative_terr"
+#define SPECENUM_VALUE12 ULR_PLAYER_DIED
+#define SPECENUM_VALUE12NAME "player_died"
+#define SPECENUM_VALUE13 ULR_ARMISTICE
+#define SPECENUM_VALUE13NAME "armistice"
+#define SPECENUM_VALUE14 ULR_SDI
+#define SPECENUM_VALUE14NAME "sdi"
+#define SPECENUM_VALUE15 ULR_DETONATED
+#define SPECENUM_VALUE15NAME "detonated"
+#define SPECENUM_VALUE16 ULR_MISSILE
+#define SPECENUM_VALUE16NAME "missile"
+#define SPECENUM_VALUE17 ULR_NUKE
+#define SPECENUM_VALUE17NAME "nuke"
+#define SPECENUM_VALUE18 ULR_HP_LOSS
+#define SPECENUM_VALUE18NAME "hp_loss"
+#define SPECENUM_VALUE19 ULR_FUEL
+#define SPECENUM_VALUE19NAME "fuel"
+#define SPECENUM_VALUE20 ULR_STACK_CONFLICT
+#define SPECENUM_VALUE20NAME "stack_conflict"
+#define SPECENUM_VALUE21 ULR_BRIBED
+#define SPECENUM_VALUE21NAME "bribed"
+#define SPECENUM_VALUE22 ULR_CAPTURED
+#define SPECENUM_VALUE22NAME "captured"
+#define SPECENUM_VALUE23 ULR_CAUGHT
+#define SPECENUM_VALUE23NAME "caught"
+#define SPECENUM_VALUE24 ULR_TRANSPORT_LOST
+#define SPECENUM_VALUE24NAME "transport_lost"
+#include "specenum_gen.h"
+
 /* battle related */
 struct unit_type *find_a_unit_type(enum unit_role_id role,
 				   enum unit_role_id role_tech);
 bool maybe_make_veteran(struct unit *punit);
 void notify_unit_experience(struct unit *punit);
 void unit_versus_unit(struct unit *attacker, struct unit *defender,
-		      bool bombard);
+                      bool bombard, int *att_hp, int *def_hp);
+void combat_veterans(struct unit *attacker, struct unit *defender);
 
 /* move check related */
 bool is_unit_being_refueled(const struct unit *punit);
@@ -66,11 +120,13 @@ struct unit *create_unit_full(struct player *pplayer, struct tile *ptile,
 			      struct unit_type *punittype, int veteran_level,
 			      int homecity_id, int moves_left, int hp_left,
 			      struct unit *ptrans);
-void wipe_unit(struct unit *punit, bool count_lost, struct player *killer);
+void wipe_unit(struct unit *punit, enum unit_loss_reason reason,
+               struct player *killer);
 void kill_unit(struct unit *pkiller, struct unit *punit, bool vet);
 
 struct unit *unit_change_owner(struct unit *punit, struct player *pplayer,
-                               int homecity) fc__warn_unused_result;
+                               int homecity, enum unit_loss_reason reason)
+                               fc__warn_unused_result;
 
 /* sending to client */
 void package_unit(struct unit *punit, struct packet_unit_info *packet);
@@ -80,7 +136,8 @@ void package_short_unit(struct unit *punit,
 			bool new_serial_num);
 void send_unit_info(struct player *dest, struct unit *punit);
 void send_unit_info_to_onlookers(struct conn_list *dest, struct unit *punit, 
-				 struct tile *ptile, bool remove_unseen);
+				 struct tile *ptile, bool remove_unseen,
+                                 bool remove_moving);
 void send_all_known_units(struct conn_list *dest);
 void unit_goes_out_of_sight(struct player *pplayer, struct unit *punit);
 
@@ -89,9 +146,9 @@ void do_nuclear_explosion(struct player *pplayer, struct tile *ptile);
 bool do_airline(struct unit *punit, struct city *city2);
 void do_explore(struct unit *punit);
 bool do_paradrop(struct unit *punit, struct tile *ptile);
-void load_unit_onto_transporter(struct unit *punit, struct unit *ptrans);
-void unload_unit_from_transporter(struct unit *punit);
-bool move_unit(struct unit *punit, struct tile *ptile, int move_cost);
+void unit_transport_load_send(struct unit *punit, struct unit *ptrans);
+void unit_transport_unload_send(struct unit *punit);
+bool unit_move(struct unit *punit, struct tile *ptile, int move_cost);
 bool execute_orders(struct unit *punit);
 
 bool unit_can_do_action_now(const struct unit *punit);
