@@ -59,7 +59,7 @@
 ****************************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
 #include <string.h>
@@ -68,6 +68,7 @@
 #include "log.h"
 #include "mem.h"
 #include "shared.h"     /* ARRAY_SIZE */
+#include "support.h"
 
 #include "genhash.h"
 
@@ -162,7 +163,7 @@ void genhash_str_free_func(void *vkey)
 ***************************************************************************/
 genhash_val_t genhash_ptr_val_func(const void *vkey, size_t num_buckets)
 {
-  unsigned long result = ((unsigned long) vkey);
+  intptr_t result = ((intptr_t) vkey);
   return (result % num_buckets);
 }
 
