@@ -620,7 +620,8 @@ const char *fc_url_encode(const char *txt)
 /************************************************************************** 
   Finds the next (lowest) free port.
 **************************************************************************/ 
-int find_next_free_port(int starting_port, enum fc_addr_family family)
+int find_next_free_port(int starting_port, enum fc_addr_family family,
+                        char *net_interface)
 {
   int port;
   int s;
@@ -666,7 +667,7 @@ int find_next_free_port(int starting_port, enum fc_addr_family family)
     hints.ai_socktype = SOCK_DGRAM;
     hints.ai_flags = AI_PASSIVE | FC_AI_NUMERICSERV;
 
-    err = getaddrinfo(NULL, servname, &hints, &res);
+    err = getaddrinfo(net_interface, servname, &hints, &res);
     if (!err) {
       struct addrinfo *current = res;
 
@@ -687,7 +688,25 @@ int find_next_free_port(int starting_port, enum fc_addr_family family)
     memset(&tmp, 0, sizeof(tmp));
     sock4->sin_family = AF_INET;
     sock4->sin_port = htons(port);
-    sock4->sin_addr.s_addr = htonl(INADDR_ANY);
+    if (net_interface != NULL) {
+#if defined(HAVE_INET_ATON)
+      if (inet_aton(net_interface, &sock4->sin_addr) == 0) {
+#else /* HAVE_INET_ATON */
+      sock4->sin_addr.s_addr = inet_addr(net_interface);
+      if (sock4->sin_addr.s_addr == INADDR_NONE) {
+#endif /* HAVE_INET_ATON */
+        struct hostent *hp;
+
+        hp = gethostbyname(net_interface);
+        if (!hp || hp->h_addrtype != AF_INET) {
+          return -1;
+        }
+
+        memcpy(&sock4->sin_addr, hp->h_addr, hp->h_length);
+      }
+    } else {
+      sock4->sin_addr.s_addr = htonl(INADDR_ANY);
+    }
 
     if (bind(s, &tmp.saddr, sockaddr_size(&tmp)) == 0) {
       found = TRUE;
@@ -695,6 +714,10 @@ int find_next_free_port(int starting_port, enum fc_addr_family family)
 #endif /* HAVE_GETADDRINFO */
   }
 
+  /* Rollback the last increment from the loop, back to port
+   * number found to be free. */
+  port--;
+
   fc_closesocket(s);
   
   return port;
