@@ -126,7 +126,8 @@ void fc_nonblock(int sockfd);
 struct fc_sockaddr_list *net_lookup_service(const char *name, int port,
 					    enum fc_addr_family family);
 fz_FILE *fc_querysocket(int sock, void *buf, size_t size);
-int find_next_free_port(int starting_port, enum fc_addr_family family);
+int find_next_free_port(int starting_port, enum fc_addr_family family,
+                        char *net_interface);
 
 const char *fc_lookup_httpd(char *server, int *port, const char *url);
 const char *fc_url_encode(const char *txt);
