@@ -32,22 +32,21 @@
 *************************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include <fc_config.h>
 #endif
 
-#include <assert.h>
-
+/* utility */
 #include "log.h"
-#include "shared.h"		/* TRUE, FALSE */
+#include "support.h"            /* TRUE, FALSE */
 
 #include "rand.h"
 
-#define LOG_RAND LOG_DEBUG
+#define log_rand log_debug
 
 /* A global random state:
- * Initialized by mysrand(), updated by myrand(),
- * Can be duplicated/saved/restored via get_myrand_state()
- * and set_myrand_state().
+ * Initialized by fc_srand(), updated by fc_rand(),
+ * Can be duplicated/saved/restored via fc_rand_state()
+ * and fc_rand_set_state().
  */
 static RANDOM_STATE rand_state;
 
@@ -78,13 +77,13 @@ static RANDOM_STATE rand_state;
   directly representable in type RANDOM_TYPE, so we do instead:
          divisor = MAX_UINT32/size
 *************************************************************************/
-RANDOM_TYPE myrand_debug(RANDOM_TYPE size,
-		     const char *called_as, int line, const char *file) 
+RANDOM_TYPE fc_rand_debug(RANDOM_TYPE size, const char *called_as,
+                          int line, const char *file) 
 {
   RANDOM_TYPE new_rand, divisor, max;
   int bailout = 0;
 
-  assert(rand_state.is_init);
+  fc_assert_ret_val(rand_state.is_init, 0);
 
   if (size > 1) {
     divisor = MAX_UINT32 / size;
@@ -110,8 +109,9 @@ RANDOM_TYPE myrand_debug(RANDOM_TYPE size,
     rand_state.v[rand_state.x] = new_rand;
 
     if (++bailout > 10000) {
-      freelog(LOG_ERROR, "%s(%lu) = %lu bailout at line %d of %s", 
-	    called_as, (unsigned long)size, (unsigned long)new_rand, line, file);
+      log_error("%s(%lu) = %lu bailout at %s:%d", 
+                called_as, (unsigned long) size,
+                (unsigned long) new_rand, file, line);
       new_rand = 0;
       break;
     }
@@ -124,8 +124,9 @@ RANDOM_TYPE myrand_debug(RANDOM_TYPE size,
     new_rand = 0;
   }
 
-  freelog(LOG_RAND, "%s(%lu) = %lu at line %d of %s",
-	    called_as, (unsigned long)size, (unsigned long)new_rand, line, file);
+  log_rand("%s(%lu) = %lu at %s:%d",
+           called_as, (unsigned long) size,
+           (unsigned long) new_rand, file, line);
 
   return new_rand;
 } 
@@ -133,7 +134,7 @@ RANDOM_TYPE myrand_debug(RANDOM_TYPE size,
 /*************************************************************************
   Initialize the generator; see comment at top of file.
 *************************************************************************/
-void mysrand(RANDOM_TYPE seed) 
+void fc_srand(RANDOM_TYPE seed) 
 { 
     int  i; 
 
@@ -150,21 +151,21 @@ void mysrand(RANDOM_TYPE seed)
     rand_state.is_init = TRUE;
 
     /* Heat it up a bit:
-     * Using modulus in myrand() this was important to pass
-     * test_random1().  Now using divisor in myrand() that particular
+     * Using modulus in fc_rand() this was important to pass
+     * test_random1().  Now using divisor in fc_rand() that particular
      * test no longer indicates problems, but this seems a good idea
      * anyway -- eg, other tests could well reveal other initial
      * problems even using divisor.
      */
     for (i=0; i<10000; i++) {
-      (void) myrand(MAX_UINT32);
+      (void) fc_rand(MAX_UINT32);
     }
 } 
 
 /*************************************************************************
   Return whether the current state has been initialized.
 *************************************************************************/
-bool myrand_is_init(void)
+bool fc_rand_is_init(void)
 {
   return rand_state.is_init;
 }
@@ -172,18 +173,18 @@ bool myrand_is_init(void)
 /*************************************************************************
   Return a copy of the current rand_state; eg for save/restore.
 *************************************************************************/
-RANDOM_STATE get_myrand_state(void)
+RANDOM_STATE fc_rand_state(void)
 {
   int i;
 
-  freelog(LOG_RAND, "get_myrand_state J=%d K=%d X=%d",
-    rand_state.j, rand_state.k, rand_state.x);
+  log_rand("fc_rand_state J=%d K=%d X=%d",
+           rand_state.j, rand_state.k, rand_state.x);
   for (i  = 0; i < 8; i++) {
-    freelog(LOG_RAND, "get_myrand_state %d, %08x %08x %08x %08x %08x %08x %08x",
-      i, rand_state.v[7 * i],
-      rand_state.v[7 * i + 1], rand_state.v[7 * i + 2],
-      rand_state.v[7 * i + 3], rand_state.v[7 * i + 4],
-      rand_state.v[7 * i + 5], rand_state.v[7 * i + 6]);
+    log_rand("fc_rand_state %d, %08x %08x %08x %08x %08x %08x %08x",
+             i, rand_state.v[7 * i],
+             rand_state.v[7 * i + 1], rand_state.v[7 * i + 2],
+             rand_state.v[7 * i + 3], rand_state.v[7 * i + 4],
+             rand_state.v[7 * i + 5], rand_state.v[7 * i + 6]);
   }
 
   return rand_state;
@@ -193,26 +194,26 @@ RANDOM_STATE get_myrand_state(void)
   Replace current rand_state with user-supplied; eg for save/restore.
   Caller should take care to set state.is_init beforehand if necessary.
 *************************************************************************/
-void set_myrand_state(RANDOM_STATE state)
+void fc_rand_set_state(RANDOM_STATE state)
 {
   int i;
 
   rand_state = state;
 
-  freelog(LOG_RAND, "set_myrand_state J=%d K=%d X=%d",
-    rand_state.j, rand_state.k, rand_state.x);
+  log_rand("fc_rand_set_state J=%d K=%d X=%d",
+           rand_state.j, rand_state.k, rand_state.x);
   for (i  = 0; i < 8; i++) {
-    freelog(LOG_RAND, "set_myrand_state %d, %08x %08x %08x %08x %08x %08x %08x",
-      i, rand_state.v[7 * i],
-      rand_state.v[7 * i + 1], rand_state.v[7 * i + 2],
-      rand_state.v[7 * i + 3], rand_state.v[7 * i + 4],
-      rand_state.v[7 * i + 5], rand_state.v[7 * i + 6]);
+    log_rand("fc_rand_set_state %d, %08x %08x %08x %08x %08x %08x %08x",
+             i, rand_state.v[7 * i],
+             rand_state.v[7 * i + 1], rand_state.v[7 * i + 2],
+             rand_state.v[7 * i + 3], rand_state.v[7 * i + 4],
+             rand_state.v[7 * i + 5], rand_state.v[7 * i + 6]);
   }
 }
 
 /*************************************************************************
   Test one aspect of randomness, using n numbers.
-  Reports results to LOG_NORMAL; with good randomness, behaviourchange
+  Reports results to LOG_TEST; with good randomness, behaviourchange
   and behavioursame should be about the same size.
   Tests current random state; saves and restores state, so can call
   without interrupting current sequence.
@@ -224,11 +225,11 @@ void test_random1(int n)
   bool didchange, olddidchange = FALSE;
   int behaviourchange = 0, behavioursame = 0;
 
-  saved_state = get_myrand_state();
-  /* mysrand(time(NULL)); */  /* use current state */
+  saved_state = fc_rand_state();
+  /* fc_srand(time(NULL)); */  /* use current state */
 
   for (i = 0; i < n+2; i++) {
-    new_value = myrand(2);
+    new_value = fc_rand(2);
     if (i > 0) {		/* have old */
       didchange = (new_value != old_value);
       if (i > 1) {		/* have olddidchange */
@@ -242,9 +243,38 @@ void test_random1(int n)
     }
     old_value = new_value;
   }
-  freelog(LOG_NORMAL, "test_random1(%d) same: %d, change: %d",
-	  n, behavioursame, behaviourchange);
+  log_test("test_random1(%d) same: %d, change: %d",
+           n, behavioursame, behaviourchange);
 
   /* restore state: */
-  set_myrand_state(saved_state);
+  fc_rand_set_state(saved_state);
+}
+
+/*************************************************************************
+  Local pseudo-random function for repeatedly reaching the same result,
+  instead of fc_rand().  Primarily needed for tiles.
+
+  Use an invariant equation for seed.
+  Result is 0 to (size - 1).
+*************************************************************************/
+RANDOM_TYPE fc_randomly_debug(RANDOM_TYPE seed, RANDOM_TYPE size,
+                              const char *called_as,
+                              int line, const char *file)
+{
+  RANDOM_TYPE result;
+
+#define LARGE_PRIME (10007)
+#define SMALL_PRIME (1009)
+
+  /* Check for overflow and underflow */
+  fc_assert_ret_val(seed < MAX_UINT32 / LARGE_PRIME, 0);
+  fc_assert_ret_val(size < SMALL_PRIME, 0);
+  fc_assert_ret_val(size > 0, 0);
+  result = ((seed * LARGE_PRIME) % SMALL_PRIME) % size;
+
+  log_rand("%s(%lu,%lu) = %lu at %s:%d",
+           called_as, (unsigned long) seed, (unsigned long) size,
+           (unsigned long) result, file, line);
+
+  return result;
 }
