@@ -930,8 +930,9 @@ static struct strvec *base_get_dirs(const char *dir_list)
 /***************************************************************************
   Returns a list of data directory paths, in the order in which they should
   be searched.  These paths are specified internally or may be set as the
-  environment variable $FREECIV_PATH (a separated list of directories,
+  environment variable $FREECIV_DATA PATH (a separated list of directories,
   where the separator itself is specified internally, platform-dependent).
+  FREECIV_PATH may also be consulted for backward compatibility.
   '~' at the start of a component (provided followed by '/' or '\0') is
   expanded as $HOME.
 
@@ -974,8 +975,9 @@ const struct strvec *get_data_dirs(void)
 /***************************************************************************
   Returns a list of save directory paths, in the order in which they should
   be searched.  These paths are specified internally or may be set as the
-  environment variable $FREECIV_PATH (a separated list of directories,
+  environment variable $FREECIV_SAVE_PATH (a separated list of directories,
   where the separator itself is specified internally, platform-dependent).
+  FREECIV_PATH may also be consulted for backward compatibility.
   '~' at the start of a component (provided followed by '/' or '\0') is
   expanded as $HOME.
 
@@ -1034,9 +1036,10 @@ const struct strvec *get_save_dirs(void)
 /***************************************************************************
   Returns a list of scenario directory paths, in the order in which they
   should be searched.  These paths are specified internally or may be set
-  as the environment variable $FREECIV_PATH (a separated list of
+  as the environment variable $FREECIV_SCENARIO_PATH (a separated list of
   directories, where the separator itself is specified internally,
-  platform-dependent).  '~' at the start of a component (provided followed
+  platform-dependent).  FREECIV_PATH may also be consulted for backward
+  compatibility.  '~' at the start of a component (provided followed
   by '/' or '\0') is expanded as $HOME.
 
   The returned pointer is static and shouldn't be modified, nor destroyed
@@ -1337,30 +1340,6 @@ struct fileinfo_list *fileinfolist_infix(const struct strvec *dirs,
   return res;
 }
 
-/***************************************************************************
-  As datafilename(), above, except die with an appropriate log
-  message if we can't find the file in the datapath.
-***************************************************************************/
-const char *fileinfoname_required(const struct strvec *dirs,
-                                  const char *filename)
-{
-  const char *dname;
-
-  fc_assert_exit(NULL != filename);
-  dname = fileinfoname(dirs, filename);
-
-  if (dname) {
-    return dname;
-  } else {
-    /* TRANS: <FREECIV_PATH> configuration error */
-    log_error(_("The path may be set via the \"%s\" environment variable."),
-              FREECIV_PATH);
-    log_error(_("Current path is: \"%s\""), fileinfoname(dirs, NULL));
-    log_fatal(_("The \"%s\" file is required ... aborting!"), filename);
-    exit(EXIT_FAILURE);
-  }
-}
-
 /***************************************************************************
   Language environmental variable (with emulation).
 ***************************************************************************/
@@ -2135,7 +2114,7 @@ int fc_vsnprintcf(char *buf, size_t buf_len, const char *format,
   const char *f = format;
   char *const max = buf + buf_len - 1;
   char *b = buf, *c;
-  const char *const cmax = cformat + sizeof(cformat - 2);
+  const char *const cmax = cformat + sizeof(cformat) - 2;
   int i, j;
 
   if ((size_t) -1 == sequences_num) {
