@@ -26,8 +26,8 @@ extern "C" {
 #include "support.h" /* bool, fc__attribute */
 
 #ifdef HAVE_CONFIG_H
-#ifndef FC_CONFIG_H            /* this should be defined in config.h */
-#error Files including fcintl.h should also include config.h directly
+#ifndef FC_CONFIG_H  /* this should be defined in fc_config.h */
+#error Files including fcintl.h should also include fc_config.h directly
 #endif
 #endif
 
@@ -163,10 +163,6 @@ struct strvec *fileinfolist(const struct strvec *dirs, const char *suffix);
 struct fileinfo_list *fileinfolist_infix(const struct strvec *dirs,
                                          const char *infix, bool nodups);
 const char *fileinfoname(const struct strvec *dirs, const char *filename);
-struct strvec *fileinfonames(const struct strvec *dirs,
-                             const char *filename);
-const char *fileinfoname_required(const struct strvec *dirs,
-                                  const char *filename);
 
 char *get_langname(void);
 void init_nls(void);
