@@ -101,6 +101,10 @@
  *   fghdf is not a valid name
  */
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 #include "log.h"        /* fc_assert. */
 #include "support.h"    /* bool type. */
 
@@ -117,7 +121,7 @@
 #define SPECENUM_FOO(suffix) SPECENUM_PASTE(SPECENUM_NAME, suffix)
 
 #ifndef SPECENUM_INVALID
-#define SPECENUM_INVALID (-1)
+#define SPECENUM_INVALID ((enum SPECENUM_NAME) -1)
 #endif
 
 #ifdef SPECENUM_BITWISE
@@ -2101,9 +2105,9 @@ static inline enum SPECENUM_NAME SPECENUM_FOO(_next)(enum SPECENUM_NAME e)
 {
   do {
 #ifdef SPECENUM_BITWISE
-    e <<= 1;
+    e = (enum SPECENUM_NAME)(e << 1);
 #else
-    e++;
+    e = (enum SPECENUM_NAME)(e + 1);
 #endif
 
     if (e > SPECENUM_FOO(_max)()) {
@@ -3552,3 +3556,7 @@ static inline enum SPECENUM_NAME SPECENUM_FOO(_by_name)
 #undef SPECENUM_VALUE122NAME
 #undef SPECENUM_VALUE123NAME
 #undef SPECENUM_VALUE124NAME
+
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
