@@ -36,11 +36,12 @@ OutFile "Output/\${APPNAME}-\${VERSION}-win32-\${GUI_ID}-setup.exe"
 
 Var STARTMENU_FOLDER
 Var DefaultLanguageCode
+Var LangName
 
 ; Pages
 
 !insertmacro MUI_PAGE_WELCOME
-!insertmacro MUI_PAGE_LICENSE "$1\COPYING"
+!insertmacro MUI_PAGE_LICENSE "$1\doc\freeciv\COPYING"
 !insertmacro MUI_PAGE_COMPONENTS
 Page custom DefaultLanguage DefaultLanguageLeave
 !insertmacro MULTIUSER_PAGE_INSTALLMODE
@@ -166,14 +167,14 @@ SectionGroup "Additional languages"
 
 EOF
 
-find $1/share/locale -mindepth 1 -maxdepth 1 -type d -printf %f\\n |
-sort |
-while read -r name
+cat ../../bootstrap/langnames.txt |
+sort -k 2 |
+while read -r code name
 do
-if test -e $1/share/locale/$name/LC_MESSAGES/freeciv.mo; then
-echo "  Section \"$name\""
-echo "  SetOutPath \$INSTDIR\\share\\locale\\$name"
-echo "  File /r $1\\share\\locale\\$name\*.*"
+if test -e $1/share/locale/$code/LC_MESSAGES/freeciv.mo; then
+echo "  Section \"$name ($code)\""
+echo "  SetOutPath \$INSTDIR\\share\\locale\\$code"
+echo "  File /r $1\\share\\locale\\$code\*.*"
 
 # install special fonts for CJK locales
 if [ "$name" = "zh_CN" ]; then
@@ -239,16 +240,17 @@ Start Menu shortcut properties."
   \${NSD_CreateDropList} 0 -60% 100% 13u ""
   Pop \$DefaultLanguageDropList
 
-  \${NSD_CB_AddString} \$DefaultLanguageDropList "auto"
-  \${NSD_CB_SelectString} \$DefaultLanguageDropList "auto"
+  \${NSD_CB_AddString} \$DefaultLanguageDropList "Autodetected"
+  \${NSD_CB_SelectString} \$DefaultLanguageDropList "Autodetected"
+  \${NSD_CB_AddString} \$DefaultLanguageDropList "US English (en_US)"
 EOF
 
-  find $1/share/locale -mindepth 1 -maxdepth 1 -type d -printf %f\\n |
-  sort |
-  while read -r name
+  cat ../../bootstrap/langnames.txt |
+  sort -k 2 |
+  while read -r code name
   do
-  if test -e $1/share/locale/$name/LC_MESSAGES/freeciv.mo; then
-  echo "  \${NSD_CB_AddString} \$DefaultLanguageDropList \"$name\""
+  if test -e $1/share/locale/$code/LC_MESSAGES/freeciv.mo; then
+  echo "  \${NSD_CB_AddString} \$DefaultLanguageDropList \"$name ($code)\""
   fi
   done
 
@@ -257,7 +259,25 @@ cat <<EOF
 FunctionEnd
 
 Function DefaultLanguageLeave
-  \${NSD_GetText} \$DefaultLanguageDropList \$DefaultLanguageCode
+  \${NSD_GetText} \$DefaultLanguageDropList \$LangName
+EOF
+
+  echo "  \${If} \$LangName == \"Autodetected\""
+  echo "    StrCpy \$DefaultLanguageCode \"auto\""
+  echo "  \${EndIf}"
+  echo "  \${If} \$LangName == \"US English (en_US)\""
+  echo "    StrCpy \$DefaultLanguageCode \"en_US\""
+  echo "  \${EndIf}"
+
+  cat ../../bootstrap/langnames.txt |
+  while read -r code name
+  do
+    echo "  \${If} \$LangName == \"$name ($code)\""
+    echo "    StrCpy \$DefaultLanguageCode \"$code\""
+    echo "  \${EndIf}"
+  done
+
+cat <<EOF
 FunctionEnd
 
 Function RunFreeciv
